unit fMHA_BatteryWizard;


interface

uses
  Windows,
  Messages,
  SysUtils,
  Classes,
  Graphics,
  Controls,
  Forms,
  ExtCtrls,
  StdCtrls,
  Grids,
  Menus,
  Buttons,
  ComCtrls,
  ImgList,
  RzLabel,
  RzPanel,
  ORCtrls,
  VirtualTrees,
  JvExControls,
  JvComponent,
  JvXPCore,
  JvXPButtons;

type

  TfrmMHA_BatteryWizard = class(TForm)
    pnlLists: TPanel;
    grpAvailable: TGroupBox;
    pnlSpacer0: TPanel;
    pnlLeft: TPanel;
    pnlSpacer5: TPanel;
    pnlSpacer4: TPanel;
    pnlRight: TPanel;
    pnlSpacer3: TPanel;
    btnUp: TBitBtn;
    btnDown: TBitBtn;
    pnlButton: TPanel;
    grpPickedInstruments: TGroupBox;
    btnRemove: TBitBtn;
    grpInstructions: TGroupBox;
    lblInstructions: TLabel;
    mnuMain: TMainMenu;
    mnuFile: TMenuItem;
    mnuExit: TMenuItem;
    ImageList1: TImageList;
    lstChosen: TListView;
    PopupMenu1: TPopupMenu;
    mnuDescription: TMenuItem;
    mnuNew: TMenuItem;
    pnlSpacer1: TPanel;
    pnlHeader: TRzPanel;
    lblHeader: TRzLabel;
    pnlBorder: TRzPanel;
    N1: TMenuItem;
    mnuRename: TMenuItem;
    mnuDelete: TMenuItem;
    mnuHelp: TMenuItem;
    mnuOnlineSupport: TMenuItem;
    N2: TMenuItem;
    mnuAbout: TMenuItem;
    N3: TMenuItem;
    mnuInstrumentDescription: TMenuItem;
    grpBatteryName: TGroupBox;
    cbxBatteryName: TComboBox;
    btnSave: TJvXPButton;
    imgBackup: TImage;
    treSelect: TVirtualStringTree;
    procedure btnSaveClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ConfigureForm;

    procedure DisplayAvailableInstruments;
    procedure LoadImageLists;
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure btnRemoveClick(Sender: TObject);
    procedure lstChosenRedundant;
    procedure UpdateButtons;
    procedure btnUpClick(Sender: TObject);
    procedure btnDownClick(Sender: TObject);
    procedure lstChosenEnter(Sender: TObject);
    procedure lstChosenDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure lstChosenDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure lstChosenMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure mnuDescriptionClick(Sender: TObject);
    procedure cbxBatteryNameChange(Sender: TObject);
    procedure BuildBattery;
    procedure DisplayBatteryNameChange;
    procedure cbxBatteryNameKeyPress(Sender: TObject; var Key: Char);
    procedure mnuNewClick(Sender: TObject);
    procedure mnuExitClick(Sender: TObject);
    procedure mnuDeleteClick(Sender: TObject);
    procedure mnuOnlineSupportClick(Sender: TObject);
    procedure mnuAboutClick(Sender: TObject);
    procedure cbxBatteryNameEnter(Sender: TObject);
    procedure btnSaveEnter(Sender: TObject);
    procedure grpBatteryNameMouseMove(Sender: TObject; Shift: TShiftState;
      X, Y: Integer);
    procedure grpPickedInstrumentsMouseMove(Sender: TObject;
      Shift: TShiftState; X, Y: Integer);
    procedure pnlButtonMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure lblInstructionsMouseMove(Sender: TObject; Shift: TShiftState;
      X, Y: Integer);
    procedure lstChosenMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure lstChosenSelectItem(Sender: TObject; Item: TListItem;
      Selected: Boolean);
    procedure mnuRenameClick(Sender: TObject);
    procedure PopulateComboBox;

    procedure treSelectGetText(Sender: TBaseVirtualTree;
      Node: PVirtualNode; Column: TColumnIndex; TextType: TVSTTextType;
      var CellText: WideString);
    function treSelectGetChildCount(NodeIn: string): Integer;
    function treSelectGetHasChildren(NodeName: string): Boolean;
    procedure treSelectInitChildren(Sender: TBaseVirtualTree;
      Node: PVirtualNode; var ChildCount: Cardinal);
    function treSelectGetItems(PNode: string; ChildNumber: integer): string;
    procedure treSelectInitNode(Sender: TBaseVirtualTree; ParentNode, Node:
      PVirtualNode; var InitialStates: TVirtualNodeInitStates);
    procedure treSelectChecked(Sender: TBaseVirtualTree;
      Node: PVirtualNode);
    procedure treSelectCheckThisNode(ThisNode: string);
    procedure treSelectUnCheckThisNode(ThisNode: string);
    procedure treSelectEnter(Sender: TObject);
    procedure treSelectMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure treSelectUnCheck;
    procedure treSelectGetImageIndex(Sender: TBaseVirtualTree;
      Node: PVirtualNode; Kind: TVTImageKind; Column: TColumnIndex;
      var Ghosted: Boolean; var ImageIndex: Integer);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure treSelectClick(Sender: TObject);
    procedure cbxBatteryNameSelect(Sender: TObject);
    procedure imgBackupClick(Sender: TObject);
    procedure treSelectDblClick(Sender: TObject);
    procedure treSelectPaintText(Sender: TBaseVirtualTree;
      const TargetCanvas: TCanvas; Node: PVirtualNode;
      Column: TColumnIndex; TextType: TVSTTextType);
    procedure FormResize(Sender: TObject);
  private
    { Private declarations }
    FChange: Boolean;
    procedure RemoveDeselectedInstrument(AInstrument: string);
    function GetBatteryID(aBatteryName: string): string;
  public
    { Public declarations }
    property Change: boolean read FChange write FChange;
  end;

var
  frmMHA_BatteryWizard        : TfrmMHA_BatteryWizard;

implementation

uses
  uIA_Const,
  uMHA_Const,
  uIA_Utils,
  uMHA_Registry,
  uIA_ThisUser,
  fMHA_InstrumentSelect,
  uMHA_Battery,
  uMHA_Battery_OPS,
  fIA_Dialogs,
  fMHA_BatteryRename,
  uIA_Instrument,
  uMHA_Instrument_OPS,
  uIA_AvailableInstrument,
  fMHA_About,
  uIA_GUIControl,
  fMHA_InstrumentDescription,
  uMHA_DM;

type
  // Node data record
  PSelectData = ^TSelectData;

  TSelectData = record
    Caption: WideString;
    Style: Char; // B,I,R Battery, Incomplete, Regular
    ImageIndex: Integer;
  end;

const
  lcColor                     : TColor = clInfoBk;


var
  // "lv" Local Variable
  lvCurrentBattery,
    lvNodeHeight,
    lvRootNode,
    lvChildren                : Integer;
  treSelectBatt               : TStringList;
  lvBattery                   : TBattery;
  lvInitialize                : Boolean;


{$R *.DFM}

//------------------------------------------------------------------------------
//            Create, Show and Close
//------------------------------------------------------------------------------

procedure TfrmMHA_BatteryWizard.FormCreate(Sender: TObject);
begin
  lvInitialize := True;
  // Keep Position out of ConfigureForm when ConfigureForm can be called
  // more than once with the form.
  Position := poDesigned;

  if gvUseScreenReader then
    treSelect.Images := nil
  else
    LoadImageLists;

  lvBattery := TBattery.Create;
  treSelectBatt := TStringList.create;

  ConfigureForm;

  DisplayAvailableInstruments;
  PopulateComboBox;

  mnuInstrumentDescription.Enabled := False;

  if cbxBatteryName.Items.Count > 0 then
    cbxBatteryNameSelect(Sender);

  lvCurrentBattery := 0;
  FChange := false;

  ShowModal;
end;


procedure TfrmMHA_BatteryWizard.FormShow(Sender: TObject);
begin
  Application.ProcessMessages;
  lvInitialize := False;
  Screen.Cursor := crDefault;

  if gvUseScreenReader then
    ActiveControl := grpBatteryName
  else
    ActiveControl := cbxBatteryName;
end;

procedure TfrmMHA_BatteryWizard.ConfigureForm;
var
  i                           : Integer;
begin
  // Add a unique color
  for i := 0 to ComponentCount - 1 do
    begin
      if Components[i] is TPanel then
        TPanel(Components[i]).Color := lcColor;
      if Components[i] is TLabel then
        TLabel(Components[i]).Color := lcColor;
      if Components[i] is TGroupBox then
        TGroupBox(Components[i]).Color := lcColor;
    end;

  // GUI Settings
  with GUI_Control do
    begin
      Scaled := True;
      AutoScroll := False;
      AutoSize := False;
      PixelsPerInch := 120;

      KeyPreview := True;
      HorzScrollBar.Range := 0; // Ensures that buttons will move when re-sized
      VertScrollBar.Range := 0;
      Font.Assign(gvFont);

      Width := gvWidth; //CheckWidth(640, gvWidth);
      Height := gvHeight; //CheckHeight(420, gvHeight);
      Left := GetLeft(Width);
      Top := GetTop(Height);
      if CheckWidth(Width) or CheckHeight(Height) then
        WindowState := wsMaximized;
      Refresh; // resets alClient dimensions

      pnlButton.Height := 3 * ButtonHeight div 2;
      btnSave.Font.Assign(gvFont);
      btnSave.Left := 0;
      btnSave.Top := ButtonHeight div 4;
      btnSave.Height := ButtonHeight;
      btnSave.Width := ButtonWidthNormal;
      pnlHeader.Height := pnlHeaderHeight;
      lblHeader.Font.Size := lblHeaderFontSize;

      pnlLeft.Width := (5 * ButtonWidthNormal) div 2;
      pnlRight.Width := 3 * ButtonWidthNormal;
      lblInstructions.Font.Assign(gvFont);
      lblInstructions.Font.Color := clNavy;
      grpInstructions.Font.Color := clNavy;
      lvNodeHeight := 3 * TextHeight('Yy', gvFont) div 2;
    end;

  lstChosen.Clear;
  UpdateButtons;
  mnuInstrumentDescription.Enabled := False;

  if gvUseScreenReader then
    begin
      Left := 0;
      Width := Screen.Width;
      imgBackup.Visible := False;
      grpBatteryName.tabstop := True;
      grpAvailable.tabstop := True;
      grpPickedInstruments.tabstop := True;

      Caption := 'MHA Battery Wizard ';
      grpAvailable.Caption := '&Available Instruments. ' +
        'Navigate with arrow keys and use space bar to check the desired ' +
        'instruments. Enter tree view with tab key.';
      grpBatteryName.Caption := 'Enter a unique name for new &battery, or ' +
        'select a battery from combo box ' +
        'you wish to change. Press tab key to continue.';
      grpPickedInstruments.Caption := '&Instruments in Battery. ' +
        'Select by pressing space bar. Buttons follow that change the order ' +
        'and remove instruments. Press tab key view list.';

      btnUp.Caption := '&Up';
      btnDown.Caption := '&Down';
      btnRemove.Caption := '&Remove';
      btnSave.caption := btnSave.Caption + ' button';

    end
  else
    begin
      Caption := MHA_Caption;
      grpBatteryName.tabstop := false;
      grpAvailable.tabstop := false;
      grpPickedInstruments.tabstop := false;
      grpAvailable.Caption := ' &Available Instruments:  ';
      grpBatteryName.Caption := '  &Name of Battery  ';
      grpPickedInstruments.Caption := ' &Instruments in Battery: ';

      btnUp.Caption := '';
      btnDown.Caption := '';
      btnRemove.Caption := '';
    end;
end;

procedure TfrmMHA_BatteryWizard.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  treSelectBatt.Free;
  lvBattery.Free;
end;


//------------------------------------------------------------------------------
//            Buttons
//------------------------------------------------------------------------------

procedure TfrmMHA_BatteryWizard.btnSaveClick(Sender: TObject);

begin

  // Close if no name and no instruments
  if ((cbxBatteryName.Text = '') or
    (cbxBatteryName.Text = ' ')) and
    (lstChosen.Items.Count = 0) then
    begin
      Close;
      exit;
    end;

  // Don't use "My Battery" as name
  if cbxBatteryName.Text = 'My Battery' then
    begin
      MsgDlgFrm('Currently, the name of the new battery is "My Battery" ' +
        'which is used a default place-marker. ' +
        'Please give the new battery a different name.',
        MHA_Caption,
        mtInformation, [mbOK], 0);
      ActiveControl := cbxBatteryName;
      exit;
    end;

  // Prompt user for name of Battery
  if ((cbxBatteryName.Text = '') or
    (cbxBatteryName.Text = ' ')) and
    (lstChosen.Items.Count > 0) then
    begin
      MsgDlgFrm('A name for this battery is required. ',
        MHA_Caption,
        mtInformation, [mbOk], 0);
      exit;
    end;

   // A prompt for no instruments selected is not needed: the save button
   // is disabled until there is an instrument selected.

  // Populate InstrumentsInBattery with instruments in lstChoice.
  BuildBattery;

  // When a battery is saved, the battery is first deleted, then a new
  // battery is added with these particulars, and a new BateryID is created.
  Battery_OPS.SaveBattery(lvBattery);

  FChange := true;
  Close;
end;


procedure TfrmMHA_BatteryWizard.UpdateButtons;
begin
  if lstChosen.Items.Count <= 0 then
    btnSave.Enabled := False
  else
    btnSave.Enabled := True;
  if lstChosen.Items.Count > 1 then
    begin
      btnUp.Enabled := true;
      btnDown.Enabled := true;
    end
  else
    begin
      btnUp.Enabled := False;
      btnDown.Enabled := False;
    end;

  if lstChosen.SelCount > 0 then
    btnRemove.Enabled := true
  else
    btnRemove.Enabled := False;
end;

procedure TfrmMHA_BatteryWizard.treSelectCheckThisNode(ThisNode: string);
// This is used to check a given instrument when the battery is selected.
var
  Node                        : PVirtualNode;
  Data                        : PSelectData;
  i                           : Integer;
begin
  Node := treSelect.GetFirst;
  for i := 1 to treSelect.RootNodeCount - 1 do
    begin
      Node := treSelect.GetNextSibling(Node);
      Data := treSelect.GetNodeData(Node);
      if Data^.Caption = ThisNode then
        treSelect.CheckState[Node] := csCheckedNormal;
    end;
  treSelect.Refresh;
end;

procedure TfrmMHA_BatteryWizard.treSelectUnCheckThisNode(ThisNode: string);
// This is used to uncheck a given instrument when it is dropped from the
// list of selected instruments
var
  Node                        : PVirtualNode;
  Data                        : PSelectData;
  i                           : Integer;
begin
  Node := treSelect.GetFirst;
  for i := 1 to treSelect.RootNodeCount - 1 do
    begin
      Node := treSelect.GetNextSibling(Node);
      Data := treSelect.GetNodeData(Node);
      if Data^.Caption = ThisNode then
        treSelect.CheckState[Node] := csUnCheckedNormal;
    end;
  treSelect.Refresh;
end;


procedure TfrmMHA_BatteryWizard.btnRemoveClick(Sender: TObject);
var
  i                           : Integer;
  tempCaption                 : string;
begin
  mnuInstrumentDescription.Enabled := False;
  tempCaption := lstChosen.Selected.Caption;

  i := lstChosen.Items.IndexOf(lstChosen.Selected);
  lstChosen.Items.Delete(i);

  treSelectUnCheckThisNode(TempCaption);

  UpdateButtons;
end;


procedure TfrmMHA_BatteryWizard.btnUpClick(Sender: TObject);
var
  i                           : Integer;
  tempstr                     : string;
begin
  mnuInstrumentDescription.Enabled := False;
  for i := 1 to lstChosen.Items.Count - 1 do
    if lstChosen.Items[i].Selected then
      begin
        tempstr := lstChosen.Items[i - 1].Caption;
        lstChosen.Items[i - 1].Caption := lstChosen.Items[i].Caption;
        lstChosen.Items[i].Caption := tempstr;
        break;
      end;
  ActiveControl := lstChosen;
  if i < 2 then
    lstChosen.Items[0].Selected := True
  else
    lstChosen.Items[i - 1].Selected := True;
end;


procedure TfrmMHA_BatteryWizard.btnDownClick(Sender: TObject);
var
  i                           : Integer;
  tempstr                     : string;
begin
  mnuInstrumentDescription.Enabled := False;
  for i := 0 to lstChosen.Items.Count - 2 do
    if lstChosen.Items[i].Selected then
      begin
        tempstr := lstChosen.Items[i + 1].Caption;
        lstChosen.Items[i + 1].Caption := lstChosen.Items[i].Caption;
        lstChosen.Items[i].Caption := tempstr;
        break;
      end;
  ActiveControl := lstChosen;
  if i > lstChosen.Items.Count - 2 then
    Exit;
  lstChosen.Items[i + 1].Selected := True;
end;


//------------------------------------------------------------------------------
//              Menu controls
//------------------------------------------------------------------------------


procedure TfrmMHA_BatteryWizard.mnuDescriptionClick(Sender: TObject);
var
  Data                        : PSelectData;
  aInstrument                 : TInstrument;
begin
  Data := treSelect.GetNodeData(treSelect.GetFirstSelected);

  aInstrument := TInstrument.Create;
  aInstrument.InstrumentName := Data^.Caption;

  if Instrument_OPS.LoadDescriptiveProperties(aInstrument) then
    begin
      try
        frmMHA_InstrumentDescription := TfrmMHA_InstrumentDescription.Create(nil);
        frmMHA_InstrumentDescription.ShowDescription(aInstrument);
        frmMHA_InstrumentDescription.ShowModal;
      finally
        frmMHA_InstrumentDescription.Free;
        aInstrument.Free;
      end;
    end //if instrument descriptive properties loaded
  else
    aInstrument.Free;
end;


//------------------------------------------------------------------------------
//              Populate Tree Views
//------------------------------------------------------------------------------

procedure TfrmMHA_BatteryWizard.DisplayAvailableInstruments;
var
  i                           : integer;
  TempStr                     : string;
begin
  treSelect.Colors.FocusedSelectionColor := clHighlight;
  treSelect.Colors.UnfocusedSelectionColor := clBtnFace;
  treSelect.Font.Assign(gvFont);
  treSelect.Font.Color := clBlack;
  treSelect.DefaultNodeHeight := lvNodeHeight;
  treSelect.Clear;

  treSelectBatt.Clear;
  lvRootNode := 0; // The first node in lvtreSelectBatt;

  TempStr := 'Root=';

  for i := 0 to Instrument_OPS.ListOfInstruments.Count - 1 do
    TempStr := TempStr + TAvailableInstrument(Instrument_OPS.ListOfInstruments[i]).AvailInstrumentName + U;

  treSelectBatt.Add(TempStr);

  treSelect.RootNodeCount := NumberOfPieces(treSelectBatt[0], U) - 1;
  treSelect.NodeDataSize := SizeOf(TSelectData);
end;


procedure TfrmMHA_BatteryWizard.PopulateComboBox;
var
  i                           : Integer;
begin
  // Add Battery names to combo box

  cbxBatteryName.Items.Clear;

  if Battery_OPS.ListOfBatteries.Count < 1 then
    begin
      cbxBatteryName.Text := 'My Battery';
      lvCurrentBattery := -1;
    end
  else
    begin
      for i := 0 to Battery_OPS.ListOfBatteries.Count - 1 do
        cbxBatteryName.Items.Add(Trim(TBattery(Battery_OPS.ListOfBatteries[i]).BatteryName));
      cbxBatteryName.Text := cbxBatteryName.Items[0];
      lvCurrentBattery := 0;
      //cbxBatteryNameChange(self);
    end;
end;

//------------------------------------------------------------------------------
//              Keypress
//------------------------------------------------------------------------------

procedure TfrmMHA_BatteryWizard.FormKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
var
  WhereMouse                  : TPoint;
begin
  if (Shift = [ssALT]) and (Key = vk_F4) then
    Close;

  if Key = vk_Escape then
    begin
      Close;
      Key := 0;
    end;

  if Shift = [ssAlt] then
    begin
      if Key in [65, 97] then // "A", "a" Available
        begin
          //ActiveControl := chkAlphaInstruments;
          WhereMouse.x := frmMHA_BatteryWizard.Left + grpAvailable.left +
            grpAvailable.width;
          WhereMouse.y := frmMHA_BatteryWizard.Top + 40 {Menu & header} +
            grpAvailable.top;
          Mouse.CursorPos := WhereMouse;
          Key := 0;
        end;
      if Key in [66, 98] then // "B", "b" Battery Name combo box
        begin
          ActiveControl := cbxBatteryName;
          WhereMouse.x := frmMHA_BatteryWizard.Left + grpAvailable.left +
            pnlSpacer4.Width + grpAvailable.width;
          WhereMouse.y := frmMHA_BatteryWizard.Top + 40 {Menu & header} +
            grpAvailable.top;
          Mouse.CursorPos := WhereMouse;
          Key := 0;
        end;
      if Key in [73, 105] then // "I", "i" Chosen instruments
        begin
          ActiveControl := lstChosen;
          Key := 0;
        end;
    end;
end;

//------------------------------------------------------------------------------
//              Setup Components
//------------------------------------------------------------------------------

(*
procedure TfrmMHA_BatteryWizard.chkAlphaInstrumentsChecked(
  Sender: TObject; Node: TTreeNTNode);
begin

  if (Node.CheckState <> csChecked) then
    begin
      RemoveDeselectedInstrument(Node.Text);
      Exit;
    end;

  PlaceInstrumentInChosenList(Node);

  UpdateButtons;
end;  *)


procedure TfrmMHA_BatteryWizard.RemoveDeselectedInstrument(AInstrument: string);
var
  i                           : integer;
begin
  for i := 0 to lstChosen.Items.Count - 1 do
    if (lstChosen.Items[i].Caption = AInstrument) then
      begin
        lstChosen.Items[i].Delete;
        Break;
      end;

  UpdateButtons;
end;

(*
procedure TfrmMHA_BatteryWizard.PlaceInstrumentInChosenList
  (Node: TTreeNTNode);
begin
  // Place the checked instrument(s) in the Chosen list
  with Node do
    lstChosen.Items.Add.Caption := Text;

  // Remove redundant items
  lstChosenRedundant;
end;*)


procedure TfrmMHA_BatteryWizard.lstChosenRedundant;
var
  i, j                        : Integer;
begin
  UpdateButtons;
  for i := 0 to lstChosen.Items.Count - 1 do
    begin
      if i = lstChosen.Items.Count - 1 then
        Exit;
      for j := i + 1 to lstChosen.Items.Count - 1 do
        if lstChosen.Items[i].Caption = lstChosen.Items[j].Caption then
          begin
            lstChosen.Items[j].Delete;
            Break;
          end;
    end;
end;


procedure TfrmMHA_BatteryWizard.LoadImageLists;

// D2 and D3 image lists are incompatible, thus I load the icons here from the resource.
// BTW. Loading icons
// this way produces much better looking images in the trees than by loading it in the IDE.

var
  Icon                        : TIcon;
  H, W                        : Integer;

begin
  // ImageList1 with predefined icons
  H := ImageList1.Height;
  W := ImageList1.Width;
  Icon := TIcon.Create;

  Icon.Handle := LoadImage(HInstance, 'USERSET', IMAGE_ICON, W, H, 0);
  ImageList1.AddIcon(Icon);
  IconBattery := ImageList1.Count - 1;

  Icon.Handle := LoadImage(HInstance, 'INSTRUMENT', IMAGE_ICON, W, H, 0);
  ImageList1.AddIcon(Icon);
  IconInstrument := ImageList1.Count - 1;

  Icon.Handle := LoadImage(HInstance, 'INCOMPLETE', IMAGE_ICON, W, H, 0);
  ImageList1.AddIcon(Icon);
  IconIncomplete := ImageList1.Count - 1;

  Icon.Free;
end;


procedure TfrmMHA_BatteryWizard.lstChosenEnter(Sender: TObject);
begin
  lblInstructions.Caption := #13 + '    This is the order in which the instruments ' +
    'will be presented. Use the arrows or drag && drop to change the order.';
  lblInstructions.Update;
  mnuInstrumentDescription.Enabled := False;
end;

//------------------------------------------------------------------------------
//            Drag and Drop
//------------------------------------------------------------------------------

procedure TfrmMHA_BatteryWizard.lstChosenDragDrop(Sender,
  Source: TObject; X, Y: Integer);
var
  OldIndex, NewIndex          : integer;
  InsertItem                  : TListItem;
  Old                         : string;
begin
  Old := lstChosen.Selected.Caption;
  OldIndex := lstChosen.Items.IndexOf(lstChosen.Selected);
  NewIndex := lstChosen.Items.IndexOf(lstChosen.GetItemAt(X, Y));
  if NewIndex = -1 then
    NewIndex := lstChosen.Items.Count - 1;
  with Source as TListView do
    begin
      Items.Delete(OldIndex);
      InsertItem := Items.Insert(NewIndex);
      InsertItem.Caption := Old;
    end;
end;


procedure TfrmMHA_BatteryWizard.lstChosenDragOver(Sender,
  Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean);
begin
  if Source is TListView then
    Accept := True;
end;


procedure TfrmMHA_BatteryWizard.lstChosenMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if Button = mbLeft then
    with Sender as TListView do
      BeginDrag(False);
end;


procedure TfrmMHA_BatteryWizard.cbxBatteryNameSelect(Sender: TObject);
// User selects a battery.
var
  i                           : Integer;
begin
  // Setup: Clear the checked boxes in list of available instruments and
  //        list of selected instruments.
  treSelectUnCheck;
  lstChosen.Items.Clear;

  if cbxBatteryName.Items.Count = 0 then
    exit;

  if lvCurrentBattery = -1 then
    Exit;

  if cbxBatteryName.ItemIndex = -1 then
    cbxBatteryName.ItemIndex := 0;

  lvCurrentBattery := cbxBatteryName.ItemIndex;

  for i := 0 to TBattery(Battery_OPS.ListOfBatteries[lvCurrentBattery]).InstrumentsInBattery.Count - 1 do
    treSelectCheckThisNode(Piece(TBattery(Battery_OPS.ListOfBatteries[lvCurrentBattery]).InstrumentsInBattery[i], U, 3));

  // The checked batteries are added to the TListView (lstChosen) by the
  // process of being checked in treSelect.
  // This is not needed:
  // lstChosen.Items.Add.Caption := Piece(TempStr, U, 3);

  UpdateButtons;

  FChange := True;
end;

procedure TfrmMHA_BatteryWizard.cbxBatteryNameChange(Sender: TObject);
begin

(*
  // Setup: Clear the checked boxes in list of available instruments and
  //        list of selected instruments.
  treSelectUnCheck;
  lstChosen.Items.Clear;

  if cbxBatteryName.Items.Count = 0 then
    exit;

  if lvCurrentBattery = -1 then
    Exit;

  lvCurrentBattery := cbxBatteryName.ItemIndex;

  for i := 0 to TBattery(Battery_OPS.ListOfBatteries[lvCurrentBattery]).InstrumentsInBattery.Count - 1 do
      treSelectCheckThisNode(Piece(TBattery(Battery_OPS.ListOfBatteries[lvCurrentBattery]).InstrumentsInBattery[i], U, 3));

  // The checked batteries are added to the TListView (lstChosen) by the
  // process of being checked in treSelect.
  // lstChosen.Items.Add.Caption := Piece(TempStr, U, 3);

  UpdateButtons;
  FChange := true;
 *)
end;

function TfrmMHA_BatteryWizard.GetBatteryID(aBatteryName: string): string;
var
  i                           : Integer;
begin
  result := '';
  for i := 0 to Battery_OPS.ListOfBatteries.Count - 1 do
    begin
      if TBattery(Battery_OPS.ListOfBatteries[i]).BatteryName = aBatteryName then
        begin
          result := TBattery(Battery_OPS.ListOfBatteries[i]).BatteryID;
          Exit;
        end;
    end;
end;

procedure TfrmMHA_BatteryWizard.BuildBattery;
var
  i                           : integer;
begin
  lvBattery.BatteryName := cbxBatteryName.Text;
  lvBattery.BatteryID := GetBatteryID(lvBattery.BatteryName);
  lvBattery.InstrumentsInBattery.clear;

  for i := 0 to lstChosen.items.count - 1 do
    lvBattery.InstrumentsInBattery.add(
      Trim(lstChosen.Items[i].Caption));
end;


procedure TfrmMHA_BatteryWizard.DisplayBatteryNameChange;
begin
  try
    frmMHA_BatteryRename := TfrmMHA_BatteryRename.Create(nil);
  finally
    frmMHA_BatteryRename.Free;
  end;

  Battery_OPS.LoadBatteries;
  treSelectUncheck;
  FChange := true;
end;

procedure TfrmMHA_BatteryWizard.cbxBatteryNameKeyPress(Sender: TObject;
  var Key: Char);
begin
  if Key in [#61, #62, #94] then // = > ^
    begin
      Key := #0;
      MsgDlgFrm('Sorry, this key cannot be used in a battery name.  ',
        MHA_Caption,
        mtInformation, [mbok], 0);
    end;
end;

procedure TfrmMHA_BatteryWizard.mnuNewClick(Sender: TObject);
begin
  cbxBatteryName.text := '';
  lstChosen.items.clear;
  treSelectUnCheck;
  ActiveControl := cbxBatteryName;
end;


procedure TfrmMHA_BatteryWizard.mnuExitClick(Sender: TObject);
begin
  Close;
end;

procedure TfrmMHA_BatteryWizard.mnuDeleteClick(Sender: TObject);
begin
  gvShowOptionTabs := 'D';

  try
    frmMHA_BatteryRename := TfrmMHA_BatteryRename.Create(nil);
  finally
    frmMHA_BatteryRename.Free;
  end;

  PopulateComboBox;
  cbxBatteryName.text := '';
  lstChosen.Items.Clear;
  treSelectUnCheck;
  UpDateButtons;
  FChange := True;

end;


procedure TfrmMHA_BatteryWizard.mnuOnlineSupportClick(Sender: TObject);
begin
  VisitURL(MHA_ONLINE_SUPPORT_URL);
end;


procedure TfrmMHA_BatteryWizard.mnuAboutClick(Sender: TObject);
begin
  frmMHA_About := TfrmMHA_About.Create(nil);
  try
    frmMHA_About.ShowModal;
  finally
    frmMHA_About.Free;
  end; //try
end;


procedure TfrmMHA_BatteryWizard.cbxBatteryNameEnter(Sender: TObject);
begin
  // Update the instructions panel.
  lblInstructions.Caption := CRLF +
    '  Enter a unique name for battery.';
  mnuInstrumentDescription.Enabled := False;
end;

procedure TfrmMHA_BatteryWizard.btnSaveEnter(Sender: TObject);
begin
  // Update the instructions panel.
  lblInstructions.Caption := CRLF +
    '  Click to save battery. You will return to ' +
    '"Instrument Administrator."';
end;

procedure TfrmMHA_BatteryWizard.grpBatteryNameMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin
  // Update the instructions panel.
  lblInstructions.Caption := CRLF +
    '  Enter a unique name for battery ' + CRLF +
    '  or select one from combo box.';
end;


procedure TfrmMHA_BatteryWizard.grpPickedInstrumentsMouseMove(
  Sender: TObject; Shift: TShiftState; X, Y: Integer);
begin
  lblInstructions.Caption := #13 + '    This is the order in which the instruments ' +
    'will be presented. Use the arrows or drag && drop to change the order.';
  lblInstructions.Update;
end;

procedure TfrmMHA_BatteryWizard.pnlButtonMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin
  btnSaveEnter(Sender);
end;

procedure TfrmMHA_BatteryWizard.lblInstructionsMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin
  // Update the instructions panel.
  lblInstructions.Caption := CRLF +
    '  A battery is a handy collection of instruments that are used ' +
    'frequently or grouped for special purposes.' + CRLF + CRLF +
    '  Check an instrument to move it to the list ' +
    'of instruments for this battery.' + CRLF + CRLF +
    '  You can have many batteries and the same instrument can be in ' +
    'several batteries.';
end;


procedure TfrmMHA_BatteryWizard.lstChosenMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin
  lstChosenEnter(Sender);
end;

procedure TfrmMHA_BatteryWizard.lstChosenSelectItem(Sender: TObject;
  Item: TListItem; Selected: Boolean);
begin
  UpdateButtons;
end;

procedure TfrmMHA_BatteryWizard.mnuRenameClick(Sender: TObject);
begin
  gvShowOptionTabs := 'R';
  DisplayBatteryNameChange;

  PopulateComboBox;

  cbxBatteryName.text := '';
  treSelectUncheck;
  lstChosen.Items.Clear;
  UpDateButtons;
  FChange := True;
end;

//------------------------------------------------------------------------------
//       Virtual Tree Stuff
//------------------------------------------------------------------------------

procedure TfrmMHA_BatteryWizard.treSelectGetText(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Column: TColumnIndex; TextType: TVSTTextType;
  var CellText: WideString);
var
  Data                        : PSelectData;
begin
  Data := Sender.GetNodeData(Node);
  CellText := Data.Caption;
end;


function TfrmMHA_BatteryWizard.treSelectGetChildCount(NodeIn: string): integer;
var
  i                           : integer;
begin
  Result := 0;

  for i := 0 to treSelectBatt.Count - 1 do
    begin
      if NodeIn = Piece(treSelectBatt[i], '=', 1) then
        begin
          Result := NumberOfPieces(treSelectBatt[i], U) - 1;
          break;
        end;
    end;
end;


procedure TfrmMHA_BatteryWizard.treSelectInitChildren(Sender: TBaseVirtualTree;
  Node: PVirtualNode; var ChildCount: Cardinal);
var
  Data                        : PSelectData;
begin
  lvChildren := 0;
  Data := Sender.GetNodeData(Node);
  ChildCount := treSelectGetChildCount(Data^.Caption);

end;

function TfrmMHA_BatteryWizard.treSelectGetItems(PNode: string; ChildNumber: integer): string;
var
  i, j                        : integer;
  TempStr                     : string;
begin

  Result := '';
  for i := 0 to treSelectBatt.Count - 1 do
    begin
      if PNode = Piece(treSelectBatt[i], '=', 1) then
        begin
          TempStr := Piece(treSelectBatt[i], U, ChildNumber + 1);
          // First child has root in its name.
          j := Pos('=', TempStr);
          if j > 0 then
            Result := Copy(TempStr, j + 1, Length(TempStr))
          else
            Result := TempStr;
        end;
    end;
end;

function TfrmMHA_BatteryWizard.treSelectGetHasChildren(NodeName: string): Boolean;
var
  i                           : integer;
begin
  Result := False;
  for i := 0 to treSelectBatt.Count - 1 do
    if NodeName = Piece(treSelectBatt[i], '=', 1) then
      begin
        Result := True;
        break;
      end;
end;

procedure TfrmMHA_BatteryWizard.treSelectInitNode(Sender: TBaseVirtualTree; ParentNode, Node:
  PVirtualNode; var InitialStates: TVirtualNodeInitStates);
var
  DataP,
    Data                      : PSelectData;
begin
  Data := Sender.GetNodeData(Node);

  if ParentNode = nil then // Parent nodes
    begin
      Data^.Caption := treSelectGetItems('Root', lvRootNode);
      if treSelectGetHasChildren(Data^.Caption) then
        begin
          Data^.Style := 'B'; // Battery
          InitialStates := [ivsHasChildren];
        end
      else
        begin
          InitialStates := [];
          if Pos('(', Data^.Caption) > 0 then
            Data^.Style := 'I' //Incomplete
          else
            Data^.Style := 'R'; //Regular
        end;
      Inc(lvRootNode);
    end
  else
    begin
      DataP := Sender.GetNodeData(ParentNode);
      Data^.Caption := treSelectGetItems(DataP^.Caption, lvChildren);

      if treSelectGetHasChildren(Data^.Caption) then
        InitialStates := [ivsHasChildren]
      else
        begin
          InitialStates := [];
          Data^.Style := 'R'; //Regular
        end;
      Inc(lvChildren);
    end;

  Node.NodeHeight := lvNodeHeight;
  Node.CheckType := ctCheckBox;
  Sender.CheckState[Node] := csUnCheckedNormal;
end;


procedure TfrmMHA_BatteryWizard.treSelectUnCheck;
// This is used to uncheck all nodes
var
  Node                        : PVirtualNode;
  i                           : Integer;
begin
  Node := treSelect.GetFirst;
  for i := 1 to treSelect.RootNodeCount - 1 do
    begin
      Node := treSelect.GetNextSibling(Node);
      treSelect.CheckState[Node] := csUnCheckedNormal;
    end;
  treSelect.Refresh;
end;

procedure TfrmMHA_BatteryWizard.treSelectChecked(
  Sender: TBaseVirtualTree; Node: PVirtualNode);
var
  Data                        : PSelectData;
begin
  Data := Sender.GetNodeData(Node);
  if Node.CheckState = csCheckedNormal then
    begin
      lstChosen.Items.Add.Caption := Trim(Data.Caption);
      // Remove redundant items
      lstChosenRedundant;
     // Node.States
    end
  else
    begin
      RemoveDeselectedInstrument(Data.Caption);
    end;
end;


procedure TfrmMHA_BatteryWizard.treSelectEnter(Sender: TObject);
begin
  // Update the instructions panel.
  lblInstructions.Caption := CRLF +
    '  Select the instrument by checking its box with the mouse, or ' +
    'use the arrow keys and space-bar to select.';
end;

procedure TfrmMHA_BatteryWizard.treSelectMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin
  // Update the instructions panel.
  treSelectEnter(Sender);
end;

procedure TfrmMHA_BatteryWizard.treSelectGetImageIndex(
  Sender: TBaseVirtualTree; Node: PVirtualNode; Kind: TVTImageKind;
  Column: TColumnIndex; var Ghosted: Boolean; var ImageIndex: Integer);
var
  Data                        : PSelectData;

begin

  if gvUseScreenReader then
    Exit;

  Data := Sender.GetNodeData(Node);
  case Data.Style of
    'R': // Regular
      ImageIndex := IconInstrument;
    'B': // Battery
      ImageIndex := IconBattery;
    'I': // Incomplete
      ImageIndex := IconIncomplete;
  end; //case

end;

procedure TfrmMHA_BatteryWizard.treSelectClick(Sender: TObject);
begin
  mnuInstrumentDescription.Enabled := True;
end;


procedure TfrmMHA_BatteryWizard.imgBackupClick(Sender: TObject);
begin
  Close;
end;

procedure TfrmMHA_BatteryWizard.treSelectDblClick(Sender: TObject);
begin
  treSelectChecked(treSelect, treSelect.GetFirstSelected);
 // Sender: TBaseVirtualTree; Node: PVirtualNode);
end;

procedure TfrmMHA_BatteryWizard.treSelectPaintText(
  Sender: TBaseVirtualTree; const TargetCanvas: TCanvas;
  Node: PVirtualNode; Column: TColumnIndex; TextType: TVSTTextType);
var
  Data                        : PSelectData;
begin

  if gvUseScreenReader then
    Exit;

  Data := Sender.GetNodeData(Node);
  case Data.Style of
    'B': // Battery are bold
      TargetCanvas.Font.Style := TargetCanvas.Font.Style + [fsBold];
    'I': // Incomplete are underlined
      begin
        TargetCanvas.Font.Style := TargetCanvas.Font.Style + [fsUnderline];
        TargetCanvas.Font.Color := clMaroon;
      end;
  end; //case
end;

procedure TfrmMHA_BatteryWizard.FormResize(Sender: TObject);
begin
  if lvInitialize then
    Exit;

  // Reset these, used in placement of dialogs
  gvLeft := Left;
  gvTop := Top;
  gvHeight := Height;
  gvWidth := Width;
end;

end.

