unit fMHA_InstrumentResource;

(*
  This unit will produce a string list that contains:
    1) Instrument description
    2) Questions
    3) Introductions
    4) Sections
    5) Rules
    6) Scales
    7) Report template

  The string list is saved to a resource file.

  This is very similar to the export unit in IAT, however, here
  the IENS are recorded.
*)

interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  uIA_Question,
  uIA_Instrument,
  uIA_Introduction,
  uIA_SectionLabel,
  StdCtrls,
  ComCtrls,
  Dialogs;

type
  TfrmMHA_ResourceFile = class(TForm)
    ReportMemo: TRichEdit;
  private
    { Private declarations }

    procedure WriteQuestions(Instrument: TInstrument);
    procedure WriteIntroductions(Instrument: TInstrument);
    procedure WriteSections(Instrument: TInstrument);
    procedure WriteRules(Instrument: TInstrument);
    //procedure WriteScales;
    //procedure WriteReport;

    procedure ReadDescriptiveProperties(Instrument: TInstrument);
    procedure ReadQuestions(Instrument: TInstrument);
    procedure ReadIntroductions(Instrument: TInstrument);
    procedure ReadSections(Instrument: TInstrument);
    procedure ReadRules(Instrument: TInstrument);
    //procedure  ReadScales(Instrument: TInstrument);

    function StringOfChoices(Choices: TStringList): string;
  public
    { Public declarations }
    procedure BuildInstrumentResourceFile(FullFileName: string; Instrument: TInstrument);
    procedure ReadInstrumentResourceFile(FullFileName: string; Instrument: TInstrument);
  end;

var
  frmMHA_ResourceFile         : TfrmMHA_ResourceFile;

implementation

{$R *.dfm}

uses
  uIA_Const,
  uIA_Utils,
  uIA_Rule,
  uIA_Display;


var
  lvStringList                : TStringList;
  lvFont                      : TFont;
  lvStartRule,
    lvStartIntro,
    lvStartSection            : Integer;
  //Script                      : TScript;

procedure TfrmMHA_ResourceFile.BuildInstrumentResourceFile(FullFileName: string; Instrument: TInstrument);
begin
  //Instrument := TInstrument.Create(nil);
  lvStringList := TStringList.Create;
  //MHA_Report := TMHA_Report.Create;
  //Script := TScript.Create;
  //TempStr := GetApplicationData;

  with Instrument do
    begin
      lvStringList.Add(' Instrument follows');
      lvStringList.Add(
        Name + U +
        PrintTitle + U +
        Version + U +
        BoolToStr(RequiresSignature) + U +
        Reference + U +
        BoolToStr(RequiresLicense) + U +
        IsOperational + U +
        BoolToStr(LicenseIsCurrent) + U +
        BoolToStr(IsLegacy) + U +
        IntToStr(NumberOfQuestions) + U +
        IEN + U +
        Publisher + U +
        Author + U +
        Copyright + U +
        PublicationDate + U +
        NormSample + U +
        R_Privilege + U +
        Purpose + U +
        TargetPopulation + U +
        EntryDate + U +
        EnteredBy + U +
        A_Privilege + U +
        LastEditedBy + U +
        LastEditDate + U +
        BoolToStr(IsNationalTest) + U +
        LastEditFMDate + U +
        BoolToStr(SubmitToNationalDB) + U +
        BoolToStr(IsCopyrighted) + U +
        BoolToStr(WriteFullText) + U);
    end;

  lvStringList.Add('End Instrument');
  lvStringList.Add('');

  lvStringList.Add('Questions follow');
  WriteQuestions(Instrument);
  lvStringList.Add('End Questions');
  lvStringList.Add('');

  lvStringList.Add('Introductions follow');
  WriteIntroductions(Instrument);
  lvStringList.Add('End Introductions');
  lvStringList.Add('');

  lvStringList.Add('Sections follow');
  WriteSections(Instrument);
  lvStringList.Add('End Sections');
  lvStringList.Add('');

  lvStringList.Add('Rules follow');
  WriteRules(Instrument);
  lvStringList.Add('End Rules');
  lvStringList.Add('');

//  lvStringList.Add('Scales follow');
//  WriteScales;
//  lvStringList.Add('End Scales');
//  lvStringList.Add('');

//  WriteReport;
//  lvStringList.Add('');

  lvStringList.Add('End of Write file');
  lvStringList.SaveToFile(FullFileName);
  lvStringList.Free;
end;

procedure TfrmMHA_ResourceFile.ReadInstrumentResourceFile(FullFileName: string;
  Instrument: TInstrument);
begin
  lvStringList := TStringList.Create;

  lvFont := TFont.Create;
  lvStringList.LoadFromFile(FullFileName);

  lvStartRule := 0;
  lvStartIntro := 0;
  lvStartSection := 0;

  ReadDescriptiveProperties(Instrument);

  Instrument.ListOfQuestions.Clear;
  Instrument.ListOfIntroductions.Clear;
  Instrument.ListOfSections.Clear;

  ReadQuestions(Instrument);
  ReadIntroductions(Instrument);

  ReadSections(Instrument);
  ReadRules(Instrument);
end;

//procedure TEditor_Import.FreeEverything;
//begin
//  lvStringList.Free;
//  lvFont.Free;
//  lvOldNewQuestionID.Free;
//
//  frmMHA7_Editor_Save.Free;
//end;

(*function TfrmMHA_ResourceFile.GetApplicationData: string;
const
  InfoNum                     = 10;
  InfoStr                     : array[1..InfoNum] of string = ('CompanyName',
  'FileDescription', 'FileVersion', 'InternalName', 'LegalCopyright',
  'LegalTradeMarks', 'OriginalFileName', 'ProductName', 'ProductVersion',
  'Comments');
var
  S                           : string;
  n, Len, i                   : DWORD;
  Buf                         : PChar;
  Value                       : PChar;
begin
  S := Application.ExeName;
  n := GetFileVersionInfoSize(PChar(S), n);
  if n > 0 then
    begin
      Buf := AllocMem(n);
      GetFileVersionInfo(PChar(S), 0, n, Buf);
      for i := 1 to InfoNum do
        if VerQueryValue(Buf, PChar('StringFileInfo\040904E4\' + InfoStr[i]), Pointer(Value), Len) then
          Result := Result + InfoStr[i] + ': ' + Value + ',  ';
      FreeMem(Buf, n);
    end
  else
    Result := 'No version information found.';
end; *)



function TfrmMHA_ResourceFile.StringOfChoices(Choices: TStringList): string;
// Each line has: Sequence^ChoiceID^Text^Legacy Value
var
  i                           : Integer;
begin
  Result := 'NONE';

  if Choices.Count = 0 then
    Exit;

  Result := '';

  for i := 0 to Choices.Count - 1 do
    Result := Result + U + Piece(Choices[i], U, 3); // Text only
end;



function ConvertChoiceIdentifierToString
  (ChoiceIdent: TChoiceIdentifier): string;
begin
  Result := 'ONE';
  if ChoiceIdent = ciZeroBased then
    Result := 'ZERO';
  if ChoiceIdent = ciNone then
    Result := 'NONE';
end;

function ConvertResponseTypeToString(AResponseType: TResponseType): string;
begin
  Result := String_RT; //make this the default
  if (AResponseType = rtMultiChoice) then
    Result := MCHOICE_RT;
  if (AResponseType = rtInteger) then
    Result := INTEGER_RT;
  if (AResponseType = rtDate) then
    Result := DATE_RT;
  if (AResponseType = rtMemo) then
    Result := MEMO_RT;
  if (AResponseType = rtCurrency) then
    Result := CURRENCY_RT;
  if (AResponseType = rtVista) then
    Result := VISTA_RT;
  if (AResponseType = rtMaskEdit) then
    Result := MASKEDIT_RT;
  if (AResponseType = rtTrackBar) then
    Result := TRACKBAR_RT;
  if (AResponseType = rtCheckList) then
    Result := CHECKLIST_RT;
end; //ConvertToResponseType

function StyleToString(StyleIn: TFontStyles): string;
begin
  Result := '';
  if fsBold in StyleIn then
    Result := 'Y' + U
  else
    Result := 'N' + U;
  if fsItalic in StyleIn then
    Result := Result + 'Y' + U
  else
    Result := Result + 'N' + U;
  if fsUnderline in StyleIn then
    Result := Result + 'Y' + U
  else
    Result := Result + 'N' + U;
end;

function AlignmentToStr(AAlignment: TAlignment): string;
begin
  case AAlignment of
    taLeftJustify: Result := LEFT_JUSTIFY;
    taCenter: Result := CENTER_JUSTIFY;
    taRightJustify: Result := RIGHT_JUSTIFY;
  end;
end;

function AnswerComponentToStr(AAnswerComponent: TAnswerComponent): string;
begin
  Result := 'NA';

  case AAnswerComponent of

    //DATA TYPE OF COMPONENT: SET OF CODES//
    //INTERNALLY-STORED CODE: RA//   WILL STAND FOR: RadioGroup//
    //INTERNALLY-STORED CODE: CK//   WILL STAND FOR: CheckList//
    //INTERNALLY-STORED CODE: CO//   WILL STAND FOR: ComboBox//
    //INTERNALLY-STORED CODE: LI//   WILL STAND FOR: List//
    //INTERNALLY-STORED CODE: ED//   WILL STAND FOR: Edit//
    //INTERNALLY-STORED CODE: MA//   WILL STAND FOR: Mask Edit//
    //INTERNALLY-STORED CODE: SP//   WILL STAND FOR: Spin Edit//
    //INTERNALLY-STORED CODE: ME//   WILL STAND FOR: Memo//
    //INTERNALLY-STORED CODE: DA//   WILL STAND FOR: Date Picker//
    //INTERNALLY-STORED CODE: TR//   WILL STAND FOR: Track Bar//
    //INTERNALLY-STORED CODE: OR//   WILL STAND FOR: ORcombo Box//
    //INTERNALLY-STORED CODE: NA//   WILL STAND FOR: Not Applicable//

    acComboBox: Result := COMBOBOX_AC;
    acCheckListBox: Result := CHECKLIST_AC;
    acListBox: Result := LISTBOX_AC;
    acRadioGroup: Result := RADIOGROUP_AC;
    acEdit: Result := EDIT_AC;
    acMaskEdit: Result := MASKEDIT_AC;
    acSpinEdit: Result := SPINEDIT_AC;
    acMemo: Result := MEMO_AC;
    acDatePicker: Result := DATEPICKER_AC;
    acTrackBar: Result := TRACKBAR_AC;
    acORCombo: Result := ORCOMBOBOX_AC;
  end;
//  ShowMessage(Result);
end;


procedure TfrmMHA_ResourceFile.WriteQuestions;
var
  i                           : Integer;

  function MultipleChoiceItems(DisplayComp: TAnswerComponent): string;
  begin
    Result := '';
    if (DisplayComp = acComboBox) or
      (DisplayComp = acListBox) or
      (DisplayComp = acRadioGroup) or
      (DisplayComp = acTrackBar) or
      (DisplayComp = acCheckListBox) then
      begin
        with TQuestion(Instrument.ListOfQuestions[i]) do
          begin
            Result := 'Choices Follow ' + U +
              ConvertChoiceIdentifierToString(ChoiceIdentifier) + U +
              IntToStr(MultiChoice.NumberOfPicks) + U +
              IntToStr(MultiChoice.NumberOfChoices) +
              StringOfChoices(MultiChoice.Choices) + U +
              'End Choices';
          end;
      end;
  end;

begin
  if Instrument.ListOfQuestions.Count = 0 then
    Exit;

  for i := 0 to Instrument.ListOfQuestions.Count - 1 do
    begin
      with TQuestion(Instrument.ListOfQuestions[i]) do
        begin
          lvStringList.Add(' ');
          lvStringList.Add('Question_' + IntToStr(i) + U +
            // Question properties
            Text + U +
            QuestionID + U +
            IntToStr(Max) + U +
            BoolToStr(IsRequired) + U +
            IntToStr(Min) + U +
            Designator + U +
            Hint + U +
            Caption + U +
            BoolToStr(IsLastQuestion) + U +
            BoolToStr(IsFirstQuestion) + U +
            ConvertResponseTypeToString(ResponseType) + U +

            // Question Display
            Display.DisplayID + U +
            Display.DisplayFont.Name + U +
            StyleToString(Display.DisplayFont.Style) +
            IntToStr(Display.DisplayFont.Size) + U +
            ColorToString(Display.DisplayFont.Color) + U +
            IntToStr(Display.DisplayLeft) + U +
            AlignmentToStr(Display.DisplayAlignment) + U +

            // Answer Display
            AnswerDisplay.DisplayID + U +
            AnswerDisplay.DisplayFont.Name + U +
            StyleToString(AnswerDisplay.DisplayFont.Style) +
            IntToStr(AnswerDisplay.DisplayFont.Size) + U +
            ColorToString(AnswerDisplay.DisplayFont.Color) + U +
            IntToStr(AnswerDisplay.DisplayLeft) + U +
            IntToStr(AnswerDisplay.DisplayWidth) + U +
            AlignmentToStr(AnswerDisplay.DisplayAlignment) + U +
            AnswerDisplay.DisplayMask + U +
            IntToStr(AnswerDisplay.DisplayColumns) + U +
            AnswerComponentToStr(AnswerDisplay.DisplayComponent) + U +
            BoolToStr(AnswerDisplay.DisplaySameLine) + U +

            // Choices
            MultipleChoiceItems(AnswerDisplay.DisplayComponent));

          lvStringList.Add('End Question_' + IntToStr(i));
        end;
    end;
end;


procedure TfrmMHA_ResourceFile.WriteIntroductions;
var
  i                           : Integer;
begin

  if Instrument.ListOfIntroductions.Count = 0 then
    Exit;

  for i := 0 to Instrument.ListOfIntroductions.Count - 1 do
    begin
      with TIntroduction(Instrument.ListOfIntroductions[i]) do
        begin
          lvStringList.Add(' ');
          lvStringList.Add('Introduction_' + IntToStr(i) + U +
            ID + U +
            Text + U +
            BoolToStr(IsNew) + U +
            Display.DisplayID + U +
            Display.DisplayFont.Name + U +
            StyleToString(Display.DisplayFont.Style) +
            IntToStr(Display.DisplayFont.Size) + U +
            ColorToString(Display.DisplayFont.Color) + U +
            IntToStr(Display.DisplayLeft) + U +
            AlignmentToStr(Display.DisplayAlignment) + U);
          lvStringList.Add('End Introduction_' + IntToStr(i));
        end; // with
    end;
end;


procedure TfrmMHA_ResourceFile.WriteSections;
var
  i                           : Integer;
begin
  for i := 0 to Instrument.ListOfSections.Count - 1 do
    begin
      with TSectionLabel(Instrument.ListOfSections[i]) do
        begin
          lvStringList.Add(' ');
          lvStringList.Add('Section_' + IntToStr(i) + U +
            QuestionID + U +
            TabCaption + U +
            SectionCaption + U +
            Display.DisplayID + U +
            Display.DisplayFont.Name + U +
            StyleToString(Display.DisplayFont.Style) +
            IntToStr(Display.DisplayFont.Size) + U +
            ColorToString(Display.DisplayFont.Color) + U +
            IntToStr(Display.DisplayLeft) + U +
            AlignmentToStr(Display.DisplayAlignment) + U);
          lvStringList.Add('End Section_' + IntToStr(i));
        end; // with
    end; // for i
end;

function DataTypeToString(ADataValueType: TValueDataType): string;
begin
  //  TValueDataType = (dtString, dtDate, dtNumber, dtTimeInterval);
  case ADataValueType of
    dtDate: Result := DATE_STR;
    dtNumber: Result := NUMBER_STR;
    dtTimeInterval: Result := INTERVAL_STR;
  else
    Result := STRING_STR;
  end; //case
end;

function QuestionOperatorToStr(ThisRule: TRule; IsFirstQuestion: BOOLEAN): string;
var
  tmpOperator                 : TOperatorType;
begin
  with ThisRule do
    begin
      if IsFirstQuestion then
        tmpOperator := FirstOperator
      else
        tmpOperator := SecondOperator;

      Result := '';

  //OPERATOR_STRINGS = 'Equals^Does not equal^Is less than^Is greater than^'
  //+ 'Equals or is less than^Equals or is greater than';

      case tmpOperator of
        toEQ: Result := Piece(OPERATOR_STRINGS, U, 1);
        toNE: Result := Piece(OPERATOR_STRINGS, U, 2);
        toLT: Result := Piece(OPERATOR_STRINGS, U, 3);
        toGT: Result := Piece(OPERATOR_STRINGS, U, 4);
        toLE: Result := Piece(OPERATOR_STRINGS, U, 5);
        toGE: Result := Piece(OPERATOR_STRINGS, U, 6);
      end;
    end;
end;


function BoolOperatorToString(ThisRule: TRule): string;
begin
  Result := 'AND';

  if (ThisRule.BoolOperator = boOR) then
    Result := 'OR';

end;


procedure TfrmMHA_ResourceFile.WriteRules;
var
  i                           : Integer;
begin
  if Instrument.ListOfRules.Count = 0 then
    Exit;

  for i := 0 to Instrument.ListOfRules.Count - 1 do
    begin
      with TRule(Instrument.ListOfRules[i]) do
        begin
          lvStringList.Add(' ');
          lvStringList.Add('Rule_' + IntToStr(i) + U +
            DataTypeToString(FirstValueDataType) + U +
            QuestionOperatorToStr(TRule(Instrument.ListOfRules[i]), True) + U +
            DataTypeToString(SecondValueDataType) + U +
            QuestionOperatorToStr(TRule(Instrument.ListOfRules[i]), False) + U +
            BoolOperatorToString(TRule(Instrument.ListOfRules[i])) + U +
            FirstQuestionID + U +
            FirstValue + U +
            SecondQuestionID + U +
            SecondValue + U +
            MessageText + U +
            'Skip*' + SkipQuestionIDs);
        end;
    end; // for i

end;

(*
procedure TfrmMHA_ResourceFile.WriteScales;
var
  i, j, k                     : Integer;
begin

  if Instrument.ListOfScaleGroups.count = 0 then
    exit;

  for i := 0 to Instrument.ListOfScaleGroups.Count - 1 do
    begin
      with TScaleGroup(Instrument.ListOfScaleGroups[i]) do
        begin
          lvStringList.Add('');
          lvStringList.Add('ScaleGroup_' + IntToStr(i) + U +
            GroupName + U +
            IEN + U +
            IntToStr(Sequence) + U +
            OrdinateTitle + U +
            IntToStr(OrdinateMin) + U +
            IntToStr(OrdinateIncrement) + U +
            IntToStr(OrdinateMax) + U +
            IntToStr(Grid1Ordinate) + U +
            IntToStr(Grid2Ordinate) + U +
            IntToStr(Grid3Ordinate) + U +
            IntToStr(NumberOfScales));

          for j := 0 to Instrument.ListOfScales.Count - 1 do
            begin
              with TScale(Instrument.ListOfScales[j]) do
                begin
                  if TScaleGroup(Instrument.ListOfScaleGroups[i]).IEN = GroupIEN then
                    begin
                      lvStringList.Add('');
                      lvStringList.Add('   Scale_' + IntToStr(j) + U +
                        Name + U +
                        ScaleGroup.IEN + U +
                        IntToStr(Sequence) + U +
                        XLabel);

                      for k := 0 to Instrument.ListOfKeys.Count - 1 do
                        begin
                          with TKey(Instrument.ListOfKeys[k]) do
                            begin
                              if ScaleID = TScale(Instrument.ListOfScales[j]).IEN then
                                begin
                                  lvStringList.Add('');
                                  lvStringList.Add('      ScoringKey_' + IntToStr(k) + U +
                                    Key.ScaleID + U +
                                    QuestionID + U +
                                    Target + U +
                                    Value);
                                end;
                            end; // with TKey(Instrument.ListOfKeys[k])
                        end; // for k
                      lvStringList.Add('   End Scale_' + IntToStr(j));
                    end; //  if IEN = GroupIEN
                end; // with TScale(Instrument.ListOfScales[j]) do
            end; //for j...
          lvStringList.Add('End ScaleGroup_' + IntToStr(i));
        end; // with Instrument.ListOfScaleGroups(i)
    end; // for i
end; *)

(*
procedure TfrmMHA_ResourceFile.WriteReport;
// This is saved in Vista as a solid text file.
var
  i                           : Integer;
  Reply,
    TempIEN                   : string;

begin

  ReportMemo.PlainText := true;
  ReportMemo.Clear;

  Reply := MHA_Report.ReadRoughReport(Instrument.Name, '', '', '', '', '',
    ReportMemo, TempIEN);

  if Reply = 'ERROR' then
    begin
      Showmessage('Unable to read report-template in VistA. ');
      ReportMemo.Free;
      exit;
    end;

  if (Reply = 'No Report') or (ReportMemo.Text = '') then
    begin
      ReportMemo.Free;
      Exit;
    end;

  lvStringList.Add('Report_follows');

  ReportMemo.SelectAll;
  lvStringList.Append(ReportMemo.SelText);

  lvStringList.Add(END_OF_REPORT);

  if MHA_Report.ListOfScripts.Count > 0 then
    for i := 0 to MHA_Report.ListOfScripts.Count - 1 do
      begin
        with TScript(MHA_Report.ListOfScripts[i]) do
          begin
            lvStringList.Add(
              ScriptName + EQ +
              FirstCaption + gvScriptDelimiter +
              FirstQuestionID + gvScriptDelimiter +
              FirstAnswer + gvScriptDelimiter +
              FirstAnswerIndex + gvScriptDelimiter +
              FirstOperation + gvScriptDelimiter +
              SecondCaption + gvScriptDelimiter +
              SecondQuestionID + gvScriptDelimiter +
              SecondAnswer + gvScriptDelimiter +
              SecondAnswerIndex + gvScriptDelimiter +
              SecondOperation + gvScriptDelimiter +
              ScriptText + RECORD_END);
          end; // with
      end; // for i

  lvStringList.Add('End_Report');
  ReportMemo.Free;
end; *)

procedure TfrmMHA_ResourceFile.ReadDescriptiveProperties(Instrument: TInstrument);
var
  i                           : Integer;
  TempStr                     : string;
begin
  with Instrument do
    begin
      //Instrument_OPS.Clear;
      i := 1;
      Name := Piece(lvStringList[1], U, i);
      Inc(i);
      PrintTitle := Piece(lvStringList[1], U, i);
      Inc(i);
      Version := Piece(lvStringList[1], U, i);
      Inc(i);
      RequiresSignature := StrToBool(Piece(lvStringList[1], U, i));
      Inc(i);
      Reference := Piece(lvStringList[1], U, i);
      Inc(i);
      RequiresLicense := StrToBool(Piece(lvStringList[1], U, i));
      Inc(i);
      TempStr := Piece(lvStringList[1], U, i);
      IsOperational := TempStr[1];
      Inc(i);
      LicenseIsCurrent := StrToBool(Piece(lvStringList[1], U, i));
      Inc(i);
      IsLegacy := StrToBool(Piece(lvStringList[1], U, i));
      Inc(i);
      NumberOfQuestions := StrToIntDef(Piece(lvStringList[1], U, i), 0);
      Inc(i);
      IEN := Piece(lvStringList[1], U, i);
      Inc(i);
      Publisher := Piece(lvStringList[1], U, i);
      Inc(i);
      Author := Piece(lvStringList[1], U, i);
      Inc(i);
      Copyright := Piece(lvStringList[1], U, i);
      Inc(i);
      PublicationDate := Piece(lvStringList[1], U, i);
      Inc(i);
      NormSample := Piece(lvStringList[1], U, i);
      Inc(i);
      R_Privilege := Piece(lvStringList[1], U, i);
      Inc(i);
      Purpose := Piece(lvStringList[1], U, i);
      Inc(i);
      TargetPopulation := Piece(lvStringList[1], U, i);
      Inc(i);
      EntryDate := Piece(lvStringList[1], U, i);
      Inc(i);
      EnteredBy := Piece(lvStringList[1], U, i);
      Inc(i);
      A_Privilege := Piece(lvStringList[1], U, i);
      Inc(i);
      LastEditedBy := Piece(lvStringList[1], U, i);
      Inc(i);
      LastEditDate := Piece(lvStringList[1], U, i);
      Inc(i);
      IsNationalTest := StrToBool(Piece(lvStringList[1], U, i));
      Inc(i);
      LastEditFMDate := Piece(lvStringList[1], U, i);
      Inc(i);
      SubmitToNationalDB := StrToBool(Piece(lvStringList[1], U, i));
      Inc(i);
      IsCopyrighted := StrToBool(Piece(lvStringList[1], U, i));
      Inc(i);
      WriteFullText := StrToBool(Piece(lvStringList[1], U, i));
    end;

end;



procedure TfrmMHA_ResourceFile.ReadQuestions(Instrument: TInstrument);
var
  Count,
    i, j, k                   : Integer;
  AQuestion                   : TQuestion;

begin

  // Start a 3; 0,1,2 for instrument object
  for i := 2 to lvStringList.Count - 1 do
    begin

      // String with Question data
      if Pos('Question_', lvStringList[i]) = 1 then
        begin
          lvStartIntro := i;
          j := 2;
          AQuestion := TQuestion.Create;

          AQuestion.Text := Piece(lvStringList[i], U, j);
          Inc(j);
          AQuestion.QuestionID := Piece(lvStringList[i], U, j);
          Inc(j);
          AQuestion.Max := StrToIntDef(Piece(lvStringList[i], U, j), 0);
          Inc(j);
          AQuestion.IsRequired := StrToBool(Piece(lvStringList[i], U, j));
          Inc(j);
          AQuestion.Min := StrToIntDef(Piece(lvStringList[i], U, j), 0);
          Inc(j);
          AQuestion.Designator := Piece(lvStringList[i], U, j);
          Inc(j);
          AQuestion.Hint := Piece(lvStringList[i], U, j);
          Inc(j);
          AQuestion.Caption := Piece(lvStringList[i], U, j);
          Inc(j);
          AQuestion.IsLastQuestion := StrToBool(Piece(lvStringList[i], U, j));
          Inc(j);
          AQuestion.IsFirstQuestion := StrToBool(Piece(lvStringList[i], U, j));
          Inc(j);
          AQuestion.ResponseType := ConvertStringToResponseType
            (Piece(lvStringList[i], U, j));
          Inc(j);

          // Build Question Display
          AQuestion.Display.DisplayID := Piece(lvStringList[i], U, j);
          Inc(j);
          AQuestion.Display.DisplayFont.Name := Piece(lvStringList[i], U, j); // Font.name
          Inc(j);
          AQuestion.Display.DisplayFont.Style := [];
          if Piece(lvStringList[i], U, j) = 'Y' then // Bold: boolean
            AQuestion.Display.DisplayFont.Style := [fsBold];
          Inc(j);
          if Piece(lvStringList[i], U, j) = 'Y' then // Italic: boolean
            AQuestion.Display.DisplayFont.Style :=
              AQuestion.Display.DisplayFont.Style + [fsItalic];
          Inc(j);
          if Piece(lvStringList[i], U, j) = 'Y' then // Underline: boolean
            AQuestion.Display.DisplayFont.Style :=
              AQuestion.Display.DisplayFont.Style + [fsUnderline];
          Inc(j);
          //Size: integer
          AQuestion.Display.DisplayFont.Size := StrToIntDef(
            Piece(lvStringList[i], U, j), 8);
          Inc(j);
          //Color: string
          AQuestion.Display.DisplayFont.Color := StringToColor(
            Piece(lvStringList[i], U, j));
          Inc(j);
           //Left: integer
          AQuestion.Display.DisplayLeft :=
            StrToIntDef(Piece(lvStringList[i], U, j), 1);
          Inc(j);
          //Align: TAlignment
          AQuestion.Display.DisplayAlignment :=
            StrToAlignment(Piece(lvStringList[i], U, j));
          Inc(j);

          // Build AQuestion.AnswerDisplay
          AQuestion.AnswerDisplay.DisplayID := Piece(lvStringList[i], U, j); // ID
          Inc(j);
          AQuestion.AnswerDisplay.DisplayFont.Name := Piece(lvStringList[i], U, j); // Font.name
          Inc(j);
          AQuestion.AnswerDisplay.DisplayFont.Style := [];
          if Piece(lvStringList[i], U, j) = 'Y' then // Bold: boolean
            AQuestion.AnswerDisplay.DisplayFont.Style := [fsBold];
          Inc(j);
          if Piece(lvStringList[i], U, j) = 'Y' then // Italic: boolean
            AQuestion.AnswerDisplay.DisplayFont.Style :=
              AQuestion.AnswerDisplay.DisplayFont.Style + [fsItalic];
          Inc(j);
          if Piece(lvStringList[i], U, j) = 'Y' then // Underline: boolean
            AQuestion.AnswerDisplay.DisplayFont.Style :=
              AQuestion.AnswerDisplay.DisplayFont.Style + [fsUnderline];
          Inc(j);
          //Size: integer
          AQuestion.AnswerDisplay.DisplayFont.Size := StrToIntDef(
            Piece(lvStringList[i], U, j), 8);
          Inc(j);
          //Color: string
          AQuestion.AnswerDisplay.DisplayFont.Color := StringToColor(
            Piece(lvStringList[i], U, j));
          Inc(j);
          //Left: integer
          AQuestion.AnswerDisplay.DisplayLeft := StrToIntDef(
            Piece(lvStringList[i], U, j), 1);
          Inc(j);
          // Width : integer
          AQuestion.AnswerDisplay.DisplayWidth := StrToIntDef(
            Piece(lvStringList[i], U, j), 100);
          Inc(j);
          //Align: TAlignment
          AQuestion.AnswerDisplay.DisplayAlignment :=
            StrToAlignment(
            Piece(lvStringList[i], U, j));
          Inc(j);
          AQuestion.AnswerDisplay.DisplayMask := Piece(lvStringList[i], U, j);
          Inc(j);
          AQuestion.AnswerDisplay.DisplayColumns := StrToIntDef(
            Piece(lvStringList[i], U, j), 1);
          Inc(j);
          AQuestion.AnswerDisplay.DisplayComponent :=
            StrToAnswerComponent(
            Piece(lvStringList[i], U, j));
          Inc(j);
          AQuestion.AnswerDisplay.DisplaySameLine :=
            StrToBool(Piece(lvStringList[i], U, j));
          Inc(j);

          // Choices
          if Pos('Choices Follow', lvStringList[i]) > 0 then
            begin
              Inc(j);
              //Inc(j);
              AQuestion.ChoiceIdentifier :=
                ConvertStringToChoiceIdentifier(Piece(lvStringList[i], U, j));
              Inc(j);
              AQuestion.MultiChoice.NumberOfPicks := StrToInt(Piece(lvStringList[i], U, j));
              Inc(j);
              AQuestion.MultiChoice.NumberOfChoices := StrToInt(Piece(lvStringList[i], U, j));
              Inc(j);

              if AQuestion.MultiChoice.NumberOfChoices > 0 then
                begin
                  for k := 1 to AQuestion.MultiChoice.NumberOfChoices do
                    begin
                      AQuestion.MultiChoice.Choices.Add(Piece(lvStringList[i], U, j));
                      Inc(j);
                    end;
                end;
            end;

// ShowMessage('AQuestion.MultiChoice.NumberOfPicks '+ IntToStr(AQuestion.MultiChoice.NumberOfPicks) +#13+
// 'AQuestion.MultiChoice.NumberOfChoices ' +IntToStr(AQuestion.MultiChoice.NumberOfChoices));

// if AQuestion.MultiChoice.Choices.Count > 1 then
// ShowMessage(
// 'AQuestion.MultiChoice.Choices[0] ' +   AQuestion.MultiChoice.Choices[0] +#13+
// 'AQuestion.MultiChoice.Choices[1] ' +  AQuestion.MultiChoice.Choices[1]);
          Instrument.ListOfQuestions.Add(AQuestion);
        end; // String with question data

    end; // for i; each string in import file
end;


procedure TfrmMHA_ResourceFile.ReadIntroductions(Instrument: TInstrument);
var
  i, j                        : Integer;
  AIntroduction               : TIntroduction;
begin

  for i := lvStartIntro to lvStringList.Count - 1 do
    begin

      if Pos('Introduction_', lvStringList[i]) = 1 then
        begin
          with AIntroduction do
            begin
              AIntroduction := TIntroduction.Create;
              lvStartSection := i;
              j := 2;

              ID := Piece(lvStringList[i], U, j);
              Inc(j);
              Text := Piece(lvStringList[i], U, j);
              Inc(j);
              IsNew := StrToBool(Piece(lvStringList[i], U, j));
              Inc(j);

              Display.DisplayID := Piece(lvStringList[i], U, j); // ID
              Inc(j);
              Display.DisplayFont.Name := Piece(lvStringList[i], U, j); // Font.name
              Inc(j);
              Display.DisplayFont.Style := [];
              if Piece(lvStringList[i], U, j) = 'Y' then // Bold: boolean
                Display.DisplayFont.Style := [fsBold];
              Inc(j);
              if Piece(lvStringList[i], U, j) = 'Y' then // Italic: boolean
                Display.DisplayFont.Style :=
                  Display.DisplayFont.Style + [fsItalic];
              Inc(j);
              if Piece(lvStringList[i], U, j) = 'Y' then // Underline: boolean
                Display.DisplayFont.Style :=
                  Display.DisplayFont.Style + [fsUnderline];
              Inc(j);
              //Size: integer
              Display.DisplayFont.Size := StrToIntDef(
                Piece(lvStringList[i], U, j), 8);
              Inc(j);
              //Color: string
              Display.DisplayFont.Color := StringToColor(
                Piece(lvStringList[i], U, j));
              Inc(j);
               //Left: integer
              Display.DisplayLeft := StrToIntDef(
                Piece(lvStringList[i], U, j), 1);
              Inc(j);
              //Align: TAlignment
              Display.DisplayAlignment :=
                StrToAlignment(Piece(lvStringList[i], U, j));

              Instrument.ListOfIntroductions.Add(AIntroduction);

            end;
        end;
    end;
end;


procedure TfrmMHA_ResourceFile.ReadSections(Instrument: TInstrument);
var

  i, j                        : Integer;
  ASection                    : TSectionLabel;
  Reply                       : string;
begin

  // Just in case there are no introductions
  if lvStartSection = 0 then
    lvStartSection := lvStartIntro;

  for i := lvStartSection to lvStringList.Count - 1 do
    begin

      if Pos('Section_', lvStringList[i]) = 1 then
        begin
          with ASection do
            begin
              ASection := TSectionLabel.Create;
              lvStartRule := i;
              j := 2;

              QuestionID := Piece(lvStringList[i], U, j);
              Inc(j);
              TabCaption := Piece(lvStringList[i], U, j);
              Inc(j);
              SectionCaption := Piece(lvStringList[i], U, j);
              Inc(j);
              Display.DisplayID := Piece(lvStringList[i], U, j); // ID
              Inc(j);
              Display.DisplayFont.Name := Piece(lvStringList[i], U, j); // Font.name
              Inc(j);
              Display.DisplayFont.Style := [];
              if Piece(lvStringList[i], U, j) = 'Y' then // Bold: boolean
                Display.DisplayFont.Style := [fsBold];
              Inc(j);
              if Piece(lvStringList[i], U, j) = 'Y' then // Italic: boolean
                Display.DisplayFont.Style :=
                  Display.DisplayFont.Style + [fsItalic];
              Inc(j);
              if Piece(lvStringList[i], U, j) = 'Y' then // Underline: boolean
                Display.DisplayFont.Style :=
                  Display.DisplayFont.Style + [fsUnderline];
              Inc(j);
              //Size: integer
              Display.DisplayFont.Size := StrToIntDef(
                Piece(lvStringList[i], U, j), 8);
              Inc(j);
              //Color: string
              Display.DisplayFont.Color := StringToColor(
                Piece(lvStringList[i], U, j));
              Inc(j);
               //Left: integer
              Display.DisplayLeft := StrToIntDef(
                Piece(lvStringList[i], U, j), 1);
              Inc(j);
              //Align: TAlignment
              Display.DisplayAlignment :=
                StrToAlignment(Piece(lvStringList[i], U, j));

              Instrument.ListOfSections.Add(ASection);

            end; // POS
        end; // with
    end; // for i
end;


procedure TfrmMHA_ResourceFile.ReadRules(Instrument: TInstrument);
var
  x, i, j, k                  : Integer;
  reply,
    OldSkipQuestions,
    NewSkipQuestions          : string;
  ARule                       : TRule;

  function MyStrToBoolOp(ABoolOperatorString: string): TBoolOperatorType;
  begin
    Result := boAND; //the default value
    if (ABoolOperatorString = 'OR') then
      Result := boOR;
  end; //MyStrToBoolOp

  function MyStrToDataType(ADataTypeString: string): TValueDataType;
  begin
    //dtString, dtDate, dtNumber
    Result := dtString;
    if (ADataTypeString = DATE_STR) then
      Result := dtDate;
    if (ADataTypeString = NUMBER_STR) then
      Result := dtNumber;
    if (ADataTypeString = INTERVAL_STR) then
      Result := dtTimeInterval;
  end; //MyStrToDataType

  function MyStrToOperator(AOperatorString: string): TOperatorType;
  var
    tmpIndex                  : integer;
    tmpString                 : string;
  begin
    //toEQ, toLT, toLE, toGT, toGE, toNE
   //  OPERATOR_STRINGS = 'Equals^Does not equal^Is less than^Is greater than^'
   //    + 'Equals or is less than^Equals or is greater than';

    tmpString := AOperatorString;
    tmpIndex := PieceIndex(OPERATOR_STRINGS, tmpString, U);

    case tmpIndex of
      2: Result := toNE;
      3: Result := toLT;
      4: Result := toGT;
      5: Result := toLE;
      6: Result := toGE;
    else
      Result := toEQ;
    end; //case

  end; //MyStrToSecondOp

begin
  // Just in case there are no Intro's or Sections
  if lvStartRule = 0 then
    lvStartRule := lvStartSection;
  if lvStartRule = 0 then
    lvStartRule := lvStartIntro;


 // for i := lvStartRule to lvStringList.Count - 1 do
  for i := lvStartRule to lvStringList.Count - 1 do
    begin

      // String with Rule data
      if Pos('Rule_', lvStringList[i]) = 1 then
        begin
          j := 2;
          ARule := TRule.Create;

          ARule.FirstValueDataType := MyStrToDataType(Piece(lvStringList[i], U, j));
          Inc(j);
          ARule.FirstOperator := MyStrToOperator(Piece(lvStringList[i], U, j));
          Inc(j);
          ARule.SecondValueDataType := MyStrToDataType(Piece(lvStringList[i], U, j));
          Inc(j);
          ARule.SecondOperator := MyStrToOperator(Piece(lvStringList[i], U, j));
          Inc(j);
          ARule.BoolOperator := MyStrToBoolOp(Piece(lvStringList[i], U, j));
          Inc(j);
          ARule.FirstQuestionID := Piece(lvStringList[i], U, j);
          Inc(j);
          ARule.FirstValue := Piece(lvStringList[i], U, j);
          Inc(j);
          ARule.SecondQuestionID := Piece(lvStringList[i], U, j);
          Inc(j);
          ARule.SecondValue := Piece(lvStringList[i], U, j);
          Inc(j);
          ARule.MessageText := Piece(lvStringList[i], U, j);
          Inc(j);
          x := Pos('Skip*', lvStringList[i]);
          ARule.SkipQuestionIDs := Copy(lvStringList[i], x, Length(lvStringList[i]));

          Instrument.ListOfRules.Add(ARule);
        end;

      ARule.Free;
    end;
end;

(*procedure TfrmMHA_ResourceFile.ReadScales(Instrument: TInstrument);
var
  i, j                        : Integer;
  TempScaleGroupID,
    TempScaleID,
    Reply                     : string;
  AScaleGroup                 : TScaleGroup;
  AScale                      : TScale;
  AKey                        : TKey;


begin

  AScaleGroup := TScaleGroup.Create;
  AScale := TScale.Create;
  AKey := TKey.Create;

  // Just in case there are no rules, Intro's or Sections
  if lvStartScale = 0 then
    lvStartScale := lvStartRule;
  if lvStartScale = 0 then
    lvStartScale := lvStartSection;
  if lvStartScale = 0 then
    lvStartScale := lvStartIntro;

  for i := lvStartScale to lvStringList.Count - 1 do
    begin

      // String with Scale Group data
      if Pos('ScaleGroup_', lvStringList[i]) = 1 then
        begin
          j := 2;

          AScaleGroup.GroupName := Piece(lvStringList[i], U, j);
          Inc(j);
          AScaleGroup.IEN := Piece(lvStringList[i], U, j);
          Inc(j);
          AScaleGroup.Sequence := StrToIntDef(Piece(lvStringList[i], U, j), 1);
          Inc(j);
          AScaleGroup.OrdinateTitle := Piece(lvStringList[i], U, j);
          Inc(j);
          AScaleGroup.OrdinateIncrement := StrToIntDef(Piece(lvStringList[i], U, j), 1);
          Inc(j);
          AScaleGroup.OrdinateMax := StrToIntDef(Piece(lvStringList[i], U, j), 10);
          Inc(j);
          AScaleGroup.OrdinateMin := StrToIntDef(Piece(lvStringList[i], U, j), 0);
          Inc(j);
          AScaleGroup.Grid1Ordinate := StrToIntDef(Piece(lvStringList[i], U, j), 0);
          Inc(j);
          AScaleGroup.Grid2Ordinate := StrToIntDef(Piece(lvStringList[i], U, j), 0);
          Inc(j);
          AScaleGroup.Grid3Ordinate := StrToIntDef(Piece(lvStringList[i], U, j), 0);
          Inc(j);
          AScaleGroup.NumberOfScales := StrToIntDef(Piece(lvStringList[i], U, j), 0);



        end; // POS ScaleGroup

      // String with Scale data
      if Pos('Scale_', lvStringList[i]) > 0 then
        begin
          j := 2;

          AScale.Name := Piece(lvStringList[i], U, j);
          Inc(j);
          AScale.IEN := Piece(lvStringList[i], U, j);
          Inc(j);
          AScale.Sequence := StrToIntDef(Piece(lvStringList[i], U, j), 0);
          Inc(j);
          AScale.XLabel := Piece(lvStringList[i], U, j);
          AScale.GroupIEN := AScaleGroup.IEN;


        end;

      // String with Scoring Key data
      if Pos('ScoringKey_', lvStringList[i]) > 0 then
        begin
          j := 2;

          AKey.ScaleID := AScale.IEN;
          AKey.QuestionID := ConvertOldToNew(Piece(lvStringList[i], U, j));
          Inc(j);
          AKey.Target := Piece(lvStringList[i], U, j);
          Inc(j);
          AKey.Value := Piece(lvStringList[i], U, j);

          reply := WriteScoringKey(True, AKey);
          if reply <> OK then
            Result := Reply;

          lvStartReport := i;
        end;
    end; // for i ...

  AScaleGroup.Free;
  AScale.Free;
  AKey.Free;
end;  *)

(*procedure TfrmMHA_ResourceFile.ReadReport(Instrument: TInstrument);
var
  i                           : Integer;
  aReport,
    TempIEN,
    Reply                     : string;
  OKToCopy                    : Boolean;
begin
  Result := OK;

  aReport := '';
  OkToCopy := False;

  for i := lvStartReport to lvStringList.Count - 1 do
    begin
      if OKToCopy then
        begin
          // Vista does not handle carriage returns well; mark with #124
          aReport := aReport + #124 + lvStringList[i];
        end;
      if Pos('Report_follows', lvStringList[i]) > 0 then
        OkToCopy := True;
      if Pos('End_Report', lvStringList[i]) > 0 then
        Break;
    end;

  if not OkToCopy then // No report
    Exit;

  // Replace the old QuestionIDs with the new ones
  for i := 0 to lvOldNewQuestionID.Count - 1 do
    begin
      StringReplace(aReport, Piece(lvOldNewQuestionID[i], EQ, 1),
        Piece(lvOldNewQuestionID[i], EQ, 2), [rfReplaceAll]);
    end;

  Reply := SaveReport(aReport, Instrument.IEN, TempIEN, True);

  if Reply <> OK then
    Result := Reply;
end; *)

end.

