unit fMHA_ProgressNote;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  StdCtrls,
  ComCtrls,
  JvExControls,
  JvComponent,
  JvXPCore,
  JvXPButtons,
  ExtCtrls,
  ORCtrls,
  uMHA_Administration;

type
  TfrmMHA_ProgressNote = class(TForm)
    pnlQuery: TPanel;
    btnSave: TJvXPButton;
    btnEdit: TJvXPButton;
    btnNoSave: TJvXPButton;
    lblQueryInfo: TLabel;
    cbxAll: TCheckBox;
    memPNote: TRichEdit;
    pnlBottom: TPanel;
    btnSaveEdit: TJvXPButton;
    btnCancel: TJvXPButton;
    lblStaff: TLabel;
    cbxStaff: TORComboBox;
    procedure FormCreate(Sender: TObject);
    procedure ConfigureForm;
    procedure ConfigureEdit;
    procedure GetProgressNote;
    function CoSignerRequired: boolean;
    procedure btnSaveClick(Sender: TObject);
    procedure btnEditClick(Sender: TObject);
    procedure btnNoSaveClick(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
    procedure btnSaveEditClick(Sender: TObject);
    procedure cbxStaffChange(Sender: TObject);
    procedure cbxStaffNeedData(Sender: TObject; const StartFrom: string;
      Direction, InsertAt: Integer);
    procedure cbxStaffDropDown(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Private declarations }
    function CheckCosigner: boolean;
  public
    { Public declarations }
  end;

var
  frmMHA_ProgressNote         : TfrmMHA_ProgressNote;

implementation

uses
  uIA_Const,
  fIA_Dialogs,
  uMHA_Registry,
  uIA_Utils,
  uIA_ThisUser,
  uMHA_Const,
  uMHA_Administration_OPS,
  uIA_Staff_OPS,
  uMHA_Report_OPS,
  uIA_GUIControl,
  uMHA_DM;

{$R *.dfm}

const
  lcProgress                  = 'Progress';
  lcConsult                   = 'Consult';

var
  lvCoSignerNeeded            : boolean;
  lvDUZ,
    lvProgNoteTitle,
    lvProgressOrConsult       : string;

procedure TfrmMHA_ProgressNote.FormCreate(Sender: TObject);
begin
  // Is this form used for a Consult or Progress Note?
  if gvAdmin.ConsultIEN <> '' then
    lvProgressOrConsult := lcConsult
  else
    lvProgressOrConsult := lcProgress;

  lvCoSignerNeeded := CoSignerRequired; // Default is false

  if gvUseScreenReader then
    Caption := 'Three choices about saving a ' + lvProgressOrConsult + ' note to Vista. ' +
      'Tab to hear choices, use enter key to select.'
  else
    Caption := 'Save ' + lvProgressOrConsult + ' Note to VistA';


  ConfigureForm;
  ShowModal;
end;

procedure TfrmMHA_ProgressNote.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  if lvCoSignerNeeded then
    MHA_Registry.SaveRegistry;
end;

procedure TfrmMHA_ProgressNote.ConfigureForm;
begin
  memPNote.Visible := false;
  pnlBottom.Visible := false;
  cbxAll.Visible := false;

  if lvCoSignerNeeded then
    begin
      cbxStaff.Visible := true;
      lblStaff.Visible := true;

      if Piece(gvCoSignerIEN, U, 1) = '' then
        begin
          cbxStaff.InitLongList('AA');
          cbxStaff.Text := '';
          lvDUZ := '';
        end
      else
        begin
          cbxStaff.Text := Piece(gvCoSignerIEN, U, 1);
          lvDUZ := Piece(gvCoSignerIEN, U, 2);
        end;
    end
  else
    begin
      cbxStaff.Visible := false;
      lblStaff.Visible := false;
    end;

  with GUI_Control do
    begin
      Scaled := True;
      AutoScroll := False;
      AutoSize := False;
      PixelsPerInch := 120;

      KeyPreview := True;
      HorzScrollBar.Range := 0; // Ensures that buttons will move when re-sized
      VertScrollBar.Range := 0;
      Font.Assign(gvFont);

      btnSave.Caption := '&Save standard ' + lvProgressOrConsult + ' note';
      btnEdit.caption := '&Edit then save ' + lvProgressOrConsult + ' note';
      btnNoSave.Caption := 'Do &not save ' + lvProgressOrConsult + ' note';

      if cbxStaff.visible then
        Height := ButtonHeight * 13
      else
        Height := ButtonHeight * 10;

      Width := 2 * TextWidth(btnEdit.Caption, gvFont);
      Left := GetLeft(Width) + (Width div 2);
      Top := GetTop(Height) + (Height div 2);
      if CheckWidth(Width) or CheckHeight(Height) then
        WindowState := wsMaximized;
      Refresh; // resets alClient dimensions

      pnlQuery.Left := 0;
      pnlQuery.Top := 0;
      pnlQuery.Width := Width;
      pnlQuery.Height := Height;

      cbxStaff.Height := 3 * TextHeight('Yy', gvFont) div 2;
      lblStaff.Height := cbxStaff.Height;
      lblQueryInfo.Height := 3 * ButtonHeight;

      lblQueryInfo.Caption := #13 + gvAdmin.InstrumentName + // Instrument Name
        #13 + 'This administration has been saved in Vista.' +
        #13 + 'Do you wish to:';

      if gvAdmin.ConsultIEN <> '' then
        begin
          cbxAll.Visible := false;
          //Height := Height + ButtonHeight;
          pnlQuery.align := alClient;
          //lblConsult.Visible := true;
        end;

      btnSave.Height := ButtonHeight;
      btnEdit.Height := btnSave.Height;
      btnNoSave.Height := btnSave.Height;
      cbxAll.Height := btnSave.Height;

      btnSave.Width := TextWidth(btnEdit.Caption + 'XXXXXX', gvFont);
      btnEdit.Width := btnSave.Width;
      btnNoSave.Width := btnSave.Width;
      cbxAll.Width := Width;
      cbxStaff.Width := btnSave.Width;
      lblStaff.Width := cbxStaff.Width;

      if cbxStaff.visible then
        begin
          lblStaff.Top := lblQueryInfo.Height + ButtonHeight div 2;
          cbxStaff.Top := lblStaff.Top + (3 * lblStaff.Height div 2);
          btnSave.Top := cbxStaff.Top + (3 * ButtonHeight div 2);
        end
      else
        btnSave.Top := lblQueryInfo.Height + ButtonHeight div 2;

      btnEdit.Top := btnSave.Top + (3 * ButtonHeight div 2);
      btnNoSave.Top := btnEdit.Top + (3 * ButtonHeight div 2);
      cbxAll.Top := btnNoSave.Top + (3 * ButtonHeight div 2);

      btnSave.Left := (Width - btnSave.Width) div 2;
      btnEdit.Left := btnSave.Left;
      btnNoSave.Left := btnSave.Left;
      lblStaff.Left := btnSave.Left;
      cbxStaff.Left := btnSave.Left;
      cbxAll.Left := btnSave.Left div 2;

      if gvUseScreenReader then
        begin
          btnCancel.Caption := btnCancel.Caption + ' button';
          btnSave.Caption := btnSave.Caption + ' button';
          btnEdit.Caption := btnEdit.Caption + ' button';
          btnNoSave.Caption := btnNoSave.Caption + ' button';
          btnSaveEdit.Caption := btnSaveEdit.Caption + ' button';
        end;

      if cbxStaff.visible then
        ActiveControl := cbxStaff
      else
        ActiveControl := btnSave;
    end;
end;

function TfrmMHA_ProgressNote.CoSignerRequired: boolean;
begin
// If the title is blank, then the cosigner will always come back false (no cosigner needed).
// Best to use the default title when blank.

  if lvProgressOrConsult = lcProgress then
    begin
      if gvAdmin.ProgNoteTitle = '' then
        lvProgNoteTitle := PROGRESS_NOTE_TITLE
      else
        lvProgNoteTitle := gvAdmin.ProgNoteTitle;
      result := ThisUser.CosignerNeeded(ThisUser.DUZ, lvProgNoteTitle,
        DataModule1.RPCBroker1);
    end
  else
    begin
      if gvAdmin.ConsultTitle = '' then
        lvProgNoteTitle := CONSULT_NOTE_TITLE
      else
        lvProgNoteTitle := gvAdmin.ConsultTitle;
      result := ThisUser.CosignerNeeded(ThisUser.DUZ, lvProgNoteTitle,
        DataModule1.RPCBroker1);
    end;
end;

procedure TfrmMHA_ProgressNote.GetProgressNote;
// This takes care of consults too.
begin

  Administration_OPS.BuildProgressNote(gvAdmin, True);

  with memPNote do
    begin
      Clear;
      memPNote.Lines.AddStrings(Administration_OPS.ProgressNoteStr);

      SelStart := 0;
      SelLength := 18000;
      SelAttributes.Name := 'Courier New';
      SelAttributes.Size := gvFont.Size;
      SelAttributes.Color := gvFont.color;
      SelAttributes.Style := gvFont.Style;

      (*
      Font.Name := 'Courier New';
      Font.Size := gvFont.Size;
      Font.Color := gvFont.color;
      Font.Style := gvFont.style;
      MaxLength := Length(Text); *)
      SelStart := 0;

    end;
end;

function TfrmMHA_ProgressNote.CheckCosigner: boolean;
begin
  result := false;

  if lvCoSignerNeeded then
  // Ensure the user has selected a co-signer; cannot be themself or
  // another student.
    begin

      if POS(UpperCase(ThisUser.Name), UpperCase(cbxStaff.Text)) > 0 then
        begin
          MsgDlgFrm('You must select a co-signer (it can not be yourself).',
            MHA_CAPTION, mtInformation, [mbOK], 0);
          exit;
        end;

      if ThisUser.DUZ = lvDUZ then
        begin
          MsgDlgFrm('You must select a co-signer (it can not be yourself).',
            MHA_CAPTION, mtInformation, [mbOK], 0);
          exit;
        end;

      if cbxStaff.Text = '' then
        begin
          MsgDlgFrm('You must select a co-signer.',
            MHA_CAPTION, mtInformation, [mbOK], 0);
          exit;
        end;

      if ThisUser.CosignerNeeded(lvDUZ, lvProgNoteTitle,
        DataModule1.RPCBroker1) then
        begin
          MsgDlgFrm(cbxStaff.Text + #13 +
            'is not authorized to cosign this document.',
            MHA_CAPTION, mtInformation, [mbOK], 0);
          exit;
        end;
    end;

  result := true;
end;

procedure TfrmMHA_ProgressNote.btnEditClick(Sender: TObject);
begin
  if not CheckCosigner then
    exit;

  // Progress notes and consults are the same here
  ConfigureEdit;
  GetProgressNote;

end;

procedure TfrmMHA_ProgressNote.btnSaveClick(Sender: TObject);
begin
  // Save the standard, boilerplate progress-note.
  // Handles legacy and regular instruments

  if not CheckCosigner then
    exit;

  if lvProgressOrConsult = lcProgress then
    Administration_OPS.SaveProgressNote(True, gvAdmin)
  else
    Administration_OPS.SaveConsultNote(True, gvAdmin);

  Close;
end;

procedure TfrmMHA_ProgressNote.btnNoSaveClick(Sender: TObject);
begin
  Close;
end;


procedure TfrmMHA_ProgressNote.btnCancelClick(Sender: TObject);
var
  reply                       : TModalResult;
begin

  Reply := MsgDlgFrm('To cancel means that no ' + lvProgressOrConsult + ' note will be saved to VistA. ' + #13 + #13 +
    'Would you like to save the standard ' + lvProgressOrConsult + ' note?',
    gvAdmin.InstrumentName,
    mtConfirmation, [mbYes, mbNo], 0);

  if reply = mrYes then
    begin
      if lvProgressOrConsult = lcProgress then
        Administration_OPS.SaveProgressNote(True, gvAdmin)
      else
        Administration_OPS.SaveConsultNote(True, gvAdmin);
    end;

  Close;
end;


procedure TfrmMHA_ProgressNote.ConfigureEdit;
begin
  pnlQuery.visible := false;

  with GUI_Control do
    begin
      Left := gvLeft;
      Top := gvTop;
      Height := gvHeight;
      Width := gvWidth;

      if CheckWidth(Width) or CheckHeight(Height) then
        WindowState := wsMaximized;
      Refresh; // resets alClient dimensions

      pnlBottom.Visible := true;
      pnlBottom.Align := alBottom;

      memPNote.Visible := true;
      memPNote.Align := alClient;
      memPNote.ReadOnly := false;
      memPNote.Enabled := true;

      pnlQuery.Height := 0;
      pnlBottom.Height := 2 * ButtonHeight;
      btnCancel.Height := ButtonHeight;
      btnSaveEdit.Height := ButtonHeight;

      btnCancel.Width := ButtonWidthNormal;
      btnSaveEdit.Width := btnCancel.Width;

      btnSaveEdit.Top := pnlBottom.Height div 4;
      btnCancel.Top := btnSaveEdit.Top;

      btnCancel.Left := btnCancel.Width div 2;
      btnSaveEdit.Left := pnlBottom.Width - (3 * btnSaveEdit.Width div 2);

      ActiveControl := memPNote;
    end;
end;

procedure TfrmMHA_ProgressNote.btnSaveEditClick(Sender: TObject);
begin

  Administration_OPS.ProgressNoteStr.Clear;
  Administration_OPS.ProgressNoteStr.AddStrings(memPNote.lines);

  if lvProgressOrConsult = lcProgress then
    Administration_OPS.SaveEditedProgressNote(True, gvAdmin)
  else
    Administration_OPS.SaveEditedConsultNote(True, gvAdmin);

  Close;
end;

procedure TfrmMHA_ProgressNote.cbxStaffNeedData(Sender: TObject;
  const StartFrom: string; Direction, InsertAt: Integer);
begin
  inherited;

  (Sender as TORComboBox).ForDataUse(Staff_OPS.LoadStaffSubSet(StartFrom,
    Direction, DataModule1.RPCBroker1));
end;

procedure TfrmMHA_ProgressNote.cbxStaffDropDown(Sender: TObject);
begin
 // OnFormShow does not load cbxStaff items because the method is very
 // slow. This method will load the items the first time the user clicks
 // on the combobox's down arrow (i.e., when item.count = 0)

 // I don't think this used.


  if (cbxStaff.Items.Count = 0) then
    begin
      if (gvCoSignerIEN = '') or (gvCoSignerIEN = '0') then
        begin
          cbxStaff.InitLongList(ThisUser.Name);
          cbxStaff.SelectByIEN(StrToInt(ThisUser.DUZ));
          lvDUZ := ThisUser.DUZ;
        end
      else
        begin
          cbxStaff.InitLongList(Piece(gvCoSignerIEN, U, 1));
          cbxStaff.SelectByIEN(StrToInt(Piece(gvCoSignerIEN, U, 2)));
          lvDUZ := Piece(gvCoSignerIEN, U, 2);
        end;
    end;

end;

procedure TfrmMHA_ProgressNote.cbxStaffChange(Sender: TObject);
begin
  lvDUZ := IntToStr(cbxStaff.ItemIEN);

  if (lvDUZ = '0') or (lvDUZ = '') or (lvDUZ = ' ') then
    begin
      gvCoSignerIEN := '';
      exit;
    end
  else
    gvCoSignerIEN := cbxStaff.text + U + lvDUZ;
end;

end.

