unit fMHA_UploadResults;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Classes,
  Graphics,
  Controls,
  Forms,
  StdCtrls,
  RzLabel,
  ExtCtrls,
  RzPanel,
  uIA_Patient,
  uMHA_Administration,
  ComCtrls,
  Menus,
  Grids,
  BaseGrid,
  AdvGrid,
  JvExControls,
  JvComponent,
  JvXPCore,
  JvXPButtons;

type

  TfrmMHA_UploadResults = class(TForm)
    pnlHeader: TRzPanel;
    lblHeader: TRzLabel;
    pnlBottom: TPanel;
    pnlOffLine: TPanel;
    lblInstruction: TLabel;
    Splitter1: TSplitter;
    MainMenu1: TMainMenu;
    mnuFile: TMenuItem;
    mnuExit: TMenuItem;
    mnuHelp: TMenuItem;
    mnuOnlineSupport: TMenuItem;
    N1: TMenuItem;
    mnuAbout: TMenuItem;
    pnlMatch: TPanel;
    pnlSpacer1: TPanel;
    pnlSpacer2: TPanel;
    grpMatch: TGroupBox;
    pnlSpacer0: TPanel;
    pnlSpacer4: TPanel;
    Splitter2: TSplitter;
    pnlLevel: TPanel;
    lblConfidence: TLabel;
    lblLevel: TLabel;
    pnlLeft: TPanel;
    grpCPRS: TGroupBox;
    pnlSearch: TPanel;
    lblSearch: TLabel;
    lblSearchInstructions: TLabel;
    edSearch: TEdit;
    grpPossible: TRzGroupBox;
    grpRecords: TRzGroupBox;
    grdOffLine: TAdvStringGrid;
    grdVista: TAdvStringGrid;
    btnGo: TJvXPButton;
    btnDelete: TJvXPButton;
    btnUpload: TJvXPButton;
    btnClose: TJvXPButton;
    imgBackup: TImage;
    grdMatch: TAdvStringGrid;
    procedure FormCreate(Sender: TObject);
    procedure btnCloseClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ConfigureForm;
    procedure grdMatchGetCellColor(Sender: TObject; ARow, ACol: Integer;
      AState: TGridDrawState; ABrush: TBrush; AFont: TFont);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure btnGoClick(Sender: TObject);
    procedure edSearchChange(Sender: TObject);
    procedure btnUploadClick(Sender: TObject);
    procedure btnDeleteClick(Sender: TObject);
    procedure mnuExitClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure mnuOnlineSupportClick(Sender: TObject);
    procedure mnuAboutClick(Sender: TObject);
    procedure grdVistaClick(Sender: TObject);
    procedure edSearchKeyPress(Sender: TObject; var Key: Char);
    procedure grdOffLineKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure grdVistaKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure imgBackupClick(Sender: TObject);
    procedure grdOffLineClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
  private
    //FUploadAdministration: TAdministration;
    procedure InitGrdMatch;
    procedure InitGrdOffLine;
    procedure InitGrdVista;
    function SexChange(ASex: string): string;
    procedure LoadOffLineAdministrations;
    procedure DisplayOfflinePatientProperties(AName: string);
    procedure DisplayMatchProperties(APatient: TPatient);
    procedure SearchForVistaPatients;
    procedure UpdateCPRSPatientProperties;
    procedure DisplayPatientProperties(AName: string);
    procedure DisplayMatchConfidenceLevel;
    procedure SettingsForNewMatch;
    procedure LoadTestNameAndDateByRow(ARow: integer);
    //function MessageText(SubText: string): string;
    procedure ClearMatchGrid;
    procedure SetButtons;
    procedure HandleProgressNote;
    { Private declarations }
  public
    //property UploadAdministration: TAdministration read FUploadAdministration
    //  write FUploadAdministration;
    procedure DisplayCPRSPatientProperties;
    { Public declarations }
  end;

var
  frmMHA_UploadResults        : TfrmMHA_UploadResults;

implementation

uses
  uIA_Utils,
  fMHA_About,
  uMHA_Registry,
  uIA_ThisUser,
  uIA_Instrument,
  uMHA_Instrument_OPS,
  uMHA_OffLineFile,
  uMHA_LocalFiles,
  uMHA_UploadResults_OPS,
  fIA_Dialogs,
  uMHA_Administration_OPS,
  uMHA_ResourceFile_OPS,
  uIA_Const,
  uMHA_Const,
  fMHA_Main,
  uIA_GUIControl,
  fMHA_UploadAdmin,
  uMHA_DM,
  fMHA_ProgressNote;

var
  lvCPRSPatient               : TPatient;
  lvInitialize                : Boolean;
  lvCurrentFile,
    lvCurrentPt               : Integer;

  lvUploadAdministration      : TAdministration;
  lvInstrument                : TInstrument;

{$R *.DFM}

procedure TfrmMHA_UploadResults.FormCreate(Sender: TObject);
begin
  // Keep Position out of ConfigureForm when ConfigureForm can be called
  // more than once with the form.
  Position := poDesigned;

  lvInitialize := True;
  lvUploadAdministration := TAdministration.Create;

  lvCPRSPatient := TPatient.Create;
  MHA_Registry.ReadRegistry;
  ConfigureForm;
  lvCurrentPt := -1;
  lvCurrentFile := -1;
end;

procedure TfrmMHA_UploadResults.FormShow(Sender: TObject);
begin
  UpdateCPRSPatientProperties;

  lblInstruction.Caption :=
    'Highlight an off-line record, then find the matching patient '
    + 'in CPRS and, finally, upload the record to VistA.';

  frmMHA_Main.Hide;

  InitGrdVista;
  InitGrdMatch;

  LoadOffLineAdministrations;
  DisplayOfflinePatientProperties(grdOffLine.Cells[0, grdOffLine.Row]);
  LoadTestNameAndDateByRow(grdOffLine.Row);

  lvInitialize := False;
  Screen.Cursor := crDefault;

  if gvUseScreenReader then
    ActiveControl := grpRecords
  else
    ActiveControl := grdOffLine;
end;


procedure TfrmMHA_UploadResults.FormDestroy(Sender: TObject);
begin
//  if lvCPRSPatient <> nil then
  lvCPRSPatient.Free;
//  if lvUploadAdministration <> nil then
  lvUploadAdministration.Free;
end;


procedure TfrmMHA_UploadResults.ConfigureForm;
var
  i                           : Integer;
begin
  with GUI_Control do
    begin
      Scaled := True;
      AutoScroll := False;
      AutoSize := False;

      PixelsPerInch := 120;
      KeyPreview := True;
      HorzScrollBar.Range := 0; // Ensures that buttons will move when re-sized
      VertScrollBar.Range := 0;
      Font.Assign(gvFont);

      Caption := MHA_Caption + ' in use by ' + ThisUser.Name;

      Height := GetHeight(28 * ButtonHeight); //CheckHeight(480, gvHeight);
      Width := GetWidth(12 * ButtonWidthNormal); //CheckWidth(640, gvWidth);
      Left := GetLeft(Width);
      Top := GetTop(Height);
      if CheckWidth(Width) or CheckHeight(Height) then
        WindowState := wsMaximized;
      Refresh; // resets alClient dimensions

      pnlHeader.Height := pnlHeaderHeight;
      lblHeader.Font.Assign(gvFont);

      pnlOffline.Height := 4 * ButtonHeight;
      pnlBottom.Height := 3 * ButtonHeight div 2;
      pnlLeft.Width := Width div 2;

      for i := 0 to ComponentCount - 1 do
        begin
          if Components[i] is TLabel then
            begin
              TLabel(Components[i]).Font.Assign(gvFont);
              TLabel(Components[i]).Font.style := [];
              TLabel(Components[i]).Layout := tlCenter;
              TLabel(Components[i]).Height := 3 * TextHeight('Yy', gvFont) div
                2;
              if TLabel(Components[i]).Name = 'lblSearchInstructions' then
                begin
                  TLabel(Components[i]).Height := GetWrapHeight(
                    TLabel(Components[i]).Caption, grpCPRS.Width, gvFont);
                  TLabel(Components[i]).Layout := tlTop;
                  TLabel(Components[i]).Font.Color := clNavy;
                end;
            end;
          if Components[i] is TGroupBox then
            begin
              TGroupBox(Components[i]).Font.Assign(gvFont);
              TGroupBox(Components[i]).Font.Style := [fsBold];
            end;
          if Components[i] is TRzGroupBox then
            begin
              TRzGroupBox(Components[i]).Font.Assign(gvFont);
              TRzGroupBox(Components[i]).Font.Style := [fsBold];
            end;
          if Components[i] is TEdit then
            begin
              TEdit(Components[i]).Font.Assign(gvFont);
              TEdit(Components[i]).Font.Style := [];
              TEdit(Components[i]).Height := ButtonHeight;
            end;
          if Components[i] is TJvXPButton then
            begin
              TJvXPButton(Components[i]).Font.Assign(gvFont);
              TJvXPButton(Components[i]).Font.style := [];
              TJvXPButton(Components[i]).Height := ButtonHeight;
              TJvXPButton(Components[i]).Width :=
                ButtonWidth(TButton(Components[i]).Caption);
              TJvXPButton(Components[i]).Top :=
                (pnlBottom.Height - ButtonHeight) div 2;
            end;
        end;

      lblHeader.Font.Size := lblHeaderFontSize;
      lblInstruction.Font.Size := lblPatientFontSize;
      lblInstruction.Width := pnlHeader.Width div 2;

      lblSearch.Top := ButtonHeight div 4;
      lblSearch.Left := lblSearch.Top;
      lblSearch.Width := TextWidth(lblSearch.caption + 'X', gvFont);

      edSearch.Top := lblSearch.Top;
      edSearch.Left := lblSearch.Left + 5 + lblSearch.Width;
      edSearch.Width := TextWidth('XXXXXXXXXXXXXXX', gvFont);

      btnGo.Top := lblSearch.Top;
      btnGo.Left := edSearch.Left + edSearch.Width + 5;
      lblSearchInstructions.Top := (2 * lblSearch.Top) + lblSearch.Height;

      lblConfidence.Left := 10;
      lblLevel.Left := lblConfidence.Left + lblConfidence.Width + 5;
      lblLevel.Top := lblConfidence.Top;

      pnlSearch.Height := lblSearch.Height + lblSearchInstructions.Height +
        (3 * lblSearch.Top);

      btnDelete.Left := btnClose.Top;
      btnClose.Left := pnlBottom.Width - btnClose.Width - btnClose.Top;
      btnUpload.Left := btnClose.Left - btnUpload.Width - btnClose.Top;
    end;

  if gvUseScreenReader then
    begin
      Left := 0;
      Width := Screen.Width;
      lblSearchInstructions.Visible := False;
      pnlLeft.Width := grpRecords.Width;
      pnlBottom.Visible := False;
      grpMatch.Visible := False;
      splitter2.Visible := False;
      pnlSpacer2.Visible := False;

      grpRecords.TabStop := True;
      grpRecords.Caption := 'The following grid is a list the off line records. '
        +
        'Each row contains the patients name, S S N, D O B, gender, name of instrument ' +
        'and the date of administration. Navigate using the arrow keys. ' +
        'Delete record by pressing delete key. Enter grid by pressing tab key.';

      grpCPRS.TabStop := True;
      grpCPRS.Caption := 'The next step is to search for this person in CPRS. ' +
        'You can use their last name or S S N number or the first letter of their last ' +
        'name plus the last four digits of S S N. The S S N is entered by default. Then press GO button or enter key. ' +
        'A list of potential matches will follow. Press tab key to continue.';

      grpPossible.TabStop := True;
      grpPossible.Caption := 'This grid contains a list of potential matches. ' +
        'Each row contains match level, name, S S N, D O B, and gender. Navigate using the arrow ' +
        'keys. Select record to upload by pressing the enter key. Enter the grid by pressing the tab key.';

      Caption := 'Off Line Results: ' + lblInstruction.Caption;

      btnGo.caption := btnGo.Caption + ' button';
      btnDelete.caption := btnDelete.Caption + ' button';
      btnUpload.caption := btnUpload.Caption + ' button';
      btnClose.caption := btnClose.Caption + ' button';

    end
  else
    begin
      grpMatch.Visible := True;
      grpRecords.TabStop := False;
      grpCPRS.tabstop := False;
      grpPossible.tabstop := False;
    end;
end;


procedure TfrmMHA_UploadResults.btnCloseClick(Sender: TObject);
begin
  Close;
end;

procedure TfrmMHA_UploadResults.InitGrdOffLine;
begin
  with grdOffLine, GUI_Control do
    begin
      Clear;
      Font.Assign(gvFont);
      FixedFont.Assign(gvFont);
      ActiveCellFont.Assign(gvFont);
      FixedFont.Style := [];
      DefaultColWidth := TextWidth('XXXXXXXXXXXXXXXX', gvFont);
      DefaultRowHeight := 3 * TextHeight('Yy', gvFont) div 2;
        // The width of Col[2] will vary depending on the width of the panel.
      ColumnSize.StretchColumn := 4;
      ColumnSize.Stretch := True;
      ColWidths[2] := TextWidth('XXXXXXXXXXX', gvFont);
      ColWidths[3] := TextWidth('XXXXX', gvFont);
      SelectionRectangle := true;
      ShowSelection := True;

      if gvUseScreenReader then
        begin
          //SelectionRectangle := True;
          //ShowSelection := True;
          Options := [goFixedVertLine, goFixedHorzLine, goHorzLine,
            goDrawFocusSelected, goColSizing, goRowSelect]
        end
      else
        begin
          //SelectionRectangle := true;
          //ShowSelection := True;
          Options := [goFixedVertLine, goFixedHorzLine, goVertLine, goHorzLine,
            goDrawFocusSelected, goColSizing, goRowSelect];
        end;

      Cells[0, 0] := 'Name';
      Cells[1, 0] := 'SSN';
      Cells[2, 0] := 'DOB';
      Cells[3, 0] := 'Sex';
      Cells[4, 0] := 'Instrument';
      Cells[5, 0] := 'Date';
    end;
end;

procedure TfrmMHA_UploadResults.InitGrdVista;
begin
  with grdVista, GUI_Control do
    begin
      Clear;
      Font.Assign(gvFont);
      FixedFont.Assign(gvFont);
      ActiveCellFont.Assign(gvFont);
      FixedRows := 1;
      FixedCols := 0;
      FixedFont.Style := [];
      ColCount := 5;
      RowCount := 5;
      DefaultColWidth := TextWidth('XXXXXXXX', gvFont);
      DefaultRowHeight := 3 * TextHeight('Yy', gvFont) div 2;
        // The width of Col[2] will vary depending on the width of the panel.
      ColumnSize.StretchColumn := 1;
      ColumnSize.Stretch := True;
      IntegralHeight := False;
      ColWidths[0] := TextWidth('XMATCHX', gvFont);
      ColWidths[2] := TextWidth('XXXXXXXXXXX', gvFont);
      SelectionRectangle := true;
      ShowSelection := True;

      if gvUseScreenReader then
        Options := [goHorzLine, goDrawFocusSelected, goColSizing, goRowSelect]
      else
        Options := [goFixedVertLine, goFixedHorzLine, goVertLine, goHorzLine,
          goDrawFocusSelected, goColSizing, goRowSelect];

      Cells[0, 0] := 'Match';
      Cells[1, 0] := 'Name';
      Cells[2, 0] := 'SSN';
      Cells[3, 0] := 'DOB';
      Cells[4, 0] := 'Sex';
    end;
end;

procedure TfrmMHA_UploadResults.InitGrdMatch;
begin

  with grdMatch, GUI_Control do
    begin
      Clear;
      Font.Assign(gvFont);
      FixedFont.Assign(gvFont);
      ActiveCellFont.Assign(gvFont);
      FixedFont.Style := [];
      FixedCols := 1;
      FixedRows := 1;
      ColCount := 3;
      RowCount := 5;

      FixedColWidth := TextWidth('XXXXXXXXXXXXXXXXXX', gvFont); // 18
      DefaultColWidth := TextWidth('XXXXXXXXXXXXXXXXXX', gvFont); // 18
      DefaultRowHeight := 3 * TextHeight('Yy', gvFont) div 2;
      ColWidths[0] := TextWidth('XNameX', gvFont);

      Height := RowCount * DefaultRowHeight + 5;
        // The width of Col[2] will vary depending on the width of the panel.
      ColumnSize.StretchColumn := 1;
      ColumnSize.Stretch := True;
      Options := [goFixedVertLine, goFixedHorzLine, goHorzLine, goVertLine,
        goColSizing];

      Cells[0, 1] := 'Name';
      Cells[0, 2] := 'SSN';
      Cells[0, 3] := 'DOB';
      Cells[0, 4] := 'Sex';

      Cells[1, 0] := 'Off-Line Patient';
      Cells[2, 0] := 'CPRS Patient';
    end;
end;

procedure TfrmMHA_UploadResults.ClearMatchGrid;
var
  i                           : integer;
begin
  for i := 1 to 4 do
    grdMatch.Cells[2, i] := '';
end;

procedure TfrmMHA_UploadResults.LoadOffLineAdministrations;
var
  i, j, k                     : integer;
begin
  InitGrdOffLine;

  UploadResults_OPS.ListOfOffLineFiles.Clear;
  UploadResults_OPS.LoadOffLineFiles;

  UploadResults_OPS.ListOfOffLinePatients.Clear;
  UploadResults_OPS.LoadOffLinePatients;

  grdOffLine.RowCount := UploadResults_OPS.ListOfOffLineFiles.Count + 1;
  k := 0;

  for j := 0 to UploadResults_OPS.ListOfOffLinePatients.Count - 1 do
    for i := 0 to UploadResults_OPS.ListOfOffLineFiles.Count - 1 do
      begin
        if TOffLineFile(UploadResults_OPS.ListOfOffLineFiles[i]).OffLinePtID =
          TPatient(UploadResults_OPS.ListOfOffLinePatients[j]).DFN then
          begin
            with grdOffLine do
              begin
              //  if (TOffLineFile(UploadResults_OPS.ListOfOffLineFiles[i]).InstrumentName <> '') and
              //    (TPatient(UploadResults_OPS.ListOfOffLinePatients[j]).Name <> '') then
                begin
                  Inc(k);
                  Cells[0, k] := TPatient(UploadResults_OPS.ListOfOffLinePatients[j]).Name;
                  Cells[1, k] := TPatient(UploadResults_OPS.ListOfOffLinePatients[j]).FormattedSSN;
                  Cells[2, k] := TPatient(UploadResults_OPS.ListOfOffLinePatients[j]).DOB;
                  Cells[3, k] := SexChange(TPatient(UploadResults_OPS.ListOfOffLinePatients[j]).Sex);
                  Cells[4, k] := TOffLineFile(UploadResults_OPS.ListOfOffLineFiles[i]).InstrumentName;
                  Cells[5, k] := TOffLineFile(UploadResults_OPS.ListOfOffLineFiles[i]).DateGiven;
                end;
              end; //with grdOffLine
          end; // if
      end; //for j, i...

  if k > 0 then
    begin
      grdOffLine.SortSettings.Direction := sdAscending;
      grdOffLine.SortByColumn(0);
      grdOffLine.Row := 1;
//    end
//  else
//    begin
//      SysUtils.DeleteFile(LocalFiles.BuildAdministrationFileName(
//        TOffLineFile(UploadResults_OPS.ListOfOffLineFiles[0]).OffLinePtID,
//        DateTimeToFileNameDateTime(StrToDateTime(TOffLineFile(UploadResults_OPS.ListOfOffLineFiles[0]).DateGiven)),
//        TOffLineFile(UploadResults_OPS.ListOfOffLineFiles[0]).InstrumentName));
//
//      SysUtils.DeleteFile(LocalFiles.AnswerFilesPath + '\' +
//        TOffLineFile(UploadResults_OPS.ListOfOffLineFiles[0]).OffLinePtID +
//        FILE_NAME_DELIMITER +
//        TOffLineFile(UploadResults_OPS.ListOfOffLineFiles[0]).OffLineAdminID +
//        '.' + ANSWER_FILE_EXT);
//      Close;
    end;

  SetButtons;
end;

procedure TfrmMHA_UploadResults.SetButtons;
begin
  btnDelete.Enabled := False;
  btnUpload.Enabled := False;

  if (grdOffLine.RowCount > 1) and
    (grdVista.RowCount > 1) then
    begin
      btnDelete.Enabled := True;
      btnUpload.Enabled := True;
    end;

  if (grdOffLine.RowCount > 1) then
    btnDelete.Enabled := True;

end;

procedure TfrmMHA_UploadResults.SearchForVistaPatients;
var
  i                           : integer;
  TempStr,
    SearchString              : string;
begin
  // This procedure searches VistA for records that match the search string.

  InitGrdVista;
  SearchString := StringReplace(edSearch.Text, '-', '', [rfReplaceAll]);

  with grdVista, UploadResults_OPS do
    begin
      SearchForPatients(SearchString);

      for i := 0 to NumberOfPatientsFound - 1 do
        begin
          Cells[1, i + 1] := Piece(PatientSearchNameByIndex(i), EQ, 1);
          TempStr := Piece(PatientSearchNameByIndex(i), EQ, 2);
          Cells[2, i + 1] := Piece(TempStr, U, 2);
          Cells[3, i + 1] := Piece(TempStr, U, 1);
          Cells[4, i + 1] := Piece(TempStr, U, 3);

          UploadResults_OPS.LoadSearchPatient(lvCPRSPatient, Cells[1, i + 1]);
          UploadResults_OPS.LoadMatchConfidenceLevel(lvCPRSPatient,
            TPatient(UploadResults_OPS.ListOfOffLinePatients[lvCurrentPt]));

          case UploadResults_OPS.MatchConfidenceLevel of
            mlLow: Cells[0, i + 1] := 'Low';
            mlGood: Cells[0, i + 1] := 'Good';
            mlHigh: Cells[0, i + 1] := 'High';
          end; // case
        end;

      RowCount := NumberOfPatientsFound + 1;

      if RowCount = 1 then
        begin
          RowCount := 2;
          Cells[0, 1] := 'None';
          ClearMatchGrid;
        end
      else
        begin
          if not gvUseScreenReader then
            begin
              DisplayPatientProperties(Cells[1, Row]);
            end;
        end;
    end;
  SetButtons;
end;

procedure TfrmMHA_UploadResults.DisplayMatchConfidenceLevel;
begin

  UploadResults_OPS.LoadMatchConfidenceLevel(lvCPRSPatient,
    TPatient(UploadResults_OPS.ListOfOffLinePatients[lvCurrentPt]));

  case UploadResults_OPS.MatchConfidenceLevel of
    mlLow:
      begin
        lblLevel.Font.Color := clRed;
        lblConfidence.Font.Color := clRed;
        lblLevel.Caption := 'Low';
      end;
    mlGood:
      begin
        lblLevel.Font.Color := clWindowText;
        lblConfidence.Font.Color := clWindowText;
        lblLevel.Caption := 'Good';
      end;
    mlHigh:
      begin
        lblLevel.Font.Color := clGreen;
        lblConfidence.Font.Color := clGreen;
        lblLevel.Caption := 'High';
      end;
  end;

end;

procedure TfrmMHA_UploadResults.DisplayCPRSPatientProperties;
begin
  // This procedure is invoked only if the CPRS patient selection
  // form is used while frmMHA3_UploadResults is active

  DisplayMatchProperties(Patient);
end;

procedure TfrmMHA_UploadResults.UpdateCPRSPatientProperties;
begin
  // This procedure is called by FormCreate and by DisplaylvCPRSPatientProperties
  // The purpose is to keep track of the patient selected in CPRS so the
  // user has a chance to reload that patient when frmMHA_UploadResults is
  // closed.
  UploadResults_OPS.CPRSPatientDFN := Patient.DFN;
  UploadResults_OPS.CPRSPatientName := Patient.Name;
end;

procedure TfrmMHA_UploadResults.DisplayPatientProperties(AName: string);
begin

  UploadResults_OPS.LoadSearchPatient(lvCPRSPatient, AName);
  DisplayMatchProperties(lvCPRSPatient);
end;

procedure TfrmMHA_UploadResults.DisplayMatchProperties(APatient: TPatient);
var
  i                           : integer;
  TempName,
    TempStr                   : string;
begin

  if (APatient = lvCPRSPatient) then
    i := 2
  else
    i := 1;

  if APatient.Name = 'Name' then
    TempName := ''
  else
    TempName := APatient.Name;

  with grdMatch do
    begin
      Cells[i, 1] := TempName;
      Cells[i, 2] := APatient.FormattedSSN;
      Cells[i, 3] := APatient.DOB;

      if Length(APatient.Sex) = 1 then
        TempStr := SexChange(APatient.Sex)
      else
        TempStr := APatient.Sex;

      Cells[i, 4] := TempStr;
    end;

  SettingsForNewMatch;
end;


procedure TfrmMHA_UploadResults.DisplayOfflinePatientProperties(AName: string);
var
  i                           : Integer;
begin

  // The grid row is not the same as lvCurrentPt.
  if UploadResults_OPS.ListOfOffLinePatients.Count = 0 then
    lvCurrentPt := -1
  else
    begin
      for i := 0 to UploadResults_OPS.ListOfOffLinePatients.Count - 1 do
        if AName =
          TPatient(UploadResults_OPS.ListOfOffLinePatients[i]).Name then
          begin
            lvCurrentPt := i;
            break;
          end;
    end;

  if not gvUseScreenReader then
    begin
      DisplayMatchProperties(TPatient(UploadResults_OPS.ListOfOffLinePatients[lvCurrentPt]));
      edSearch.Text := copy(TPatient(UploadResults_OPS.ListOfOffLinePatients[lvCurrentPt]).Name, 1, 1) +
        copy(TPatient(UploadResults_OPS.ListOfOffLinePatients[lvCurrentPt]).SSN, 6, 4);
      SearchForVistaPatients;
    end
  else
    edSearch.Text := TPatient(UploadResults_OPS.ListOfOffLinePatients[lvCurrentPt]).SSN;

end;

function TfrmMHA_UploadResults.SexChange(ASex: string): string;
begin
  if (ASex = 'F') then
    Result := 'FEMALE'
  else
    Result := 'MALE';
end;

procedure TfrmMHA_UploadResults.LoadTestNameAndDateByRow(ARow: integer);
var
  i                           : Integer;
begin
  for i := 0 to UploadResults_OPS.ListOfOffLineFiles.Count do
    begin
      if (grdOffLine.Cells[4, ARow] =
        TOffLineFile(UploadResults_OPS.ListOfOffLineFiles[i]).InstrumentName) and
        (grdOffLine.Cells[5, ARow] =
        TOffLineFile(UploadResults_OPS.ListOfOffLineFiles[i]).DateGiven) then
        begin
          lvCurrentFile := i;
          break;
        end;
    end;
end;

procedure TfrmMHA_UploadResults.grdMatchGetCellColor(Sender: TObject;
  ARow, ACol: Integer; AState: TGridDrawState; ABrush: TBrush;
  AFont: TFont);
begin

  //Change text color based on match
  if (ARow > 0) and (ACol = 2) then
    begin
      if (Trim(grdMatch.Cells[1, ARow]) = Trim(grdMatch.Cells[2, ARow])) then
        AFont.Color := clWindowText
      else
        AFont.Color := clRed;
    end;

  if (ARow > 0) and (ACol = 1) then
    ABrush.Color := clSkyBlue;

  if (ARow > 0) and (ACol = 2) then
    ABrush.Color := clInfoBk;
end;

procedure TfrmMHA_UploadResults.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  if not (Patient.DFN = UploadResults_OPS.CPRSPatientDFN) then
    if MsgDlgFrm('The patient you selected in CPRS was '
      + UploadResults_OPS.CPRSPatientName + ', but the patient who is '
      + 'now selected is ' + Patient.Name + '.' + CRLF + CRLF
      + 'Would you like work with ' + UploadResults_OPS.CPRSPatientName
      + ' now?', MHA_Caption, mtInformation, [mbYes, mbNo], 0) = mrYes then
      Patient.Load(UploadResults_OPS.CPRSPatientDFN,
        DataModule1.RPCBroker1);

end;

procedure TfrmMHA_UploadResults.btnGoClick(Sender: TObject);
begin
  SearchForVistaPatients;

  if gvUseScreenReader then
    ActiveControl := grpPossible;
end;

procedure TfrmMHA_UploadResults.edSearchChange(Sender: TObject);
begin
  btnGo.Enabled := Length(edSearch.Text) > 0;
  SetButtons;
end;

procedure TfrmMHA_UploadResults.SettingsForNewMatch;
begin
  btnUpload.Enabled := (UploadResults_OPS.NumberOfPatientsFound > 0);
  DisplayMatchConfidenceLevel;
end;

procedure TfrmMHA_UploadResults.btnUploadClick(Sender: TObject);
var
  Discontinue                 : boolean;
  ReplyStr                    : string;

begin

  if (UploadResults_OPS.NumberOfPatientsFound = 0) then
    begin
      MsgDlgFrm('No matching CPRS patient was found, and you cannot upload '
        + 'a "non-match."', MHA_Caption, mtInformation, [mbOK], 0);
      exit;
    end; //no patients

  if (UploadResults_OPS.MatchConfidenceLevel = mlLow) then
    if MsgDlgFrm('The match confidence level is low.  '
      + 'Are you sure the records are for the same person?', MHA_Caption,
      mtConfirmation, [mbYes, mbNo], 0) = mrNo then
      exit; //no confidence

  Administration_OPS.ClearAdmin(lvUploadAdministration);

  // OffLinePatient.DFN is the temporary, arbitrary number used for off-line identification.
  // It is not the PatientID used in Vista.
  UploadResults_OPS.LoadOffLineAdministration(
    TPatient(UploadResults_OPS.ListOfOffLinePatients[lvCurrentPt]).DFN,
    TOffLineFile(UploadResults_OPS.ListOfOffLineFiles[lvCurrentFile]).DateGiven,
    TOffLineFile(UploadResults_OPS.ListOfOffLineFiles[lvCurrentFile]).InstrumentName,
    lvCPRSPatient.DFN,
    lvUploadAdministration);

  UploadResults_OPS.LoadOffLineAnswers(
    TPatient(UploadResults_OPS.ListOfOffLinePatients[lvCurrentPt]).DFN,
    lvUploadAdministration);

  gvLeft := Left;
  gvTop := Top;
  gvHeight := Height;
  gvWidth := Width;

  // Get Administered By, Ordered By and Location
  frmMHA_UploadAdmin := TfrmMHA_UploadAdmin.Create(nil);
  try
    frmMHA_UploadAdmin.lblPatient.Caption :=
      lvCPRSPatient.Name + '   ' + #13 + 'SSN: ' + lvCPRSPatient.FormattedSSN;

    frmMHA_UploadAdmin.lblHeader.Caption := '   ' +
      TOffLineFile(UploadResults_OPS.ListOfOffLineFiles[lvCurrentFile]).InstrumentName;

    frmMHA_UploadAdmin.PatientDFN := lvCPRSPatient.DFN;

    if gvUseScreenReader then
      frmMHA_UploadAdmin.Caption :=
        'Upload ' + TOffLineFile(UploadResults_OPS.ListOfOffLineFiles[lvCurrentFile]).InstrumentName +
        ' for ' + lvCPRSPatient.Name
    else
      frmMHA_UploadAdmin.Caption := MHA_Caption;

    frmMHA_UploadAdmin.ShowModal;

    Discontinue := frmMHA_UploadAdmin.Discontinue;

    // This will finish the populating of the administation object
    lvUploadAdministration.LocationIEN := frmMHA_UploadAdmin.LocationIEN;
    lvUploadAdministration.LocationStr := frmMHA_UploadAdmin.Location;
    lvUploadAdministration.OrdererDUZ := frmMHA_UploadAdmin.OrdererDUZ;
    lvUploadAdministration.AdministeredBy := frmMHA_UploadAdmin.AdminBy;
    lvUploadAdministration.ConsultIEN := frmMHA_UploadAdmin.ConsultIEN;

  finally
    frmMHA_UploadAdmin.Free;
  end;

  if Discontinue then
    exit;

  if UploadResults_OPS.SaveOffLineInVista(
    TPatient(UploadResults_OPS.ListOfOffLinePatients[lvCurrentPt]).DFN,
    // OffLinePatient.DFN is used to identify the admin data file to be erased.
    TOffLineFile(UploadResults_OPS.ListOfOffLineFiles[lvCurrentFile]).DateGiven,
    lvUploadAdministration) then

    // Reset the list of off-line admins
    LoadOffLineAdministrations
  else
    begin
      MsgDlgFrm('Unable to save answers', MHA_Caption, mtError, [mbOK], 0);
      exit;
    end;

  Administration_OPS.NotifyOrderer(lvUploadAdministration.OrdererDUZ,
    lvCPRSPatient.DFN, lvUploadAdministration.InstrumentName);

  lvInstrument := TInstrument.Create;
  lvInstrument.InstrumentName := lvUploadAdministration.InstrumentName;

  // Look for the instrument on the local directory
  if not ResourceFile_OPS.ReadResourceFile(lvInstrument) then
    begin
      // Get instrument from Vista: slower
      Instrument_OPS.Load(lvInstrument);
      // Make a copy on the local drive for next time
      ResourceFile_OPS.WriteResourceFile(lvInstrument);
    end;


  // All questions answered, even though some may have been skipped.
  // ---------------------------------------------------------------

  if (lvUploadAdministration.NumberAnswered =
    lvInstrument.NumberOfQuestions) then
    begin
      lvUploadAdministration.IsComplete := True;

      ReplyStr := Administration_OPS.SaveScore(lvUploadAdministration);

      if gvUseScreenReader then
        MsgDlgFrm('Administration saved.', 'Administration saved.',
          mtInformation, [mbOk], 0);

      HandleProgressNote;
    end;

end;

procedure TfrmMHA_UploadResults.HandleProgressNote;
begin

  // No progress or consult notes for restricted instruments
  if lvInstrument.R_Privilege <> '' then
    begin
      if (lvUploadAdministration.ConsultIEN <> '') then
        MsgDlgFrm(lvInstrument.InstrumentName + ' data saved. ' + #13 + #13 +
          '(A consult note was not ' +
          'written to VistA as this is a restricted instrument.)',
          lvInstrument.InstrumentName, mtInformation, [mbOk], 0)
      else
        MsgDlgFrm(lvInstrument.InstrumentName + ' data saved. ' + #13 + #13 +
          '(A progress note was not ' +
          'written to VistA as this is a restricted instrument.)',
          lvInstrument.InstrumentName, mtInformation, [mbOk], 0);
      lvInstrument.Free;
      exit;
    end;

  lvUploadAdministration.IsaLegacy := lvInstrument.IsLegacy;

  // If User has identified a consult note, then do not produce a progress note.
  if (lvUploadAdministration.ConsultIEN <> '') then
    begin

      if (POS('PATIENT DOES NOT HAVE', lvUploadAdministration.ConsultIEN) > 0) then
        begin
          lvInstrument.Free;
          exit;
        end;

       // User can save a standard consult note, edit one, or have no consult note.
       // This form takes care of co-signer issues.
      if lvInstrument.GenerateProgressNote then
        begin
          gvAdmin := TAdministration.create;
          Administration_OPS.CopyAdmin(lvUploadAdministration,
            gvAdmin);
          try
            frmMHA_ProgressNote := TfrmMHA_ProgressNote.Create(nil);
          finally
            frmMHA_ProgressNote.Free;
          end; //try
          gvAdmin.Free;
        end;
      lvInstrument.Free;
      exit;
    end;

  //----------------  Progress note --------------------------------------

  // User can save a standard progress note, edit one, or have no progress note.
  // This form takes care of co-signer issues.
  if lvInstrument.GenerateProgressNote then
    begin
      gvAdmin := TAdministration.create;
      Administration_OPS.CopyAdmin(lvUploadAdministration,
        gvAdmin);
      try
        frmMHA_ProgressNote := TfrmMHA_ProgressNote.Create(nil);
      finally
        frmMHA_ProgressNote.Free;
      end; //try
      gvAdmin.Free;
      lvInstrument.Free;
    end;
end;


(*function TfrmMHA_UploadResults.MessageText(SubText: string): string;
var
  tmpMsg                      : string;
begin
  tmpMsg := 'The ' +
    TOffLineFile(UploadResults_OPS.ListOfOffLineFiles[lvCurrentFile]).InstrumentName
    + ' administered on ' +
    TOffLineFile(UploadResults_OPS.ListOfOffLineFiles[lvCurrentFile]).DateGiven
    + ' to ' + TPatient(UploadResults_OPS.ListOfOffLinePatients[lvCurrentPt]).Name + ' is selected '
    + SubText + CRLF + CRLF;

  if UploadAdministration.IsComplete then
    tmpMsg := tmpMsg + 'All questions have been answered.'
  else
    begin
      if UploadAdministration.IsRestartable then
        tmpMsg := tmpMsg +
          'It is possible to restart this administration.  A total of '
          + IntToStr(UploadAdministration.NumberAnswered) +
          ' questions have '
          + 'been answered.'
      else
        tmpMsg := tmpMsg +
          'It is not possible to complete this administration.';
    end;

  Result := tmpMsg;
end;*)

procedure TfrmMHA_UploadResults.btnDeleteClick(Sender: TObject);
begin

  //  Note that local files are deleted in two cases:

  //  1) If the local administration file does not exist, then
  //  the local answer file will be deleted.
  //  2) If the local administration file does exist and the user wants to continue,
  //  then both the local administration and local answer file will be deleted.

  //  It's hard to imagine the first case ever being true, though, since
  //  the administration wouldn't appear on the list if the file didn't exist.
  //  It is theoretically possible, but highly unlikely, that something happened
  //  to the administration file between loading the form and clicking the
  //  delete button.

  with grdOffline do
    begin
      if row < 1 then
        Exit;

      if MsgDlgFrm('Are you sure you want to delete the off-line record for ' + #13 +
        Cells[0, row] + #13 +
        Cells[4, row] + #13 +
        Cells[5, row] + '?' + #13 + #13 +
        '(This can not be reversed later.)', MHA_Caption, mtConfirmation,
        [mbYes, mbNo], 0) = mrNo then
        Exit;

    end; // with grdOffline

  SysUtils.DeleteFile(LocalFiles.BuildAdministrationFileName(
    TOffLineFile(UploadResults_OPS.ListOfOffLineFiles[lvCurrentFile]).OffLinePtID,
    DateTimeToFileNameDateTime(StrToDateTime(TOffLineFile(UploadResults_OPS.ListOfOffLineFiles[lvCurrentFile]).DateGiven)),
    TOffLineFile(UploadResults_OPS.ListOfOffLineFiles[lvCurrentFile]).InstrumentName));

  SysUtils.DeleteFile(LocalFiles.AnswerFilesPath + '\' +
    TOffLineFile(UploadResults_OPS.ListOfOffLineFiles[lvCurrentFile]).OffLinePtID +
    FILE_NAME_DELIMITER +
    TOffLineFile(UploadResults_OPS.ListOfOffLineFiles[lvCurrentFile]).OffLineAdminID +
    '.' + ANSWER_FILE_EXT);

  // Clear the decks
  edSearch.Text := '';
  ClearMatchGrid;
  grdVista.Clear;

  // Clear the Off-Line string-grid of this administration.
  LoadOffLineAdministrations;

  if grdOffLine.Row > 0 then
    begin
      DisplayOfflinePatientProperties(grdOffLine.Cells[0, grdOffLine.Row]);
      LoadTestNameAndDateByRow(grdOffLine.Row);
    end;
end;

procedure TfrmMHA_UploadResults.mnuExitClick(Sender: TObject);
begin
  Close;
end;

procedure TfrmMHA_UploadResults.FormKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
var
  WhereMouse                  : TPoint;
begin
  if (Shift = [ssALT]) and (Key = vk_F4) then
    Close;

  if Key = vk_Escape then
    begin
      Close;
      Key := 0;
    end;

  if Shift = [ssAlt] then
    begin
      if Key in [83, 115] then // Search
        begin
          ActiveControl := edSearch;
          WhereMouse.x := frmMHA_UploadResults.Left + pnlSearch.Left +
            edSearch.left +
            edSearch.width;
          WhereMouse.y := pnlSearch.Top + 20 +
            edSearch.top;
          Mouse.CursorPos := WhereMouse;
          Key := 0;
        end;
    end;
end;

procedure TfrmMHA_UploadResults.mnuOnlineSupportClick(Sender: TObject);
begin
  VisitURL(MHA_ONLINE_SUPPORT_URL);
end;

procedure TfrmMHA_UploadResults.mnuAboutClick(Sender: TObject);
begin
  frmMHA_About := TfrmMHA_About.Create(nil);
  try
    frmMHA_About.ShowModal;
  finally
    frmMHA_About.Free;
  end; //try
end;

procedure TfrmMHA_UploadResults.grdVistaClick(Sender: TObject);
begin
  if lvInitialize then
    Exit;
  if grdVista.Row > 0 then
    if not gvUseScreenReader then
      DisplayPatientProperties(grdVista.Cells[1, grdVista.Row]);
  SetButtons;
end;

procedure TfrmMHA_UploadResults.edSearchKeyPress(Sender: TObject;
  var Key: Char);
begin
  if (Key = #13) and (Length(edSearch.Text) < 2) then
    MessageDlg('At least three characters are needed for a search.', mtInformation,
      [mbOK], 0);

  if (Key = #13) and (Length(edSearch.Text) > 2) then
    begin
      SearchForVistaPatients;
      if gvUseScreenReader then
        ActiveControl := grpPossible;
    end;
end;

procedure TfrmMHA_UploadResults.grdOffLineKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if key = vk_Delete then
    btnDeleteClick(Sender);
end;

procedure TfrmMHA_UploadResults.grdVistaKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if gvUseScreenReader then
    if Key = vk_Return then
      btnUploadClick(Sender);
end;

procedure TfrmMHA_UploadResults.imgBackupClick(Sender: TObject);
begin
  Close;
end;

procedure TfrmMHA_UploadResults.grdOffLineClick(Sender: TObject);
begin
  if lvInitialize then
    Exit;

  if (grdOffLine.Row > 0) then
    begin
      DisplayOfflinePatientProperties(grdOffLine.Cells[0, grdOffLine.Row]);
      LoadTestNameAndDateByRow(grdOffLine.Row);
    end;

  SetButtons;
end;

procedure TfrmMHA_UploadResults.FormResize(Sender: TObject);
begin
  if lvInitialize then
    Exit;

  // Reset these, used in placement of dialogs
  gvLeft := Left;
  gvTop := Top;
  gvHeight := Height;
  gvWidth := Width;
end;




end.

