unit uMHA_ASI_Build_508;

interface

uses
  Classes,
  ComCtrls,
  Controls,
  fIA_Dialogs,
  ExtCtrls,
  Forms,
  Graphics,
  Mask,
  Messages,
  StdCtrls,
  Windows,
  StExpr,
  ORCtrls,
  Fmcmpnts,
  Fmcntrls,
  SysUtils,
  uMHA_SpnEdt,
  Buttons;

type
  {**
  Construct GUI for ASI that conforms to 508 specs.
  Form is different for Full, Lite and Followup
  versions.
  }
  TASI_Build_508 = class(TObject)
  private
    procedure LoadCommentFields(WhichIEN: string);
  protected
  public
    constructor Create;
    procedure BuildBasics;
    procedure InsertItems;
    procedure BuildForm;
    procedure SetupComboBoxItems;
    procedure DefaultValuesForItems;
    procedure Setup_ASIFull_508;
    procedure Setup_ASILite_508;
    procedure Setup_ASIFollowup_508;
    //procedure PositionASIItems(WhichForm: Char);
    procedure MakePartialList;

    destructor Destroy; override;
  published
  end; { TASI_Build }

var
  ASI_Build_508               : TASI_Build_508;

implementation

uses
  RzCmboBx,
  RzEdit,
  uIA_ThisUser,
  fMHA_ASI_Entry_508,
  uMHA_ASI,
  uMHA_ASI_OPS,
  uIA_Utils,
  uIA_Const,
  Trpcb,
  uMHA_Const,
  uMHA_Answer_IO,
  uIA_GUIControl,
  uMHA_DM;

var
  lvPartialList               : TStringList;


constructor TASI_Build_508.Create;
begin
  inherited Create;
end;

destructor TASI_Build_508.Destroy;
begin
  inherited Destroy;
end;

procedure TASI_Build_508.BuildBasics;
begin


  if TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType = itFull then
    gvASI_Type := 0;
  if TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType = itLite then
    gvASI_Type := 1;
  if TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType = itFU then
    gvASI_Type := 2;
  if TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType = itMV then
    gvASI_Type := 3;

  // Run-time construciton of most combo box strings
  SetupComboBoxItems;

  with frmMHA_ASI_Entry_508 do
    begin
      //dtpG5.DateTime := Date;

      // Setup Default values
      //dtpG4.MaxDate := Now;
      //dtpG5.MaxDate := Now;
      dtpG4.text := DateTimeToStr(Date);
      dtpG5.text := DateTimeToStr(Date);

      DefaultValuesForItems;

      (*
      with cbxG3.Items do
        begin
          Clear;
          //Type 7: G3
          Add('1=Consultation/Liaison');
          Add('2=Early Intervention/Education/Referral');
          Add('3=Detoxification Services');
          Add('4=Ambulatory except Opioid Substitution');
          Add('5=Opioid Substitution');
          Add('6=Inpatient Acute Care');
          Add('7=Substance Abuse Residential Rehabilitation Treatment Program');
          Add('8=Compensated Work Therapy/Transitional Residence');
          Add('9=Therapeutic Community');
          Add('10=Domiciliary');
          Add('11=VA Halfway House (Not Contracted)');
          Add('12=Combination: Inpatient Acute, Residential, Ambulatory Care');
          Add('13=Combination: Inpatient Acute Care, Ambulatory');
          Add('14=Combination: Inpatient Acute Care, Residential');
          Add('15=Combination: Residential, Ambulatory');
          Add('16=Primary Care (Non-Substance Abuse)');
          Add('17=General Medicine');
          Add('18=General Psychiatry');
          Add('19=General Surgery');
          Add('20=Domiciliary (Non-Substance Abuse)');
          Add('21=SCI Program');
        end; *)

//      if ((gvLastASIProgram < 0) or (gvLastASIProgram > 100)) then
//        gvLastASIProgram := 0;
//      cbxG3.ItemIndex := gvLastASIProgram;
      //cbxG3.Text := Piece(cbxG3.Items[gvLastASIProgram], U, 2);

      // Hide the verify window
      pnlVerify.Height := 0;

      // setttings from Windows registry
      (*
      SpltGeneral.percent := gvASIGeneral;
      SpltMedical.percent := gvASIMedical;
      SpltEmploy.percent := gvASIEmploy;
      SpltDrug.percent := gvASIDrug;
      SpltLegal.percent := gvASILegal;
      SpltFamily.percent := gvASIFamily;
      SpltSocial.percent := gvASISocial;
      SpltPsych.percent := gvASIPsych;*)

      // default settings
      sbGeneral1.VertScrollBar.Position := 0;
      sbMedical1.VertScrollBar.Position := 0;
      sbEmploy1.VertScrollBar.Position := 0;
      sbLegal1.VertScrollBar.Position := 0;
      sbDrug1.VertScrollBar.Position := 0;
      sbFamily1.VertScrollBar.Position := 0;
      sbSocial1.VertScrollBar.Position := 0;
      sbPsych1.VertScrollBar.Position := 0;
      //pgctrlASI.ActivePage := TabGeneral;

    end;
end;

procedure TASI_Build_508.MakePartialList;
// The broker calls are limited to 80 fields at a time.
// This list uses three calls and contains the fields for
// Full, Lite or Follow-up ASI.
// The IEN is found in frmMain.grdOldASI.Cells[5, frmMain.grdOldASI.Row]
var
  n                           : integer;
  S                           : string;

  procedure ReadAnswer(StringIn: string);
  var
    i                         : integer;
  begin
    with frmMHA_ASI_Entry_508 do
      begin
        // the end of the list as been reached.
        if StringIn = 'Done' then
          begin
            DataModule1.ASI_FMGets1.FieldNumbers.Clear;
            DataModule1.ASI_FMGets1.AddField(S);
            DataModule1.ASI_FMGets1.GetData;
            for i := 0 to DataModule1.ASI_FMGets1.Results.Count - 1 do
              lvPartialList.Add(DataModule1.ASI_FMGets1.results[i] + ';' +
                DataModule1.ASI_FMGets1.GetField(DataModule1.ASI_FMGets1.Results[i]).FMDBInternal);
            exit;
          end;
        // add the new item to the growing string
        S := S + StringIn;
        // Keep track of how many items, no more than 80
        Inc(n);
        if n > 80 then
          begin
            DataModule1.ASI_FMGets1.FieldNumbers.Clear;
            DataModule1.ASI_FMGets1.AddField(S);
            DataModule1.ASI_FMGets1.GetData;
            for i := 0 to DataModule1.ASI_FMGets1.Results.Count - 1 do
              lvPartialList.Add(DataModule1.ASI_FMGets1.results[i] + ';' +
                DataModule1.ASI_FMGets1.GetField(DataModule1.ASI_FMGets1.Results[i]).FMDBInternal);
            S := '';
            n := 0;
          end;
      end;
  end;

begin
  with frmMHA_ASI_Entry_508 do
    begin
      lvPartialList := TStringList.Create;
      lvPartialList.Clear;

      DataModule1.RPCBroker1.CreateContext('YS BROKER1');
      DataModule1.ASI_FMGets1.FileNumber := '604';
      DataModule1.ASI_FMGets1.IENS := TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).IENS;
      S := '';
      n := 0;

          // General  17 items -------------------------------------------
          //'NAME
      ReadAnswer('.02;');
          //INTRCODE  This is Interviewer DUZ
      ReadAnswer('.81;');
          //ORDEREDBY
      ReadAnswer('.09;');
          //PROGRAM
      ReadAnswer('2;');
          //ADMITED
      ReadAnswer('1;');
          //INTDATE
      ReadAnswer('.05;');
          //CLASS
      ReadAnswer('.04;');
          //CONTACT
      ReadAnswer('.07;');
          //SPECIAL
      ReadAnswer('.11;');
          //  ReadComboBoxYEARS
      ReadAnswer('1.201;');
          //  ReadComboBoxMONTS
      ReadAnswer('1.202;');
          //  OWNHOME
      ReadAnswer('1.203;');
          //  RACE
      ReadAnswer('.16;');
          //RELIGION
      ReadAnswer('.17;');
          //CONENV
      ReadAnswer('2.01;');
          //OTHER
      ReadAnswer('2.02;');
          //CONENTIM
      ReadAnswer('2.03;');


          // Medical 15 items ----------------------------------------------
          //HOSPLIFE
      ReadAnswer('8.01;');
          //DC    if frmMain.Partial_ASI = 'A' then // Full
      if (TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType = itFull) then
        begin
              //HOSPYEAR
          ReadAnswer('8.02;');
              //HOSPMONT
          ReadAnswer('8.025;');
        end;
          //    if ((frmMain.Partial_ASI = 'A') or (frmMain.Partial_ASI = 'L')) then
      if ((TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType = itFull) or
        (TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType = itLite)) then
        begin
              //CHRONMED
          ReadAnswer('8.03;');
              //CHRONMEDSPEC
          ReadAnswer('8.04;');
        end;
          //MEDICATS
      ReadAnswer('8.05;');
          //PHYSDIS
      ReadAnswer('8.06;');
          //PHYSDISSPEC
      ReadAnswer('8.07;');
          //DAYMEDP
      ReadAnswer('8.08;');
          //TROUBLED
      ReadAnswer('8.09;');
          //IMPTMT
      ReadAnswer('8.11;');
          //DC   if ((frmMain.Partial_ASI = 'A') or (frmMain.Partial_ASI = 'F')) then
      if ((TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType = itFull) or
        (TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType = itFU)) then
            //ISRMED
        ReadAnswer('8.12;');
          //MPATMISR
      ReadAnswer('8.14;');
          //MEDUNDER
      ReadAnswer('8.15;');

          // Employment 27 Items ----------------------------------------------
          //EDUCYEAR
      ReadAnswer('9.01;');
          //EDUCMONT
      ReadAnswer('9.02;');
          //TRAINING
      ReadAnswer('9.03;');
          //DC    if frmMain.Partial_ASI = 'A' then
      if (TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType = itFull) then
        begin
              //PROFN
          ReadAnswer('9.04;');
              //SPECPTS
          ReadAnswer('9.05;');
        end;
          //LICENSE
      ReadAnswer('9.06;');
          //CAR
      ReadAnswer('9.09;');
          //DC    if ((frmMain.Partial_ASI = 'A') or (frmMain.Partial_ASI = 'L')) then
      if ((TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType = itFull) or (TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType = itLite)) then
        begin
              //JOBYEAR
          ReadAnswer('9.11;');
              //JOBMONTH
          ReadAnswer('9.12;');
        end;
          //OCCUP
      ReadAnswer('9.14;');
          //SPECOCCUP
      ReadAnswer('9.145;');
          //DC    if ((frmMain.Partial_ASI = 'A') or (frmMain.Partial_ASI = 'F')) then
      if ((TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType = itFull) or (TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType = itFU)) then
        begin
              //SUPPORT
          ReadAnswer('9.15;');
        end;
          //MAJORITY
      ReadAnswer('9.16;');

          //DC    if ((frmMain.Partial_ASI = 'A') or (frmMain.Partial_ASI = 'L')) then
      if ((TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType = itFull) or (TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType = itLite)) then
            //PATTERNS
        ReadAnswer('9.17;');
          //DAYSPAID
      ReadAnswer('9.18;');
          //EMPL$
      ReadAnswer('9.19;');
          //UNEMPLOY
      ReadAnswer('9.21;');
          //DPA
      ReadAnswer('9.22;');
          //PENSION
      ReadAnswer('9.23;');
          //FAMILY
      ReadAnswer('9.24;');
          //ILLEGAL
      ReadAnswer('9.25;');
          //DEPENDS
      ReadAnswer('9.29;');
          //DC    if ((frmMain.Partial_ASI = 'A') or (frmMain.Partial_ASI = 'F')) then
      if ((TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType = itFull) or (TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType = itFU)) then
        begin
              //EMPROB
          ReadAnswer('9.31;');
              //EMPTROUB
          ReadAnswer('9.32;');
        end;
          //EMPCOUNS
      ReadAnswer('9.33;');

          //DC    if ((frmMain.Partial_ASI = 'A') or (frmMain.Partial_ASI = 'F')) then
      if ((TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType = itFull) or (TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType = itFU)) then
            //EMPLOYRA
        ReadAnswer('9.34;');
          //EPATMISR
      ReadAnswer('9.35;');
          //EMPUNDER
      ReadAnswer('9.36;');

          // Drug and Alcohol 65 items ----------------------------------------
          //ALC30
      ReadAnswer('10.01;');
          //ALCLIYR
      ReadAnswer('10.02;');
          //ALCRT
      ReadAnswer('10.03;');
          //ALCIN30
      ReadAnswer('10.04;');
          //ALCINLIY
      ReadAnswer('10.05;');
          //INTOXRT
      ReadAnswer('10.06;');
          //HEROIN30
      ReadAnswer('10.07;');
          //HERONLIY
      ReadAnswer('10.08;');
          //HERNRT
      ReadAnswer('10.09;');
          //METHDN30
      ReadAnswer('10.11;');
          //METDNLIY
      ReadAnswer('10.12;');
          //METHRT
      ReadAnswer('10.14;');
          //OPIAT30
      ReadAnswer('10.15;');
          //OPIATLIY
      ReadAnswer('10.16;');
          //OPIRT
      ReadAnswer('10.17;');
          //BARB30
      ReadAnswer('10.18;');
          //BARBLIFY
      ReadAnswer('10.19;');
          //BARBRT
      ReadAnswer('10.21;');
          //SEDS30
      ReadAnswer('10.22;');
          //SEDLIFYR
      ReadAnswer('10.23;');
          //SEDSRT
      ReadAnswer('10.24;');
          //COCAN30
      ReadAnswer('10.25;');
          //COCLIFYR
      ReadAnswer('10.26;');
          //COCAINRT
      ReadAnswer('10.27;');
          //AMPH30
      ReadAnswer('10.28;');
          //AMPLIFYR
      ReadAnswer('10.29;');
          //AMPRT
      ReadAnswer('10.31;');
          //MJ30
      ReadAnswer('10.32;');
          //MJLIFEYR
      ReadAnswer('10.33;');
          //MJRT
      ReadAnswer('10.34;');
          //HALLU30
      ReadAnswer('10.35;');
          //HALLIFYR
      ReadAnswer('10.36;');
          //HALRT
      ReadAnswer('10.37;');
          //INHA30
      ReadAnswer('10.38;');
          //INHALIYR
      ReadAnswer('10.39;');
          //INHALRT
      ReadAnswer('10.41;');
          //GT130
      ReadAnswer('10.42;');
          //GT1LIYR
      ReadAnswer('10.43;');
          //DC    if ((frmMain.Partial_ASI = 'A') or (frmMain.Partial_ASI = 'F')) then
      if ((TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType = itFull) or (TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType = itFU)) then
            //MAJPROB
        ReadAnswer('10.45;');
          //DC      if rgrpG8.ItemIndex = 0 {Full} then
      if (TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType = itFull) then
        begin
              //LONGABST
          ReadAnswer('11.01;');
              //LASTABST
          ReadAnswer('11.02;');
        end;
          //    if ((frmMain.Partial_ASI = 'A') or (frmMain.Partial_ASI = 'F')) then
      if ((TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType = itFull) or (TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType = itFU)) then
        begin
              //DTS
          ReadAnswer('11.03;');
              //ODS
          ReadAnswer('11.04;');
        end;
          //ALCTXS
      ReadAnswer('11.05;');
          //DRUGTXS
      ReadAnswer('11.06;');
          //DC    if ((frmMain.Partial_ASI = 'A') or (frmMain.Partial_ASI = 'F')) then
      if ((TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType = itFull) or (TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType = itFU)) then
        begin
              //ALDETOX
          ReadAnswer('11.07;');
              //DRUDETOX
          ReadAnswer('11.08;');
        end;
          //ALCMONEY
      ReadAnswer('11.09;');
          //DRUMONEY
      ReadAnswer('11.11;');
          //OPDAYS
      ReadAnswer('11.12;');
          //ALPROBS
      ReadAnswer('11.14;');
          //DRUPROBS
      ReadAnswer('11.15;');
          //ALBOTHR
      ReadAnswer('11.16;');
          //DRUGBOTH
      ReadAnswer('11.17;');
          //ALTMT
      ReadAnswer('11.165;');
          //DRUTMT
      ReadAnswer('11.175;');
          //DC    if ((frmMain.Partial_ASI = 'A') or (frmMain.Partial_ASI = 'F')) then
      if ((TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType = itFull) or (TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType = itFU)) then
        begin
              //ISRALC
          ReadAnswer('11.18;');
              //ISRDRUG
          ReadAnswer('11.185;');
        end;
          //DAMISREP
      ReadAnswer('11.19;');
          //DAUNDER
      ReadAnswer('11.21;');

          // Legal  34 Items -----------------------------------------------------
        //DC    if ((frmMain.Partial_ASI = 'A') or (frmMain.Partial_ASI = 'L')) then
      if ((TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType = itFull) or (TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType = itLite)) then
            //CJINVOLV
        ReadAnswer('14.01;');
          //ONPROBAT
      ReadAnswer('14.02;');
          //SHOPLIFT
      ReadAnswer('14.03;');
          //PAROLVIL
      ReadAnswer('14.04;');
          //DRUGCHAR
      ReadAnswer('14.05;');
          //FORGE
      ReadAnswer('14.06;');
          //WEAPONS
      ReadAnswer('14.07;');
          //B_N_E
      ReadAnswer('14.08;');
          //ROBBERY
      ReadAnswer('14.09;');
          //ASSAULT
      ReadAnswer('14.11;');
          //ARSON
      ReadAnswer('14.12;');
          //RAPE
      ReadAnswer('14.14;');
          //MURDER
      ReadAnswer('14.15;');
          //PROSTITU
      ReadAnswer('14.16;');
          //CONTEMPT
      ReadAnswer('14.17;');
          //OTHER
      ReadAnswer('14.18;');
          //CONVICTS
      ReadAnswer('14.19;');
          //DISORDER
      ReadAnswer('14.21;');
          //DWI
      ReadAnswer('14.22;');
          //DRIVING
      ReadAnswer('14.23;');
          //INCARLI
      ReadAnswer('14.24;');
          //DC    if ((frmMain.Partial_ASI = 'A') or (frmMain.Partial_ASI = 'F')) then
      if ((TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType = itFull) or (TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType = itFU)) then
        begin
              //LASTINCA
          ReadAnswer('14.25;');
              //REASONIN
          ReadAnswer('14.26;');
        end;
          //WAITRIAL
      ReadAnswer('14.27;');
          //REASTRIA
      ReadAnswer('14.28;');
          //DETAIN30
      ReadAnswer('14.29;');
          //ILLDAY30
      ReadAnswer('14.31;');
          //PRALEGAL
      ReadAnswer('14.32;');
          //PRACOUNS
      ReadAnswer('14.33;');
          //DC    if ((frmMain.Partial_ASI = 'A') or (frmMain.Partial_ASI = 'F')) then
      if ((TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType = itFull) or (TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType = itFU)) then
            //ISRLEGAL
        ReadAnswer('14.34;');
          //LPMISREP
      ReadAnswer('14.35;');
          //LPTUNDER
      ReadAnswer('14.36;');

          // Family History 37 Items ------------------------------------------
        //DC    if frmMain.Partial_ASI = 'A' then
      if (TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType = itFull) then
        begin
              //FHMGM
          ReadAnswer('15.01;');
              //FHMGM
          ReadAnswer('15.02;');
              //FHMGM
          ReadAnswer('15.03;');
              //FHMGF
          ReadAnswer('15.04;');
              //FHMGF
          ReadAnswer('15.05;');
              //FHMGF
          ReadAnswer('15.06;');
              //FHMM
          ReadAnswer('15.07;');
              //FHMM
          ReadAnswer('15.08;');
              //FHMM
          ReadAnswer('15.09;');
              //FHMA
          ReadAnswer('15.11;');
              //FHMA
          ReadAnswer('15.12;');
              //FHMA
          ReadAnswer('15.14;');
              //FHMU
          ReadAnswer('15.15;');
              //FHMU
          ReadAnswer('15.16;');
              //FHMU
          ReadAnswer('15.17;');
              //FHFGM
          ReadAnswer('15.18;');
              //FHFGM
          ReadAnswer('15.19;');
              //FHFGM
          ReadAnswer('15.21;');
              //FHFGF
          ReadAnswer('15.22;');
              //FHFGF
          ReadAnswer('15.23;');
              //FHFGF
          ReadAnswer('15.24;');
              //FHFF
          ReadAnswer('15.25;');
              //FHFF
          ReadAnswer('15.26;');
              //FHFF
          ReadAnswer('15.27;');
              //FHFA
          ReadAnswer('16.01;');
              //FHFA
          ReadAnswer('16.02;');
              //FHFA
          ReadAnswer('16.03;');
              //FHFU
          ReadAnswer('16.04;');
              //FHFU
          ReadAnswer('16.05;');
              //FHFU
          ReadAnswer('16.06;');
              //FHIBB
          ReadAnswer('16.07;');
              //FHIBB
          ReadAnswer('16.08;');
              //FHIBB
          ReadAnswer('16.09;');
              //FHIBS
          ReadAnswer('16.11;');
              //FHIBS
          ReadAnswer('16.12;');
              //FHIBS
          ReadAnswer('16.14;');
        end; // Full ASI

          // Social 53 items -----------------------------------------------
          //MARSTAT
      ReadAnswer('17.01;');
          //DC    if frmMain.Partial_ASI = 'A' then
      if (TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType = itFull) then
        begin
              //MARYRS
          ReadAnswer('17.02;');
              //MARMOS
          ReadAnswer('17.03;');
        end;
          //MARSATF
      ReadAnswer('17.04;');
          //LIVARRGN
      ReadAnswer('17.05;');
          //DC    if frmMain.Partial_ASI = 'A' then
      if (TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType = itFull) then
        begin
              //LIVYRS
          ReadAnswer('17.06;');
              //LIVMONTH
          ReadAnswer('17.07;');
        end;
          //LIVSATF
      ReadAnswer('17.08;');
          //LIVWALC
      ReadAnswer('17.09;');
          //LIVWDRUG
      ReadAnswer('17.11;');
          //DC    if ((frmMain.Partial_ASI = 'A') or (frmMain.Partial_ASI = 'F')) then
      if ((TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType = itFull) or (TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType = itFU)) then
        begin
              //FREETIME
          ReadAnswer('17.12;');
              //FREESATF
          ReadAnswer('17.14;');
              //CLOSFREN
          ReadAnswer('17.15;');
        end;
          //DC    if frmMain.Partial_ASI = 'A' then
      if (TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType = itFull) then
        begin
              //CLOSMOTR
          ReadAnswer('17.16;');
              //CLOSFATR
          ReadAnswer('17.17;');
              //CLOSSIBS
          ReadAnswer('17.18;');
              //CLOSPRTN
          ReadAnswer('17.19;');
              //CLOSKIDS
          ReadAnswer('17.21;');
              //CLOSFREN
          ReadAnswer('17.22;');
        end;
          //MOM30
      ReadAnswer('18.01;');
          //MOMLIFE
      ReadAnswer('18.02;');
          //DAD30
      ReadAnswer('18.03;');
          //DADLIFE
      ReadAnswer('18.04;');
          //SIBS30
      ReadAnswer('18.05;');
          //SIBSLIFE
      ReadAnswer('18.06;');
          //SPOUS30
      ReadAnswer('18.07;');
          //SPOULIFE
      ReadAnswer('18.08;');
          //KIDS30
      ReadAnswer('18.09;');
          //KIDSLIFE
      ReadAnswer('18.11;');
          //OTHFAM30
      ReadAnswer('18.12;');
          //OTHFAMLI
      ReadAnswer('18.14;');
          //FREN30
      ReadAnswer('18.15;');
          //FRENLIFE
      ReadAnswer('18.16;');
          //NEIGH30
      ReadAnswer('18.17;');
          //NEIGHLIF
      ReadAnswer('18.18;');
          //COWOR30
      ReadAnswer('18.185;');
          //COWORLIF
      ReadAnswer('18.186;');
          //DC    if ((frmMain.Partial_ASI = 'A') or (frmMain.Partial_ASI = 'F')) then
      if ((TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType = itFull) or
        (TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType = itFU)) then
            //EMOABU30
        ReadAnswer('18.19;');
          //EMOABULI
      ReadAnswer('18.195;');
          //PHYABU30
      ReadAnswer('18.21;');
          //PHYABULI
      ReadAnswer('18.215;');
          //SEXABU30
      ReadAnswer('18.22;');
          //SEXABULI
      ReadAnswer('18.225;');
          //FAMCONFL
      ReadAnswer('18.23;');
          //OTHCONFL
      ReadAnswer('18.24;');
          //PRAFPROB
      ReadAnswer('18.25;');
          //DC    if ((frmMain.Partial_ASI = 'A') or (frmMain.Partial_ASI = 'F')) then
      if ((TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType = itFull) or
        (TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType = itFU)) then
            //PRASPROB
        ReadAnswer('18.26;');
          //FAMTMT
      ReadAnswer('18.27;');
          //DC    if ((frmMain.Partial_ASI = 'A') or (frmMain.Partial_ASI = 'F')) then
      if ((TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType = itFull) or
        (TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType = itFU)) then
        begin
              //SOCTMT
          ReadAnswer('18.28;');
              //ISRFS
          ReadAnswer('18.29;');
        end;
          //FSMISREP
      ReadAnswer('18.31;');
          //FSPUND
      ReadAnswer('18.32;');

          // Psych ----------------------------------------------------------
          //INPSYC
      ReadAnswer('19.01;');
          //OUTPSYC
      ReadAnswer('19.02;');
          //PSYCPEN
      ReadAnswer('19.03;');
          //DEPR30
      ReadAnswer('19.04;');
          //DEPRLIFE
      ReadAnswer('19.05;');
          //ANXT30
      ReadAnswer('19.06;');
          //ANXTLIFE
      ReadAnswer('19.07;');
          //HALL30
      ReadAnswer('19.08;');
          //HALLLIFE
      ReadAnswer('19.09;');
          //UNDER30
      ReadAnswer('19.11;');
          //UNDERLIF
      ReadAnswer('19.12;');
          //VIOL30
      ReadAnswer('19.14;');
          //VIOLLIFE
      ReadAnswer('19.15;');
          //SUITH30
      ReadAnswer('19.16;');
          //SUITHLIF
      ReadAnswer('19.17;');
          //SUIAT30
      ReadAnswer('19.18;');
          //SUIATLIF
      ReadAnswer('19.19;');
          //PMED30
      ReadAnswer('19.21;');
          //PMEDLIFE
      ReadAnswer('19.22;');
          //PSYCDAYS
      ReadAnswer('19.23;');
          //PRATPSY
      ReadAnswer('19.24;');
          //PSYTMT
      ReadAnswer('19.25;');
          //DC    if ((frmMain.Partial_ASI = 'A') or (frmMain.Partial_ASI = 'F')) then
      if ((TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType = itFull) or
        (TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType = itFU)) then
        begin
              //DEPNOW
          ReadAnswer('19.26;');
              //HOSTNOW
          ReadAnswer('19.27;');
              //ANXNOW
          ReadAnswer('19.28;');
              //REALNOW
          ReadAnswer('19.29;');
              //COMPRNOW
          ReadAnswer('19.31;');
              //SUICNOW
          ReadAnswer('19.32;');
              //ISRPSYC
          ReadAnswer('19.33;');
        end;
          //PMISREP
      ReadAnswer('19.34;');
          //PSPUND
      ReadAnswer('19.35;');

      ReadAnswer('Done');
          //    PartialList.SaveToFile(UserSession.TempFolderName + '00PL.txt');

    end;
end;

procedure TASI_Build_508.InsertItems;
// This puts the data from the dark-side into the ASI form. This occurs when the
// user wants to edit and/or sign an incomplete ASI.
var
  Response                    : string;
  S                           : string[10];
  i                           : integer;
label
  Skip;

  procedure RadioGroup(Item: TListBox; Start, X, N: integer; Answer: string);
  var
    j                         : integer;
  begin
    with frmMHA_ASI_Entry_508 do
      begin
        for j := 0 to ComponentCount - 1 do
          if Components[j] is TListBox then
            if TListBox(Components[j]) = Item then
              begin
                // Legal List code 604.3, unique for these items
                if ((UpperCase(TListBox(Components[j]).Name) = 'RGRPL23') or
                  (UpperCase(TListBox(Components[j]).Name) = 'RGRPL25')) then
                  begin
                    if Answer = '3' then
                      TListBox(Components[j]).ItemIndex := 0;
                    if Answer = '4' then
                      TListBox(Components[j]).ItemIndex := 1;
                    if Answer = '5' then
                      TListBox(Components[j]).ItemIndex := 2;
                    if Answer = '6' then
                      TListBox(Components[j]).ItemIndex := 3;
                    if Answer = '7' then
                      TListBox(Components[j]).ItemIndex := 4;
                    if Answer = '8' then
                      TListBox(Components[j]).ItemIndex := 5;
                    if Answer = '9' then
                      TListBox(Components[j]).ItemIndex := 6;
                    if Answer = '10' then
                      TListBox(Components[j]).ItemIndex := 7;
                    if Answer = '11' then
                      TListBox(Components[j]).ItemIndex := 8;
                    if Answer = '12' then
                      TListBox(Components[j]).ItemIndex := 9;
                    if Answer = '13' then
                      TListBox(Components[j]).ItemIndex := 10;
                    if Answer = '14.1' then
                      TListBox(Components[j]).ItemIndex := 11;
                    if Answer = '14.2' then
                      TListBox(Components[j]).ItemIndex := 12;
                    if Answer = '14.3' then
                      TListBox(Components[j]).ItemIndex := 13;
                    if Answer = '16' then
                      TListBox(Components[j]).ItemIndex := 14;
                    if Answer = '17' then
                      TListBox(Components[j]).ItemIndex := 15;
                    if Answer = '18' then
                      TListBox(Components[j]).ItemIndex := 16;
                    if Answer = '21' then
                      TListBox(Components[j]).ItemIndex := 17;
                    if Answer = '20' then
                      TListBox(Components[j]).ItemIndex := 18;
                    exit;
                  end;

                if UpperCase(TListBox(Components[j]).Name) = 'RGRPD14' then
                  begin
                    if Answer = '100' then
                      TListBox(Components[j]).ItemIndex := 0;
                    if Answer = '1' then
                      TListBox(Components[j]).ItemIndex := 1;
                    if Answer = '2' then
                      TListBox(Components[j]).ItemIndex := 2;
                    if Answer = '3' then
                      TListBox(Components[j]).ItemIndex := 3;
                    if Answer = '4' then
                      TListBox(Components[j]).ItemIndex := 4;
                    if Answer = '5' then
                      TListBox(Components[j]).ItemIndex := 5;
                    if Answer = '6' then
                      TListBox(Components[j]).ItemIndex := 6;
                    if Answer = '7' then
                      TListBox(Components[j]).ItemIndex := 7;
                    if Answer = '8' then
                      TListBox(Components[j]).ItemIndex := 8;
                    if Answer = '9' then
                      TListBox(Components[j]).ItemIndex := 9;
                    if Answer = '10' then
                      TListBox(Components[j]).ItemIndex := 10;
                    if Answer = '11' then
                      TListBox(Components[j]).ItemIndex := 11;
                    if Answer = '12' then
                      TListBox(Components[j]).ItemIndex := 12;
                    if Answer = '15' then
                      TListBox(Components[j]).ItemIndex := 13;
                    if Answer = '16' then
                      TListBox(Components[j]).ItemIndex := 14;
                    if Answer = 'X' then
                      TListBox(Components[j]).ItemIndex := 15;
                    if Answer = 'N' then
                      TListBox(Components[j]).ItemIndex := 16;
                    exit;
                  end;

                if Answer = 'X' then
                  begin
                    TListBox(Components[j]).ItemIndex := X;
                    exit;
                  end;
                if Answer = 'N' then
                  begin
                    TListBox(Components[j]).ItemIndex := N;
                    exit;
                  end;

                if ((Answer = '') or (Answer = ' ')) then
                  Answer := '-1';

                TListBox(Components[j]).ItemIndex := StrToInt(Answer);

                // The follow makes no sense but it is necessary.
                if ((TListBox(Components[j]).Name = 'rgrpG8') or
                  (TListBox(Components[j]).Name = 'rgrpG9') or
                  (TListBox(Components[j]).Name = 'rgrpG12') or
                  (TListBox(Components[j]).Name = 'rgrpG17') or
                  (TListBox(Components[j]).Name = 'rgrpG18') or
                  (TListBox(Components[j]).Name = 'rgrpG19') or
                  (TListBox(Components[j]).Name = 'rgrpM7') or
                  (TListBox(Components[j]).Name = 'rgrpM8') or
                  (TListBox(Components[j]).Name = 'rgrpE7') or
                  (TListBox(Components[j]).Name = 'rgrpE10') or
                  (TListBox(Components[j]).Name = 'rgrpE20') or
                  (TListBox(Components[j]).Name = 'rgrpE21') or
                  (TListBox(Components[j]).Name = 'rgrpD28') or
                  (TListBox(Components[j]).Name = 'rgrpD29') or
                  (TListBox(Components[j]).Name = 'rgrpD30') or
                  (TListBox(Components[j]).Name = 'rgrpD31') or
                  (TListBox(Components[j]).Name = 'rgrpL28') or
                  (TListBox(Components[j]).Name = 'rgrpL29') or

                  (TListBox(Components[j]).Name = 'rgrpF1') or
                  (TListBox(Components[j]).Name = 'rgrpF4') or
                  (TListBox(Components[j]).Name = 'rgrpF9') or
                  (TListBox(Components[j]).Name = 'rgrpF32') or
                  (TListBox(Components[j]).Name = 'rgrpF33') or
                  (TListBox(Components[j]).Name = 'rgrpF34') or
                  (TListBox(Components[j]).Name = 'rgrpF35') or

                  (TListBox(Components[j]).Name = 'rgrpP12') or
                  (TListBox(Components[j]).Name = 'rgrpP13')) then
                  begin
                    TListBox(Components[j]).ItemIndex := StrToInt(Answer) - 1;
                    exit;
                  end;

                if start = 1 then
                  TListBox(Components[j]).ItemIndex := StrToInt(Answer) - 1;
              end;
      end;
  end; // Radio Group

  procedure ComboBox(Item: TRzComboBox; X, N: integer; Answer: string);
  var
    m, z                      : integer;
    valid                     : boolean;
  begin
    with frmMHA_ASI_Entry_508 do
      begin
        if Length(Answer) > 2 then
          exit;
        for m := 0 to ComponentCount - 1 do
          if Components[m] is TRzComboBox then
            if TRzComboBox(Components[m]) = Item then
              begin
                // cbxD1r...cbxD12r use '98' to refer to 'X'
                if Answer = '98' then
                  begin
                    TRzComboBox(Components[m]).ItemIndex := X;
                    exit;
                  end;
                if Answer = 'X' then
                  begin
                    TRzComboBox(Components[m]).ItemIndex := X;
                    exit;
                  end;
                if Answer = 'N' then
                  begin
                    TRzComboBox(Components[m]).ItemIndex := N;
                    exit;
                  end;

                Valid := false;
                for z := 1 to Length(Answer) do
                  if Answer[z] in ['0'..'9'] then
                    Valid := true
                  else
                    Valid := false;

                if Valid then
                  TRzComboBox(Components[m]).ItemIndex := StrToInt(Answer) - 1
                else
                  TRzComboBox(Components[m]).ItemIndex := -1;
              end;
      end;
  end; // TRzComboBox

  procedure Combo(Item: TComboBox; Answer: string);
  var
    h                         : integer;
  begin
    with frmMHA_ASI_Entry_508 do
      begin
        if Length(Answer) = 0 then
          exit;
        for h := 0 to ComponentCount - 1 do
          if Components[h] is TComboBox then
            if TComboBox(Components[h]) = Item then
              TComboBox(Components[h]).text := Answer;
      end;
  end; // Combo

  procedure ORCombo(Item: TORComboBox; Answer: string);
    // This is the IENS for the program or staff item
  var
    h                         : integer;
  begin
    with frmMHA_ASI_Entry_508 do
      begin
        if Length(Answer) = 0 then
          exit;
        for h := 0 to ComponentCount - 1 do
          if Components[h] is TORComboBox then
            if TORComboBox(Components[h]) = Item then
              begin
                //if TORComboBox(Components[h]).Name = 'cbxG3' then
                //  cbxG3.SelectByIEN(StrToInt(Answer));

                if TORComboBox(Components[h]).Name = 'cbxG11Interviewerx' then
                  begin
                    cbxG11Interviewerx.SelectByIEN(StrToInt(Answer));
                    TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewerDUZ := Answer;
                  end;

                if TORComboBox(Components[h]).Name = 'cbxG11OrderedByx' then
                  begin
                    cbxG11OrderedByx.SelectByIEN(StrToInt(Answer));
                    TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).OrderedByDUZ := Answer;
                  end;
              end; // ORCombo
      end;
  end;

  procedure Edit(Item, Answer: string);
  var
    k                         : integer;
  begin
    with frmMHA_ASI_Entry_508 do
      begin
        if Length(Answer) < 1 then
          exit;
        for k := 0 to ComponentCount - 1 do
          begin
            if Components[k].ClassName = 'TRzEdit' then
              if TRzEdit(Components[k]).Name = Item then
                TRzEdit(Components[k]).text := Answer;

            if Components[k].ClassName = 'TEdit' then
              if TEdit(Components[k]).Name = Item then
                TEdit(Components[k]).text := Answer;

            if Components[k].ClassName = 'TMHA_SpnEdt' then
              if TMHA_SpnEdt(Components[k]).Name = Item then
                TMHA_SpnEdt(Components[k]).text := Answer;
          end;
      end;
  end; // Edit

  procedure Date(Item: TDateTimePicker; Answer: string);
    //date is string of numbers, M style.
  var
    l                         : integer;
  begin
    with frmMHA_ASI_Entry_508 do
      begin
        if Length(Answer) < 1 then
          exit;
        for l := 0 to ComponentCount - 1 do
          begin
            if Components[l] is TDateTimePicker then
              if TDateTimePicker(Components[l]) = Item then
                TDateTimePicker(Components[l]).date :=
                  FMDateTimeToDateTime(StrToFloat(Answer)); //Convert M date to Tdate
          end;
      end;
  end; // Date


begin
//  frmMHA_ASI_Entry_508.initialize := True;
  with frmMHA_ASI_Entry_508 do
    begin

      MakePartialList;

      for i := 0 to lvPartialList.Count - 1 do
        begin
          S := '';
          Response := '';
          S := Piece(lvPartialList[i], ';', 1);
          Response := Piece(lvPartialList[i], ';', 2);
          // ignore the blank responses
          if Response = '' then
            goto Skip;

          // General ----------------------------------------------------
          if S = '1' then
            //Date(dtpG4, Response);
            Edit('dtpG4', Response);
          if S = '2' then
            RadioGroup(cbxG3, 1, -1, -1, Response);
          //    if S = '.02' then Patient.DFN := Response;

          if S = '.04' then
            begin
              if Response = '1' then
                gvASI_Type := 0;
              if Response = '2' then
                gvASI_Type := 1;
              if Response = '3' then
                gvASI_Type := 2;
              if Response = '4' then
                gvASI_Type := 3;
            end;

          //  RadioGroup(rgrpG8, 1, -1, -1, Response);
          if S = '.05' then
            //Date(dtpG5, Response);
            Edit('dtpG5', Response);
          if S = '.07' then
            RadioGroup(rgrpG9, 1, -1, -1, Response);
          if S = '.81' then // Interviewer
            ORCombo(cbxG11Interviewerx, Response);
          if S = '.09' then // Ordered by
            ORCombo(cbxG11OrderedByx, Response);
          if S = '.11' then
            RadioGroup(rgrpG12, 1, -1, 3, Response);
          if S = '1.201' then
            Edit('edtG14y', Response);
          if S = '1.202' then
            Edit('edtG14m', Response);
          if S = '1.203' then
            RadioGroup(rgrpG15, 0, 2, -1, Response);
          if S = '.16' then
            RadioGroup(rgrpG17, 1, -1, -1, Response);
          if S = '.17' then
            RadioGroup(rgrpG18, 1, -1, -1, Response);
          if S = '2.01' then
            RadioGroup(rgrpG19, 1, 6, 7, Response);
          if S = '2.02' then
            Edit('edtG19e', Response);
          if S = '2.03' then
            Edit('edtG20', Response);

          // Medical ----------------------------------------------------
          if S = '8.01' then
            Edit('edtM1', Response);
          if S = '8.02' then
            Edit('edtM2y', Response);
          if S = '8.025' then
            Edit('edtM2m', Response);
          if S = '8.03' then
            RadioGroup(rgrpM3, 0, 2, -1, Response);
          if S = '8.04' then
            Edit('edtM3e', Response);
          if S = '8.05' then
            RadioGroup(rgrpM4, 0, 2, -1, Response);
          if S = '8.06' then
            RadioGroup(rgrpM5, 0, 2, -1, Response);
          if S = '8.07' then
            Edit('edtM5e', Response);
          if S = '8.08' then
            Edit('edtM6', Response);
          if S = '8.09' then
            RadioGroup(rgrpM7, 0, 5, -1, Response);
          if S = '8.11' then
            RadioGroup(rgrpM8, 0, 5, -1, Response);
          if S = '8.12' then
            RadioGroup(rgrpM9, 0, 10, -1, Response);
          if S = '8.14' then
            RadioGroup(rgrpM10, 0, 2, -1, Response);
          if S = '8.15' then
            RadioGroup(rgrpM11, 0, 2, -1, Response);
          // Comment field is below

    // Education --------------------------------------------------
          if S = '9.01' then
            Edit('edtE1y', Response);
          if S = '9.02' then
            Edit('edtE1m', Response);
          if S = '9.03' then
            Edit('edtE2', Response);
          if S = '9.04' then
            RadioGroup(rgrpE3, 0, 2, -1, Response);
          if S = '9.05' then
            Edit('edtE3e', Response);
          if S = '9.06' then
            RadioGroup(rgrpE4, 0, 2, -1, Response);
          if S = '9.09' then
            RadioGroup(rgrpE5, 0, 2, -1, Response);
          if S = '9.11' then
            Edit('edtE6y', Response);
          if S = '9.12' then
            Edit('edtE6m', Response);
          if S = '9.14' then
            RadioGroup(rgrpE7, 1, -1, 9, Response);
          if S = '9.145' then
            Edit('edtE7e', Response);
          if S = '9.15' then
            RadioGroup(rgrpE8, 0, 2, -1, Response);
          if S = '9.16' then
            RadioGroup(rgrpE9, 0, 2, 3, Response);
          if S = '9.17' then
            RadioGroup(rgrpE10, 1, 8, 9, Response);
          if S = '9.18' then
            Edit('edtE11', Response);
          if S = '9.19' then
            Edit('edtE12', Response);
          if S = '9.21' then
            Edit('edtE13', Response);
          if S = '9.22' then
            Edit('edtE14', Response);
          if S = '9.23' then
            Edit('edtE15', Response);
          if S = '9.24' then
            Edit('edtE16', Response);
          if S = '9.25' then
            Edit('edtE17', Response);
          if S = '9.29' then
            Edit('edtE18', Response);
          if S = '9.31' then
            Edit('edtE19', Response);
          if S = '9.32' then
            RadioGroup(rgrpE20, 0, 5, 6, Response);
          if S = '9.33' then
            RadioGroup(rgrpE21, 0, 5, 6, Response);
          if S = '9.34' then
            RadioGroup(rgrpE22, 0, 10, -1, Response);
          if S = '9.35' then
            RadioGroup(rgrpE23, 0, 2, -1, Response);
          if S = '9.36' then
            RadioGroup(rgrpE24, 0, 2, -1, Response);
          // Comment field is below

   // Drug -----------------------------------------------------
          if S = '10.01' then
            Edit('edtD1d', Response);
          if S = '10.02' then
            Edit('edtD1y', Response);
          if S = '10.03' then
            RadioGroup(lbD1r, 1, 5, -1, Response);
          if S = '10.04' then
            Edit('edtD2d', Response);
          if S = '10.05' then
            Edit('edtD2y', Response);
          if S = '10.06' then
            RadioGroup(lbD2r, 1, 5, -1, Response);
          if S = '10.07' then
            Edit('edtD3d', Response);
          if S = '10.08' then
            Edit('edtD3y', Response);
          if S = '10.09' then
            RadioGroup(lbD3r, 1, 5, -1, Response);
          if S = '10.11' then
            Edit('edtD4d', Response);
          if S = '10.12' then
            Edit('edtD4y', Response);
          if S = '10.14' then
            RadioGroup(lbD4r, 1, 5, -1, Response);
          if S = '10.15' then
            Edit('edtD5d', Response);
          if S = '10.16' then
            Edit('edtD5y', Response);
          if S = '10.17' then
            RadioGroup(lbD5r, 1, 5, -1, Response);
          if S = '10.18' then
            Edit('edtD6d', Response);
          if S = '10.19' then
            Edit('edtD6y', Response);
          if S = '10.21' then
            RadioGroup(lbD6r, 1, 5, -1, Response);
          if S = '10.22' then
            Edit('edtD7d', Response);
          if S = '10.23' then
            Edit('edtD7y', Response);
          if S = '10.24' then
            RadioGroup(lbD7r, 1, 5, -1, Response);
          if S = '10.25' then
            Edit('edtD8d', Response);
          if S = '10.26' then
            Edit('edtD8y', Response);
          if S = '10.27' then
            RadioGroup(lbD8r, 1, 5, -1, Response);
          if S = '10.28' then
            Edit('edtD9d', Response);
          if S = '10.29' then
            Edit('edtD9y', Response);
          if S = '10.31' then
            RadioGroup(lbD9r, 1, 5, -1, Response);
          if S = '10.32' then
            Edit('edtD10d', Response);
          if S = '10.33' then
            Edit('edtD10y', Response);
          if S = '10.34' then
            RadioGroup(lbD10r, 1, 5, -1, Response);
          if S = '10.35' then
            Edit('edtD11d', Response);
          if S = '10.36' then
            Edit('edtD11y', Response);
          if S = '10.37' then
            RadioGroup(lbD11r, 1, 5, -1, Response);
          if S = '10.38' then
            Edit('edtD12d', Response);
          if S = '10.39' then
            Edit('edtD12y', Response);
          if S = '10.41' then
            RadioGroup(lbD12r, 1, 5, -1, Response);
          if S = '10.42' then
            Edit('edtD13d', Response);
          if S = '10.43' then
            Edit('edtD13y', Response);
          if S = '10.45' then
            RadioGroup(rgrpD14, 0, -1, -1, Response);

          if S = '11.01' then
            Edit('edtD15', Response);
          if S = '11.02' then
            Edit('edtD16', Response);
          if S = '11.03' then
            Edit('edtD17', Response);
          if S = '11.04' then
            Edit('edtD18', Response);
          if S = '11.05' then
            Edit('edtD19', Response);
          if S = '11.06' then
            Edit('edtD20', Response);
          if S = '11.07' then
            Edit('edtD21', Response);
          if S = '11.08' then
            Edit('edtD22', Response);
          if S = '11.09' then
            Edit('edtD23', Response);
          if S = '11.11' then
            Edit('edtD24', Response);
          if S = '11.12' then
            Edit('edtD25', Response);
          if S = '11.14' then
            Edit('edtD26', Response);
          if S = '11.15' then
            Edit('edtD27', Response);
          if S = '11.16' then
            RadioGroup(rgrpD28, 0, 5, 6, Response);
          if S = '11.165' then
            RadioGroup(rgrpD29, 0, 5, 6, Response);
          if S = '11.17' then
            RadioGroup(rgrpD30, 0, 5, 6, Response);
          if S = '11.175' then
            RadioGroup(rgrpD31, 0, 5, 6, Response);
          if S = '11.18' then
            RadioGroup(rgrpD32, 0, 10, 11, Response);
          if S = '11.185' then
            RadioGroup(rgrpD33, 0, 10, -1, Response);
          if S = '11.19' then
            RadioGroup(rgrpD34, 0, 2, -1, Response);
          if S = '11.21' then
            RadioGroup(rgrpD35, 0, -1, -1, Response);
          // Comment field is below

      //Legal -------------------------------------------------------
          if S = '14.01' then
            RadioGroup(rgrpL1, 0, 2, -1, Response);
          if S = '14.02' then
            RadioGroup(rgrpL2, 0, 2, -1, Response);
          if S = '14.03' then
            Edit('edtL3', Response);
          if S = '14.04' then
            Edit('edtL4', Response);
          if S = '14.05' then
            Edit('edtL5', Response);
          if S = '14.06' then
            Edit('edtL6', Response);
          if S = '14.07' then
            Edit('edtL7', Response);
          if S = '14.08' then
            Edit('edtL8', Response);
          if S = '14.09' then
            Edit('edtL9', Response);
          if S = '14.11' then
            Edit('edtL10', Response);
          if S = '14.12' then
            Edit('edtL11', Response);
          if S = '14.14' then
            Edit('edtL12', Response);
          if S = '14.15' then
            Edit('edtL13', Response);
          if S = '14.16' then
            Edit('edtL14', Response);
          if S = '14.17' then
            Edit('edtL15', Response);
          if S = '14.18' then
            Edit('edtL16', Response);
          if S = '14.19' then
            Edit('edtL17', Response);
          if S = '14.21' then
            Edit('edtL18', Response);
          if S = '14.22' then
            Edit('edtL19', Response);
          if S = '14.23' then
            Edit('edtL20', Response);
          if S = '14.24' then
            Edit('edtL21', Response);
          if S = '14.25' then
            Edit('edtL22', Response);
          if S = '14.26' then
            RadioGroup(rgrpL23, 0, 0, 0, Response);
          if S = '14.27' then
            RadioGroup(rgrpL24, 0, 2, -1, Response);
          if S = '14.28' then
            RadioGroup(rgrpL25, 0, 0, 0, Response);
          if S = '14.29' then
            Edit('edtL26', Response);
          if S = '14.31' then
            Edit('edtL27', Response);
          if S = '14.32' then
            RadioGroup(rgrpL28, 0, 5, 6, Response);
          if S = '14.33' then
            RadioGroup(rgrpL29, 0, 5, 6, Response);
          if S = '14.34' then
            RadioGroup(rgrpL30, 0, 10, -1, Response);
          if S = '14.35' then
            RadioGroup(rgrpL31, 0, 2, -1, Response);
          if S = '14.36' then
            RadioGroup(rgrpL32, 0, 2, -1, Response);
          // Comment field is below

   // Family -----------------------------------------------------
          if S = '15.01' then
            RadioGroup(lbH1a, 1, 2, 3, Response);
          if S = '15.02' then
            RadioGroup(lbH1d, 1, 2, 3, Response);
          if S = '15.03' then
            RadioGroup(lbH1p, 1, 2, 3, Response);
          if S = '15.04' then
            RadioGroup(lbH2a, 1, 2, 3, Response);
          if S = '15.05' then
            RadioGroup(lbH2d, 1, 2, 3, Response);
          if S = '15.06' then
            RadioGroup(lbH2p, 1, 2, 3, Response);
          if S = '15.07' then
            RadioGroup(lbH3a, 1, 2, 3, Response);
          if S = '15.08' then
            RadioGroup(lbH3d, 1, 2, 3, Response);
          if S = '15.09' then
            RadioGroup(lbH3p, 1, 2, 3, Response);
          if S = '15.11' then
            RadioGroup(lbH4a, 1, 2, 3, Response);
          if S = '15.12' then
            RadioGroup(lbH4d, 1, 2, 3, Response);
          if S = '15.14' then
            RadioGroup(lbH4p, 1, 2, 3, Response);
          if S = '15.15' then
            RadioGroup(lbH5a, 1, 2, 3, Response);
          if S = '15.16' then
            RadioGroup(lbH5d, 1, 2, 3, Response);
          if S = '15.17' then
            RadioGroup(lbH5p, 1, 2, 3, Response);
          if S = '15.18' then
            RadioGroup(lbH6a, 1, 2, 3, Response);
          if S = '15.19' then
            RadioGroup(lbH6d, 1, 2, 3, Response);
          if S = '15.21' then
            RadioGroup(lbH6p, 1, 2, 3, Response);
          if S = '15.22' then
            RadioGroup(lbH7a, 1, 2, 3, Response);
          if S = '15.23' then
            RadioGroup(lbH7d, 1, 2, 3, Response);
          if S = '15.24' then
            RadioGroup(lbH7p, 1, 2, 3, Response);
          if S = '15.25' then
            RadioGroup(lbH8a, 1, 2, 3, Response);
          if S = '15.26' then
            RadioGroup(lbH8d, 1, 2, 3, Response);
          if S = '15.27' then
            RadioGroup(lbH8p, 1, 2, 3, Response);

          if S = '16.01' then
            RadioGroup(lbH9a, 1, 2, 3, Response);
          if S = '16.02' then
            RadioGroup(lbH9d, 1, 2, 3, Response);
          if S = '16.03' then
            RadioGroup(lbH9p, 1, 2, 3, Response);
          if S = '16.04' then
            RadioGroup(lbH10a, 1, 2, 3, Response);
          if S = '16.05' then
            RadioGroup(lbH10d, 1, 2, 3, Response);
          if S = '16.06' then
            RadioGroup(lbH10p, 1, 2, 3, Response);
          if S = '16.07' then
            RadioGroup(lbH11a, 1, 2, 3, Response);
          if S = '16.08' then
            RadioGroup(lbH11d, 1, 2, 3, Response);
          if S = '16.09' then
            RadioGroup(lbH11p, 1, 2, 3, Response);
          if S = '16.11' then
            RadioGroup(lbH12a, 1, 2, 3, Response);
          if S = '16.12' then
            RadioGroup(lbH12d, 1, 2, 3, Response);
          if S = '16.14' then
            RadioGroup(lbH12p, 1, 2, 3, Response);
          // Comment field is below

   // Social -----------------------------------------------------
          if S = '17.01' then
            RadioGroup(rgrpF1, 1, 6, -1, Response);
          if S = '17.02' then
            Edit('edtF2y', Response);
          if S = '17.03' then
            Edit('edtF2m', Response);
          if S = '17.04' then
            RadioGroup(rgrpF3, 0, 3, 4, Response);
          if S = '17.05' then
            RadioGroup(rgrpF4, 1, 9, 10, Response);
          if S = '17.06' then
            Edit('edtF5y', Response);
          if S = '17.07' then
            Edit('edtF5m', Response);
          if S = '17.08' then
            RadioGroup(rgrpF6, 0, 3, 4, Response);
          if S = '17.09' then
            RadioGroup(rgrpF7, 0, 2, -1, Response);
          if S = '17.11' then
            RadioGroup(rgrpF8, 0, 2, -1, Response);
          if S = '17.12' then
            RadioGroup(rgrpF9, 1, 3, 4, Response);
          if S = '17.14' then
            RadioGroup(rgrpF10, 0, 3, 4, Response);
          if S = '17.15' then
            Edit('edtF11', Response);
          if S = '17.16' then
            RadioGroup(lbF12, 1, 2, 3, Response);
          if S = '17.17' then
            RadioGroup(lbF13, 1, 2, 3, Response);
          if S = '17.18' then
            RadioGroup(lbF14, 1, 2, 3, Response);
          if S = '17.19' then
            RadioGroup(lbF15, 1, 2, 3, Response);
          if S = '17.21' then
            RadioGroup(lbF16, 1, 2, 3, Response);
          if S = '17.22' then
            RadioGroup(lbF17, 1, 2, 3, Response);

          if S = '18.01' then
            RadioGroup(rgrpF18, 0, 2, 3, Response);
          if S = '18.02' then
            RadioGroup(rgrpF18l, 0, 2, 3, Response);
          if S = '18.03' then
            RadioGroup(rgrpF19, 0, 2, 3, Response);
          if S = '18.04' then
            RadioGroup(rgrpF19l, 0, 2, 3, Response);
          if S = '18.05' then
            RadioGroup(rgrpF20, 0, 2, 3, Response);
          if S = '18.06' then
            RadioGroup(rgrpF20l, 0, 2, 3, Response);
          if S = '18.07' then
            RadioGroup(rgrpF21, 0, 2, 3, Response);
          if S = '18.08' then
            RadioGroup(rgrpF21l, 0, 2, 3, Response);
          if S = '18.09' then
            RadioGroup(rgrpF22, 0, 2, 3, Response);
          if S = '18.11' then
            RadioGroup(rgrpF22l, 0, 2, 3, Response);
          if S = '18.12' then
            RadioGroup(rgrpF23, 0, 2, 3, Response);
          if S = '18.14' then
            RadioGroup(rgrpF23l, 0, 2, 3, Response);
          if S = '18.15' then
            RadioGroup(rgrpF24, 0, 2, 3, Response);
          if S = '18.16' then
            RadioGroup(rgrpF24l, 0, 2, 3, Response);
          if S = '18.17' then
            RadioGroup(rgrpF25, 0, 2, 3, Response);
          if S = '18.18' then
            RadioGroup(rgrpF25l, 0, 2, 3, Response);
          if S = '18.185' then
            RadioGroup(rgrpF26, 0, 2, 3, Response);
          if S = '18.186' then
            RadioGroup(rgrpF26l, 0, 2, 3, Response);
          if S = '18.19' then
            RadioGroup(rgrpF27, 0, 2, 3, Response);
          if S = '18.195' then
            RadioGroup(rgrpF27l, 0, 2, 3, Response);
          if S = '18.21' then
            RadioGroup(rgrpF28, 0, 2, 3, Response);
          if S = '18.215' then
            RadioGroup(rgrpF28l, 0, 2, 3, Response);
          if S = '18.22' then
            RadioGroup(rgrpF29, 0, 2, 3, Response);
          if S = '18.225' then
            RadioGroup(rgrpF29l, 0, 2, 3, Response);
          if S = '18.23' then
            Edit('edtF30', Response);
          if S = '18.24' then
            Edit('edtF31', Response);
          if S = '18.25' then
            RadioGroup(rgrpF32, 0, 5, 6, Response);
          if S = '18.26' then
            RadioGroup(rgrpF33, 0, 5, 6, Response);
          if S = '18.27' then
            RadioGroup(rgrpF34, 0, 5, 6, Response);
          if S = '18.28' then
            RadioGroup(rgrpF35, 0, 5, 6, Response);
          if S = '18.29' then
            RadioGroup(rgrpF36, 0, 10, -1, Response);
          if S = '18.31' then
            RadioGroup(rgrpF37, 0, 2, -1, Response);
          if S = '18.32' then
            RadioGroup(rgrpF38, 0, 2, -1, Response);
          // Comment field is below

   // Psych ------------------------------------------------------
          if S = '19.01' then
            Edit('edtP1a', Response);
          if S = '19.02' then
            Edit('edtP1b', Response);
          if S = '19.03' then
            RadioGroup(rgrpP2, 0, 2, -1, Response);
          if S = '19.04' then
            RadioGroup(rgrpP3, 0, 2, -1, Response);
          if S = '19.05' then
            RadioGroup(rgrpP3l, 0, 2, -1, Response);
          if S = '19.06' then
            RadioGroup(rgrpP4, 0, 2, -1, Response);
          if S = '19.07' then
            RadioGroup(rgrpP4l, 0, 2, -1, Response);
          if S = '19.08' then
            RadioGroup(rgrpP5, 0, 2, -1, Response);
          if S = '19.09' then
            RadioGroup(rgrpP5l, 0, 2, -1, Response);
          if S = '19.11' then
            RadioGroup(rgrpP6, 0, 2, -1, Response);
          if S = '19.12' then
            RadioGroup(rgrpP6l, 0, 2, -1, Response);
          if S = '19.14' then
            RadioGroup(rgrpP7, 0, 2, -1, Response);
          if S = '19.15' then
            RadioGroup(rgrpP7l, 0, 2, -1, Response);
          if S = '19.16' then
            RadioGroup(rgrpP8, 0, 2, -1, Response);
          if S = '19.17' then
            RadioGroup(rgrpP8l, 0, 2, -1, Response);
          if S = '19.18' then
            RadioGroup(rgrpP9, 0, 2, -1, Response);
          if S = '19.19' then
            RadioGroup(rgrpP9l, 0, 2, -1, Response);
          if S = '19.21' then
            RadioGroup(rgrpP10, 0, 2, -1, Response);
          if S = '19.22' then
            RadioGroup(rgrpP10l, 0, 2, -1, Response);
          if S = '19.23' then
            Edit('edtP11', Response);
          if S = '19.24' then
            RadioGroup(rgrpP12, 0, 5, -1, Response);
          if S = '19.25' then
            RadioGroup(rgrpP13, 0, 5, -1, Response);
          if S = '19.26' then
            RadioGroup(rgrpP14, 0, 2, -1, Response);
          if S = '19.27' then
            RadioGroup(rgrpP15, 0, 2, -1, Response);
          if S = '19.28' then
            RadioGroup(rgrpP16, 0, 2, -1, Response);
          if S = '19.29' then
            RadioGroup(rgrpP17, 0, 2, -1, Response);
          if S = '19.31' then
            RadioGroup(rgrpP18, 0, 2, -1, Response);
          if S = '19.32' then
            RadioGroup(rgrpP19, 0, 2, -1, Response);
          if S = '19.33' then
            RadioGroup(rgrpP20, 0, 10, -1, Response);
          if S = '19.34' then
            RadioGroup(rgrpP21, 0, 2, -1, Response);
          if S = '19.35' then
            RadioGroup(rgrpP22, 0, 2, -1, Response);
          // Comment field is below
          skip:
        end;

      LoadCommentFields(TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).IENS);
    end;

  if lvPartialList <> nil then
    lvPartialList.Free;
end;


procedure TASI_Build_508.BuildForm;
begin
  with frmMHA_ASI_Entry_508 do
    begin

      with cbxG11Interviewerx do
        begin
          InitLongList(TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewerName);
          SelectByIEN(StrToInt(TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewerDUZ));
        end;

      with cbxG11OrderedByx do
        begin
          InitLongList(TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).OrderedByName);
          SelectByIEN(StrToInt(TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).OrderedbyDUZ));
        end;

      case TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).InterviewType of
        itFull:
          begin
            if not TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).IENSExist then
              begin
                //rgrpG8.ItemIndex := 0;
                gvASI_Type := 0;
                TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).IENS := '';
                Setup_ASIFull_508;
              end
            else

              begin
                //rgrpG8.ItemIndex := 0;
                gvASI_Type := 0;
                Setup_ASIFull_508;
                InsertItems; // Data pulled from dark-side
              end;
          end;
        itLite, itMV:
          begin
            //rgrpG8.ItemIndex := 1;
            gvASI_Type := 1;
            Setup_ASILite_508;
            if TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).IENSExist then
              InsertItems; // Data pulled from dark-side
          end;

        itFU:
          begin
            //rgrpG8.ItemIndex := 2;
            gvASI_Type := 2;
            Setup_ASIFollowup_508;
            if TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).IENSExist then
              InsertItems; // Data pulled from dark-side
          end;
      end; //case

    end;
end;


procedure TASI_Build_508.SetupComboBoxItems;
// run-time contruction of the strings for combo boxes
var
  TempStr                     : TstringList;
begin
  with frmMHA_ASI_Entry_508 do
    begin
      TempStr := TStringList.Create;
      TempStr.Add('0. Clearly NO for all relatives in that category');
      TempStr.Add('1. Clearly YES for any relatives in that category');
      TempStr.Add('X. Uncertain or don''t know');
      TempStr.Add('N. No relative in that category');

      lbH1a.Items.Assign(TempStr);
      lbH2a.Items.Assign(TempStr);
      lbH3a.Items.Assign(TempStr);
      lbH4a.Items.Assign(TempStr);
      lbH5a.Items.Assign(TempStr);
      lbH6a.Items.Assign(TempStr);
      lbH7a.Items.Assign(TempStr);
      lbH8a.Items.Assign(TempStr);
      lbH9a.Items.Assign(TempStr);
      lbH10a.Items.Assign(TempStr);
      lbH11a.Items.Assign(TempStr);
      lbH12a.Items.Assign(TempStr);

      lbH1d.Items.Assign(TempStr);
      lbH2d.Items.Assign(TempStr);
      lbH3d.Items.Assign(TempStr);
      lbH4d.Items.Assign(TempStr);
      lbH5d.Items.Assign(TempStr);
      lbH6d.Items.Assign(TempStr);
      lbH7d.Items.Assign(TempStr);
      lbH8d.Items.Assign(TempStr);
      lbH9d.Items.Assign(TempStr);
      lbH10d.Items.Assign(TempStr);
      lbH11d.Items.Assign(TempStr);
      lbH12d.Items.Assign(TempStr);

      lbH1p.Items.Assign(TempStr);
      lbH2p.Items.Assign(TempStr);
      lbH3p.Items.Assign(TempStr);
      lbH4p.Items.Assign(TempStr);
      lbH5p.Items.Assign(TempStr);
      lbH6p.Items.Assign(TempStr);
      lbH7p.Items.Assign(TempStr);
      lbH8p.Items.Assign(TempStr);
      lbH9p.Items.Assign(TempStr);
      lbH10p.Items.Assign(TempStr);
      lbH11p.Items.Assign(TempStr);
      lbH12p.Items.Assign(TempStr);

      lbF12.Items.Assign(TempStr);
      lbF13.Items.Assign(TempStr);
      lbF14.Items.Assign(TempStr);
      lbF15.Items.Assign(TempStr);
      lbF16.Items.Assign(TempStr);
      lbF17.Items.Assign(TempStr);

      TempStr.Clear;
      TempStr.Add('1. Oral ');
      TempStr.Add('2. Nasal');
      TempStr.Add('3. Smoking');
      TempStr.Add('4. Non IV injection');
      TempStr.Add('5. IV injection');
      TempStr.Add('X. Not Answered');

      lbD1r.Items.Assign(TempStr);
      lbD2r.Items.Assign(TempStr);
      lbD3r.Items.Assign(TempStr);
      lbD4r.Items.Assign(TempStr);
      lbD5r.Items.Assign(TempStr);
      lbD6r.Items.Assign(TempStr);
      lbD7r.Items.Assign(TempStr);
      lbD8r.Items.Assign(TempStr);
      lbD9r.Items.Assign(TempStr);
      lbD10r.Items.Assign(TempStr);
      lbD11r.Items.Assign(TempStr);
      lbD12r.Items.Assign(TempStr);

      TempStr.Free;
    end;
end;


//-------------------------------------------------------------------
//       Defaults & Captions: Full, Lite & Followup
//-------------------------------------------------------------------

procedure TASI_Build_508.DefaultValuesForItems;
// These values are NOT from Allan Finkelstein's, ASI Quick Text.txt.
// Edit items are set to blank, spin edits to zero, list boxes to unselected
// When JAWS scans a fresh page, it will read all the list-box selections--very
// confusing for the user.

begin

  with frmMHA_ASI_Entry_508 do
    begin
      //frmMHA_ASI_Entry_508.initialize := True;
      // Put a header in the comment
      (*
      if length(memGeneral.text) = 0 then
        memGeneral.text := 'General Comments:';
      if length(memMedical.text) = 0 then
        memMedical.text := 'Medical Status Comments:';
      if length(memEmploy.text) = 0 then
        memEmploy.text := 'Employment/Support Status Comments:';
      if length(memDrug.text) = 0 then
        memDrug.text := 'Drug/Alcohol Use Comments:';
      if length(memFamily.text) = 0 then
        memFamily.text := 'Family History Comments:';
      if length(memLegal.text) = 0 then
        memLegal.text := 'Legal Status Comments:';
      if length(memPsych.text) = 0 then
        memPsych.text := 'Psychiatric Status Comments:';
      if length(memSocial.text) = 0 then
        memSocial.text := 'Family/Social Relationships Comments:'; *)

      // No dates here
      //rgrpG8.ItemIndex  := 0; Do use a value here
      (*
      if rgrpG9.ItemIndex = -1 then
        rgrpG9.ItemIndex := 0;
      if rgrpG12.ItemIndex = -1 then
        rgrpG12.ItemIndex := 3;
        *)
      if ((edtG14y.Enabled) and (edtG14y.text <> '')) then
        edtG14y.text := '';
      if ((edtG14m.Enabled) and (edtG14m.text <> '')) then
        edtG14m.text := '';
      if ((rgrpG15.Enabled) and (rgrpG15.ItemIndex <> -1)) then
        rgrpG15.ItemIndex := -1;
      rgrpG17.ItemIndex := -1;
      rgrpG18.ItemIndex := -1;
      rgrpG19.ItemIndex := -1;
      edtG19e.Text := '';
      edtG20.text := '';

      edtM1.Text := '';
      edtM2y.text := '';
      edtM2m.text := '';
      rgrpM3.ItemIndex := -1;
      edtM3e.text := '';
      rgrpM4.ItemIndex := -1;
      rgrpM5.ItemIndex := -1;
      edtM5e.text := '';
      edtM6.Text := '';
      rgrpM7.ItemIndex := -1;
      rgrpM8.ItemIndex := -1;
      rgrpM9.ItemIndex := -1;
      rgrpM10.ItemIndex := -1;
      rgrpM11.ItemIndex := -1;

      edtE1y.text := '';
      edtE1m.text := '';
      edtE2.text := '';
      rgrpE3.ItemIndex := -1;
      edtE3e.text := '';
      rgrpE4.ItemIndex := -1;
      rgrpE5.ItemIndex := -1;
      edtE6y.text := '';
      edtE6m.text := '';
      rgrpE7.ItemIndex := -1;
      edtE7e.text := '';
      rgrpE8.ItemIndex := -1;
      rgrpE9.ItemIndex := -1;
      rgrpE10.ItemIndex := -1;
      edtE11.text := '';
      edtE12.text := '0';
      edtE13.text := '0';
      edtE14.text := '0';
      edtE15.text := '0';
      edtE16.text := '0';
      edtE17.text := '0';
      edtE18.text := '';
      edtE19.text := '';
      rgrpE20.ItemIndex := -1;
      rgrpE21.ItemIndex := -1;
      rgrpE22.ItemIndex := -1;
      rgrpE23.ItemIndex := -1;
      rgrpE24.ItemIndex := -1;
      // Drug values are from Vista defaults
      if edtD1d.Text = '' then
        edtD1d.text := '0';
      edtD1y.text := '0';

      lbD1r.ItemIndex := -1; // 0;
      if edtD2d.Text = '' then
        edtD2d.text := '0';
      edtD2y.text := '0';

      lbD2r.ItemIndex := -1; // 0;
      edtD3d.text := '0';
      edtD3y.text := '0';

      lbD3r.ItemIndex := -1; // 4;
      edtD4d.text := '0';
      edtD4y.text := '0';

      lbD4r.ItemIndex := -1; // 0;
      edtD5d.text := '0';
      edtD5y.text := '0';

      lbD5r.ItemIndex := -1; // 0;
      edtD6d.text := '0';
      edtD6y.text := '0';

      lbD6r.ItemIndex := -1; // 0;
      edtD7d.text := '0';
      edtD7y.text := '0';

      lbD7r.ItemIndex := -1; // 0;
      edtD8d.text := '0';
      edtD8y.text := '0';

      lbD8r.ItemIndex := -1; // 1;
      edtD9d.text := '0';
      edtD9y.text := '0';

      lbD9r.ItemIndex := -1; // 0;
      edtD10d.text := '0';
      edtD10y.text := '0';

      lbD10r.ItemIndex := -1; // 2;
      edtD11d.text := '0';
      edtD11y.text := '0';

      lbD11r.ItemIndex := -1; // 0;
      edtD12d.text := '0';
      edtD12y.text := '0';

      lbD12r.ItemIndex := -1; // 1;
      edtD13d.text := '0';
      edtD13y.text := '0';

      rgrpD14.ItemIndex := -1;
      edtD15.text := '';
      edtD16.text := '';
      edtD17.text := '';
      edtD18.text := '';
      edtD19.text := ''; // blank required by program logic
      edtD20.text := ''; // blank required by program logic
      edtD21.text := '';
      edtD22.text := '';
      edtD23.text := '0';
      edtD24.text := '0';
      edtD25.text := '';
      edtD26.text := ''; // blank required by program logic
      edtD27.text := ''; // blank required by program logic
      rgrpD28.ItemIndex := -1;
      rgrpD29.ItemIndex := -1;
      rgrpD30.ItemIndex := -1;
      rgrpD31.ItemIndex := -1;
      rgrpD32.ItemIndex := -1;
      rgrpD33.ItemIndex := -1;
      rgrpD34.ItemIndex := -1;
      rgrpD35.ItemIndex := -1;

      rgrpL1.ItemIndex := -1;
      rgrpL2.ItemIndex := -1;
      edtL3.text := '0';
      edtL8.text := '0';
      edtL13.text := '0';
      edtL4.text := '0';
      edtL9.text := '0';
      edtL14.text := '0';
      edtL5.text := '0';
      edtL10.text := '0';
      edtL15.text := '0';
      edtL6.text := '0';
      edtL11.text := '0';
      edtL16.text := '0';
      edtL7.text := '0';
      edtL12.text := '0';
      edtL17.text := '0';
      edtL18.text := '0';
      edtL19.text := '0';
      edtL20.text := '0';
      edtL21.text := '';
      edtL22.text := '';
      rgrpL23.ItemIndex := -1;
      rgrpL24.ItemIndex := -1;
      rgrpL25.ItemIndex := -1;
      edtL26.text := '0';
      edtL27.text := '0';
      rgrpL28.ItemIndex := -1;
      rgrpL29.ItemIndex := -1;
      rgrpL30.ItemIndex := -1;
      rgrpL31.ItemIndex := -1;
      rgrpL32.ItemIndex := -1;
      if lbH1a.Enabled then
        begin
          lbH1a.ItemIndex := -1;
          lbH1d.ItemIndex := -1;
          lbH1p.ItemIndex := -1;
          lbH2a.ItemIndex := -1;
          lbH2d.ItemIndex := -1;
          lbH2p.ItemIndex := -1;
          lbH3a.ItemIndex := -1;
          lbH3d.ItemIndex := -1;
          lbH3p.ItemIndex := -1;
          lbH4a.ItemIndex := -1;
          lbH4d.ItemIndex := -1;
          lbH4p.ItemIndex := -1;
          lbH5a.ItemIndex := -1;
          lbH5d.ItemIndex := -1;
          lbH5p.ItemIndex := -1;
          lbH6a.ItemIndex := -1;
          lbH6d.ItemIndex := -1;
          lbH6p.ItemIndex := -1;
          lbH7a.ItemIndex := -1;
          lbH7d.ItemIndex := -1;
          lbH7p.ItemIndex := -1;
          lbH8a.ItemIndex := -1;
          lbH8d.ItemIndex := -1;
          lbH8p.ItemIndex := -1;
          lbH9a.ItemIndex := -1;
          lbH9d.ItemIndex := -1;
          lbH9p.ItemIndex := -1;
          lbH10a.ItemIndex := -1;
          lbH10d.ItemIndex := -1;
          lbH10p.ItemIndex := -1;
          lbH11a.ItemIndex := -1;
          lbH11d.ItemIndex := -1;
          lbH11p.ItemIndex := -1;
          lbH12a.ItemIndex := -1;
          lbH12d.ItemIndex := -1;
          lbH12p.ItemIndex := -1;
        end;

      rgrpF1.ItemIndex := -1;
      edtF2y.Text := '';
      edtF2m.text := '';
      rgrpF3.ItemIndex := -1;
      rgrpF4.ItemIndex := -1;
      edtF5y.Text := '';
      edtF5m.text := '';
      rgrpF6.ItemIndex := -1;
      rgrpF7.ItemIndex := -1;
      rgrpF8.ItemIndex := -1;
      rgrpF9.ItemIndex := -1;
      rgrpF10.ItemIndex := -1;
      edtF11.text := '';
      lbF12.ItemIndex := -1;
      lbF13.ItemIndex := -1;
      lbF14.ItemIndex := -1;
      lbF15.ItemIndex := -1;
      lbF16.ItemIndex := -1;
      lbF17.ItemIndex := -1;
      rgrpF18.ItemIndex := -1;
      rgrpF18l.ItemIndex := -1;
      rgrpF19.ItemIndex := -1;
      rgrpF19l.ItemIndex := -1;
      rgrpF20.ItemIndex := -1;
      rgrpF20l.ItemIndex := -1;
      rgrpF21.ItemIndex := -1;
      rgrpF21l.ItemIndex := -1;
      rgrpF22.ItemIndex := -1;
      rgrpF22l.ItemIndex := -1;
      rgrpF23.ItemIndex := -1;
      rgrpF23l.ItemIndex := -1;
      rgrpF24.ItemIndex := -1;
      rgrpF24l.ItemIndex := -1;
      rgrpF25.ItemIndex := -1;
      rgrpF25l.ItemIndex := -1;
      rgrpF26.ItemIndex := -1;
      rgrpF26l.ItemIndex := -1;
      rgrpF27.ItemIndex := -1;
      rgrpF27l.ItemIndex := -1;
      rgrpF28.ItemIndex := -1;
      rgrpF28l.ItemIndex := -1;
      rgrpF29.ItemIndex := -1;
      rgrpF29l.ItemIndex := -1;
      edtF30.Text := '';
      edtF31.text := '';
      rgrpF32.ItemIndex := -1;
      rgrpF33.ItemIndex := -1;
      rgrpF34.ItemIndex := -1;
      rgrpF35.ItemIndex := -1;
      rgrpF36.ItemIndex := -1;
      rgrpF37.ItemIndex := -1;
      rgrpF38.ItemIndex := -1;

      edtP1a.text := '0';
      edtP1b.text := '0';
      rgrpP2.ItemIndex := -1;
      rgrpP3.ItemIndex := -1;
      rgrpP3l.ItemIndex := -1;
      rgrpP4.ItemIndex := -1;
      rgrpP4l.ItemIndex := -1;
      rgrpP5.ItemIndex := -1;
      rgrpP5l.ItemIndex := -1;
      rgrpP6.ItemIndex := -1;
      rgrpP6l.ItemIndex := -1;
      rgrpP7.ItemIndex := -1;
      rgrpP7l.ItemIndex := -1;
      rgrpP8.ItemIndex := -1;
      rgrpP8l.ItemIndex := -1;
      rgrpP9.ItemIndex := -1;
      rgrpP9l.ItemIndex := -1;
      rgrpP10.ItemIndex := -1;
      rgrpP10l.ItemIndex := -1;
      edtP11.text := '';
      rgrpP12.ItemIndex := -1;
      rgrpP13.ItemIndex := -1;
      rgrpP14.ItemIndex := -1;
      rgrpP15.ItemIndex := -1;
      rgrpP16.ItemIndex := -1;
      rgrpP17.ItemIndex := -1;
      rgrpP18.ItemIndex := -1;
      rgrpP19.ItemIndex := -1;
      rgrpP20.ItemIndex := -1;
      rgrpP21.ItemIndex := -1;
      rgrpP22.ItemIndex := -1;
    end;
end;

//-------------------------------------------------------------------
//             ASI items are placed on scroll box
//-------------------------------------------------------------------

procedure TASI_Build_508.Setup_ASIFull_508;
begin
  with frmMHA_ASI_Entry_508 do
    begin
      Screen.Cursor := crHourglass;
      TabFamily.TabVisible := true;

      // Make sure these questions are Visible
      GroupBox8.Visible := true;
      GroupBox9.Visible := True;
      edtG14y.Visible := true;
      edtG14m.Visible := true;

      GroupBox10.Visible := True;
      rgrpG15.Visible := True;
      GroupBox11.Visible := True;
      rgrpG17.Visible := True;
      GroupBox12.Visible := True;
      rgrpG18.Visible := True;
      edtG19e.Enabled := false;

      gbM2.Visible := True;
      gbM2a.Visible := True;
      edtM2y.Visible := True;
      edtM2m.Visible := True;
      gbM3.Visible := True;
      rgrpM3.Visible := True;
      edtM3e.Visible := true;
      gbM3S.Visible := True;
      edtM3e.Enabled := False;
      gbM5S.Visible := True;
      edtM5e.Visible := true;
      edtM5e.Enabled := False;
      gbM9.Visible := True;
      rgrpM9.Visible := True;

      gbE3.Visible := True;
      rgrpE3.Visible := True;
      gbE3S.Visible := True;
      edtE3e.Visible := true;
      edtE3e.Enabled := False;
      gbE6y.Visible := True;
      gbE6m.Visible := True;
      edtE6y.Visible := True;
      edtE6m.Visible := True;
      gbE8.Visible := True;
      rgrpE8.Visible := True;
      gbE10.Visible := True;
      rgrpE10.Visible := True;
      edtE19.Visible := true;
      gbE19.Visible := true;
      gbE20.Visible := True;
      rgrpE20.Visible := true;
      gbE22.Visible := True;
      rgrpE22.Visible := True;
      edtD1y.Visible := true;
      edtD2y.Visible := true;
      edtD3y.Visible := true;
      edtD4y.Visible := true;
      edtD5y.Visible := true;
      edtD6y.Visible := true;
      edtD7y.Visible := true;
      edtD8y.Visible := true;
      edtD9y.Visible := true;
      edtD10y.Visible := true;
      edtD11y.Visible := true;
      edtD12y.Visible := true;
      edtD13y.Visible := True;
      gbD14.Visible := True;
      rgrpD14.Visible := true;
      gbD15.Visible := true;
      edtD15.Visible := True;
      gbD16.Visible := true;
      edtD16.Visible := True;
      gbD17.Visible := true;
      edtD17.Visible := True;
      gbD18.Visible := true;
      edtD18.Visible := True;
      gbD21.Visible := true;
      edtD21.Visible := True;
      gbD22.Visible := true;
      edtD22.Visible := True;
      gbD32.Visible := True;
      rgrpD32.Visible := true;
      rgrpD33.Visible := true;

      gbL1.Visible := True;
      rgrpL1.Visible := True;
      edtL22.Visible := true;
      gbL22.Visible := true;
      gbL23.Visible := True;
      rgrpL23.Visible := true;
      gbL30.Visible := True;
      rgrpL30.Visible := true;

      pnlFamily.Visible := True;
      lbH1a.Visible := True;
      lbH1d.Visible := True;
      lbH1p.Visible := True;
      lbH2a.Visible := True;
      lbH2d.Visible := True;
      lbH2p.Visible := True;
      lbH3a.Visible := True;
      lbH3d.Visible := True;
      lbH3p.Visible := True;
      lbH4a.Visible := True;
      lbH4d.Visible := True;
      lbH4p.Visible := True;
      lbH5a.Visible := True;
      lbH5d.Visible := True;
      lbH5p.Visible := True;
      lbH6a.Visible := True;
      lbH6d.Visible := True;
      lbH6p.Visible := True;
      lbH7a.Visible := True;
      lbH7d.Visible := True;
      lbH7p.Visible := True;
      lbH8a.Visible := True;
      lbH8d.Visible := True;
      lbH8p.Visible := True;
      lbH9a.Visible := True;
      lbH9d.Visible := True;
      lbH9p.Visible := True;
      lbH10a.Visible := True;
      lbH10d.Visible := True;
      lbH10p.Visible := True;
      lbH11a.Visible := True;
      lbH11d.Visible := True;
      lbH11p.Visible := True;
      lbH12a.Visible := True;
      lbH12d.Visible := True;
      lbH12p.Visible := True;

      gbH1a.Visible := True;
      gbH1d.Visible := True;
      gbH1p.Visible := True;
      gbH2a.Visible := True;
      gbH2d.Visible := True;
      gbH2p.Visible := True;
      gbH3a.Visible := True;
      gbH3d.Visible := True;
      gbH3p.Visible := True;
      gbH4a.Visible := True;
      gbH4d.Visible := True;
      gbH4p.Visible := True;
      gbH5a.Visible := True;
      gbH5d.Visible := True;
      gbH5p.Visible := True;
      gbH6a.Visible := True;
      gbH6d.Visible := True;
      gbH6p.Visible := True;
      gbH7a.Visible := True;
      gbH7d.Visible := True;
      gbH7p.Visible := True;
      gbH8a.Visible := True;
      gbH8d.Visible := True;
      gbH8p.Visible := True;
      gbH9a.Visible := True;
      gbH9d.Visible := True;
      gbH9p.Visible := True;
      gbH10a.Visible := True;
      gbH10d.Visible := True;
      gbH10p.Visible := True;
      gbH11a.Visible := True;
      gbH11d.Visible := True;
      gbH11p.Visible := True;
      gbH12a.Visible := True;
      gbH12d.Visible := True;
      gbH12p.Visible := True;

      gbF2y.Visible := True;
      gbF2m.Visible := True;
      edtF2y.Visible := True;
      edtF2m.Visible := True;
      gbF5m.Visible := True;
      edtF5y.Visible := True;
      edtF5m.Visible := True;
      gbF5y.Visible := True;
      gbF9.Visible := True;
      rgrpF9.Visible := true;
      gbF10.Visible := True;
      rgrpF10.Visible := true;
      edtF11.Visible := true;
      gbF11.Visible := True;
      gbF12.Visible := true;
      lbF12.Visible := true;
      gbF13.Visible := true;
      lbF13.Visible := true;
      gbF14.Visible := true;
      lbF14.Visible := true;
      gbF15.Visible := true;
      lbF15.Visible := true;
      gbF16.Visible := true;
      lbF16.Visible := true;
      gbF17.Visible := true;
      lbF17.Visible := true;
      gbF27.Visible := True;
      rgrpF27.Visible := true;
      gbF27l.Visible := True;
      rgrpF27l.Visible := true;
      gbF18l.Visible := True;
      rgrpF18l.Visible := true;
      gbF19l.Visible := True;
      rgrpF19l.Visible := true;
      gbF20l.Visible := True;
      rgrpF20l.Visible := true;
      gbF21l.Visible := True;
      rgrpF21l.Visible := true;
      gbF22l.Visible := True;
      rgrpF22l.Visible := true;
      gbF23l.Visible := True;
      rgrpF23l.Visible := true;
      gbF24l.Visible := True;
      rgrpF24l.Visible := true;
      gbF25l.Visible := True;
      rgrpF25l.Visible := true;
      gbF26l.Visible := True;
      rgrpF26l.Visible := true;
      gbF33.Visible := True;
      rgrpF33.Visible := true;
      gbF35.Visible := True;
      rgrpF35.Visible := true;
      gbF36.Visible := True;
      rgrpF36.Visible := true;

      gbP14.Visible := True;
      rgrpP14.Visible := true;
      gbP15.Visible := True;
      rgrpP15.Visible := true;
      gbP16.Visible := True;
      rgrpP16.Visible := true;
      gbP17.Visible := True;
      rgrpP17.Visible := true;
      gbP18.Visible := True;
      rgrpP18.Visible := true;
      gbP19.Visible := True;
      rgrpP19.Visible := true;
      gbP20.Visible := True;
      rgrpP20.Visible := true;
      gbP3l.Visible := True;
      rgrpP3l.visible := true;
      gbP4l.Visible := True;
      rgrpP4l.visible := true;
      gbP5l.Visible := True;
      rgrpP5l.visible := true;
      gbP6l.Visible := True;
      rgrpP6l.visible := true;
      gbP7l.Visible := True;
      rgrpP7l.visible := true;
      gbP8l.Visible := True;
      rgrpP8l.visible := true;
      gbP9l.Visible := True;
      rgrpP9l.visible := true;
      gbP10l.Visible := True;
      rgrpP10l.visible := true;

      // Set these captions
      gbM1.Caption := '*M1. How many times in your life have you been hospitalized for ' +
        'medical problems?, edit box, press tab to enter';
      gbE1y.Caption := '*E1. Years education completed?, edit box, press tab to enter';
      gbE2.Caption := '*E2. Months of training or technical education completed?, edit box, press tab to enter';
      gbE7.Caption := '*E7. Usual or last occupation?, edit box, press tab to enter';
      gbD17.Caption :=
        'D17. How many times have you had alcohol dts?, edit box, press tab to enter';
      gbD18.Caption :=
        'D18. How many times have you overdosed on drugs?, edit box, press tab to enter';
      gbD19.Caption :=
        '*D19. How many times in your life have you been treated for alcohol abuse?, edit box, press tab to enter';
      gbD20.Caption :=
        '*D20. How many times in your life have you been treated for drug abuse?, edit box, press tab to enter';
      gbD21.Caption :=
        'D21. How many of these were for alcohol detox only?, edit box, press tab to enter';
      gbD22.Caption :=
        'D22. How many of these were for drug detox only?, edit box, press tab to enter';
      gbD23.Caption :=
        '*D23. How much would you say you spent in dollars during the past thirty days for alcohol?, edit box, press tab to enter';
      gbD24.Caption :=
        '*D24. How much would you say you spent in dollars during the past thirty days for drugs?, edit box, press tab to enter';
      gbL17.Caption := '*L17. How many of these charges resulted in convictions?, edit box, press tab to enter';
      gbF4.Caption := '*F4. Usual living arrangements in past three years, list box, press tab to enter';
      gbP1a.Caption := '*P1. How many times have you been treated for any psychological ' +
        'or emotional problems?, edit box, press tab to enter';

      sbGeneral1.VertScrollBar.Range := 1600;
      sbMedical1.VertScrollBar.Range := 1400;
      sbEmploy1.VertScrollBar.Range := 2800;
      sbDrug1.VertScrollBar.Range := 4500;
      sbLegal1.VertScrollBar.Range := 3300;
      sbSocial1.VertScrollBar.Range := 6200;
      sbPsych1.VertScrollBar.Range := 3400;

      //PositionASIItems('F');
      Screen.Cursor := crDefault;
    end;
end;

procedure TASI_Build_508.Setup_ASILite_508;
begin
  with frmMHA_ASI_Entry_508 do
    begin
      Screen.Cursor := crHourglass;
      TabFamily.TabVisible := false;
      GroupBox8.Visible := true;
      GroupBox9.Visible := true;
      edtG14y.Visible := true;
      edtG14m.Visible := True;
      GroupBox10.Visible := False;
      rgrpG15.Visible := False;
      GroupBox11.Visible := True;
      rgrpG17.Visible := true;
      GroupBox12.Visible := True;
      rgrpG18.Visible := True;
      edtG19e.Enabled := false;

      gbM2.Visible := False;
      gbM2a.Visible := False;
      edtM2y.Visible := False;
      edtM2m.Visible := False;
      gbM3.Visible := True;
      rgrpM3.Visible := true;
      gbM3S.Visible := True;
      edtM3e.Visible := True;
      edtM3e.Enabled := false;
      edtM5e.Visible := true;
      gbM5S.Visible := True;
      edtM5e.Enabled := False;
      gbM9.Visible := False;
      rgrpM9.Visible := False;

      gbE3.Visible := False;
      rgrpE3.Visible := False;
      gbE3S.Visible := False;
      edtE3e.Visible := false;
      gbE6y.Visible := true;
      gbE6m.Visible := true;
      edtE6y.Visible := true;
      edtE6m.Visible := true;
      gbE8.Visible := False;
      rgrpE8.Visible := False;
      gbE10.Visible := True;
      rgrpE10.Visible := true;
      edtE19.Visible := false;
      gbE19.Visible := False;
      gbE20.Visible := False;
      rgrpE20.Visible := false;
      gbE22.Visible := False;
      rgrpE22.Visible := false;

      edtD1y.Visible := true;
      edtD2y.Visible := true;
      edtD3y.Visible := true;
      edtD4y.Visible := true;
      edtD5y.Visible := true;
      edtD6y.Visible := true;
      edtD7y.Visible := true;
      edtD8y.Visible := true;
      edtD9y.Visible := true;
      edtD10y.Visible := true;
      edtD11y.Visible := true;
      edtD12y.Visible := true;
      edtD13y.Visible := True;
      gbD14.Visible := False;
      rgrpD14.Visible := false;
      gbD15.Visible := False;
      edtD15.Visible := False;
      gbD16.Visible := False;
      edtD16.Visible := False;
      gbD17.Visible := False;
      edtD17.Visible := False;
      gbD18.Visible := False;
      edtD18.Visible := False;
      gbD21.Visible := false;
      edtD21.Visible := False;
      gbD22.Visible := false;
      edtD22.Visible := false;
      gbD32.Visible := False;
      rgrpD32.Visible := false;
      gbD33.Visible := False;
      rgrpD33.Visible := false;

      gbL1.Visible := True;
      rgrpL1.Visible := true;
      edtL22.Visible := false;
      gbL22.Visible := False;
      gbL23.Visible := False;
      rgrpL23.Visible := false;
      gbL30.Visible := False;
      rgrpL30.Visible := false;

      pnlFamily.Visible := False;
      lbH1a.Visible := False;
      lbH1d.Visible := False;
      lbH1p.Visible := False;
      lbH2a.Visible := False;
      lbH2d.Visible := False;
      lbH2p.Visible := False;
      lbH3a.Visible := False;
      lbH3d.Visible := False;
      lbH3p.Visible := False;
      lbH4a.Visible := False;
      lbH4d.Visible := False;
      lbH4p.Visible := False;
      lbH5a.Visible := False;
      lbH5d.Visible := False;
      lbH5p.Visible := False;
      lbH6a.Visible := False;
      lbH6d.Visible := False;
      lbH6p.Visible := False;
      lbH7a.Visible := False;
      lbH7d.Visible := False;
      lbH7p.Visible := False;
      lbH8a.Visible := False;
      lbH8d.Visible := False;
      lbH8p.Visible := False;
      lbH9a.Visible := False;
      lbH9d.Visible := False;
      lbH9p.Visible := False;
      lbH10a.Visible := False;
      lbH10d.Visible := False;
      lbH10p.Visible := False;
      lbH11a.Visible := False;
      lbH11d.Visible := False;
      lbH11p.Visible := False;
      lbH12a.Visible := False;
      lbH12d.Visible := False;
      lbH12p.Visible := False;

      gbH1a.Visible := False;
      gbH1d.Visible := False;
      gbH1p.Visible := False;
      gbH2a.Visible := False;
      gbH2d.Visible := False;
      gbH2p.Visible := False;
      gbH3a.Visible := False;
      gbH3d.Visible := False;
      gbH3p.Visible := False;
      gbH4a.Visible := False;
      gbH4d.Visible := False;
      gbH4p.Visible := False;
      gbH5a.Visible := False;
      gbH5d.Visible := False;
      gbH5p.Visible := False;
      gbH6a.Visible := False;
      gbH6d.Visible := False;
      gbH6p.Visible := False;
      gbH7a.Visible := False;
      gbH7d.Visible := False;
      gbH7p.Visible := False;
      gbH8a.Visible := False;
      gbH8d.Visible := False;
      gbH8p.Visible := False;
      gbH9a.Visible := False;
      gbH9d.Visible := False;
      gbH9p.Visible := False;
      gbH10a.Visible := False;
      gbH10d.Visible := False;
      gbH10p.Visible := False;
      gbH11a.Visible := False;
      gbH11d.Visible := False;
      gbH11p.Visible := False;
      gbH12a.Visible := False;
      gbH12d.Visible := False;
      gbH12p.Visible := False;

      gbF2y.Visible := False;
      gbF2m.Visible := False;
      edtF2y.Visible := False;
      edtF2m.Visible := False;
      gbF5m.Visible := False;
      edtF5y.Visible := False;
      edtF5m.Visible := False;
      gbF5y.Visible := False;
      gbF9.Visible := False;
      rgrpF9.Visible := false;
      gbF10.Visible := False;
      rgrpF10.Visible := false;
      edtF11.Visible := false;
      gbF11.Visible := False;
      gbF12.Visible := False;
      lbF12.Visible := False;
      gbF13.Visible := False;
      lbF13.Visible := False;
      gbF14.Visible := False;
      lbF14.Visible := False;
      gbF15.Visible := False;
      lbF15.Visible := False;
      gbF16.Visible := False;
      lbF16.Visible := False;
      gbF17.Visible := False;
      lbF17.Visible := False;
      gbF27.Visible := False;
      rgrpF27.Visible := false;
      gbF27l.Visible := False;
      rgrpF27l.Visible := false;
      gbF18l.Visible := True;
      rgrpF18l.Visible := true;
      gbF19l.Visible := True;
      rgrpF19l.Visible := true;
      gbF20l.Visible := True;
      rgrpF20l.Visible := true;
      gbF21l.Visible := True;
      rgrpF21l.Visible := true;
      gbF22l.Visible := True;
      rgrpF22l.Visible := true;
      gbF23l.Visible := True;
      rgrpF23l.Visible := true;
      gbF24l.Visible := True;
      rgrpF24l.Visible := true;
      gbF25l.Visible := True;
      rgrpF25l.Visible := true;
      gbF26l.Visible := True;
      rgrpF26l.Visible := true;
      gbF33.Visible := False;
      rgrpF33.Visible := false;
      gbF35.Visible := False;
      rgrpF35.Visible := false;
      gbF36.Visible := False;
      rgrpF36.Visible := false;

      gbP14.Visible := False;
      rgrpP14.Visible := false;
      gbP15.Visible := False;
      rgrpP15.Visible := false;
      gbP16.Visible := False;
      rgrpP16.Visible := false;
      gbP17.Visible := False;
      rgrpP17.Visible := false;
      gbP18.Visible := False;
      rgrpP18.Visible := false;
      gbP19.Visible := False;
      rgrpP19.Visible := false;
      gbP20.Visible := False;
      rgrpP20.Visible := False;
      gbP3l.Visible := True;
      rgrpP3l.visible := true;
      gbP4l.Visible := True;
      rgrpP4l.visible := true;
      gbP5l.Visible := True;
      rgrpP5l.visible := true;
      gbP6l.Visible := True;
      rgrpP6l.visible := true;
      gbP7l.Visible := True;
      rgrpP7l.visible := true;
      gbP8l.Visible := True;
      rgrpP8l.visible := true;
      gbP9l.Visible := True;
      rgrpP9l.visible := true;
      gbP10l.Visible := True;
      rgrpP10l.visible := true;

      // Set these captions
      gbM1.Caption := '*M1. How many times in your life have you been hospitalized for ' +
        'medical problems?, edit box, press tab to enter';
      gbE1y.Caption := '*E1. Years education completed?, edit box, press tab to enter';
      gbE2.Caption := '*E2. Months training or technical education completed?, edit box, press tab to enter';
      gbE7.Caption := '*E7. Usual or last occupation, edit box, press tab to enter';
      gbD17.Caption :=
        'D17. How many times have you had alcohol dts?, edit box, press tab to enter';
      gbD18.Caption :=
        'D18. How many times have you overdosed on drugs?, edit box, press tab to enter';
      gbD19.Caption :=
        '*D19. How many times in your life have you been treated for alcohol abuse?, edit box, press tab to enter';
      gbD20.Caption :=
        '*D20. How many times in your life have you been treated for drug abuse?, edit box, press tab to enter';
      gbD21.Caption :=
        'D21. How many of these were for alcohol detox only?, edit box, press tab to enter';
      gbD22.Caption :=
        'D22. How many of these were for drug detox only?, edit box, press tab to enter';
      gbD23.Caption :=
        '*D23. How much would you say you spent in dollars during the past thirty days for alcohol?, edit box, press tab to enter';
      gbD24.Caption :=
        '*D24. How much would you say you spent in dollars during the past thirty days for drugs?, edit box, press tab to enter';
      gbL17.Caption :=
        '*L17. How many of these charges resulted in convictions?, edit box, press tab to enter';
      gbF4.Caption := '*F4. Usual living arrangements in past three years, list box, press tab to enter';
      gbP1a.Caption := '*P1. How many times have you been treated for any psychological ' +
        'or emotional problems?, edit box, press tab to enter';

      sbGeneral1.VertScrollBar.Range := 1600;
      sbMedical1.VertScrollBar.Range := 1400;
      sbEmploy1.VertScrollBar.Range := 2800;
      sbDrug1.VertScrollBar.Range := 4500;
      sbLegal1.VertScrollBar.Range := 3300;
      sbSocial1.VertScrollBar.Range := 6200;
      sbPsych1.VertScrollBar.Range := 3400;

      //PositionASIItems('L');
      Screen.Cursor := crDefault;
    end;
end;

procedure TASI_Build_508.Setup_ASIFollowup_508; // Followup
begin
  with frmMHA_ASI_Entry_508 do
    begin
      Screen.Cursor := crHourglass;
      TabFamily.TabVisible := false;
      // Make sure these questions are Visible
      GroupBox8.Visible := False;
      GroupBox9.Visible := False;
      edtG14y.Visible := False;
      edtG14m.Visible := False;
      GroupBox10.Visible := False;
      rgrpG15.Visible := False;
      GroupBox11.Visible := False;
      rgrpG17.Visible := False;
      GroupBox12.Visible := False;
      rgrpG18.Visible := False;
      edtG19e.Enabled := false;

      gbM2.Visible := False;
      gbM2a.Visible := False;
      edtM2y.Visible := False;
      edtM2m.Visible := False;
      gbM3.Visible := False;
      rgrpM3.Visible := False;
      edtM3e.Visible := false;
      gbM3S.Visible := False;
      edtM5e.Visible := False;
      gbM5S.Visible := False;
      gbM9.Visible := True;
      rgrpM9.Visible := true;

      gbE3.Visible := False;
      rgrpE3.Visible := False;
      gbE3S.Visible := False;
      edtE3e.Visible := false;
      gbE6y.Visible := False;
      gbE6m.Visible := False;
      edtE6y.Visible := False;
      edtE6m.Visible := False;
      gbE8.Visible := True;
      rgrpE8.Visible := true;
      gbE10.Visible := False;
      rgrpE10.Visible := False;
      edtE19.Visible := true;
      gbE19.Visible := true;
      gbE20.Visible := true;
      rgrpE20.Visible := true;
      gbE22.Visible := True;
      rgrpE22.Visible := true;

      gbD1y.Visible := false;
      edtD1y.Visible := false;
      gbD2y.Visible := False;
      edtD2y.Visible := false;
      gbD3y.Visible := False;
      edtD3y.Visible := false;
      gbD4y.Visible := False;
      edtD4y.Visible := false;
      gbD5y.Visible := False;
      edtD5y.Visible := false;
      gbD6y.Visible := False;
      edtD6y.Visible := false;
      gbD7y.Visible := False;
      edtD7y.Visible := false;
      gbD8y.Visible := False;
      edtD8y.Visible := false;
      gbD9y.Visible := False;
      edtD9y.Visible := false;
      gbD10y.Visible := False;
      edtD10y.Visible := false;
      gbD11y.Visible := False;
      edtD11y.Visible := false;
      gbD12y.Visible := False;
      edtD12y.Visible := false;
      gbD13y.Visible := False;
      edtD13y.Visible := false;
      gbD14.Visible := True;
      rgrpD14.Visible := true;
      gbD15.Visible := false;
      edtD15.Visible := False;
      gbD16.Visible := false;
      edtD16.Visible := False;
      gbD17.Visible := true;
      edtD17.Visible := True;
      gbD18.Visible := true;
      edtD18.Visible := True;
      gbD21.Visible := true;
      edtD21.Visible := True;
      gbD22.Visible := true;
      edtD22.Visible := True;
      gbD32.Visible := True;
      rgrpD32.Visible := true;
      gbD33.Visible := True;
      rgrpD33.Visible := true;

      gbL1.Visible := False;
      rgrpL1.Visible := False;
      edtL22.Visible := true;
      gbL22.Visible := true;
      gbL23.Visible := True;
      rgrpL23.Visible := true;
      gbL30.Visible := True;
      rgrpL30.Visible := true;

      pnlFamily.Visible := False;
      lbH1a.Visible := False;
      lbH1d.Visible := False;
      lbH1p.Visible := False;
      lbH2a.Visible := False;
      lbH2d.Visible := False;
      lbH2p.Visible := False;
      lbH3a.Visible := False;
      lbH3d.Visible := False;
      lbH3p.Visible := False;
      lbH4a.Visible := False;
      lbH4d.Visible := False;
      lbH4p.Visible := False;
      lbH5a.Visible := False;
      lbH5d.Visible := False;
      lbH5p.Visible := False;
      lbH6a.Visible := False;
      lbH6d.Visible := False;
      lbH6p.Visible := False;
      lbH7a.Visible := False;
      lbH7d.Visible := False;
      lbH7p.Visible := False;
      lbH8a.Visible := False;
      lbH8d.Visible := False;
      lbH8p.Visible := False;
      lbH9a.Visible := False;
      lbH9d.Visible := False;
      lbH9p.Visible := False;
      lbH10a.Visible := False;
      lbH10d.Visible := False;
      lbH10p.Visible := False;
      lbH11a.Visible := False;
      lbH11d.Visible := False;
      lbH11p.Visible := False;
      lbH12a.Visible := False;
      lbH12d.Visible := False;
      lbH12p.Visible := False;

      gbH1a.Visible := False;
      gbH1d.Visible := False;
      gbH1p.Visible := False;
      gbH2a.Visible := False;
      gbH2d.Visible := False;
      gbH2p.Visible := False;
      gbH3a.Visible := False;
      gbH3d.Visible := False;
      gbH3p.Visible := False;
      gbH4a.Visible := False;
      gbH4d.Visible := False;
      gbH4p.Visible := False;
      gbH5a.Visible := False;
      gbH5d.Visible := False;
      gbH5p.Visible := False;
      gbH6a.Visible := False;
      gbH6d.Visible := False;
      gbH6p.Visible := False;
      gbH7a.Visible := False;
      gbH7d.Visible := False;
      gbH7p.Visible := False;
      gbH8a.Visible := False;
      gbH8d.Visible := False;
      gbH8p.Visible := False;
      gbH9a.Visible := False;
      gbH9d.Visible := False;
      gbH9p.Visible := False;
      gbH10a.Visible := False;
      gbH10d.Visible := False;
      gbH10p.Visible := False;
      gbH11a.Visible := False;
      gbH11d.Visible := False;
      gbH11p.Visible := False;
      gbH12a.Visible := False;
      gbH12d.Visible := False;
      gbH12p.Visible := False;

      gbF2y.Visible := False;
      gbF2m.Visible := False;
      edtF2y.Visible := False;
      edtF2m.Visible := False;
      gbF5m.Visible := False;
      edtF5y.Visible := False;
      edtF5m.Visible := False;
      gbF5y.Visible := False;
      gbF9.Visible := True;
      rgrpF9.Visible := true;
      gbF10.Visible := True;
      rgrpF10.Visible := true;
      edtF11.Visible := true;
      gbF11.Visible := True;
      gbF12.Visible := False;
      lbF12.Visible := False;
      gbF13.Visible := False;
      lbF13.Visible := False;
      gbF14.Visible := False;
      lbF14.Visible := False;
      gbF15.Visible := False;
      lbF15.Visible := False;
      gbF16.Visible := False;
      lbF16.Visible := False;
      gbF17.Visible := False;
      lbF17.Visible := False;
      gbF27.Visible := True;
      rgrpF27.Visible := true;
      gbF27l.Visible := True;
      rgrpF27l.Visible := true;
      gbF18l.Visible := False;
      rgrpF18l.Visible := false;
      gbF19l.Visible := False;
      rgrpF19l.Visible := false;
      gbF20l.Visible := False;
      rgrpF20l.Visible := false;
      gbF21l.Visible := False;
      rgrpF21l.Visible := false;
      gbF22l.Visible := False;
      rgrpF22l.Visible := false;
      gbF23l.Visible := False;
      rgrpF23l.Visible := false;
      gbF24l.Visible := False;
      rgrpF24l.Visible := false;
      gbF25l.Visible := False;
      rgrpF25l.Visible := false;
      gbF26l.Visible := False;
      rgrpF26l.Visible := false;
      gbF33.Visible := True;
      rgrpF33.Visible := true;
      gbF35.Visible := True;
      rgrpF35.Visible := true;
      gbF36.Visible := True;
      rgrpF36.Visible := true;

      gbP14.Visible := True;
      rgrpP14.Visible := true;
      gbP15.Visible := True;
      rgrpP15.Visible := true;
      gbP16.Visible := True;
      rgrpP16.Visible := true;
      gbP17.Visible := True;
      rgrpP17.Visible := true;
      gbP18.Visible := True;
      rgrpP18.Visible := true;
      gbP19.Visible := True;
      rgrpP19.Visible := true;
      gbP20.Visible := True;
      rgrpP20.Visible := true;
      gbP3l.Visible := False;
      rgrpP3l.visible := false;
      gbP4l.Visible := False;
      rgrpP4l.visible := false;
      gbP5l.Visible := False;
      rgrpP5l.visible := false;
      gbP6l.Visible := False;
      rgrpP6l.visible := false;
      gbP7l.Visible := False;
      rgrpP7l.visible := false;
      gbP8l.Visible := False;
      rgrpP8l.visible := false;
      gbP9l.Visible := False;
      rgrpP9l.visible := false;
      gbP10l.Visible := False;
      rgrpP10l.visible := false;

      // Set these captions:  LastASIDate is a property of ASI_List
      //NB: I don't know how often this will occur in real life, but if the Followup
      //begin edited isn't the most recent ASI administered, LastASIDate will be a date
      //prior to the putative date of the Followup interview.
      with ASI_OPS do
        begin
          gbM1.Caption := '*M1. How many times since ' + LastASIDate + ' have you been ' +
            'hospitalized for medical problems?, edit box, press tab to enter';
          gbE1y.Caption := '*E1. Years education completed since ' + LastASIDate +
            ' , edit box, press tab to enter';
          gbE2.Caption := '*E2. Months of training or technical education completed since ' + LastASIDate +
            ' , edit box, press tab to enter';

          gbE7.Caption := '*E7. Occupation since ' + LastASIDate + ' , edit box, press tab to enter';
          gbD17.Caption :=
            'D17. How many times since ' + LastASIDate + ' have you had alcohol dts?, edit box, press tab to enter';
          gbD18.Caption :=
            'D18. How many times since ' + LastASIDate + ' have you overdosed on drugs?, edit box, press tab to enter';
          gbD19.Caption :=
            '*D19. How many times since ' + LastASIDate + ' have you been treated for alcohol abuse?, edit box, press tab to enter';
          gbD20.Caption :=
            '*D20. How many times since ' + LastASIDate + ' have you been treated for drug abuse?, edit box, press tab to enter';
          gbD21.Caption :=
            'D21. Since ' + LastASIDate + ' how many of these were for alcohol detox only?, edit box, press tab to enter';
          gbD22.Caption :=
            'D22. Since ' + LastASIDate + ' how many of these were for drug detox only?, edit box, press tab to enter';
          gbD23.Caption :=
            '*D23. How much would you say you spent in dollars during the past thirty days for alcohol?, edit box, press tab to enter';
          gbD24.Caption :=
            '*D24. How much would you say you spent in dollars during the past thirty days for drugs?';
          gbL17.Caption := '*L17. Since ' + LastASIDate + ' how many of these charges resulted in convictions?, edit box, press tab to enter';
          gbF4.Caption := '*F4. Usual living arrangements since ' + LastASIDate +
            ' , list box, press tab to enter';
          gbP1a.Caption := '*P1. How many times since ' + LastASIDate + ' have you been treated for any psychological ' +
            'or emotional problems?, edit box, press tab to enter';
        end;

      sbGeneral1.VertScrollBar.Range := 1600;
      sbMedical1.VertScrollBar.Range := 1400;
      sbEmploy1.VertScrollBar.Range := 2800;
      sbDrug1.VertScrollBar.Range := 4500;
      sbLegal1.VertScrollBar.Range := 3300;
      sbSocial1.VertScrollBar.Range := 6200;
      sbPsych1.VertScrollBar.Range := 3400;

      //PositionASIItems('U');
      Screen.Cursor := crDefault;
    end;
end;


procedure TASI_Build_508.LoadCommentFields(WhichIEN: string);

  function CleanText(TextIn: string): string;
    // M-side strings have ^ as hard carriage return. Not needed here.
  var
    TempStr                   : string;
  begin
    Result := '';
    if Trim(TextIn) = BAD_DATA then
      Exit;
    if Length(TextIn) = 0 then
      Exit;
    TempStr := TextIn;
    // Mark the double carriage returns. This keeps the paragraphs intact.
    TempStr := StringReplace(TempStr, '^^', #240, [rfReplaceAll, rfIgnoreCase]);
    // Remove all carriage returns
    TempStr := StringReplace(TempStr, '^', '', [rfReplaceAll, rfIgnoreCase]);
    // Replace the paragraph markers
    TempStr := StringReplace(TempStr, #240, #13 + #13, [rfReplaceAll, rfIgnoreCase]);
    Result := TempStr;
  end;

begin
  with frmMHA_ASI_Entry_508.memGeneral.Lines do
    begin
      Clear;
      Add(AnswerReadComment('604', '.6', WhichIEN, DataModule1.RPCBroker1));
      Text := CleanText(Text);
    end;
  with frmMHA_ASI_Entry_508.MemMedical.Lines do
    begin
      Clear;
      Add(AnswerReadComment('604', '8.5', WhichIEN, DataModule1.RPCBroker1));
      Text := CleanText(Text);
    end;
  with frmMHA_ASI_Entry_508.MemEmploy.Lines do
    begin
      Clear;
      Add(AnswerReadComment('604', '9.5', WhichIEN, DataModule1.RPCBroker1));
      Text := CleanText(Text);
    end;
  with frmMHA_ASI_Entry_508.MemDrug.Lines do
    begin
      Clear;
      Add(AnswerReadComment('604', '10.5', WhichIEN, DataModule1.RPCBroker1));
      Text := CleanText(Text);
    end;
  with frmMHA_ASI_Entry_508.MemLegal.Lines do
    begin
      Clear;
      Add(AnswerReadComment('604', '14.5', WhichIEN, DataModule1.RPCBroker1));
      Text := CleanText(Text);
    end;
  with frmMHA_ASI_Entry_508.MemFamily.Lines do
    begin
      Clear;
      Add(AnswerReadComment('604', '16.5', WhichIEN, DataModule1.RPCBroker1));
      Text := CleanText(Text);
    end;
  with frmMHA_ASI_Entry_508.MemSocial.Lines do
    begin
      Clear;
      Add(AnswerReadComment('604', '18.5', WhichIEN, DataModule1.RPCBroker1));
      Text := CleanText(Text);
    end;
  with frmMHA_ASI_Entry_508.MemPsych.Lines do
    begin
      Clear;
      Add(AnswerReadComment('604', '19.5', WhichIEN, DataModule1.RPCBroker1));
      Text := CleanText(Text);
    end;
  with frmMHA_ASI_Entry_508.MemSpirit.Lines do
    begin
      Clear;
      Add(AnswerReadComment('604', '188', WhichIEN, DataModule1.RPCBroker1));
      Text := CleanText(Text);
    end;
  with frmMHA_ASI_Entry_508.MemLeisure.Lines do
    begin
      Clear;
      Add(AnswerReadComment('604', '187', WhichIEN, DataModule1.RPCBroker1));
      Text := CleanText(Text);
    end;
end;



(*
procedure TASI_Build_508.PositionASIItems(WhichForm: Char);
var
  i,
    T, // Top
    L, // Label height
    S // Space between items;
                              : integer;
begin
  with frmMHA_ASI_Entry_508 do
    begin
      T := 5;
      S := 5;
      L := 3 * GUI_Control.TextHeight('Yy', gvFont) div 2;

      for i := 0 to ComponentCount - 1 do
        begin
          if Components[i] is TLabel then
            begin
              TLabel(Components[i]).AutoSize := true;
              TLabel(Components[i]).Left := 5;
              TLabel(Components[i]).Layout := tlBottom;
              TLabel(Components[i]).ShowHint := False;
              TLabel(Components[i]).ParentFont := True;
              TLabel(Components[i]).ShowAccelChar := False;
              TLabel(Components[i]).Alignment := taLeftJustify;
              TLabel(Components[i]).WordWrap := False;
            end;
          if Components[i] is TRzEdit then
            begin
              TRzEdit(Components[i]).Left := 5;
              TRzEdit(Components[i]).OnExit := RzEditOnExit;
            end;
          if Components[i] is TEdit then
            begin
              TEdit(Components[i]).Left := 5;
              TEdit(Components[i]).OnExit := EditOnExit;
            end;
          if Components[i] is TListBox then
            begin
              TListBox(Components[i]).Left := 5;
              TListBox(Components[i]).Height :=
                TListBox(Components[i]).Items.Count * L;
              TListBox(Components[i]).Width :=
                sbGeneral.Width div 2;
              TListBox(Components[i]).OnExit := ListBoxOnExit;
            end;
          if Components[i] is TRzComboBox then
            begin
              TRzComboBox(Components[i]).Left := 5;
              TRzComboBox(Components[i]).Width :=
                sbGeneral.Width div 2;
              TRzComboBox(Components[i]).OnExit := RzComboBoxOnExit;
            end;
          if Components[i] is TMHA_SpnEdt then
            begin
              TMHA_SpnEdt(Components[i]).Left := 5;
              TMHA_SpnEdt(Components[i]).Value := 0;
              TMHA_SpnEdt(Components[i]).CheckRange := true;
              TMHA_SpnEdt(Components[i]).AllowKeyEdit := True;
              TMHA_SpnEdt(Components[i]).OnExit := SpinEditOnExit;
            end;
          if Components[i] is TORComboBox then
            begin
              TORComboBox(Components[i]).Left := 5;
              TORComboBox(Components[i]).Width :=
                sbGeneral.Width div 2;
              TORComboBox(Components[i]).OnExit := ORComboBoxOnExit;
            end;
          if Components[i] is TDateTimePicker then
            begin
              TDateTimePicker(Components[i]).Left := 5;
              TDateTimePicker(Components[i]).OnExit := DateOnExit;
            end;
        end; // for i

      // Reset the Scroll bars
      sbGeneral.VertScrollBar.Position := 0;
      sbMedical.VertScrollBar.Position := 0;
      sbEmploy.VertScrollBar.Position := 0;
      sbLegal.VertScrollBar.Position := 0;
      sbDrug.VertScrollBar.Position := 0;
      sbFamily.VertScrollBar.Position := 0;
      sbSocial.VertScrollBar.Position := 0;
      sbPsych.VertScrollBar.Position := 0;


      // General Information Items --------------------------------------
      lblG3.top := T;

      cbxG3.Left := lblG3.Left + lblG3.Width + 5;
      cbxG3.top := T;
      T := T + cbxG3.Height + S;
      lblG4.top := T;
      T := T + L + S;
      dtpG4.top := T;
      T := T + dtpG4.Height + S;
      lblG5.top := T;
      T := T + L + S;
      dtpG5.Top := T;
      T := T + dtpG5.Height + S;
      lblG8.Top := T;
      T := T + L + S;
      rgrpG8.top := T;
      T := T + rgrpG8.Height + S;
      lblG9.Top := T;
      T := T + L + S;
      rgrpG9.Top := T;
      T := T + rgrpG9.Height + S;
      lblG11Interviewerx.top := T;

      cbxG11Interviewerx.Left := lblG11Interviewerx.Left + lblG11Interviewerx.Width + 5;
      cbxG11Interviewerx.top := T;
      T := T + cbxG11Interviewerx.Height + S;
      lblG11OrderedByx.top := T;

      cbxG11OrderedByx.Left := lblG11OrderedByx.Left + lblG11OrderedByx.Width + 5;
      cbxG11OrderedByx.top := T;
      T := T + cbxG11OrderedByx.Height + S;
      lblG12.Top := T;
      T := T + L + S;
      rgrpG12.top := T;
      T := T + rgrpG12.Height + S;
      if not (WhichForm = 'U') then
        begin
          lblG14y.Top := T;
          edtG14y.Left := lblG14y.Left + lblG14y.Width + 5;
          edtG14y.Top := T;
          T := T + edtG14y.Height + S;
          lblG14m.top := T;
          T := T + L + S;
          edtG14m.top := T;
          T := T + edtG14y.Height + S;
        end;
      if WhichForm = 'F' then
        begin
          lblG15.Top := T;
          T := T + L + S;
          rgrpG15.Top := T;
          T := T + rgrpG15.Height + S;
        end;
      if not (WhichForm = 'U') then
        begin
          lblG17.Top := T;

          rgrpG17.Left := lblG17.Width + lblG17.Left + 5;
          rgrpG17.Top := T;
          T := T + rgrpG17.Height + S;
          lblG18.Top := T;
          T := T + L + S;
          rgrpG18.Top := T;
          T := T + rgrpG18.Height + S;
        end;
      lblG19.Top := T;
      T := T + L + S;
      rgrpG19.Top := T;
      T := T + S + rgrpG19.Height;
      lblG19e.Top := T;
      edtG19e.Left := lblG19e.Left + lblG19e.Width + 5;
      edtG19e.Top := T;
      T := T + L + S;
      lblG20.top := T;

      edtG20.Left := lblG20.Left + lblG20.Width + 5;
      edtG20.Top := T;
      T := T + S * 3 + edtG20.Height;
      GeneralNextPage.top := T + 30;
      sbGeneral.VertScrollBar.Range := T + 30;

      // Medical Status -------------------------------------------------
      T := 5;
      lblM1.Top := T;

      edtM1.Top := T;
      edtM1.Left := lblM1.Left + lblM1.Width + 5;
      T := T + S + edtM1.height;
      if WhichForm = 'F' then
        begin
          lblM2.Top := T;

          edtM2y.Top := T;
          edtM2y.Left := lblM2.Left + lblM2.Width + 5;
          T := T + edtM2y.Height + S;
          lblM2m.Top := T;
          T := T + L + S;
          edtM2m.Top := T;
          T := T + S + edtM2y.height;
        end;
      if not (WhichForm = 'U') then
        begin
          lblM3.Top := T;
          T := T + L + S;
          rgrpM3.top := T;
          T := T + S + rgrpM3.height;
          edtM3e.top := T;  lblM3Spec.top := T;
          T := T + S + edtM3e.height;  edtM3e.left := 40;
        end;
      lblM4.Top := T;
      T := T + L + S;
      rgrpM4.top := T;
      T := T + S + rgrpM4.height;
      lblM5.Top := T;
      T := T + L + S;
      rgrpM5.top := T;
      T := T + S + rgrpM5.height;
      edtM5e.top := T; lblM3Spec.top := T;
      T := T + S + edtM5e.height;  edtM5e.left := 40;
      lblM6.Top := T;

      edtM6.Left := lblM6.Left + lblM6.Width + 5;
      edtM6.Top := T;
      T := T + S + edtM6.height;
      lblM7.Top := T;
      T := T + L + S;
      rgrpM7.top := T;
      T := T + S + rgrpM7.height;
      lblM8.Top := T;
      T := T + L + S;
      rgrpM8.top := T;
      T := T + S + rgrpM8.height;
      if ((WhichForm = 'U') or (WhichForm = 'F')) then
        begin
          lblM9.Top := T;
          T := T + L + S;
          rgrpM9.top := T;
          T := T + S + rgrpM9.height;
        end;
      lblM10.Top := T;
      T := T + L + S;
      rgrpM10.top := T;
      T := T + S + rgrpM10.height;
      lblM11.Top := T;
      T := T + L + S;
      rgrpM11.top := T;
      T := T + S * 3 + rgrpM11.Height;
      MedicalNextPage.top := T + 30;
      sbMedical.VertScrollBar.Range := T + 30;


      // Employment -----------------------------------------------------
      T := 5;
      lblE1.Top := T;

      edtE1y.Top := T;
      edtE1y.Left := lblE1.Left + lblE1.Width + 5;
      T := T + S + edtE1y.Height;
      lblE1m.Top := T;

      edtE1m.Top := T;
      edtE1m.Left := lblE1m.Left + lblE1m.Width + 5;
      T := T + S + edtE1m.height;
      lblE2.Top := T;

      edtE2.Left := lblE2.Left + lblE2.Width + 5;
      edtE2.Top := T;
      //spnE2.Top := T;
      T := T + S + edtE2.height;
      if WhichForm = 'F' then
        begin
          lblE3.Top := T;
          T := T + L + S;
          rgrpE3.top := T;
          T := T + S + rgrpE3.height;
          edtE3e.top := T; lblE3Spec.top := T;
          T := T + S + edtE3e.height;  edtE3e.left := 40;
        end;
      lblE4.Top := T;
      T := T + L + S;
      rgrpE4.top := T;
      T := T + S + rgrpE4.height;
      lblE5.Top := T;
      T := T + L + S;
      rgrpE5.top := T;
      T := T + S + rgrpE5.height;
      if not (WhichForm = 'U') then
        begin
          lblE6.Top := T;

          edtE6y.Left := lblE6.Left + lblE6.Width + 5;
          edtE6y.Top := T;
          T := T + L + S;
          lblE6m.Top := T;
          T := T + L + S;
          edtE6m.Top := T;
          T := T + S + edtE6y.height;
        end;
      lblE7a.Top := T;

      rgrpE7.Left := lblE7a.Left + lblE7a.Width + 5;
      rgrpE7.top := T;
      T := T + S + rgrpE7.height;
      btnHollingshead.top := T;
      T := T + btnHollingshead.height + S;
      lblE7.top := T;
      T := T + L + S;
      edtE7e.top := T;
      T := T + S + edtE7e.height;
      if not (WhichForm = 'L') then
        begin
          lblE8.Top := T;
          T := T + L + S;
          rgrpE8.top := T;
          T := T + S + rgrpE8.height;
        end;
      lblE9.Top := T;
      T := T + L + S;
      rgrpE9.top := T;
      T := T + S + rgrpE9.height;
      if not (WhichForm = 'U') then
        begin
          lblE10.Top := T;
          T := T + L + S;
          rgrpE10.top := T;
          T := T + S + rgrpE10.height;
        end;
      lblE11.Top := T;

      edtE11.Left := lblE11.Left + lblE11.Width + 5;
      edtE11.Top := T;
      T := T + S + edtE11.height;
      lblE12.Top := T;
      T := T + L + S;
      edtE12.Top := T;
      T := T + S + edtE12.height;
      lblE13.Top := T;
      T := T + L + S;
      edtE13.Top := T;
      T := T + S + edtE13.height;
      lblE14.Top := T;
      T := T + L + S;
      edtE14.Top := T;
      T := T + S + edtE14.height;
      lblE15.Top := T;
      T := T + L + S;
      edtE15.Top := T;
      T := T + S + edtE15.height;
      lblE16.Top := T;
      T := T + L + S;
      edtE16.Top := T;
      T := T + S + edtE16.height;
      lblE17.Top := T;
      T := T + L + S;
      edtE17.Top := T;
      T := T + S + edtE17.height;
      lblE18.Top := T;

      edtE18.Left := lblE18.Left + lblE18.Width + 5;
      edtE18.Top := T;

      T := T + S + edtE18.height;
      if not (WhichForm = 'L') then
        begin
          lblE19.Top := T;

          edtE19.Left := lblE19.Left + lblE19.Width + 5;
          edtE19.Top := T;
          T := T + S + edtE19.height;
          lblE20.Top := T;
          T := T + L + S;
          rgrpE20.top := T;
          T := T + S + rgrpE20.height;
        end;
      lblE21.Top := T;
      T := T + L + S;
      rgrpE21.top := T;
      T := T + S + rgrpE21.height;
      if not (WhichForm = 'L') then
        begin
          lblE22.Top := T;
          T := T + L + S;
          rgrpE22.top := T;
          T := T + S + rgrpE22.height;
        end;
      lblE23.Top := T;
      T := T + L + S;
      rgrpE23.top := T;
      T := T + S + rgrpE23.height;
      lblE24.Top := T;
      T := T + L + S;
      rgrpE24.top := T;
      T := T + S * 3 + rgrpE24.Height;
      EmployNextPage.top := T + 30;
      sbEmploy.VertScrollBar.Range := T + 30;

      // Drug -------------------------------------------------
      T := 5;
      lblD1d.Top := T;

      edtD1d.Left := lblD1d.Left + lblD1d.Width + 5;
      edtD1d.top := T;
      T := T + S + edtD1d.Height;
      lblD1y.Top := T;

      edtD1y.Left := lblD1y.Left + lblD1y.Width + 5;
      edtD1y.top := T;
      T := T + S + edtD1y.Height;
      lblD1r.Top := T;
      T := T + L + S;
      cbxD1r.top := T;
      T := T + S + cbxD1r.Height;

      lblD2d.Top := T;

      edtD2d.Left := lblD2d.Left + lblD2d.Width + 5;
      edtD2d.top := T;
      T := T + S + edtD1d.Height;
      lblD2y.Top := T;

      edtD2y.Left := lblD2y.Left + lblD2y.Width + 5;
      edtD2y.top := T;
      T := T + S + edtD1y.Height;
      lblD2r.Top := T;
      T := T + L + S;
      cbxD2r.top := T;
      T := T + S + cbxD1r.Height;

      lblD3d.Top := T;

      edtD3d.Left := lblD3d.Left + lblD3d.Width + 5;
      edtD3d.top := T;
      T := T + S + edtD1d.Height;
      lblD3y.Top := T;

      edtD3y.Left := lblD3y.Left + lblD3y.Width + 5;
      edtD3y.top := T;
      T := T + S + edtD1y.Height;
      lblD3r.Top := T;
      T := T + L + S;
      cbxD3r.top := T;
      T := T + S + cbxD1r.Height;

      lblD4d.Top := T;

      edtD4d.Left := lblD4d.Left + lblD4d.Width + 5;
      edtD4d.top := T;
      T := T + S + edtD1d.Height;
      lblD4y.Top := T;

      edtD4y.Left := lblD4y.Left + lblD4y.Width + 5;
      edtD4y.top := T;
      T := T + S + edtD1y.Height;
      lblD4r.Top := T;
      T := T + L + S;
      cbxD4r.top := T;
      T := T + S + cbxD1r.Height;

      lblD5d.Top := T;

      edtD5d.Left := lblD5d.Left + lblD5d.Width + 5;
      edtD5d.top := T;
      T := T + S + edtD1d.Height;
      lblD5y.Top := T;

      edtD5y.Left := lblD5y.Left + lblD5y.Width + 5;
      edtD5y.top := T;
      T := T + S + edtD1y.Height;
      lblD5r.Top := T;
      T := T + L + S;
      cbxD5r.top := T;
      T := T + S + cbxD1r.Height;

      lblD6d.Top := T;

      edtD6d.Left := lblD6d.Left + lblD6d.Width + 5;
      edtD6d.top := T;
      T := T + S + edtD1d.Height;
      lblD6y.Top := T;

      edtD6y.Left := lblD6y.Left + lblD6y.Width + 5;
      edtD6y.top := T;
      T := T + S + edtD1y.Height;
      lblD6r.Top := T;
      T := T + L + S;
      cbxD6r.top := T;
      T := T + S + cbxD1r.Height;

      lblD7d.Top := T;

      edtD7d.Left := lblD7d.Left + lblD7d.Width + 5;
      edtD7d.top := T;
      T := T + S + edtD1d.Height;
      lblD7y.Top := T;

      edtD7y.Left := lblD7y.Left + lblD7y.Width + 5;
      edtD7y.top := T;
      T := T + S + edtD1y.Height;
      lblD7r.Top := T;
      T := T + L + S;
      cbxD7r.top := T;
      T := T + S + cbxD1r.Height;

      lblD8d.Top := T;

      edtD8d.Left := lblD8d.Left + lblD8d.Width + 5;
      edtD8d.top := T;
      T := T + S + edtD1d.Height;
      lblD8y.Top := T;

      edtD8y.Left := lblD8y.Left + lblD8y.Width + 5;
      edtD8y.top := T;
      T := T + S + edtD1y.Height;
      lblD8r.Top := T;
      T := T + L + S;
      cbxD8r.top := T;
      T := T + S + cbxD1r.Height;

      lblD9d.Top := T;

      edtD9d.Left := lblD9d.Left + lblD9d.Width + 5;
      edtD9d.top := T;
      T := T + S + edtD1d.Height;
      lblD9y.Top := T;

      edtD9y.Left := lblD9y.Left + lblD9y.Width + 5;
      edtD9y.top := T;
      T := T + S + edtD1y.Height;
      lblD9r.Top := T;
      T := T + L + S;
      cbxD9r.top := T;
      T := T + S + cbxD1r.Height;

      lblD10d.Top := T;

      edtD10d.Left := lblD10d.Left + lblD10d.Width + 5;
      edtD10d.top := T;
      T := T + S + edtD1d.Height;
      lblD10y.Top := T;

      edtD10y.Left := lblD10y.Left + lblD10y.Width + 5;
      edtD10y.top := T;
      T := T + S + edtD1y.Height;
      lblD10r.Top := T;
      T := T + L + S;
      cbxD10r.top := T;
      T := T + S + cbxD1r.Height;

      lblD11d.Top := T;

      edtD11d.Left := lblD11d.Left + lblD11d.Width + 5;
      edtD11d.top := T;
      T := T + S + edtD1d.Height;
      lblD11y.Top := T;

      edtD11y.Left := lblD11y.Left + lblD11y.Width + 5;
      edtD11y.top := T;
      T := T + S + edtD1y.Height;
      lblD11r.Top := T;
      T := T + L + S;
      cbxD11r.top := T;
      T := T + S + cbxD1r.Height;

      lblD12d.Top := T;

      edtD12d.Left := lblD12d.Left + lblD12d.Width + 5;
      edtD12d.top := T;
      T := T + S + edtD1d.Height;
      lblD12y.Top := T;

      edtD12y.Left := lblD12y.Left + lblD12y.Width + 5;
      edtD12y.top := T;
      T := T + S + edtD1y.Height;
      lblD12r.Top := T;
      T := T + L + S;
      cbxD12r.top := T;
      T := T + S + cbxD1r.Height;

      lblD13d.Top := T;

      edtD13d.Left := lblD13d.Left + lblD13d.Width + 5;
      edtD13d.top := T;
      T := T + S + edtD1d.Height;
      lblD13y.Top := T;

      edtD13y.Left := lblD13y.Left + lblD13y.Width + 5;
      edtD13y.top := T;
      T := T + S + edtD1y.Height;

      if not (WhichForm = 'L') then
        begin
          lblD14.Top := T;
          T := T + L + S;
          rgrpD14.top := T;
          T := T + S + rgrpD14.Height;
        end;
      if WhichForm = 'F' then
        begin
          lblD15.Top := T;

          edtD15.Left := lblD15.Left + lblD15.Width + 5;
          edtD15.top := T;
          T := T + S + edtD15.height;
          lblD16.Top := T;

          edtD16.Left := lblD16.Left + lblD16.Width + 5;
          edtD16.top := T;
          T := T + S + edtD16.height;
        end;
      if not (WhichForm = 'L') then
        begin
          lblD17.Top := T;

          edtD17.top := T;
          edtD17.Left := lblD17.Left + lblD17.Width + 5;
          T := T + S + edtD17.height;
          lblD18.Top := T;

          edtD18.top := T;
          edtD18.Left := lblD18.Left + lblD18.Width + 5;
          T := T + S + edtD18.height;
        end;
      lblD19.Top := T;

      edtD19.top := T;
      edtD19.Left := lblD19.Left + lblD19.Width + 5;
      T := T + S + edtD19.height;
      lblD20.Top := T;

      edtD20.Left := lblD20.Left + lblD20.Width + 5;
      edtD20.top := T;
      T := T + S + edtD20.height;
      if not (WhichForm = 'L') then
        begin
          lblD21.Top := T;

          edtD21.top := T;
          edtD21.Left := lblD21.Left + lblD21.Width + 5;
          T := T + S + edtD21.height;
          lblD22.Top := T;

          edtD22.top := T;
          edtD22.Left := lblD22.Left + lblD22.Width + 5;
          T := T + S + edtD22.height;
        end;
      lblD23.Top := T;

      edtD23.top := T;
      edtD23.Left := lblD23.Left + lblD23.Width + 5;
      T := T + S + edtD23.height;
      lblD24.Top := T;

      edtD24.top := T;
      edtD24.Left := lblD24.Left + lblD24.Width + 5;
      T := T + S + edtD24.height;
      lblD25.Top := T;

      edtD25.top := T;
      edtD25.Left := lblD25.Left + lblD25.Width + 5;
      T := T + S + edtD25.height;
      lblD26.Top := T;

      edtD26.top := T;
      edtD26.Left := lblD26.Left + lblD26.Width + 5;
      T := T + S + edtD26.height;
      lblD27.Top := T;

      edtD27.top := T;
      edtD27.Left := lblD27.Left + lblD27.Width + 5;
      T := T + S + edtD27.height;
      lblD28.Top := T;
      T := T + L + S;
      rgrpD28.top := T;
      T := T + S + rgrpD28.height;
      lblD29.Top := T;
      T := T + L + S;
      rgrpD29.top := T;
      T := T + S + rgrpD29.height;
      lblD30.Top := T;
      T := T + L + S;
      rgrpD30.top := T;
      T := T + S + rgrpD30.height;
      lblD31.Top := T;
      T := T + L + S;
      rgrpD31.top := T;
      T := T + S + rgrpD31.height;
      if not (WhichForm = 'L') then
        begin
          lblD32.Top := T;
          T := T + L + S;
          rgrpD32.top := T;
          T := T + S + rgrpD32.height;
          lblD33.Top := T;
          T := T + L + S;
          rgrpD33.top := T;
          T := T + S + rgrpD33.height;
        end;
      lblD34.Top := T;
      T := T + L + S;
      rgrpD34.top := T;
      T := T + S + rgrpD34.height;
      lblD35.Top := T;
      T := T + L + S;
      rgrpD35.top := T;
      T := T + S * 3 + rgrpD35.Height;
      DrugNextPage.top := T + 30;
      sbDrug.VertScrollBar.Range := T + 30;

      //Legal -------------------------------------------------
      T := 5;
      if not (WhichForm = 'U') then
        begin
          lblL1.Top := T;
          T := T + L + S;
          rgrpL1.top := T;
          T := T + S + rgrpL1.height;
        end;
      lblL2.Top := T;
      T := T + L + S;
      rgrpL2.top := T;
      T := T + S + rgrpL2.height;
      lblL3.Top := T;

      edtL3.Left := lblL3.Left + lblL3.Width + 5;
      edtL3.top := T;
      T := T + S + edtL3.Height;
      lblL4.Top := T;

      edtL4.Left := lblL4.Left + lblL4.Width + 5;
      edtL4.top := T;
      T := T + S + edtL3.Height;
      lblL5.Top := T;

      edtL5.Left := lblL5.Left + lblL5.Width + 5;
      edtL5.top := T;
      T := T + S + edtL3.Height;
      lblL6.Top := T;

      edtL6.Left := lblL6.Left + lblL6.Width + 5;
      edtL6.top := T;
      T := T + S + edtL3.Height;
      lblL7.Top := T;

      edtL7.Left := lblL7.Left + lblL7.Width + 5;
      edtL7.top := T;
      T := T + S + edtL3.Height;
      lblL8.Top := T;

      edtL8.Left := lblL8.Left + lblL8.Width + 5;
      edtL8.top := T;
      T := T + S + edtL3.Height;
      lblL9.Top := T;

      edtL9.Left := lblL9.Left + lblL9.Width + 5;
      edtL9.top := T;
      T := T + S + edtL3.Height;
      lblL10.Top := T;

      edtL10.Left := lblL10.Left + lblL10.Width + 5;
      edtL10.top := T;
      T := T + S + edtL3.Height;
      lblL11.Top := T;

      edtL11.Left := lblL11.Left + lblL11.Width + 5;
      edtL11.top := T;
      T := T + S + edtL3.Height;
      lblL12.Top := T;

      edtL12.Left := lblL12.Left + lblL12.Width + 5;
      edtL12.top := T;
      T := T + S + edtL3.Height;
      lblL13.Top := T;

      edtL13.Left := lblL13.Left + lblL13.Width + 5;
      edtL13.top := T;
      T := T + S + edtL3.Height;
      lblL14.Top := T;

      edtL14.Left := lblL14.Left + lblL14.Width + 5;
      edtL14.top := T;
      T := T + S + edtL3.Height;
      lblL15.Top := T;

      edtL15.Left := lblL15.Left + lblL15.Width + 5;
      edtL15.top := T;
      T := T + S + edtL3.Height;
      lblL16.Top := T;

      edtL16.Left := lblL16.Left + lblL16.Width + 5;
      edtL16.top := T;
      T := T + S + edtL3.Height;
      lblL17.Top := T;

      edtL17.Left := lblL17.Left + lblL17.Width + 5;
      edtL17.top := T;
      T := T + S + edtL3.Height;
      lblL18.Top := T;

      edtL18.Left := lblL18.Left + lblL18.Width + 5;
      edtL18.top := T;
      T := T + S + edtL18.height;
      lblL19.Top := T;

      edtL19.Left := lblL19.Left + lblL19.Width + 5;
      edtL19.top := T;
      T := T + S + edtL19.height;
      lblL20.Top := T;

      edtL20.Left := lblL20.Left + lblL20.Width + 5;
      edtL20.top := T;
      T := T + S + edtL20.height;
      lblL21.Top := T;

      edtL21.Left := lblL21.Left + lblL21.Width + 5;
      edtL21.top := T;
      T := T + S + edtL21.height;
      if not (WhichForm = 'L') then
        begin
          lblL22.Top := T;

          edtL22.Left := lblL22.Left + lblL22.Width + 5;
          edtL22.top := T;

          T := T + S + edtL22.height;
          lblL23.Top := T;
          T := T + L + S;
          rgrpL23.top := T;
          T := T + S + rgrpL23.height;
        end;
      lblL24.Top := T;
      T := T + L + S;
      rgrpL24.top := T;
      T := T + S + rgrpL24.height;
      lblL25.Top := T;
      T := T + L + S;
      rgrpL25.top := T;
      T := T + S + rgrpL25.height;

      lblL26.Top := T;

      edtL26.top := T;
      edtL26.Left := lblL26.Left + lblL26.Width + 5;
      T := T + S + edtL26.height;
      lblL27.Top := T;

      edtL27.top := T;
      edtL27.Left := lblL27.Left + lblL27.Width + 5;
      T := T + S + edtL27.height;
      lblL28.Top := T;
      T := T + L + S;
      rgrpL28.top := T;
      T := T + S + rgrpL28.height;
      lblL29.Top := T;
      T := T + L + S;
      rgrpL29.top := T;
      T := T + S + rgrpL29.height;
      if not (WhichForm = 'L') then
        begin
          lblL30.Top := T;
          T := T + L + S;
          rgrpL30.top := T;
          T := T + S + rgrpL30.height;
        end;
      lblL31.Top := T;
      T := T + L + S;
      rgrpL31.top := T;
      T := T + S + rgrpL31.height;
      lblL32.Top := T;
      T := T + L + S;
      rgrpL32.top := T;
      T := T + S * 3 + rgrpL32.Height;
      LegalNextPage.top := T + 30;
      sbLegal.VertScrollBar.Range := T + 30;

      //Family -------------------------------------------------
      T := 5;
      if WhichForm = 'F' then
        begin
          lblH1a.top := T;
          T := T + L + S;
          cbxH1a.top := T;
          T := T + L + S;
          lblH1d.top := T;
          T := T + L + S;
          cbxH1d.top := T;
          T := T + L + S;
          lblH1p.top := T;
          T := T + L + S;
          cbxH1p.top := T;
          T := T + S + cbxH1a.height;


          lblH2a.top := T;
          T := T + L + S;
          cbxH2a.top := T;
          T := T + L + S;
          lblH2d.top := T;
          T := T + L + S;
          cbxH2d.top := T;
          T := T + L + S;
          lblH2p.top := T;
          T := T + L + S;
          cbxH2p.top := T;
          T := T + S + cbxH2a.height;

          lblH3a.top := T;
          T := T + L + S;
          cbxH3a.top := T;
          T := T + L + S;
          lblH3d.top := T;
          T := T + L + S;
          cbxH3d.top := T;
          T := T + L + S;
          lblH3p.top := T;
          T := T + L + S;
          cbxH3p.top := T;
          T := T + S + cbxH3a.height;

          lblH4a.top := T;
          T := T + L + S;
          cbxH4a.top := T;
          T := T + L + S;
          lblH4d.top := T;
          T := T + L + S;
          cbxH4d.top := T;
          T := T + L + S;
          lblH4p.top := T;
          T := T + L + S;
          cbxH4p.top := T;
          T := T + S + cbxH4a.height;

          lblH5a.top := T;
          T := T + L + S;
          cbxH5a.top := T;
          T := T + L + S;
          lblH5d.top := T;
          T := T + L + S;
          cbxH5d.top := T;
          T := T + L + S;
          lblH5p.top := T;
          T := T + L + S;
          cbxH5p.top := T;
          T := T + S + cbxH5a.height;


          lblH6a.top := T;
          T := T + L + S;
          cbxH6a.top := T;
          T := T + L + S;
          lblH6d.top := T;
          T := T + L + S;
          cbxH6d.top := T;
          T := T + L + S;
          lblH6p.top := T;
          T := T + L + S;
          cbxH6p.top := T;
          T := T + S + cbxH6a.height;

          lblH7a.top := T;
          T := T + L + S;
          cbxH7a.top := T;
          T := T + L + S;
          lblH7d.top := T;
          T := T + L + S;
          cbxH7d.top := T;
          T := T + L + S;
          lblH7p.top := T;
          T := T + L + S;
          cbxH7p.top := T;
          T := T + S + cbxH7a.height;

          lblH8a.top := T;
          T := T + L + S;
          cbxH8a.top := T;
          T := T + L + S;
          lblH8d.top := T;
          T := T + L + S;
          cbxH8d.top := T;
          T := T + L + S;
          lblH8p.top := T;
          T := T + L + S;
          cbxH8p.top := T;
          T := T + S + cbxH8a.height;

          lblH9a.top := T;
          T := T + L + S;
          cbxH9a.top := T;
          T := T + L + S;
          lblH9d.top := T;
          T := T + L + S;
          cbxH9d.top := T;
          T := T + L + S;
          lblH9p.top := T;
          T := T + L + S;
          cbxH9p.top := T;
          T := T + S + cbxH9a.height;

          lblH10a.top := T;
          T := T + L + S;
          cbxH10a.top := T;
          T := T + L + S;
          lblH10d.top := T;
          T := T + L + S;
          cbxH10d.top := T;
          T := T + L + S;
          lblH10p.top := T;
          T := T + L + S;
          cbxH10p.top := T;
          T := T + S + cbxH10a.Height;

          lblH11a.top := T;
          T := T + L + S;
          cbxH11a.top := T;
          T := T + L + S;
          lblH11d.top := T;
          T := T + L + S;
          cbxH11d.top := T;
          T := T + L + S;
          lblH11p.top := T;
          T := T + L + S;
          cbxH11p.top := T;
          T := T + S + cbxH11a.height;

          lblH12a.top := T;
          T := T + L + S;
          cbxH12a.top := T;
          T := T + L + S;
          lblH12d.top := T;
          T := T + L + S;
          cbxH12d.top := T;
          T := T + L + S;
          lblH12p.top := T;
          T := T + L + S;
          cbxH12p.top := T;
          T := T + S * 3 + cbxH12a.height;
          FamilyNextPage.top := T + 30;
          sbFamily.VertScrollBar.Range := T + 30;
        end;

      //Social -------------------------------------------------
      T := 5;
      lblF1.Top := T;
      T := T + L + S;
      rgrpF1.top := T;
      T := T + S + rgrpF1.height;
      if WhichForm = 'F' then
        begin
          lblF2.Top := T;

          edtF2y.Left := lblF2.Left + lblF2.Width + 5;
          edtF2y.Top := T;
          T := T + edtF2y.height + S;
          lblF2m.Top := T;
          T := T + L + S;
          edtF2m.Top := T;
          T := T + S + edtF2y.height;
        end;
      lblF3.Top := T;
      T := T + L + S;
      rgrpF3.top := T;
      T := T + S + rgrpF3.height;
      lblF4.Top := T;
      T := T + L + S;
      rgrpF4.top := T;
      T := T + S + rgrpF4.height;
      if WhichForm = 'F' then
        begin
          lblF5y.Top := T;

          edtF5y.Left := lblF5y.Left + lblF5y.Width + 5;
          edtF5y.Top := T;
          T := T + edtF5y.height + S;
          lblF5m.Top := T;
          T := T + L + S;
          edtF5m.Top := T;
          T := T + S + edtF5y.height;
        end;
      lblF6.Top := T;
      T := T + L + S;
      rgrpF6.top := T;
      T := T + S + rgrpF6.height;
      lblF7.Top := T;
      T := T + L + S;
      rgrpF7.top := T;
      T := T + S + rgrpF7.height;
      lblF8.Top := T;
      T := T + L + S;
      rgrpF8.top := T;
      T := T + S + rgrpF8.height;
      if not (WhichForm = 'L') then
        begin
          lblF9.Top := T;
          T := T + L + S;
          rgrpF9.top := T;
          T := T + S + rgrpF9.height;
          lblF10.Top := T;
          T := T + L + S;
          rgrpF10.top := T;
          T := T + S + rgrpF10.height;
          lblF11.Top := T;

          edtF11.Left := lblF11.Left + lblF11.Width + 5;
          edtF11.Top := T;

          T := T + S + edtF11.height;
        end;
      if WhichForm = 'F' then
        begin
          lblF12.top := T;
          T := T + L + S;
          cbxF12.top := T;
          T := T + S + cbxF12.Height;
          lblF13.top := T;
          T := T + L + S;
          cbxF13.top := T;
          T := T + S + cbxF12.Height;
          lblF14.top := T;
          T := T + L + S;
          cbxF14.top := T;
          T := T + S + cbxF12.Height;
          lblF15.top := T;
          T := T + L + S;
          cbxF15.top := T;
          T := T + S + cbxF12.Height;
          lblF16.top := T;
          T := T + L + S;
          cbxF16.top := T;
          T := T + S + cbxF12.Height;
          lblF17.top := T;
          T := T + L + S;
          cbxF17.top := T;
          T := T + S + cbxF12.Height;
        end;

      lblF18.Top := T;
      T := T + L + S;
      rgrpF18.top := T;
      T := T + S + rgrpF18.Height;
      lblF18l.Top := T;
      T := T + L + S;
      rgrpF18l.top := T;
      T := T + S + rgrpF18l.height;
      lblF19.Top := T;
      T := T + L + S;
      rgrpF19.top := T;
      T := T + S + rgrpF19.Height;
      lblF19l.Top := T;
      T := T + L + S;
      rgrpF19l.top := T;
      T := T + S + rgrpF19l.height;
      lblF20.Top := T;
      T := T + L + S;
      rgrpF20.top := T;
      T := T + S + rgrpF20.Height;
      lblF20l.Top := T;
      T := T + L + S;
      rgrpF20l.top := T;
      T := T + S + rgrpF20l.height;
      lblF21.Top := T;
      T := T + L + S;
      rgrpF21.top := T;
      T := T + S + rgrpF21.Height;
      lblF21l.Top := T;
      T := T + L + S;
      rgrpF21l.top := T;
      T := T + S + rgrpF21l.height;
      lblF22.Top := T;
      T := T + L + S;
      rgrpF22.top := T;
      T := T + S + rgrpF22.Height;
      lblF22l.Top := T;
      T := T + L + S;
      rgrpF22l.top := T;
      T := T + S + rgrpF22l.height;
      lblF23.Top := T;
      T := T + L + S;
      rgrpF23.top := T;
      T := T + S + rgrpF23.Height;
      lblF23l.Top := T;
      T := T + L + S;
      rgrpF23l.top := T;
      T := T + S + rgrpF23l.height;
      lblF24.Top := T;
      T := T + L + S;
      rgrpF24.top := T;
      T := T + S + rgrpF24.Height;
      lblF24l.Top := T;
      T := T + L + S;
      rgrpF24l.top := T;
      T := T + S + rgrpF24l.height;
      lblF25.Top := T;
      T := T + L + S;
      rgrpF25.top := T;
      T := T + S + rgrpF25.Height;
      lblF25l.Top := T;
      T := T + L + S;
      rgrpF25l.top := T;
      T := T + S + rgrpF25l.height;
      lblF26.Top := T;
      T := T + L + S;
      rgrpF26.top := T;
      T := T + S + rgrpF26.Height;
      lblF26l.Top := T;
      T := T + L + S;
      rgrpF26l.top := T;
      T := T + S + rgrpF26l.height;

      if not (WhichForm = 'L') then
        begin
          lblF27.Top := T;
          T := T + L + S;
          rgrpF27.top := T;
          T := T + S + rgrpF27.Height;
          lblF27l.Top := T;
          T := T + L + S;
          rgrpF27l.top := T;
          T := T + S + rgrpF27l.height;
        end;
      lblF28.Top := T;
      T := T + L + S;
      rgrpF28.top := T;
      T := T + S + rgrpF28.Height;
      lblF28l.Top := T;
      T := T + L + S;
      rgrpF28l.top := T;
      T := T + S + rgrpF28l.height;
      lblF29.Top := T;
      T := T + L + S;
      rgrpF29.top := T;
      T := T + S + rgrpF29.Height;
      lblF29l.Top := T;
      T := T + L + S;
      rgrpF29l.top := T;
      T := T + S + rgrpF29l.height;
      lblF30.Top := T;

      edtF30.top := T;
      edtF30.Left := lblF30.Left + lblF30.Width + 5;
      T := T + S + edtF30.height;
      lblF31.Top := T;

      edtF31.top := T;
      edtF31.Left := lblF31.Left + lblF31.Width + 5;
      T := T + S + edtF31.height;
      lblF32.Top := T;
      T := T + L + S;
      rgrpF32.top := T;
      T := T + S + rgrpF32.height;
      if not (WhichForm = 'L') then
        begin
          lblF33.Top := T;
          T := T + L + S;
          rgrpF33.top := T;
          T := T + S + rgrpF33.height;
        end;
      lblF34.Top := T;
      T := T + L + S;
      rgrpF34.top := T;
      T := T + S + rgrpF34.height;
      if not (WhichForm = 'L') then
        begin
          lblF35.Top := T;
          T := T + L + S;
          rgrpF35.top := T;
          T := T + S + rgrpF35.height;
          lblF36.Top := T;
          T := T + L + S;
          rgrpF36.top := T;
          T := T + S + rgrpF36.height;
        end;
      lblF37.Top := T;
      T := T + L + S;
      rgrpF37.top := T;
      T := T + S + rgrpF37.height;
      lblF38.Top := T;
      T := T + L + S;
      rgrpF38.top := T;
      T := T + S * 3 + rgrpF38.height;
      SocialNextPage.top := T + 30;
      sbSocial.VertScrollBar.Range := T + 30;

      //Psych -------------------------------------------------
      T := 5;
      lblP1a.top := T;

      edtP1a.Left := lblP1a.Left + lblP1a.Width + 5;
      edtP1a.top := T;
      T := T + S + edtP1a.height;
      lblP1b.top := T;

      edtP1b.Left := lblP1b.Left + lblP1b.Width + 5;
      edtP1b.top := T;
      T := T + S + edtP1b.height;
      lblP2.Top := T;
      T := T + L + S;
      rgrpP2.top := T;
      T := T + S + rgrpP2.height;

      lblP3.Top := T;
      T := T + L + S;
      rgrpP3.top := T;
      T := T + S + rgrpP3.Height;
      lblP3l.Top := T;
      T := T + L + S;
      rgrpP3l.top := T;
      T := T + S + rgrpP3l.height;
      lblP4.Top := T;
      T := T + L + S;
      rgrpP4.top := T;
      T := T + S + rgrpP4.Height;
      lblP4l.Top := T;
      T := T + L + S;
      rgrpP4l.top := T;
      T := T + S + rgrpP4l.height;
      lblP5.Top := T;
      T := T + L + S;
      rgrpP5.top := T;
      T := T + S + rgrpP5.Height;
      lblP5l.Top := T;
      T := T + L + S;
      rgrpP5l.top := T;
      T := T + S + rgrpP5l.height;
      lblP6.Top := T;
      T := T + L + S;
      rgrpP6.top := T;
      T := T + S + rgrpP6.Height;
      lblP6l.Top := T;
      T := T + L + S;
      rgrpP6l.top := T;
      T := T + S + rgrpP6l.height;
      lblP7.Top := T;
      T := T + L + S;
      rgrpP7.top := T;
      T := T + S + rgrpP7.Height;
      lblP7l.Top := T;
      T := T + L + S;
      rgrpP7l.top := T;
      T := T + S + rgrpP7l.height;
      lblP8.Top := T;
      T := T + L + S;
      rgrpP8.top := T;
      T := T + S + rgrpP8.Height;
      lblP8l.Top := T;
      T := T + L + S;
      rgrpP8l.top := T;
      T := T + S + rgrpP8l.height;
      lblP9.Top := T;
      T := T + L + S;
      rgrpP9.top := T;
      T := T + S + rgrpP9.Height;
      lblP9l.Top := T;
      T := T + L + S;
      rgrpP9l.top := T;
      T := T + S + rgrpP9l.height;
      lblP10.Top := T;
      T := T + L + S;
      rgrpP10.top := T;
      T := T + S + rgrpP10.Height;
      lblP10l.Top := T;
      T := T + L + S;
      rgrpP10l.top := T;
      T := T + S + rgrpP10l.height;
      lblP11.Top := T;

      edtP11.Left := lblP11.Left + lblP11.Width + 5;
      edtP11.top := T + S;
      T := T + S + edtP11.height;
      lblP12.Top := T;
      T := T + L + S;
      rgrpP12.top := T;
      T := T + S + rgrpP12.height;
      lblP13.Top := T;
      T := T + L + S;
      rgrpP13.top := T;
      T := T + S + rgrpP13.height;
      if not (WhichForm = 'L') then
        begin
          lblP14.Top := T;
          T := T + L + S;
          rgrpP14.top := T;
          T := T + S + rgrpP14.height;
          lblP15.Top := T;
          T := T + L + S;
          rgrpP15.top := T;
          T := T + S + rgrpP15.height;
          lblP16.Top := T;
          T := T + L + S;
          rgrpP16.top := T;
          T := T + S + rgrpP16.height;
          lblP17.Top := T;
          T := T + L + S;
          rgrpP17.top := T;
          T := T + S + rgrpP17.height;
          lblP18.Top := T;
          T := T + L + S;
          rgrpP18.top := T;
          T := T + S + rgrpP18.height;
          lblP19.Top := T;
          T := T + L + S;
          rgrpP19.top := T;
          T := T + S + rgrpP19.height;
          lblP20.Top := T;
          T := T + L + S;
          rgrpP20.top := T;
          T := T + S + rgrpP20.height;
        end;
      lblP21.Top := T;
      T := T + L + S;
      rgrpP21.top := T;
      T := T + S + rgrpP21.height;
      lblP22.Top := T;
      T := T + L + S;
      rgrpP22.top := T;
      T := T + S + rgrpP22.height;

      btnFinished.Top := T;
      T := T + btnFinished.Height + S;
      sbPsych.VertScrollBar.Range := T + 30;
    end;
end;*)


end.

