unit uMHA_ASI_IO;

interface

function ASIReadEvaluationFactors(aIENS: string): string;
function ASIReadItemValues(aIENS: string; ItemFieldNumbers: string):
  string;
function ASIReadList(aDFN: string): Boolean;
function ASIReadReport(aIENS: string; aProcedure: string): string;

function ASIWriteChoicesMade(aIENS: string): string;
//function ASIWriteNewInterview(aIENS: string): string;
function ASIWriteNewInterview(
  aPatientDFN,
  aInterviewType,
  aDate,
  aInterviewerDUZ,
  aOrderedbyDUZ: string): string;
// returns the IENS
function ASIWriteProgressNote(aIENS: string): BOOLEAN;
function ASISign(ASignature, aIENS: string): BOOLEAN;
function ASIStatusOfSavedRecord(aIENS: string): string;

implementation

uses
  SysUtils,
  fIA_dialogs,
  Trpcb,
  uIA_Utils,
  uIA_Const,
  uMHA_Const,
  uMHA_LocalFiles,
  uMHA_ASI,
  uMHA_ASI_OPS,
  uMHA_DM;


function ASIStatusOfSavedRecord(aIENS: string): string;
// Possible results are:
//  0^MISSING REQUIRED FIELDS
//  1^OK TO SIGN
//  2^A G12 RECORD
begin
  result := '';

  //For now, anyway, only Vista mode is supported
  if not (DataBaseMode = dbVista) then
    Exit;

  with DataModule1.RPCBroker1 do
    begin
      ClearParameters := True;
      ClearResults := True;
      CreateContext('YS BROKER1');
      //* LocalFiles.UpdateAPILog('Call to YTQ ASI SIGNOK: IENS=' + aIENS);
      RemoteProcedure := 'YTQ ASI SIGNOK';
      Param[0].Mult['"IENS"'] := aIENS;
      Param[0].PType := List;

      try
        Call;

      except
        on E: EBrokerError do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTQ ASI SIGNOK: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin

            //* LocalFiles.UpdateAPILog('Call to YTQ ASI SIGNOK: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

      if Results.Count > 0 then
        begin
          Result := Trim(Results[1]);
          //* LocalFiles.UpdateAPILog('Call to YTQ ASI SIGNOK: Successful');
        end;
     //*  else
        //* LocalFiles.UpdateAPILog('Call to YTQ ASI SIGNOK: Error ');

    end;

end;


function ASIWriteNewInterview(
  aPatientDFN,
  aInterviewType,
  aDate,
  aInterviewerDUZ,
  aOrderedbyDUZ: string): string;
// returns the IENS
var
  TempIENS                    : string;

begin
  Result := BAD_DATA;

  //For now, anyway, only Vista mode is supported
  if not (DataBaseMode = dbVista) then
    Exit;

  with DataModule1.RPCBroker1 do
    begin
      //Generate new ASI.IENS

      ClearParameters := True;
      ClearResults := True;
      CreateContext('YS BROKER1');
      //* LocalFiles.UpdateAPILog('Call to YTQ NEW ASI');
      RemoteProcedure := 'YTAPI NEW ASI';

      try
        Call;

      except
        on E: EBrokerError do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTQ SAVE: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTQ SAVE: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

      if Results[0] = DATA then
        begin
          //TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).IENS := Results[1];
          TempIENS := Results[1];
          //* LocalFiles.UpdateAPILog('Call to YTQ NEW ASI: Successful');
        end
      else
        begin
          //* LocalFiles.UpdateAPILog('Call to YTQ NEW ASI: Error, new ASI IENS not returned.');
          Exit;
        end;

      ClearParameters := True;
      ClearResults := True;
      CreateContext('YS BROKER1');
      //* LocalFiles.UpdateAPILog('ASI: Call to YTQ SAVE:' +
      //*   ' FILEN=604' +
      //*   ' IENS=' + TempIENS +
      //*   ' 2=' + aPatientDFN +
      //*   ' 3=' + aInterviewType +
      //*   ' 4=' + aDate +
      //*   ' 5=' + aInterviewerDUZ +
      //*   ' 6=' + aOrderedbyDUZ +
      //*   ' 7=.11^N');
      RemoteProcedure := 'YTQ SAVE';
      Param[0].Mult['"FILEN"'] := '604';
      Param[0].Mult['"IEN"'] := TempIENS;
      Param[0].Mult['"2"'] := '.02^`' + aPatientDFN;
      Param[0].Mult['"3"'] := '.04^' + aInterviewType; // String: 1, 2, 3 or 4.
      Param[0].Mult['"4"'] := '.05^' + aDate;
      Param[0].Mult['"5"'] := '.81^`' + aInterviewerDUZ;
      Param[0].Mult['"6"'] := '.09^`' + aOrderedbyDUZ;
      //Default interviewer is user
      Param[0].Mult['"7"'] := '.11^N'; //Default G12 is Interview Complete

      Param[0].PType := List;

      try
        Call;

      except
        on E: EBrokerError do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTQ SAVE: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTQ SAVE: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

      if (Results[0] = DATA)
        and (pos('UPDATE OK', UpperCase(Results[1])) > 0) then
        begin
          //* LocalFiles.UpdateAPILog('Call to YTQ SAVE: Successful');
          result := TempIENS;
        end;
     //*  else
        //* LocalFiles.UpdateAPILog('Call to YTQ SAVE: Error - ASI not updated.');
    end;

end;

function ASIWriteChoicesMade(aIENS: string): string;
var
  i                           : integer;
  TempStr                     : string;
begin

  //For now, anyway, only Vista mode is supported
  if not (DataBaseMode = dbVista) then
    begin
      Result := BAD_DATA;
      Exit;
    end;

  with DataModule1.RPCBroker1 do
    begin

        //TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).IENS);
      ClearParameters := True;
      ClearResults := True;
      CreateContext('YS BROKER1');
      //* LocalFiles.UpdateAPILog('Call to YTAPI ASI SAVE DATA: YSIENS=' + aIENS);
      RemoteProcedure := 'YTAPI ASI SAVE DATA';
      Param[0].Mult['"YSIEN"'] := aIENS + ',';
      //TASI(ASI_OPS.ListOfASIResults[gvCurrentResult]).IENS + ','; // Add comma to end of RecordIEN
      for i := 0 to ASI_OPS.ChoicesMade.Count - 1 do
        Param[0].Mult[IntToStr(i + 1)] :=
          Piece(ASI_OPS.ChoicesMade.Strings[i], '^', 2) + '^' +
          UpperCase(Trim(Piece(ASI_OPS.ChoicesMade.Strings[i], '^', 3)));

      Param[0].PType := List;

      try
        Call;

      except
        on E: EBrokerError do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTAPI ASI SAVE DATA: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTAPI ASI SAVE DATA: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

      if pos(OK, Results[1]) > 0 then
        begin
          //* LocalFiles.UpdateAPILog('Call to YTAPI ASI SAVE DATA: Successful');
          // When successful this is returned:
          // "OK ASI SAVED XXXX," where XXXX is the RecordIEN

          TempStr := Results[1];
          Delete(TempStr, 1, 12);
          Delete(TempStr, Length(TempStr), 1);
          Result := TempStr;
        end
      else
        begin
          //* LocalFiles.UpdateAPILog('Call to YTAPI ASI SAVE DATA: Error');
          //* for i := 0 to Results.Count - 1 do
            //* LocalFiles.UpdateAPILog('   ' + Results[i]);
          tempstr := '';
          for i := 0 to Results.Count - 1 do
            tempstr := tempstr + Results[i];
          ShowMessage('Error: YTAPI ASI SAVE DATA' + #13 + tempstr);
        end;
    end; // With RPCBroker

end;


function ASIReadList(aDFN: string): Boolean;
var
  i                           : integer;
  TempStr                     : string;
  TempDate                    : TDateTime;
  TempASI                     : TASI;
begin
// Returns:
// AdministrationIEN=12/08/2006^LITE^INTERVIEW COMPLETED^YES^SCHULTZ,ROGER^
// 6627^SCHULTZ,ROGER^6627

  Result := False;
  TempDate := 0;

  //For now, anyway, only Vista mode is supported
  if not (DataBaseMode = dbVista) then
    Exit;

  with DataModule1.RPCBroker1 do
    begin
      ClearResults := True;
      ClearParameters := True;
      CreateContext('YS BROKER1');
      //* LocalFiles.UpdateAPILog('Call to YTQ ASI LISTER: Successful');
      RemoteProcedure := 'YTQ ASI LISTER';
      Param[0].Mult['"DFN"'] := aDFN;
      Param[0].PType := List;

      try
        Call;

      except
        on E: EBrokerError do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTQ ASI LISTER: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTQ ASI LISTER: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

      if POS('DATA', Results[0]) > 0 then
        begin
          Result := True;
          //* LocalFiles.UpdateAPILog('Call to YTQ ASI LISTER: Successful');
          for i := 1 to Results.Count - 1 do
            begin
              TempASI := TASI.Create;

              with TempASI do
                begin
                  IENS := Piece(Results[i], EQ, 1);
                  TempStr := Piece(Results[i], EQ, 2);

                  ASIDate := StrToDate(Piece(TempStr, U, 1));
                  InterviewTypeStr := Piece(TempStr, U, 2);
                  InterviewType := ASI_OPS.StrToInterviewType(InterviewTypeStr);
 
                  G12 := Piece(TempStr, U, 3);
                  IsSigned := StrToBool(Piece(TempStr, U, 4));
                  if IsSigned then
                    ASI_OPS.SignedInterviewsExist := True;
                  OrderedByName := Piece(TempStr, U, 5);
                  OrderedByDUZ := Piece(TempStr, U, 6);
                  InterviewerName := Piece(TempStr, U, 7);
                  InterviewerDUZ := Piece(TempStr, U, 8);
                  IENSExist := True;
                  // Find most recent ASI (largest number)
                  if TempDate < ASIDate then
                    TempDate := ASIDate;
                end; // with TempASI

              ASI_OPS.ListOfASIResults.Add(TempASI);
            end; // for i

          if TempDate <> 0 then
            ASI_OPS.LastASIDate := DateToStr(TempDate);
        end;
      //* else
      //*   begin
          //* LocalFiles.UpdateAPILog('Call to YTQ ASI LISTER: Error');
          //* for i := 0 to Results.Count - 1 do
            //* LocalFiles.UpdateAPILog('   ' + Results[i]);
       //*  end;
    end;
end;

function ASIReadEvaluationFactors(aIENS: string): string;
var
  i                           : integer;
begin
  //For now, anyway, only Vista mode is supported
  if not (DataBaseMode = dbVista) then
    begin
      Result := BAD_DATA;
      Exit;
    end;

  //Load ASI Eval Factors
  with DataModule1.RPCBroker1 do
    begin
      ClearResults := True;
      ClearParameters := True;
      CreateContext('YS BROKER1');
      //* LocalFiles.UpdateAPILog('Call to YTAPI ASI FACTORS: IEN=' + aIENS);
      RemoteProcedure := 'YTAPI ASI FACTORS';
      Param[0].Mult['"IEN"'] := AIENS;
      Param[0].PType := List;

      try
        Call;

      except
        on E: EBrokerError do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTAPI ASI FACTORS: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTAPI ASI FACTORS: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

      Result := '';

      if (Results[0] = DATA) then
        begin
          //* LocalFiles.UpdateAPILog('Call to YTAPI ASI FACTORS: Successful');
          for i := 1 to 5 do
            Result := Result + Piece(Results[i], U, 3) + U;

        end
      else
        begin
          Result := BAD_DATA;
          //* LocalFiles.UpdateAPILog('Call to YTAPI ASI FACTORS: Error ');
          //* for i := 0 to Results.Count - 1 do
            //* LocalFiles.UpdateAPILog('   ' + Results[i]);
        end;
    end; //with DataModule1.RPCBroker1 do

end;


function ASIReadReport(aIENS: string; aProcedure: string): string;
var
  tmpReport                   : string;
  i                           : integer;
begin
  //For now, anyway, only Vista mode is supported
  if not (DataBaseMode = dbVista) then
    begin
      Result := BAD_DATA;
      Exit;
    end;

  tmpReport := '';

  with DataModule1.RPCBroker1 do
    begin

      ClearResults := True;
      ClearParameters := True;
      CreateContext('YS BROKER1');
      //* LocalFiles.UpdateAPILog('Call to ' + aprocedure +
      //*   ' Value=' + aIENS);
      RemoteProcedure := aProcedure;
      Param[0].Value := aIENS;
      Param[0].PType := literal;

      try
        Call;

      except
        on E: EBrokerError do
          begin
            //* LocalFiles.UpdateAPILog('Call to ' + aprocedure + ': Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            //* LocalFiles.UpdateAPILog('Call to ' + aprocedure + ': Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

      //Skip string[0], which contains only control characters
      for i := 1 to Results.Count - 1 do
        tmpReport := tmpReport + Results[i] + CRLF;
    end; //with DataModule1.RPCBroker1 do

  Result := DropTrailer(tmpReport);

end;


function ASIReadItemValues(aIENS: string; ItemFieldNumbers: string): string;
var
  tmpRecord, tmpResult        : string;
  i                           : integer;
begin
  //For now, anyway, only Vista mode is supported
  if not (DataBaseMode = dbVista) then
    begin
      Result := BAD_DATA;
      Exit;
    end;

  with DataModule1.RPCBroker1 do
    begin

      ClearResults := True;
      ClearParameters := True;
      CreateContext('YS BROKER1');
      //* LocalFiles.UpdateAPILog('Call to YTQ CAPIE: FILEN=604' +
      //*   ' Fields= ' + ItemFieldNumbers +
      //*   ' IENS=' + aIENS);
      RemoteProcedure := 'YTQ CAPIE';
      Param[0].Mult['"FILEN"'] := '604';
      Param[0].Mult['"FIELDS"'] := ItemFieldNumbers;
      Param[0].Mult['"IENS"'] := aIENS;
      Param[0].PType := List;

      try
        Call;

      except
        on E: EBrokerError do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTQ CAPIE: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTQ CAPIE: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

      //Returns:
      //[DATA]
      //.65^CS FAMILY^  0.0722
      //8.12^INTERVIEWER'S RATING MEDICAL^4
      //11.18^INTERVIEWER'S RATING ALCOHOL^6

      if not (Results[0] = DATA) then
        begin
          Result := BAD_DATA;
          //* LocalFiles.UpdateAPILog('Call to YTQ CAPIE: Error ');
          //* for i := 0 to Results.Count - 1 do
            //* LocalFiles.UpdateAPILog('   ' + Results[i]);
          Exit;
        end;
      //* else
        //* LocalFiles.UpdateAPILog('Call to YTQ CAPIE: Successful');

      tmpResult := '';

      for i := 1 to Results.Count - 1 do
        begin
          tmpRecord := Results[i];
          tmpResult := tmpResult + Piece(tmpRecord, U, 1) + '='
            + Piece(tmpRecord, U, 3) + U;
        end;
    end; //with DataModule1

  Result := DropTrailer(tmpResult);

end;


function ASIWriteProgressNote(aIENS: string): BOOLEAN;
begin
  result := false;

  //For now, anyway, only Vista mode is supported
  if not (DataBaseMode = dbVista) then
    Exit;

  with DataModule1.RPCBroker1 do
    begin

      ClearParameters := True;
      ClearResults := True;
      CreateContext('YS BROKER1');
      //* LocalFiles.UpdateAPILog('Call to YTAPI ASI PNOTE: Successful');
      RemoteProcedure := 'YTAPI ASI PNOTE';
      Param[0].Mult['"YSIEN"'] := aIENS; // Add comma to end of RecordIEN   ??
      Param[0].PType := List;

      try
        Call;

      except
        on E: EBrokerError do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTAPI ASI PNOTE: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTAPI ASI PNOTE: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

      // "OK ...
      if pos(OK, Results[1]) > 0 then
        begin
          //* LocalFiles.UpdateAPILog('Call to YTAPI ASI PNOTE: Successful');
          Result := True;
        end
      else
        begin
          Result := False;
          //* LocalFiles.UpdateAPILog('Call to YTAPI ASI PNOTE: Error ');
          //* for i := 0 to Results.Count - 1 do
            //* LocalFiles.UpdateAPILog('   ' + Results[i]);
        end;
    end;
end;


function ASISign(ASignature, aIENS: string): BOOLEAN;
begin
  Result := false;

  with DataModule1.RPCBroker1 do
    begin

      ClearResults := True;
      ClearParameters := True;
      CreateContext('YS BROKER1');
      //* LocalFiles.UpdateAPILog('Call to YTAPI ASI SIGNER: CODE=' + ASignature +
      //*   ' YSIEN=' + aIENS);
      RemoteProcedure := 'YTAPI ASI SIGNER';
      Param[0].Mult['"CODE"'] := ASignature;
      Param[0].Mult['"YSIEN"'] := aIENS;
      Param[0].PType := List;

      try
        Call;

      except
        on E: EBrokerError do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTAPI ASI SIGNER: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTAPI ASI SIGNER: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try
//ShowMessage(aIENS+#13+ASignature+#13+results[1]);
      if Results[1] = '1^ASI SIGNED' then
        begin
          Result := True;
          //* LocalFiles.UpdateAPILog('Call to YTAPI ASI SIGNER: Successful');
        end
      else
        begin
          Result := False;
          //* LocalFiles.UpdateAPILog('Call to YTAPI ASI SIGNER: Error');
          //* for i := 0 to Results.Count - 1 do
            //* LocalFiles.UpdateAPILog('   ' + Results[i]);
        end;
    end;
end;

end.

