unit uMHA_ASI_Vista;

interface
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, dialogs;

type
  TASI_Vista = class(TObject)
  private
    FVistAFileNumber: string;
    FResponsesSaved: BOOLEAN;
  //  FRecordIEN : string;
    FChoicesMade: TStrings;
    function GetVistAFileNumber: string;
    property VistAFileNumber: string read GetVistAFileNumber
      write FVistAFileNumber;
  protected
  public
    constructor Create;
    procedure SaveToVistA(RecordIEN: string);
    procedure SaveProgressNote(RecordIEN: string);
    destructor Destroy; override;
    property ChoicesMade: TStrings read FChoicesMade write FChoicesMade;
    property ResponsesSaved: BOOLEAN read FResponsesSaved write FResponsesSaved;
//    property RecordIEN: String read FRecordIEN write FRecordIEN;
  published
  end; { TASI }

var
  ASI_Vista : TASI_Vista;

implementation

uses
  fIA_Dialogs,
  //uCore,   // For core objects: patient, user, etc.
  //uMHA_DM, // Container for fileMan components
  uMHA_Utils,
  uMHA_ASI,
//  MFunStr, // For piece
  TRPCB,   // for RPCBroker functions
  ORNet;


procedure TASI_Vista.SaveProgressNote(RecordIEN: string);
var
  Return : string;
begin
  with RPCBrokerV do
    begin
      ClearParameters := True;
      ClearResults := True;
      CreateContext('YS BROKER1');
      RemoteProcedure := 'YTAPI ASI PNOTE';
      Param[0].Mult['"YSIEN"'] := RecordIEN; // Add comma to end of RecordIEN
      Param[0].PType := List;
      Call;
      Return := RPCBrokerV.Results[1]; //When successful this is returned:
        // "OK ...
      if ((Return[1] = 'O') and (Return[2] = 'K')) then
        else
        MsgDlg('The ASI progress note was not written to VistA database.',
          'Mental Health Assistant', mtError, [mbOK], 0);
    end;
end;


  //******************************************************************************
  // TASIMVFile.SaveToVistA [X]USED [ ]DONE [X]TESTED
  // PURPOSE:  Upload the previous administration's local data file to VistA
  // AUTHOR: R. FELDMAN // REVIEWER:
  // LAST MODIFICATION/REVIEW DATE: 02/26/01
  //******************************************************************************


procedure TASI_Vista.SaveToVistA(RecordIEN: string);
var
  i: Integer; //For field loop counter
  Return : string; // RPC Broker results
begin
  Return := '';
  FResponsesSaved := false; // All clear toggle

  //Create a blank, new ASI VistA record and get IEN entry for .01 field
  with RPCBrokerV do
    begin
      if RecordIEN = '' then // New record: get the new RecordIEN
        begin
          ClearParameters := True;
          ClearResults := True;
          CreateContext('YS BROKER1');
          RemoteProcedure := 'YTAPI NEW ASI';
          Call;
          if Results[0] = '[DATA]' then
            RecordIEN := Results[1]
            else
            exit;
        end;

      ASI.IENS := RecordIEN; // Pass this new/old number to FASI functions
      ClearParameters := True;
      ClearResults := True;
      CreateContext('YS BROKER1');
      RemoteProcedure := 'YTAPI ASI SAVE DATA';
      Param[0].Mult['"YSIEN"'] := ASI.IENS + ','; // Add comma to end of RecordIEN
      for i := 0 to FChoicesMade.Count - 1 do
        Param[0].Mult[inttostr(i + 1)] := Piece(FChoicesMade.Strings[i], '^', 2) + '^' +
              UpperCase(Trim(Piece(FChoicesMade.Strings[i], '^', 3)));

      Param[0].PType := List;
      Call;
      Return := RPCBrokerV.Results[1]; //When successful this is returned:
        // "OK ASI SAVED XXXX," where XXXX is the RecordIEN
      if ((Return[1] = 'O') and (Return[2] = 'K')) then
        FResponsesSaved := true
        else
        MsgDlg('The ASI was not saved in the VistA database. Please try to save it again.',
          'Mental Health Assistant', mtError, [mbOK], 0);
    end; // With RPCBroker
end;

function TASI_Vista.GetVistAFileNumber: string;
begin
  result := FVistAFileNumber;
end;

constructor TASI_Vista.Create;
begin
  inherited Create;
  VistaFileNumber := '604';
  ChoicesMade := TStringList.Create;
end;

destructor TASI_Vista.Destroy;
begin
  ChoicesMade.destroy;
  inherited Destroy;
end;


end.

