unit uMHA_Build;

interface

uses
  Windows,
  dialogs,
  Messages,
  classes,
  SysUtils,
  Graphics,
  Controls,
  Forms,
  StdCtrls,
  ExtCtrls,
  ComCtrls,
  Mask,
  RzPanel,
  RzRadGrp,
  Menus,
  RzChkLst,
  RzLstBox,
  Buttons,
  ExTrackBar,
  JvValidateEdit,
  JvExMask,
  JvToolEdit,
  JvMaskEdit,
  JvCheckedMaskEdit,
  JvDatePickerEdit,
  uMHA_Display,
  RzSpnEdt,
  ORCtrls,
  uMHA_Question;

//function GetSameLine(StringIn: string): Char;
//function GetMemoSameLine(StringIn: string): Char;

procedure Build_Title(LabelIn: TLabel; Display: TDisplay;
  TagIn: integer;
  StringIn, HintIn: string; WhichScrollBox: TScrollBox);

procedure Build_Section(LabelIn: TLabel; Display: TDisplay;
  TagIn: integer;
  StringIn: string; WhichScrollBox: TScrollBox);

procedure Build_Instructions(LabelIn: TLabel; Display: TDisplay;
  TagIn: integer;
  StringIn: string; WhichScrollBox: TScrollBox);

function Build_Label(LabelIn: TLabel; Display, AnswerDisplay: TDisplay;
  StringIn, HintIn: string; TagIn: Integer; AResponseType: TResponseType;
  WhichScrollBox: TScrollBox): integer;

procedure Build_RadioGroup(Radio: TRzRadioGroup;
  CaptionIn, HintIn: string;
  TagIn: Integer;
  Display,
  AnswerDisplay: TDisplay;
  IsLegacy: Boolean;
  WhichScrollBox: TScrollBox;
  AItems: TStringList);

procedure Build_CheckListBox(CheckList: TRzCheckList; AnswerDisplay: TDisplay;
  HintIn: string; TempHeight, TagIn: integer; WhichScrollBox: TScrollBox;
  AItems: TStringList);

procedure Build_ListBox(ListBox: TRzListBox; AnswerDisplay: TDisplay;
  HintIn: string; TempHeight, TagIn: integer; WhichScrollBox: TScrollBox;
  AItems: TStringList);

procedure Build_ComboBox(ComboBox: TComboBox; AnswerDisplay: TDisplay;
  HintIn: string; TempHeight, TagIn: integer; WhichScrollBox: TScrollBox;
  AItems: TStringList);

procedure Build_TrackBar(TrackBar: TExTrackBar; AnswerDisplay: TDisplay;
  HintIn: string; TempHeight, TagIn, MinIn, MaxIn: integer; WhichScrollBox: TScrollBox);

procedure Build_TrackLabel2(LabelIn: TLabel; AnswerDisplay: TDisplay;
  TagIn: integer;
  WhichScrollBox: TScrollBox;
  Choices: TStringList);

procedure Build_Date(DateIn: TJvDatePickerEdit; AnswerDisplay: TDisplay;
  HintIn: string; TempHeight, TagIn: integer; WhichScrollBox: TScrollBox);

procedure Build_Edit(Edit: TEdit; AnswerDisplay: TDisplay;
  HintIn: string; TempHeight, TagIn: integer; WhichScrollBox: TScrollBox);

procedure Build_VistaComboBox(VistaComboBox: TORComboBox; AnswerDisplay: TDisplay;
  HintIn: string; TempHeight, TagIn: integer; WhichScrollBox: TScrollBox);

procedure Build_MaskEdit(MaskEdit: TMaskEdit; AnswerDisplay: TDisplay;
  HintIn: string; TempHeight, TagIn: integer; WhichScrollBox: TScrollBox);

procedure Build_Memo(MemoIn: TMemo; AnswerDisplay: TDisplay;
  HintIn: string; TempHeight, TagIn: integer; WhichScrollBox: TScrollBox);

procedure Build_SpinEdit(SpinEdit: TRzSpinEdit; AnswerDisplay: TDisplay;
  HintIn: string; TempHeight, TagIn, Min, Max: integer; WhichKind: TResponseType;
  WhichScrollBox: TScrollBox);

procedure Build_Currency(Currency: TJvValidateEdit; AnswerDisplay: TDisplay;
  HintIn: string; TempHeight, TagIn, Min, Max: integer; WhichScrollBox: TScrollBox);

procedure Build_Items(Designator: TChoiceIdentifier;
  var StringListIn: TStringList);

implementation

uses
  uMHA_Const,
  uMHA_Utils,
  uMHA_GUIControl,
  uMHA_Introduction,
  uMHA_SectionLabel;


function GetWidth(StringIn: string): integer;
var
  i                           : Integer;
  TempStr                     : string;
  Ok                          : Boolean;
begin
  Result := DEFAULT_EDIT_BOX_WIDTH;
  TempStr := Piece(StringIn, DELIMITER, 1);
  if (TempStr = 'NA') or (TempStr = '') then
    Exit;
  Ok := true;
  for i := 1 to length(TempStr) do
    if not (TempStr[i] in ['0'..'9']) then
      Ok := False;
  if Ok then
    result := strtoint(TempStr);
  if result <= 0 then
    result := DEFAULT_EDIT_BOX_WIDTH;
end;


procedure Build_Items(Designator: TChoiceIdentifier;
  var StringListIn: TStringList);
var
  i                           : Integer;
  TempSL                      : TStringList;
begin
  if StringListIn.Count = 0 then
    Exit;

  TempSL := TStringList.Create;

  for i := 0 to StringListIn.Count - 1 do
    begin
      case Designator of
        ciNone: TempSL.Add(Piece(StringListIn[i], U, 3));
        ciZeroBased: TempSL.Add(inttostr(i) + '. ' + Piece(StringListIn[i], U, 3));
        ciOneBased: TempSL.Add(inttostr(i + 1) + '. ' + Piece(StringListIn[i], U, 3));
      end; //case Question.ChoiceIdentifier
    end;

  StringListIn.Clear;
  StringListIn.AddStrings(TempSL);
  TempSL.Free;
end;

function Build_Label(LabelIn: TLabel; Display, AnswerDisplay: TDisplay;
  StringIn, HintIn: string; TagIn: Integer; AResponseType: TResponseType;
  WhichScrollBox: TScrollBox): integer;
begin
  //Result := 0;
  with LabelIn, GUI_Control do
    begin
      Parent := WhichScrollBox;
      if AResponseType = rtTrackBar then
        Name := cTrackBarLabel1 + IntToStr(gvQuestionOrder)
      else
        Name := cLabel + IntToStr(gvQuestionOrder);

      Tag := TagIn;
      AutoSize := False;
      Top := gvQuestionTop;

      Left := Display.DisplayLeft;
      if Left < 2 then
        Left := 2;

      if (not AnswerDisplay.DisplaySameLine) or // Component on next line
        (AResponseType = rtTrackBar) then
        Width := WhichScrollBox.Width - gvScrollBarWidthWS - Left
      else
        Width := AnswerDisplay.DisplayLeft - Left - gvScrollBarWidthWS;

      if Width <= 0 then
        Width := 1;

      Font.Assign(Display.DisplayFont);
      Hint := HintIn;
      Alignment := Display.DisplayAlignment;
      WordWrap := True;
      Caption := StringIn;

      // Check for wrapping of the caption
      Height := GetWrapHeight(Caption, Width, Font);
      if Height > GUI_Control.GetStandardHeight(Font) then
        Layout := tlTop
      else
        Layout := tlCenter;
      Result := Height;
    end;
end;

procedure Build_Title(LabelIn: TLabel; Display: TDisplay;
  TagIn: integer;
  StringIn, HintIn: string; WhichScrollBox: TScrollBox);
begin
  with LabelIn, GUI_Control do
    begin
      Parent := WhichScrollBox;
      Name := cTitle + IntToStr(gvQuestionOrder);
      //TabStop := True;
      //TabOrder := gvQuestionOrder;
      Tag := TagIn;
      AutoSize := False;
      Top := gvQuestionTop;
      Left := Display.DisplayLeft;
      if Left < 2 then
        Left := 2;
      Width := WhichScrollBox.Width - Left - gvScrollBarWidthWS;
      Font.Assign(Display.DisplayFont);
      Hint := HintIn;
      Alignment := Display.DisplayAlignment;
      WordWrap := True;
      Caption := StringIn;
      // Check for wrapping the caption
      Height := GetWrapHeight(Caption, Width, Font);
      if Height > GUI_Control.GetStandardHeight(Font) then
        Layout := tlTop
      else
        Layout := tlCenter;
      gvQuestionTop := gvQuestionTop + Height;
    end;
end;

procedure Build_Instructions(LabelIn: TLabel; Display: TDisplay;
  TagIn: integer;
  StringIn: string; WhichScrollBox: TScrollBox);
begin
  with LabelIn, GUI_Control do
    begin
      Parent := WhichScrollBox;
      Name := cIntroduction + IntToStr(gvQuestionOrder);
      Tag := TagIn;
      AutoSize := False;
      Caption := StringIn;
      Top := gvQuestionTop;
      Left := Display.DisplayLeft;
      if Left < 2 then
        Left := 2;
      Alignment := Display.DisplayAlignment;
      Font.Assign(Display.DisplayFont);
      Width := WhichScrollBox.Width - Left - gvScrollBarWidthWS;
      WordWrap := True;
      // Check for wrapping the caption
      Height := GetWrapHeight(Caption, Width, Font);
      if Height > GUI_Control.GetStandardHeight(Font) then
        Layout := tlTop
      else
        Layout := tlCenter;
      gvQuestionTop := gvQuestionTop + Height;
    end;
end;

procedure Build_Section(LabelIn: TLabel; Display: TDisplay;
  TagIn: integer;
  StringIn: string; WhichScrollBox: TScrollBox);
begin
  with LabelIn, GUI_Control do
    begin
      Parent := WhichScrollBox;
      Name := cSection + IntToStr(gvQuestionOrder);
      Tag := TagIn;
      AutoSize := False;
      Top := gvQuestionTop;
      Left := Display.DisplayLeft;
      if Left < 2 then
        Left := 2;
      Alignment := Display.DisplayAlignment;
      Caption := StringIn;
      Width := WhichScrollBox.Width - Left - gvScrollBarWidthWS;
      Font.Assign(Display.DisplayFont);
      // Check for wrapping the caption
      Height := GetWrapHeight(Caption, Width, Font);
      if Height > GUI_Control.GetStandardHeight(Font) then
        Layout := tlTop
      else
        Layout := tlCenter;
      gvQuestionTop := gvQuestionTop + Height;
    end;
end;

procedure Build_RadioGroup(Radio: TRzRadioGroup;
  CaptionIn, HintIn: string;
  TagIn: Integer;
  Display,
  AnswerDisplay: TDisplay;
  IsLegacy: Boolean;
  WhichScrollBox: TScrollBox;
  AItems: TStringList);
var
  lColumns,
    lItemHeight,
    lHeight,
    lStartYPos                : Integer;
begin
  with Radio, GUI_Control do
    begin
      Parent := WhichScrollBox;
      Name := cRadioGroup + IntToStr(gvQuestionOrder);
      GroupStyle := gsCustom;
      Tag := TagIn;
      TabStop := True;
      TabOrder := gvQuestionOrder;

      Top := gvQuestionTop;
      Left := Display.DisplayLeft;
      if Left < 2 then
        Left := 2;

      Width := GetWidth(AnswerDisplay.DisplayMask);
      if Width = DEFAULT_EDIT_BOX_WIDTH then
        Width := WhichScrollBox.Width - gvScrollBarWidthWS - Left;

      ParentColor := false;
      ShowHint := True;
      Hint := HintIn;
      BorderSides := [];
      SpaceEvenly := True; // allows wrapping of items.
      ThemeAware := true;
      Font.Assign(Display.DisplayFont);
      ItemFont.Assign(AnswerDisplay.DisplayFont);
      UseCustomGlyphs := False;
      Caption := CaptionIn; //Question.Caption;
      Items.AddStrings(AItems);
      Columns := AnswerDisplay.DisplayColumns;
      VerticalSpacing := TextHeight('Yy', Font) div 4;

      if IsLegacy then // Indent multi-choice items
        StartXPos := 3 * gvQuestionLeft
      else
        StartXPos := AnswerDisplay.DisplayLeft;
      lColumns := Columns;
      lItemHeight := ItemHeight;
      lHeight := Height;
      lStartYPos := StartYPos;
      GUI_Control.ConfigureRadioGroup(Caption, Items, Font, ItemFont,
        gvDPI, Width, StartXPos, lColumns, lItemHeight, lHeight, lStartYPos);
      Columns := lColumns;
      ItemHeight := lItemHeight;
      StartYPos := lStartYPos;
      Height := lHeight;
      gvQuestionTop := gvQuestionSpace + gvQuestionTop + Height;
    end;
end;

procedure Build_ComboBox(ComboBox: TComboBox; AnswerDisplay: TDisplay;
  HintIn: string; TempHeight, TagIn: integer; WhichScrollBox: TScrollBox;
  AItems: TStringList);
begin

  with ComboBox, GUI_Control do
    begin
      Parent := WhichScrollBox;
      Name := cComboBox + IntToStr(gvQuestionOrder);
      Tag := TagIn;
      TabStop := True;
      TabOrder := gvQuestionOrder;
      Text := '';
      Top := gvQuestionTop;
      Width := GetWidth(AnswerDisplay.DisplayMask);
      if Width = DEFAULT_EDIT_BOX_WIDTH then
        Width := 3 * gvDefaultEditWidth div 2;
      Left := AnswerDisplay.DisplayLeft;
      //AutoSize := False;
      Font.Assign(AnswerDisplay.DisplayFont);
      Height := GUI_Control.GetStandardHeight(Font);
      ShowHint := True;
      Hint := HintIn;
      Style := csDropDownList;
      ItemIndex := -1;
      ItemHeight := GetItemHeight(Font);
      Sorted := false;
      DropDownCount := 8;
      Items.AddStrings(AItems);

      if not AnswerDisplay.DisplaySameLine then // next line
        begin
          gvQuestionTop := gvQuestionSpace + gvQuestionTop + Height;
          Exit;
        end;

      if TempHeight > Height then
        gvQuestionTop := gvQuestionSpace + gvQuestionTop + TempHeight
      else
        gvQuestionTop := gvQuestionSpace + gvQuestionTop + Height;
    end;
end;

procedure Build_CheckListBox(CheckList: TRzCheckList; AnswerDisplay: TDisplay;
  HintIn: string; TempHeight, TagIn: integer; WhichScrollBox: TScrollBox;
  AItems: TStringList);
begin
  with CheckList, GUI_Control do
    begin
      Parent := WhichScrollBox;
      //Style := lbOwnerDrawFixed;
      Name := cCheckList + IntToStr(gvQuestionOrder);
      Tag := TagIn;
      TabOrder := gvQuestionOrder;
      TabOnEnter := False;
      TabStop := True;
      //AutoSize := False;
      Top := gvQuestionTop;
      Left := AnswerDisplay.DisplayLeft;
      Width := GetWidth(AnswerDisplay.DisplayMask);
      if Width = DEFAULT_EDIT_BOX_WIDTH then
        Width := 3 * gvDefaultEditWidth div 2;
      Font.Assign(AnswerDisplay.DisplayFont);
      BorderStyle := bsSingle;
      Color := clWindow;
      Sorted := False;
      ShowHint := True;
      Hint := HintIn;
      MultiSelect := True;
      ExtendedSelect := True; // Allow shift key to work when selecting many items.
      IntegralHeight := false;
      ItemIndex := -1;
      ItemHeight := GetItemHeight(Font);
      ToggleOnItemClick := True;
      UseCustomGlyphs := False;
      Items.AddStrings(AItems);
      Columns := AnswerDisplay.DisplayColumns;
      Height := GUI_Control.HeightListBox(Width, Columns, Font,
        AItems);

      if not AnswerDisplay.DisplaySameLine then // next line
        begin
          gvQuestionTop := gvQuestionSpace + gvQuestionTop + Height;
          Exit;
        end;

      if TempHeight > Height then
        gvQuestionTop := gvQuestionSpace + gvQuestionTop + TempHeight
      else
        gvQuestionTop := gvQuestionSpace + gvQuestionTop + Height;
    end;
end;

procedure Build_ListBox(ListBox: TRzListBox; AnswerDisplay: TDisplay;
  HintIn: string; TempHeight, TagIn: integer; WhichScrollBox: TScrollBox;
  AItems: TStringList);
begin
  with ListBox, GUI_Control do
    begin
      Parent := WhichScrollBox;
      Style := lbOwnerDrawFixed;
      Name := cListBox + IntToStr(gvQuestionOrder);
      TabStop := True;
      TabOrder := gvQuestionOrder;
      Tag := TagIn;
      //AutoSize := False;
      Top := gvQuestionTop;
      Left := AnswerDisplay.DisplayLeft;
      Width := GetWidth(AnswerDisplay.DisplayMask);
      if Width = DEFAULT_EDIT_BOX_WIDTH then
        Width := 3 * gvDefaultEditWidth div 2;
      Font.Assign(AnswerDisplay.DisplayFont);
      ShowHint := True;
      Hint := HintIn;
      BorderStyle := bsSingle;
      Color := clWindow;
      Sorted := False;
      MultiSelect := False;
      IntegralHeight := false;
      ItemIndex := -1;
      ItemHeight := GetItemHeight(Font);
      TabOnEnter := False;
      //ToggleOnItemClick := true;
      //UseCustomGlyphs := False;
      Items.AddStrings(AItems);
      Columns := AnswerDisplay.DisplayColumns;
      Height := GUI_Control.HeightListBox(Width, Columns, Font,
        AItems);

      if not AnswerDisplay.DisplaySameLine then // next line
        begin
          gvQuestionTop := gvQuestionSpace + gvQuestionTop + Height;
          Exit;
        end;

      if TempHeight > Height then
        gvQuestionTop := gvQuestionSpace + gvQuestionTop + TempHeight
      else
        gvQuestionTop := gvQuestionSpace + gvQuestionTop + Height;
    end;
end;

procedure Build_VistaComboBox(VistaComboBox: TORComboBox; AnswerDisplay: TDisplay;
  HintIn: string; TempHeight, TagIn: integer; WhichScrollBox: TScrollBox);
begin
  with VistaComboBox, GUI_Control do
    begin
      Parent := WhichScrollBox;
      Name := 'cbv' + IntToStr(gvQuestionOrder);
      Tag := TagIn;
      TabStop := True;
      TabOrder := gvQuestionOrder;
      ShowHint := True;
      Hint := HintIn;
      Color := clWindow;
      Text := '';
      Top := gvQuestionTop;
      Left := AnswerDisplay.DisplayLeft;
      Width := GetWidth(AnswerDisplay.DisplayMask);
      if Width = DEFAULT_EDIT_BOX_WIDTH then
        Width := 3 * gvDefaultEditWidth div 2;
      //AutoSize := False;
      Font.Assign(AnswerDisplay.DisplayFont);
      Height := GUI_Control.GetStandardHeight(Font);
      CheckBoxes := False;
      CheckEntireLine := False;
      LongList := True;
      HideSynonyms := True;
      Sorted := True;
      Style := orcsDropDown;
      Pieces := '2';

      if not AnswerDisplay.DisplaySameLine then // next line
        begin
          gvQuestionTop := gvQuestionSpace + gvQuestionTop + Height;
          Exit;
        end;

      if TempHeight > Height then
        gvQuestionTop := gvQuestionSpace + gvQuestionTop + TempHeight
      else
        gvQuestionTop := gvQuestionSpace + gvQuestionTop + Height;

    end;
end;

procedure Build_TrackLabel2(LabelIn: TLabel; AnswerDisplay: TDisplay;
  TagIn: integer;
  WhichScrollBox: TScrollBox;
  Choices: TStringList);
// Labels for the slider-scale
var
  i                           : Integer;
  TempStr                     : string;
begin
  with LabelIn, GUI_Control do
    begin
      Parent := WhichScrollBox;
      Name := cTrackBarLabel2 + IntToStr(gvQuestionOrder);
      AutoSize := False;
      Top := gvQuestionTop;
      Tag := TagIn;
      Layout := tlCenter;
      Left := AnswerDisplay.DisplayLeft;
      if Left < gvTrackBarIndent div 2 then
        Left := gvTrackBarIndent div 2;
      //Width := GetWidth(Display.DisplayMask);
      //if Width = DEFAULT_EDIT_BOX_WIDTH then
      Width := WhichScrollBox.Width - gvScrollBarWidthWS - (2 * Left);
      Font.Assign(AnswerDisplay.DisplayFont);
      Height := GetTrackBarLabel2Height(Font);
      Alignment := taLeftJustify;

      WordWrap := False;
      TempStr := '';
      // Accept High & Low descriptions or High, Medium & Low.
      if ((Choices.Count > 1) or
        (Choices.Count < 4)) then
        begin
          for i := 0 to Choices.Count - 1 do
            TempStr := TempStr + Choices[i] + DELIMITER;
        end;

      Caption := ExpandTrackLabels(Width, TempStr, Font);
      if Caption = '' then
        Height := 0;
      gvQuestionTop := gvQuestionSpace + gvQuestionTop + Height;
    end;
end;

procedure Build_TrackBar(TrackBar: TExTrackBar; AnswerDisplay: TDisplay;
  HintIn: string; TempHeight, TagIn, MinIn, MaxIn: integer; WhichScrollBox: TScrollBox);
begin
  with TrackBar do
    begin
      Parent := WhichScrollBox;
      Name := cTrackBar + IntToStr(gvQuestionOrder);
      Tag := TagIn;
      TabStop := True;
      TabOrder := gvQuestionOrder;
      gvQuestionTop := gvQuestionTop; // + TempHeight;
      Top := gvQuestionTop;
      Left := AnswerDisplay.DisplayLeft + gvTrackBarIndent;
      if Left < gvTrackBarIndent then
        Left := gvTrackBarIndent;
      //Width := GetWidth(Display.DisplayMask);
      //if Width = DEFAULT_EDIT_BOX_WIDTH then
      Width := WhichScrollBox.Width - gvScrollBarWidthWS - (2 * Left);
      Font.Assign(AnswerDisplay.DisplayFont);
      LabelsLeftTop := 30;
      Height := GUI_Control.GetTrackHeight(Font);
      ShowHint := True;
      Hint := HintIn;
      LineSize := 1;
      PageSize := 1;
      Labels := True;
      LabelsInterval := 1;
      //LabelsLeftTop := Height - (Height div 3);
      Min := MinIn;
      Max := MaxIn;
      gvQuestionTop := gvQuestionTop + Height;
    end;
end;

procedure Build_Date(DateIn: TJvDatePickerEdit; AnswerDisplay: TDisplay;
  HintIn: string; TempHeight, TagIn: integer; WhichScrollBox: TScrollBox);
begin

  with DateIn do
    begin

      Parent := WhichScrollBox;
      Name := cDate + IntToStr(gvQuestionOrder);
      (*
      //-------------
      AllowNoDate := True;
      //DateFormat :=
      DirectInput := True;
      EnableValidation := True;
      HotTrack := False;
      NoDateText := '';
      ReadOnly := False;
      ShowCheckBox := False;
      StoreDate := False;
      StoreDateFormat := False;
        *)
      //------------------
      Tag := TagIn;
      TabStop := True;
      TabOrder := gvQuestionOrder;
      Top := gvQuestionTop;
      Font.Assign(AnswerDisplay.DisplayFont);
      Width := GetWidth(AnswerDisplay.DisplayMask);
      if Width = DEFAULT_EDIT_BOX_WIDTH then
        Width := GUI_Control.TextWidth('XXXX\XX\XXXXXX', Font);
      Left := AnswerDisplay.DisplayLeft;
      //AutoSize := False;
      Height := GUI_Control.GetStandardHeight(Font);
      ShowHint := True;
      Hint := HintIn;
      Color := clWindow;

      if not AnswerDisplay.DisplaySameLine then // next line
        begin
          gvQuestionTop := gvQuestionSpace + gvQuestionTop + Height;
          Exit;
        end;

      if TempHeight > Height then
        gvQuestionTop := gvQuestionSpace + gvQuestionTop + TempHeight
      else
        gvQuestionTop := gvQuestionSpace + gvQuestionTop + Height;
    end;
end;

procedure Build_Edit(Edit: TEdit; AnswerDisplay: TDisplay;
  HintIn: string; TempHeight, TagIn: integer; WhichScrollBox: TScrollBox);
begin
  with Edit do
    begin
      Parent := WhichScrollBox;
      Name := cEdit + IntToStr(gvQuestionOrder);
      Tag := TagIn; //This is Question.ID
      TabStop := True;
      TabOrder := gvQuestionOrder;
      Top := gvQuestionTop;
      Left := AnswerDisplay.DisplayLeft;
      Font.Assign(AnswerDisplay.DisplayFont);
      Height := GUI_Control.GetStandardHeight(Font);
      Width := GetWidth(AnswerDisplay.DisplayMask);
      if Width = DEFAULT_EDIT_BOX_WIDTH then
        Width := 3 * gvDefaultEditWidth div 2;
      ShowHint := True;
      Hint := HintIn;
      Color := clWindow;
      Text := '';

      if not AnswerDisplay.DisplaySameLine then // next line
        begin
          gvQuestionTop := gvQuestionSpace + gvQuestionTop + Height;
          Exit;
        end;

      if TempHeight > Height then
        gvQuestionTop := gvQuestionSpace + gvQuestionTop + TempHeight
      else
        gvQuestionTop := gvQuestionSpace + gvQuestionTop + Height;
    end;
end;

procedure Build_MaskEdit(MaskEdit: TMaskEdit; AnswerDisplay: TDisplay;
  HintIn: string; TempHeight, TagIn: integer; WhichScrollBox: TScrollBox);
begin
  with MaskEdit do
    begin
      Parent := WhichScrollBox;
      Name := cMaskEdit + IntToStr(gvQuestionOrder);
      AutoSize := False;
      Tag := TagIn;
      Top := gvQuestionTop;
      Left := AnswerDisplay.DisplayLeft;
      Font.Assign(AnswerDisplay.DisplayFont);
      PasswordChar := #0; // Allows text to seen
      EditMask := Piece(AnswerDisplay.DisplayMask, DELIMITER, 1);
      if EditMask = '' then
        EditMask := MASK_BLANK;
      Width := GUI_Control.TextWidth(EditMask + 'XX', Font);
      Height := GUI_Control.GetStandardHeight(Font);
      BorderStyle := bsSingle;
      ShowHint := True;
      Hint := HintIn;
      Color := clWindow;

      if not AnswerDisplay.DisplaySameLine then // next line
        begin
          gvQuestionTop := gvQuestionSpace + gvQuestionTop + Height;
          Exit;
        end;

      if TempHeight > Height then
        gvQuestionTop := gvQuestionSpace + gvQuestionTop + TempHeight
      else
        gvQuestionTop := gvQuestionSpace + gvQuestionTop + Height;
    end;
end;

procedure Build_Currency(Currency: TJvValidateEdit; AnswerDisplay: TDisplay;
  HintIn: string; TempHeight, TagIn, Min, Max: integer; WhichScrollBox: TScrollBox);
begin
  with Currency do
    begin
      Parent := WhichScrollBox;
      Name := cCurrency + IntToStr(gvQuestionOrder);
      AutoSize := False;
      Tag := TagIn;
      TabStop := True;
      TabOrder := gvQuestionOrder;
      Top := gvQuestionTop;
      Left := AnswerDisplay.DisplayLeft;
      Width := GetWidth(AnswerDisplay.DisplayMask);
      Font.Assign(AnswerDisplay.DisplayFont);
      Height := GUI_Control.GetStandardHeight(Font);
      ShowHint := True;
      ShowHint := True;
      Hint := HintIn;
      Color := clWindow;

      ReadOnly := False;
      DisplayFormat := dfCurrency;
      ZeroEmpty := true;
      HasMinValue := true;
      HasMaxValue := true;
      MinValue := Min;
      MaxValue := Max;

      Alignment := AnswerDisplay.DisplayAlignment;

      if not AnswerDisplay.DisplaySameLine then // next line
        begin
          gvQuestionTop := gvQuestionSpace + gvQuestionTop + Height;
          Exit;
        end;

      if TempHeight > Height then
        gvQuestionTop := gvQuestionSpace + gvQuestionTop + TempHeight
      else
        gvQuestionTop := gvQuestionSpace + gvQuestionTop + Height;
    end;
end;

procedure Build_SpinEdit(SpinEdit: TRzSpinEdit; AnswerDisplay: TDisplay;
  HintIn: string; TempHeight, TagIn, Min, Max: integer; WhichKind: TResponseType; WhichScrollBox: TScrollBox);
// Presently the only type of WhichKind is "rtInteger," later there maybe "rtReal"
begin
  with SpinEdit do
    begin
      Parent := WhichScrollBox;
      Name := cSpinEdit + IntToStr(gvQuestionOrder);
      AutoSize := False;
      Tag := TagIn;
      TabStop := True;
      TabOrder := gvQuestionOrder;
      Top := gvQuestionTop;
      Left := AnswerDisplay.DisplayLeft;
      Width := GetWidth(AnswerDisplay.DisplayMask);
      Font.Assign(AnswerDisplay.DisplayFont);
      Height := GUI_Control.GetStandardHeight(Font);
      Hint := HintIn;
      Color := clWindow;

      ReadOnly := False;
      AllowKeyEdit := True;
      //Alignment := Display.AnswerAlignment;
      Alignment := taLeftJustify;
      AllowBlank := True;
      AutoSelect := True;
      Min := Min;
      Max := Max;
//      if (0 > MinValue) and (0 < MaxValue) then
//        Value := 0
//      else
//        Value := MinValue;
      //Text := ''; //AnswerString(Question.Tag);

      if WhichKind <> rtInteger then
        begin
          IntegersOnly := false;
          Decimals := 2;
          Increment := 1;
        end
      else
        begin
          IntegersOnly := True;
          Decimals := 0;
          Increment := 1;
        end;
      if not AnswerDisplay.DisplaySameLine then // next line
        begin
          gvQuestionTop := gvQuestionSpace + gvQuestionTop + Height;
          Exit;
        end;
      if TempHeight > Height then
        gvQuestionTop := gvQuestionSpace + gvQuestionTop + TempHeight
      else
        gvQuestionTop := gvQuestionSpace + gvQuestionTop + Height;
    end;
end;

procedure Build_Memo(MemoIn: TMemo; AnswerDisplay: TDisplay;
  HintIn: string; TempHeight, TagIn: integer; WhichScrollBox: TScrollBox);
begin
  with MemoIn do
    begin
      Parent := WhichScrollBox;
      Name := cMemo + IntToStr(gvQuestionOrder);
      Tag := TagIn;
      TabStop := True;
      TabOrder := gvQuestionOrder;
      ShowHint := True;
      Hint := HintIn;
      Color := clWindow;
      Top := gvQuestionTop;
      Left := AnswerDisplay.DisplayLeft;
      Width := GetWidth(AnswerDisplay.DisplayMask);
      if Width = DEFAULT_EDIT_BOX_WIDTH then
        Width := WhichScrollBox.Width - gvScrollBarWidthWS - Left;
      Font.Assign(AnswerDisplay.DisplayFont);
      Height := GUI_Control.GetMemoHeight(Font);
      WordWrap := True;
      WantTabs := false;
      WantReturns := True;
      ScrollBars := ssVertical;
      ReadOnly := False;

      if not AnswerDisplay.DisplaySameLine then // Next line
        begin
          gvQuestionTop := gvQuestionSpace + gvQuestionTop + Height;
          Exit;
        end;

      if TempHeight > Height then
        gvQuestionTop := gvQuestionSpace + gvQuestionTop + TempHeight
      else
        gvQuestionTop := gvQuestionSpace + gvQuestionTop + Height;
    end;
end;


(*function GetSameLine(StringIn: string): Char;
// Input is Answer.Mask. GetSameLine can be "S" = same line or
// "N" = next line.
var
  TempStr                     : string;
begin
  Result := SAME_LINE; // Same line
  TempStr := Piece(StringIn, DELIMITER, 3);

  if (Length(Trim(TempStr)) > 1) or (Length(Trim(TempStr)) < 1) then
    exit;
  if (TempStr[1] = NEXT_LINE) or (TempStr[1] = SAME_LINE) then
    Result := TempStr[1];
end;

function GetMemoSameLine(StringIn: string): Char;
// Input is Answer.Mask. GetSameLine can be "S" = same line or
// "N" = next line.
var
  TempStr                     : string;
begin
  Result := NEXT_LINE; // Next line
  TempStr := Piece(StringIn, DELIMITER, 3);

  if (Length(Trim(TempStr)) > 1) or (Length(Trim(TempStr)) < 1) then
    exit;
  if (TempStr[1] = NEXT_LINE) or (TempStr[1] = SAME_LINE) then
    Result := TempStr[1];
end; *)

end.

