
unit uMHA_GAF_IO;

interface

uses
  SysUtils,
  DateUtils,
  Classes,
  Diaccess,
  dialogs,
  TRPCb;


function DeleteGAF(aIENS: string;
  Broker: TRPCBroker): string;
function WriteNewGAFRating(aDFN, aDUZ, aDate, aGAFScore: string;
  Broker: TRPCBroker): string;
function WriteEditedGAFRating(aIENS: string; aRating: integer;
  Broker: TRPCBroker): boolean;
procedure WriteGAFEnteredInError(aIENS: string; AUserName: string;
  Broker: TRPCBroker);
function ReadPreviousGAFRatings(ADFN: string;
  Broker: TRPCBroker): string;


implementation

uses
  uIA_Const,
  uMHA_Const,
  uIA_Utils,
  uMHA_GAF,
  uMHA_GAF_OPS,
  uMHA_DM,
  uMHA_LocalFiles,
  uIA_ThisUser;

function DeleteGAF(aIENS: string;
  Broker: TRPCBroker): string;

//  NUMBER: 2813                          NAME: YTQ DELETE ENTRY
//  TAG: DELENTRY                         ROUTINE: YTQAPI6
//  RETURN VALUE TYPE: ARRAY              AVAILABILITY: SUBSCRIPTION
//  INACTIVE: ACTIVE
//DESCRIPTION:
// Deletes an entry from a file.


// Input: YSDATA=Array name to return success/error response
//        YS("FILE")=File number (e.g., 627.8)
//        YS("IEN")=record internal entry number (e.g., 100123)
// Output: YSDATA(1)="[DATA]"
//         or
//         YSDATA(1)="[ERROR]"
//         YSDATA(2)="error message"
//
// Example:
//  > S YSDATA="TEST",YS("FILE")=627.8,YS("IEN")=100123
//  > D DELENTRY^YTQAPI(YSDATA,.YS)


var
    FMFile,
    tmpIENS                   : string;
begin
  Result := '';
  FMFile := '627.8';
  tmpIENS := aIENS;

  with Broker do
    begin
      ClearParameters := True;
      ClearResults := True;
      CreateContext('YS BROKER1');
      RemoteProcedure := 'YTQ DELETE ENTRY';
      Param[0].Mult['"FILE"'] := FMFile;
      Param[0].Mult['"IEN"'] := tmpIENS;
      Param[0].PType := List;

      try
        Call;

      except
        on E: EBrokerError do
          begin
              //* LocalFiles.UpdateAPILog('Call to YTQ DELETE ENTRY: Error ');
              //* LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            result := 'Error: ' + E.message;
            exit;
          end;
        on E: EAccessViolation do
          begin
              //* LocalFiles.UpdateAPILog('Call to YTQ DELETE ENTRY: Error ');
              //* LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            result := 'Error: ' + E.message;
            exit;
          end;
      end; // try

      if results[0] <> DATA then
        begin
          result := results[1];
            //* LocalFiles.UpdateAPILog('Call to YTQ DELETE ENTRY: ' + result);
        end
      else
        Result := DATA;
          //* LocalFiles.UpdateAPILog('Call to YTQ DELETE ENTRY: Successful');
    end; //if...
end;


function WriteNewGAFRating(aDFN, aDUZ, aDate, aGAFScore: string;
  Broker: TRPCBroker): string;

begin
  //NB:  This procedure does not set the GAF.IENS except for dbOffLine
  //In MHA3, this doesn't matter because the method is followed immediately
  //by ReadPreviousRatings, which gets the IENS.
  //The failure of this procedure to set the IENS would be a problem if it
  //were not followed by ReadPreviousRatings in some future application.

  with Broker do
    begin
      ClearResults := True;
      ClearParameters := True;
      CreateContext('YS BROKER1');
      //* LocalFiles.UpdateAPILog('Call to YTAPI NEW GAF: ' +
      //*   ' DFN=' + aDFN +
      //*   ' GAF=' + aGAFScore +
      //*   ' DATE=' + aDate +
      //*   ' STAFF=' + aDUZ);
      RemoteProcedure := 'YTAPI NEW GAF';
      Param[0].Mult['"DFN"'] := aDFN;
      Param[0].Mult['"GAF"'] := aGAFScore;
      Param[0].Mult['"DATE"'] := aDate;
      Param[0].Mult['"STAFF"'] := aDUZ;
      Param[0].PType := List;

      try
        Call;

      except
        on E: EBrokerError do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTAPI NEW GAF: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTAPI NEW GAF: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

      if results[0] = '[DATA]' then
        begin
          result := OK;
          //* LocalFiles.UpdateAPILog('Call to YTAPI NEW GAF: Successful');
        end
      else
        begin
          result := results[0];
          //* LocalFiles.UpdateAPILog('Call to YTAPI NEW GAF: Error');
          //* LocalFiles.UpdateAPILog('   ' + Results[0]);
        end;
    end; //with Broker
end;


function WriteEditedGAFRating(aIENS: string; aRating: Integer;
  Broker: TRPCBroker): boolean;
var
  strScore                    : string;
begin
  result := True;
  strScore := IntToStr(aRating);

  case DataBaseMode of
    dbVista:
      begin
        DataModule1.FMFiler1.RPCBroker := Broker;
        DataModule1.FMValidator1.RPCBroker := Broker;

        //Validate the value of the field
        DataModule1.FMValidator1.Setup('627.8', aIENS, '65', strScore);
        if not DataModule1.FMValidator1.Validate then
          DataModule1.FMValidator1.DisplayErrors
        else
          begin {file}
          //Put validated field for record on Filer's change list
            DataModule1.FMFiler1.AddFDA('627.8', aIENS, '65', strScore);
          //(FileNumber, IENS, FieldNumber, NewScore)
          //File record
            if not DataModule1.FMFiler1.Update then
              begin
                DataModule1.FMFiler1.DisplayErrors;
                result := False;
              end;
          end;
      end; //casedbVista
  end; //case DatabaseMode
end;

procedure WriteGAFEnteredInError(aIENS: string; AUserName: string;
  Broker: TRPCBroker);
var
  ErrorMsg                    : string;

begin

  ErrorMsg := 'Error: entered in error noted on ' // ex. FEB 7,2002@09:45:03
    + FormatFMDateTime('mmm dd, yyyy' + ' @ ' + 'hh:mm:ss', DateTimeToFMDateTime(Now))
    + ' by ' + AUserName; // ex. MHPROVIDER,ONE

  case DataBaseMode of
    dbVista:
      begin
        ErrorMsg := StringToPieces(ErrorMsg, U, 200);

        with Broker do
          begin
            ClearResults := True;
            ClearParameters := True;
            CreateContext('YS BROKER1');
            //* LocalFiles.UpdateAPILog('Call to YTQ WP FILER:' +
            //*   ' FILEN=627.8' +
            //*   ' IEN=' + aIENS +
            //*   ' FIELD=80' +
            //*   ' 1=' + ErrorMsg);
            RemoteProcedure := 'YTQ WP FILER';
            Param[0].Mult['"FILEN"'] := '627.8';
            Param[0].Mult['"IEN"'] := aIENS;
            Param[0].Mult['"FIELD"'] := '80';
            Param[0].Mult['"1"'] := ErrorMsg;

//            for i := 1 to NumberOfPieces(ErrorMsg, U) do
//              begin
//                YS := '"' + IntToStr(i) + '"';
//                Param[0].Mult[YS] := Piece(ErrorMsg, U, i);
//              end;

            Param[0].PType := List;

            try
              Call;

            except
              on E: EBrokerError do
                begin
                  //* LocalFiles.UpdateAPILog('Call to YTQ WP FILER: Error ');
                  //* LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
                  exit;
                end;
              on E: EAccessViolation do
                begin
                  //* LocalFiles.UpdateAPILog('Call to YTQ WP FILER: Error ');
                  //* LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
                  exit;
                end;
            end; // try
          end;
        //* LocalFiles.UpdateAPILog('Call to YTQ WP FILER: Successful');
      end; //dbVista
  end; //case DatabaseMode
end;


function ReadPreviousGAFRatings(aDFN: string;
  Broker: TRPCBroker): string;
var
  TempStr                     : string;
  i, x                        : integer;
  TempGAF                     : TGAF;

begin
  Result := OK;

  with Broker do
    begin
      ClearParameters := True;
      ClearResults := True;
      CreateContext('YS BROKER1');
      //* LocalFiles.UpdateAPILog('Call to YTQ GAF HX: DFN=' + aDFN);
      RemoteProcedure := 'YTQ GAF HX';
      Param[0].Mult['"DFN"'] := aDFN;
      // Possible parameters, but not used
      // Param[0].Mult['"END"'] := DateToStr(Date);// Date of last GAF requested [optional]
      // Param[0].Mult['"BEGIN"'] := '01/01/1994';// Date of first GAF requested [optional]
      // Param[0].Mult['"LIMIT"'] := '20'; // To limit output to the last x diagnosis
      Param[0].PType := List;

      try
        Call;

      except
        on E: EBrokerError do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTQ GAF HX: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTQ GAF HX: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

      // Sample return:
      // [DATA]
      // 11358=01/12/2004^85^11678^MHPROVIDER,ONE^
      // 11355=01/06/2004^45^5325^MHPROVIDER,TWO^
      // 11356=01/06/2004^85^5325^MHPROVIDER,TWO^
      // 11357=01/01/2004^25^5325^MHPROVIDER,TWO^Error: entered in error noted on Jan 07,2004@07:01:29 by MHPROVIDER,TWO

      if Results[0] = DATA then
        begin
          //* LocalFiles.UpdateAPILog('Call to YTQ GAF HX: Successful');

          GAF_OPS.ListOfGAFs.Clear;
          GAF_OPS.RatingIsDue := True;

          if Results.Count < 2 then
            Exit;

          for i := 1 to Results.Count - 1 do
            begin

              TempGAF := TGAF.Create;

              with TempGAF do
                begin
                  IENS := Piece(Results[i], EQ, 1);
                  TempStr := Piece(Results[i], EQ, 2);
                  DateString := Piece(TempStr, U, 1);
                  x := Pos('@', DateString);
                  if x > 0 then
                    DateString := Copy(DateString, 1, x - 1);
                  GAFDate := StrToDate(DateString);
                  Rating := StrToIntDef(Piece(TempStr, U, 2), 1);
                  if (Rating > 100) then
                    Rating := 100;
                  RaterDUZ := Piece(TempStr, U, 3);
                  Rater := Piece(TempStr, U, 4);
                  EnteredInError := Pos('Error', Piece(TempStr, U, 5)) > 0;
                  UserCanModify := (RaterDUZ = ThisUser.DUZ) and not EnteredInError;
                  CanBeEdited := UserCanModify and (DaysBetween(Now, GAFDate) < 3);
                  CanBeMarkedInError := UserCanModify and not CanBeEdited;
                  CanBeModified := CanBeEdited or CanBeMarkedInError;
                  IsSaved := True;
                  if DaysBetween(Date, GAFDate) < 90 then
                    GAF_OPS.RatingIsDue := False;
                end; // with TempGAF

              GAF_OPS.ListOfGAFs.Add(TempGAF);
            end;

        end // if [DATA]
      else
        begin
          //* LocalFiles.UpdateAPILog('Call to YTQ GAF HX: Error');
          //* LocalFiles.UpdateAPILog('   ' + results[0]);
          Result := BAD_DATA;
        end;

    end; // with DataModule1.RPCBroker1
end;

end.

