unit uMHA_AUX_BASIS;

interface

uses
  SysUtils;

function ScoreAnswers_BASIS(
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string): string;

function ProgressNote_BASIS(
  PatientName,
  PatientDOB,
  PatientSex,
  ScoreString,
  AnswerString: string): string;


implementation

uses
  uMHA_AUX_Const,
  fIA_dialogs,
  uMHA_AUX_Utils;


{*
  The format for output is used by Clinical reminders as their preferred method.
  The second field in the output is the Scale IEN, which is not used in IAT.

  The BASIS-24 has six scales and an overall score.
  A score is not reported if there are two or more questions not answered;
  this includes the overall score.

*}



// Weighted sum scoring method provided by author

function ScoreAnswers_BASIS(
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string): string;

const
  Spacer                      : string = '      ';

var
  Count, i,
    Dep, Inter, Psy, Alc, Emo, Harm: Integer;

  DepWt, InterWt, PsyWt, AlcWt, EmoWt, HarmWt, TotalWt,
    DepSum, InterSum, PsySum, AlcSum, EmoSum, HarmSum,
    TotalSum
                              : Extended;
  TempResult,
    TempAnswer                : string;

begin
  // AnswerString Format:
  //      83=1^1330~
  //      84=0^214~...
  //
  // '83' is QuestionID,
  // '1' is ItemIndex for radio group, but can be text from edit field
  // '1130' is ChoiceID (not used here)

  // This is used by InstrumentResults to do a quick check on the
  // instrument as to its use of the dll
  if AnswerString = 'BASIS-24' then
  begin
     Result := 'YES';
     exit;
  end;

  Result := '';
  Count := NumberOfPieces(AnswerString, RECORD_END);

  if Count < 1 then
    Exit;

  Dep := 0;
  Inter := 0;
  Psy := 0;
  Alc := 0;
  Emo := 0;
  Harm := 0;

  DepWt := 0;
  InterWt := 0;
  PsyWt := 0;
  AlcWt := 0;
  EmoWt := 0;
  HarmWt := 0;
  TotalWt := 0;

  DepSum := 0;
  InterSum := 0;
  PsySum := 0;
  AlcSum := 0;
  EmoSum := 0;
  HarmSum := 0;
  TotalSum := 0;

  for i := 1 to Count - 1 do
    begin
      TempResult := Piece(AnswerString, RECORD_END, i);
      TempAnswer := Piece(Piece(TempResult, U, 1), EQ, 2);

      // Count the number of skipped (Blank) answers for each scale
      if (TempAnswer = '') or (TempAnswer = ' ') or
        (TempAnswer = '-1') or
        (TempAnswer = NOT_ASKED) or
        (TempAnswer = SKIPPED) or
        (TempAnswer = CHOICE_ID_OF_SKIPPED_MULTIPLE_CHOICE_QUESTION) or
        (TempAnswer = CHOICE_ID_OF_SKIPPED_BY_RULE_MC_QUESTION) or
        (TempAnswer = CHOICE_ID_OF_SKIPPED_REQUIRED_ANSWER) then
        begin
          TempAnswer := '0';
          // Depression
          if (i = 1) then
            begin
              DepWt := 0.186;
              TotalWt := 0.0696;
              Inc(Dep);
            end;
          if (i = 2) then
            begin
              DepWt := 0.2612;
              TotalWt := 0.1061;
              Inc(Dep);
            end;
          if (i = 3) then
            begin
              DepWt := 0.1763;
              TotalWt := 0.07;
              Inc(Dep);
            end;
          if (i = 9) then
            begin
              DepWt := 0.0882;
              TotalWt := 0.0565;
              Inc(Dep);
            end;

          if (i = 10) then
            begin
              DepWt := 0.1679;
              TotalWt := 0.0942;
              Inc(Dep);
            end;
          if (i = 12) then
            begin
              DepWt := 0.1204;
              TotalWt := 0.0586;
              Inc(Dep);
            end;
          // Interpersonal Problems
          if (i = 4) then
            begin
              InterWt := 0.1583;
              TotalWt := 0.0251;
              Inc(Inter);
            end;
          if (i = 5) then
            begin
              InterWt := 0.3299;
              TotalWt := 0.0279;
              Inc(Inter);
            end;
          if (i = 6) then
            begin
              InterWt := 0.2754;
              TotalWt := 0.0297;
              Inc(Inter);
            end;
          if (i = 7) then
            begin
              InterWt := 0.1362;
              TotalWt := 0.027;
              Inc(Inter);
            end;
          if (i = 8) then
            begin
              InterWt := 0.1002;
              TotalWt := 0.0241;
              Inc(Inter);
            end;

          // Psycholtic Symptoms
          if (i = 14) then
            begin
              PsyWt := 0.1049;
              TotalWt := 0.0164;
              Inc(Psy);
            end;
          if (i = 15) then
            begin
              PsyWt := 0.136;
              TotalWt := 0.023;
              Inc(Psy);
            end;
          if (i = 16) then
            begin
              PsyWt := 0.4636;
              TotalWt := 0.0243;
              Inc(Psy);
            end;
          if (i = 17) then
            begin
              PsyWt := 0.2955;
              TotalWt := 0.0304;
              Inc(Psy);
            end;
          // Alcohol/Drug Use
          if (i = 21) then
            begin
              AlcWt := 0.34;
              TotalWt := 0.0178;
              Inc(Alc);
            end;
          if (i = 22) then
            begin
              AlcWt := 0.234;
              TotalWt := 0.0135;
              Inc(Alc);
            end;
          if (i = 23) then
            begin
              AlcWt := 0.1556;
              TotalWt := 0.01;
              Inc(Alc);
            end;
          if (i = 24) then
            begin
              AlcWt := 0.2704;
              TotalWt := 0.0128;
              Inc(Alc);
            end;
          // Emotional Lability
          if (i = 13) then
            begin
              EmoWt := 0.1102;
              TotalWt := 0.0384;
              Inc(Emo);
            end;
          if (i = 18) then
            begin
              EmoWt := 0.6132;
              TotalWt := 0.065;
              Inc(Emo);
            end;
          if (i = 19) then
            begin
              EmoWt := 0.2766;
              TotalWt := 0.0589;
              Inc(Emo);
            end;
          // Self Harm
          if (i = 11) then
            begin
              HarmWt := 0.4175;
              TotalWt := 0.0483;
              Inc(Harm);
            end;
          if (i = 20) then
            begin
              HarmWt := 0.5825;
              TotalWt := 0.0524;
              Inc(Harm);
            end;
        end // Skipped questions
      else
        // Tally the scale's sum and total sum
        begin
          // Depression
          if (i = 1) then
            begin
              DepSum := DepSum + StrToIntDef(TempAnswer, 0) * 0.1860;
              TotalSum := TotalSum + StrToIntDef(TempAnswer, 0) * 0.0696;
            end;
          if (i = 2) then
            begin
              DepSum := DepSum + StrToIntDef(TempAnswer, 0) * 0.2612;
              TotalSum := TotalSum + StrToIntDef(TempAnswer, 0) * 0.1061;
            end;
          if (i = 3) then
            begin
              DepSum := DepSum + StrToIntDef(TempAnswer, 0) * 0.1763;
              TotalSum := TotalSum + StrToIntDef(TempAnswer, 0) * 0.07;
            end;
          // Reverse this item
          if (i = 9) then
            begin
              DepSum := DepSum + (4 - StrToIntDef(TempAnswer, 0)) * 0.0882;
              TotalSum := TotalSum + (4 - StrToIntDef(TempAnswer, 0)) * 0.0565;
            end;
          if (i = 10) then
            begin
              DepSum := DepSum + StrToIntDef(TempAnswer, 0) * 0.1679;
              TotalSum := TotalSum + StrToIntDef(TempAnswer, 0) * 0.0942;
            end;
          if (i = 12) then
            begin
              DepSum := DepSum + StrToIntDef(TempAnswer, 0) * 0.1204;
              TotalSum := TotalSum + StrToIntDef(TempAnswer, 0) * 0.0586;
            end;


          // Interpersonal Problems: All reversed in scoring
          if (i = 4) then
            begin
              InterSum := InterSum + (4 - StrToIntDef(TempAnswer, 0)) * 0.1583;
              TotalSum := TotalSum + (4 - StrToIntDef(TempAnswer, 0)) * 0.0251;
            end;
          if (i = 5) then
            begin
              InterSum := InterSum + (4 - StrToIntDef(TempAnswer, 0)) * 0.3299;
              TotalSum := TotalSum + (4 - StrToIntDef(TempAnswer, 0)) * 0.0279;
            end;
          if (i = 6) then
            begin
              InterSum := InterSum + (4 - StrToIntDef(TempAnswer, 0)) * 0.2754;
              TotalSum := TotalSum + (4 - StrToIntDef(TempAnswer, 0)) * 0.0297;
            end;
          if (i = 7) then
            begin
              InterSum := InterSum + (4 - StrToIntDef(TempAnswer, 0)) * 0.1362;
              TotalSum := TotalSum + (4 - StrToIntDef(TempAnswer, 0)) * 0.027;
            end;
          if (i = 8) then
            begin
              InterSum := InterSum + (4 - StrToIntDef(TempAnswer, 0)) * 0.1002;
              TotalSum := TotalSum + (4 - StrToIntDef(TempAnswer, 0)) * 0.0241;
            end;

          // Psycholtic Symptoms
          if (i = 14) then
            begin
              PsySum := PsySum + StrToIntDef(TempAnswer, 0) * 0.1049;
              TotalSum := TotalSum + StrToIntDef(TempAnswer, 0) * 0.0164;
            end;
          if (i = 15) then
            begin
              PsySum := PsySum + StrToIntDef(TempAnswer, 0) * 0.136;
              TotalSum := TotalSum + StrToIntDef(TempAnswer, 0) * 0.023;
            end;
          if (i = 16) then
            begin
              PsySum := PsySum + StrToIntDef(TempAnswer, 0) * 0.4636;
              TotalSum := TotalSum + StrToIntDef(TempAnswer, 0) * 0.0243;
            end;
          if (i = 17) then
            begin
              PsySum := PsySum + StrToIntDef(TempAnswer, 0) * 0.2955;
              TotalSum := TotalSum + StrToIntDef(TempAnswer, 0) * 0.0304;
            end;

          // Alcohol/Drug Use
          if (i = 21) then
            begin
              AlcSum := AlcSum + StrToIntDef(TempAnswer, 0) * 0.34;
              TotalSum := TotalSum + StrToIntDef(TempAnswer, 0) * 0.0178;
            end;
          if (i = 22) then
            begin
              AlcSum := AlcSum + StrToIntDef(TempAnswer, 0) * 0.234;
              TotalSum := TotalSum + StrToIntDef(TempAnswer, 0) * 0.0135;
            end;
          if (i = 23) then
            begin
              AlcSum := AlcSum + StrToIntDef(TempAnswer, 0) * 0.1556;
              TotalSum := TotalSum + StrToIntDef(TempAnswer, 0) * 0.01;
            end;
          if (i = 24) then
            begin
              AlcSum := AlcSum + StrToIntDef(TempAnswer, 0) * 0.2704;
              TotalSum := TotalSum + StrToIntDef(TempAnswer, 0) * 0.0128;
            end;

          // Emotional Lability
          if (i = 13) then
            begin
              EmoSum := EmoSum + StrToIntDef(TempAnswer, 0) * 0.1102;
              TotalSum := TotalSum + StrToIntDef(TempAnswer, 0) * 0.0384;
            end;
          if (i = 18) then
            begin
              EmoSum := EmoSum + StrToIntDef(TempAnswer, 0) * 0.6132;
              TotalSum := TotalSum + StrToIntDef(TempAnswer, 0) * 0.065;
            end;
          if (i = 19) then
            begin
              EmoSum := EmoSum + StrToIntDef(TempAnswer, 0) * 0.2766;
              TotalSum := TotalSum + StrToIntDef(TempAnswer, 0) * 0.0589;
            end;

          // Self Harm                        
          if (i = 11) then
            begin
              HarmSum := HarmSum + StrToIntDef(TempAnswer, 0) * 0.4175;
              TotalSum := TotalSum + StrToIntDef(TempAnswer, 0) * 0.0483;
            end;
          if (i = 20) then
            begin
              HarmSum := HarmSum + StrToIntDef(TempAnswer, 0) * 0.5825;
              TotalSum := TotalSum + StrToIntDef(TempAnswer, 0) * 0.0524;
            end;

        end; // Tally scale's sum
    end; // for i

  if Dep = 0 then // Number of skipped questions is nil
    Result := Spacer + 'Depression and Functioning^^' + FloatToStrF(DepSum, ffFixed, 8, 2) + RECORD_END;
  if Dep = 1 then
    Result := Spacer + 'Depression and Functioning^^' + FloatToStrF(DepSum / (1 - DepWt), ffFixed, 8, 2) + RECORD_END;
  if Dep > 1 then
    Result := Spacer + 'Depression and Functioning^^Too many skipped questions' + RECORD_END;

  if Inter = 0 then
    Result := Result + Spacer + 'Interpersonal Problems^^' + FloatToStrF(InterSum, ffFixed, 8, 2) + RECORD_END;
  if Inter = 1 then
    Result := Result + Spacer + 'Interpersonal Problems^^' + FloatToStrF(InterSum / (1 - InterWt), ffFixed, 8, 2) + RECORD_END;
  if Inter > 1 then
    Result := Result + Spacer + 'Interpersonal Problems^^Too many skipped questions' + RECORD_END;

  if Psy = 0 then
    Result := Result + Spacer + 'Psychotic Symptoms^^' + FloatToStrF(PsySum, ffFixed, 8, 2) + RECORD_END;
  if Psy = 1 then
    Result := Result + Spacer + 'Psychotic Symptoms^^' + FloatToStrF(PsySum / (1 - PsyWt), ffFixed, 8, 2) + RECORD_END;
  if Psy > 1 then
    Result := Result + Spacer + 'Psychotic Symptoms^^Too many skipped questions' + RECORD_END;

  if Alc = 0 then
    Result := Result + Spacer + 'Alcohol/Drug Use^^' + FloatToStrF(AlcSum, ffFixed, 8, 2) + RECORD_END;
  if Alc = 1 then
    Result := Result + Spacer + 'Alcohol/Drug Use^^' + FloatToStrF(AlcSum / (1 - AlcWt), ffFixed, 8, 2) + RECORD_END;
  if Alc > 1 then
    Result := Result + Spacer + 'Alcohol/Drug Use^^Too many skipped questions' + RECORD_END;

  if Emo = 0 then
    Result := Result + Spacer + 'Emotional Lability^^' + FloatToStrF(EmoSum, ffFixed, 8, 2) + RECORD_END;
  if Emo = 1 then
    Result := Result + Spacer + 'Emotional Lability^^' + FloatToStrF(EmoSum / (1 - EmoWt), ffFixed, 8, 2) + RECORD_END;
  if Emo > 1 then
    Result := Result + Spacer + 'Emotional Lability^^Too many skipped questions' + RECORD_END;

  if Harm = 0 then
    Result := Result + Spacer + 'Self-Harm^^' + FloatToStrF(HarmSum, ffFixed, 8, 2) + RECORD_END;
  if Harm = 1 then
    Result := Result + Spacer + 'Self-Harm^^' + FloatToStrF(HarmSum / (1 - HarmWt), ffFixed, 8, 2) + RECORD_END;
  if Harm > 1 then
    Result := Result + Spacer + 'Self-Harm^^Too many skipped questions' + RECORD_END;

  if (Dep + Inter + Psy + Alc + Emo + Harm) = 0 then
    Result := Result + Spacer + 'Overall^^' + FloatToStrF(TotalSum, ffFixed, 8, 2) + RECORD_END;

  if (Dep + Inter + Psy + Alc + Emo + Harm) = 1 then
    Result := Result + Spacer + 'Overall^^' + FloatToStrF(TotalSum / (1 - TotalWt), ffFixed, 8, 2) + RECORD_END;

  if (Dep + Inter + Psy + Alc + Emo + Harm) > 1 then
    Result := Result + Spacer + 'Overall^^Too many skipped questions' + RECORD_END;
end;


function ProgressNote_BASIS(
  PatientName,
  PatientDOB,
  PatientSex,
  ScoreString,
  AnswerString: string): string;

// The header, footer, questions and answers are handled elsewhere; avoiding
// broker calls here. <.DLL_String.> provides location for this string in the
// report.

const
  Indent                      : string = '   ';
var
  i                           : Integer;
  TempStr                     : string;
begin
  Result := '';

  if (ScoreString = '') or (AnswerString = '') then
    Exit;

  Result := Indent + 'BASIS-24 Scales' + #13;

  for i := 1 to NumberOfPieces(ScoreString, RECORD_END) - 1 do
    begin
      TempStr := Piece(Piece(ScoreString, RECORD_END, i), U, 1);
      while Length(TempStr) < 35 do
        TempStr := TempStr + ' ';
      Result := Result + Indent + TempStr +
        Piece(Piece(ScoreString, RECORD_END, i), U, 3) + #13;
    end;

  Result := Result + #13 + Indent +
    'Note: Lower scores indicate less frequent symptoms or difficulty' + #13 +
    Indent +
    '      whereas higher scores reflect more serious symptoms or difficulty.'+ #13 +
    Indent +
    '      Range of scores: 0 to 4.';

end;


(*
// Averaging scoring method: not used

function ScoreAnswers_BASIS(
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string): string;

var
  Dep, Inter, Psy, Alc, Emo, Harm, Total,
    DepSum, InterSum, PsySum, AlcSum, EmoSum, HarmSum,
    Count, i                  : Integer;
  DepSc, InterSc, PsySc, AlcSc, EmoSc, HarmSc, TotalSc
                              : Extended;
  TempResult,
    TempAnswer                : string;
  PChr                        : PChar;
begin
  // AnswerString Format:
  //      83=1^1330~
  //      84=0^214~...
  //
  // '83' is QuestionID,
  // '1' is ItemIndex for radio group, but can be text from edit field
  // '1130' is ChoiceID (not used here)

  Result := '';
  Count := NumberOfPieces(AnswerString, V);

  if Count < 1 then
    Exit;

  Dep := 0;
  Inter := 0;
  Psy := 0;
  Alc := 0;
  Emo := 0;
  Harm := 0;
  Total := 0;

  DepSum := 0;
  InterSum := 0;
  PsySum := 0;
  AlcSum := 0;
  EmoSum := 0;
  HarmSum := 0;

  DepSc := 0;
  InterSc := 0;
  PsySc := 0;
  AlcSc := 0;
  EmoSc := 0;
  HarmSc := 0;
  TotalSc := 0;

  for i := 0 to Count do
    begin
      TempResult := Piece(AnswerString, V, i);
      TempAnswer := Piece(Piece(TempResult, U, 1), EQ, 2);

      // Count the number of skipped (Blank) answers for each scale
      if (TempAnswer = '') or (TempAnswer = ' ') or
        (TempAnswer = '-1') or
        (TempAnswer = CHOICE_ID_OF_SKIPPED_MULTIPLE_CHOICE_QUESTION) or
        (TempAnswer = CHOICE_ID_OF_SKIPPED_BY_RULE_MC_QUESTION) or
        (TempAnswer = CHOICE_ID_OF_SKIPPED_REQUIRED_ANSWER) then
        begin
          // Depression
          if (i = 0) or (i = 1) or (i = 2) or (i = 8) or
            (i = 9) or (i = 11) then
            Inc(Dep);
          // Interpersonal Problems
          if (i = 3) or (i = 4) or (i = 5) or (i = 6) or
            (i = 7) then
            Inc(Inter);
          // Psycholtic Symptoms
          if (i = 13) or (i = 14) or (i = 15) or (i = 16) then
            Inc(Psy);
          // Alcohol/Drug Use
          if (i = 20) or (i = 21) or (i = 22) or (i = 23) then
            Inc(Alc);
          // Emotional Lability
          if (i = 12) or (i = 17) or (i = 18) then
            Inc(Emo);
          // Self Harm
          if (i = 10) or (i = 19) then
            Inc(Harm);
        end // Skipped questions
      else
        // Tally the scale's raw sum
        begin
          // Depression
          if (i = 0) or (i = 1) or (i = 2) or
            (i = 9) or (i = 11) then
            DepSum := DepSum + StrToIntDef(TempAnswer, 0);
          // Reverse this item
          if (i = 8) then
            begin
              if TempAnswer = '0' then
                DepSum := DepSum + 4;
              if TempAnswer = '1' then
                DepSum := DepSum + 3;
              if TempAnswer = '2' then
                DepSum := DepSum + 2;
              if TempAnswer = '3' then
                DepSum := DepSum + 1;
            end;
          // Interpersonal Problems: All reversed in scoring
          if (i = 3) or (i = 4) or (i = 5) or (i = 6) or
            (i = 7) then
            begin
              if TempAnswer = '0' then
                InterSum := InterSum + 4;
              if TempAnswer = '1' then
                InterSum := InterSum + 3;
              if TempAnswer = '2' then
                InterSum := InterSum + 2;
              if TempAnswer = '3' then
                InterSum := InterSum + 1;
            end;
          // Psycholtic Symptoms
          if (i = 13) or (i = 14) or (i = 15) or (i = 16) then
            PsySum := PsySum + StrToIntDef(TempAnswer, 0);
          // Alcohol/Drug Use
          if (i = 20) or (i = 21) or (i = 22) or (i = 23) then
            AlcSum := AlcSum + StrToIntDef(TempAnswer, 0);
          // Emotional Lability
          if (i = 12) or (i = 17) or (i = 18) then
            EmoSum := EmoSum + StrToIntDef(TempAnswer, 0);
          // Self Harm
          if (i = 10) or (i = 19) then
            HarmSum := HarmSum + StrToIntDef(TempAnswer, 0);

        end; // Tally scale's raw sum
    end; // for i

  // Calculate the average for each scale, given that there are no
  // more than two skipped questions for that scale.

  if Dep < 2 then
    begin
      DepSc := DepSum / (6 - Dep);
      Result := 'Depression^^' + FloatToStrF(DepSc, ffFixed, 8, 2);
    end
  else
    Result := 'Depression^^Too many skipped questions';

  if Inter < 2 then
    begin
      InterSc := InterSum / (5 - Inter);
      Result := Result + '*Interpersonal Problems^^' + FloatToStrF(InterSc, ffFixed, 8, 2);
    end
  else
    Result := Result + '*Interpersonal Problems^^Too many skipped questions';

  if Psy < 2 then
    begin
      PsySc := PsySum / (4 - Psy);
      Result := Result + '*Psychotic Symptoms^^' + FloatToStrF(PsySc, ffFixed, 8, 2);
    end
  else
    Result := Result + '*Psychotic Symptoms^^Too many skipped questions';

  if Alc < 2 then
    begin
      AlcSc := AlcSum / (4 - Alc);
      Result := Result + '*Alcohol/Drug Use^^' + FloatToStrF(AlcSc, ffFixed, 8, 2);
    end
  else
    Result := Result + '*Alcohol/Drug Use^^Too many skipped questions';

  if Emo < 2 then
    begin
      EmoSc := EmoSum / (3 - Emo);
      Result := Result + '*Emotional Lability^^' + FloatToStrF(EmoSc, ffFixed, 8, 2);
    end
  else
    Result := Result + '*Emotional Lability^^Too many skipped questions';

  if Harm < 2 then
    begin
      HarmSc := HarmSum / (2 - Harm);
      Result := Result + '*Self Harm^^' + FloatToStrF(HarmSc, ffFixed, 8, 2);
    end
  else
    Result := Result + '*Self Harm^^Too many skipped questions';

  Total := Dep + Inter + Psy + Alc + Emo + Harm;
  if Total < 2 then
    begin
      TotalSc := (DepSum + InterSum + PsySum + AlcSum + EmoSum + HarmSum) /
        (24 - Total);
      Result := Result + '*Overall^^' + FloatToStrF(TotalSc, ffFixed, 8, 2);
    end
  else
    Result := Result + '*Overall^^Too many skipped questions';
end;
*)
end.

