unit uMHA_AUX_BDI2;
// Beck Depression Inventory version 2
// 21 Questions, four possible answers scored 0 -3 except
// Questions 16 and 18, they have seven possible answers
// scored 0, 1a, 1b, 2a, 2b, 3a, 3b.

interface

uses
  SysUtils,
  fIA_dialogs,
  uMHA_AUX_Const,
  uMHA_AUX_Utils;

function ProgressNote_BDI2(
  ScoreString: string): string;

function ScoreAnswers_BDI2(
  AnswerString: string): string;

implementation


function ScoreAnswers_BDI2(
  AnswerString: string): string;

var
  Score, Ans,
    Skipped,
    Count, i                  : Integer;

  TempResult,
    TempAnswer,
    ResultString              : string;

begin
  // AnswerString Format:
  //      83=1^1330^234~
  //      84=0^214~...

  // '83' is QuestionID,
  // '1' is ItemIndex for radio group, but can be text from edit field
  // '1330' is ChoiceID (not used here)
  // '234' is AnswerID (not used here)

  // This is used by InstrumentResults to do a quick check on the
  // instrument as to its use of the dll
  if AnswerString = 'BDI2' then
    begin
      Result := 'YES';
      exit;
    end;

  Result := '';
  Count := NumberOfPieces(AnswerString, RECORD_END);
  Skipped := 0;
  ResultString := '';

  if Count < 1 then
    Exit;

  Score := 0;

  for i := 1 to Count - 1 do
    begin
      TempResult := Piece(AnswerString, RECORD_END, i);
      TempAnswer := Piece(Piece(TempResult, U, 1), EQ, 2);

      // ignore (Blank) answers
      if (TempAnswer = '') or (TempAnswer = ' ') or
        (TempAnswer = '-1') or
        (TempAnswer = CHOICE_ID_OF_SKIPPED_MULTIPLE_CHOICE_QUESTION) or
        (TempAnswer = CHOICE_ID_OF_SKIPPED_BY_RULE_MC_QUESTION) or
        (TempAnswer = CHOICE_ID_OF_SKIPPED_REQUIRED_ANSWER) then
        begin
          Inc(Skipped);
          ResultString := ResultString + IntToStr(i) + '. Not answered.' + U;
        end // Skipped questions
      else
        begin
          Ans := StrToIntDef(TempAnswer, 0);

          if (i = 1) then
            begin
              Score := Score + Ans;
              if Ans = 0 then
                ResultString := ResultString + IntToStr(i) + '. I do not feel sad (0 point)' + U;
              if Ans = 1 then
                ResultString := ResultString + IntToStr(i) + '. I feel sad much of the time (1 point)' + U;
              if Ans = 2 then
                ResultString := ResultString + IntToStr(i) + '. I am sad all the time (2 points)' + U;
              if Ans = 3 then
                ResultString := ResultString + IntToStr(i) + '. I am so sad or unhappy that I can''t stand it (3 points)' + U;
            end;

          if (i = 2) then
            begin
              Score := Score + Ans;
              if Ans = 0 then
                ResultString := ResultString + IntToStr(i) + '. I am not discouraged about my future (0 point)' + U;
              if Ans = 1 then
                ResultString := ResultString + IntToStr(i) + '. I feel more discouraged about my future than I used to be (1 point)' + U;
              if Ans = 2 then
                ResultString := ResultString + IntToStr(i) + '. I do not expect things to work out for me (2 points)' + U;
              if Ans = 3 then
                ResultString := ResultString + IntToStr(i) + '. I feel my future is hopeless and will only get worse (3 points)' + U;
            end;

          if (i = 3) then
            begin
              Score := Score + Ans;
              if Ans = 0 then
                ResultString := ResultString + IntToStr(i) + '. I do not feel like a failure (0 point)' + U;
              if Ans = 1 then
                ResultString := ResultString + IntToStr(i) + '. I have failed more than I should have (1 point)' + U;
              if Ans = 2 then
                ResultString := ResultString + IntToStr(i) + '. As I look back, I see a lot of failures (2 points)' + U;
              if Ans = 3 then
                ResultString := ResultString + IntToStr(i) + '. I feel I am a total failure as a person (3 points)' + U;
            end;

          if (i = 4) then
            begin
              Score := Score + Ans;
              if Ans = 0 then
                ResultString := ResultString + IntToStr(i) + '. I get as much pleasure as I ever did from the things I enjoy (0 point)' + U;
              if Ans = 1 then
                ResultString := ResultString + IntToStr(i) + '. I don''t enjoy things as much as I used to (1 point)' + U;
              if Ans = 2 then
                ResultString := ResultString + IntToStr(i) + '. I get very little pleasure from the things I used to enjoy (2 points)' + U;
              if Ans = 3 then
                ResultString := ResultString + IntToStr(i) + '. I can''t get any pleasure from the things I used to enjoy (3 points)' + U;
            end;

          if (i = 5) then
            begin
              Score := Score + Ans;
              if Ans = 0 then
                ResultString := ResultString + IntToStr(i) + '. I don''t feel particularly guilty (0 point)' + U;
              if Ans = 1 then
                ResultString := ResultString + IntToStr(i) + '. I feel guilty over many things I have done or should have done (1 point)' + U;
              if Ans = 2 then
                ResultString := ResultString + IntToStr(i) + '. I feel quite guilty most of the time (2 points)' + U;
              if Ans = 3 then
                ResultString := ResultString + IntToStr(i) + '. I feel guilty all of the time (3 points)' + U;
            end;

          if (i = 6) then
            begin
              Score := Score + Ans;
              if Ans = 0 then
                ResultString := ResultString + IntToStr(i) + '. I don''t feel I am being punished (0 point)' + U;
              if Ans = 1 then
                ResultString := ResultString + IntToStr(i) + '. I feel I may be punished (1 point)' + U;
              if Ans = 2 then
                ResultString := ResultString + IntToStr(i) + '. I expect to be punished (2 points)' + U;
              if Ans = 3 then
                ResultString := ResultString + IntToStr(i) + '. I feel I am being punished (3 points)' + U;
            end;

          if (i = 7) then
            begin
              Score := Score + Ans;
              if Ans = 0 then
                ResultString := ResultString + IntToStr(i) + '. I feel the same about myself as ever (0 point)' + U;
              if Ans = 1 then
                ResultString := ResultString + IntToStr(i) + '. I have lost confidence in myself (1 point)' + U;
              if Ans = 2 then
                ResultString := ResultString + IntToStr(i) + '. I am disappointed in myself (2 points)' + U;
              if Ans = 3 then
                ResultString := ResultString + IntToStr(i) + '. I dislike myself (3 points)' + U;
            end;

          if (i = 8) then
            begin
              Score := Score + Ans;
              if Ans = 0 then
                ResultString := ResultString + IntToStr(i) + '. I don''t criticize or blame myself more than usual (0 point)' + U;
              if Ans = 1 then
                ResultString := ResultString + IntToStr(i) + '. I am more critical of myself than I used to be (1 point)' + U;
              if Ans = 2 then
                ResultString := ResultString + IntToStr(i) + '. I criticize myself for all my faults (2 points)' + U;
              if Ans = 3 then
                ResultString := ResultString + IntToStr(i) + '. I blame myself for everything bad that happens (3 points)' + U;
            end;

          if (i = 9) then
            begin
              Score := Score + Ans;
              if Ans = 0 then
                ResultString := ResultString + IntToStr(i) + '. I don''t have any thoughts of killing myself (0 point)' + U;
              if Ans = 1 then
                ResultString := ResultString + IntToStr(i) + '. I have thoughts of killing myself, but I would not carry them out (1 point)' + U;
              if Ans = 2 then
                ResultString := ResultString + IntToStr(i) + '. I would like to kill myself (2 points)' + U;
              if Ans = 3 then
                ResultString := ResultString + IntToStr(i) + '. I would kill myself if I had the chance (3 points)' + U;
            end;

          if (i = 10) then
            begin
              Score := Score + Ans;
              if Ans = 0 then
                ResultString := ResultString + IntToStr(i) + '. I don''t cry anymore than I used to (0 point)' + U;
              if Ans = 1 then
                ResultString := ResultString + IntToStr(i) + '. I cry more than I used to (1 point)' + U;
              if Ans = 2 then
                ResultString := ResultString + IntToStr(i) + '. I cry over every little thing (2 points)' + U;
              if Ans = 3 then
                ResultString := ResultString + IntToStr(i) + '. I feel like crying, but I can''t (3 points)' + U;
            end;

          if (i = 11) then
            begin
              Score := Score + Ans;
              if Ans = 0 then
                ResultString := ResultString + IntToStr(i) + '. I am no more restless or wound up than usual (0 point)' + U;
              if Ans = 1 then
                ResultString := ResultString + IntToStr(i) + '. I feel more restless or wound up than usual (1 point)' + U;
              if Ans = 2 then
                ResultString := ResultString + IntToStr(i) + '. I am so restless or agitated that it sometimes hard to stay still (2 points)' + U;
              if Ans = 3 then
                ResultString := ResultString + IntToStr(i) + '. I am so restless or agitated that I have to keep moving or doing something (3 points)' + U;
            end;

          if (i = 12) then
            begin
              Score := Score + Ans;
              if Ans = 0 then
                ResultString := ResultString + IntToStr(i) + '. I have not lost interest in other people or activities (0 point)' + U;
              if Ans = 1 then
                ResultString := ResultString + IntToStr(i) + '. I am less interested in other people or things than before (1 point)' + U;
              if Ans = 2 then
                ResultString := ResultString + IntToStr(i) + '. I have lost most of my interest in other people or things (2 points)' + U;
              if Ans = 3 then
                ResultString := ResultString + IntToStr(i) + '. It''s hard to get interested in anything (3 points)' + U;
            end;

          if (i = 13) then
            begin
              Score := Score + Ans;
              if Ans = 0 then
                ResultString := ResultString + IntToStr(i) + '. I make decisions about as well as ever (0 point)' + U;
              if Ans = 1 then
                ResultString := ResultString + IntToStr(i) + '. I find it more difficult to make decisions than usual (1 point)' + U;
              if Ans = 2 then
                ResultString := ResultString + IntToStr(i) + '. I have much greater difficulty in making decisions than I used to (2 points)' + U;
              if Ans = 3 then
                ResultString := ResultString + IntToStr(i) + '. I have trouble making any decisions (3 points)' + U;
            end;

          if (i = 14) then
            begin
              Score := Score + Ans;
              if Ans = 0 then
                ResultString := ResultString + IntToStr(i) + '. I do not feel I am worthless (0 point)' + U;
              if Ans = 1 then
                ResultString := ResultString + IntToStr(i) + '. I don''t consider myself as worthwhile and useful as I used to (1 point)' + U;
              if Ans = 2 then
                ResultString := ResultString + IntToStr(i) + '. I feel more worthless as compared to other people (2 points)' + U;
              if Ans = 3 then
                ResultString := ResultString + IntToStr(i) + '. I feel utterly worthless (3 points)' + U;
            end;

          if (i = 15) then
            begin
              Score := Score + Ans;
              if Ans = 0 then
                ResultString := ResultString + IntToStr(i) + '. I have as much energy as ever (0 point)' + U;
              if Ans = 1 then
                ResultString := ResultString + IntToStr(i) + '. I have less energy than I used to have (1 point)' + U;
              if Ans = 2 then
                ResultString := ResultString + IntToStr(i) + '. I don''t have enough energy to do very much (2 points)' + U;
              if Ans = 3 then
                ResultString := ResultString + IntToStr(i) + '. I don''t have enough energy to do anything (3 points)' + U;
            end;

          if (i = 16) then
            begin
              if Ans = 0 then
                ResultString := ResultString + IntToStr(i) + '. I have not experienced any change in my sleeping pattern (0 point)' + U;
              if Ans = 1 then
                begin
                  Score := Score + 1;
                  ResultString := ResultString + IntToStr(i) + '. I sleep somewhat more than usual (1 point)' + U;
                end;
              if Ans = 2 then
                begin
                  Score := Score + 1;
                  ResultString := ResultString + IntToStr(i) + '. I sleep somewhat less than usual (1 point)' + U;
                end;
              if Ans = 3 then
                begin
                  Score := Score + 2;
                  ResultString := ResultString + IntToStr(i) + '. I sleep a lot more than usual (2 points)' + U;
                end;
              if Ans = 4 then
                begin
                  Score := Score + 2;
                  ResultString := ResultString + IntToStr(i) + '. I sleep a lot less than usual (2 points)' + U;
                end;
              if Ans = 5 then
                begin
                  Score := Score + 3;
                  ResultString := ResultString + IntToStr(i) + '. I sleep most of the day (3 points)' + U;
                end;
              if Ans = 6 then
                begin
                  Score := Score + 3;
                  ResultString := ResultString + IntToStr(i) + '. I wake up 1-2 hours early and can''t get back to sleep (3 points)' + U;
                end;
            end;

          if (i = 17) then
            begin
              Score := Score + Ans;
              if Ans = 0 then
                ResultString := ResultString + IntToStr(i) + '. I am no more irritable than usual (0 point)' + U;
              if Ans = 1 then
                ResultString := ResultString + IntToStr(i) + '. I am more irritable than usual (1 point)' + U;
              if Ans = 2 then
                ResultString := ResultString + IntToStr(i) + '. I am much more irritable than usual (2 points)' + U;
              if Ans = 3 then
                ResultString := ResultString + IntToStr(i) + '. I am irritable all the time (3 points)' + U;
            end;

          if (i = 18) then
            begin
              if Ans = 0 then
                ResultString := ResultString + IntToStr(i) + '. I have not experienced any change in my appetite (0 point)' + U;
              if Ans = 1 then
                begin
                  Score := Score + 1;
                  ResultString := ResultString + IntToStr(i) + '. My appetite is somewhat less than usual (1 point)' + U;
                end;
              if Ans = 2 then
                begin
                  Score := Score + 1;
                  ResultString := ResultString + IntToStr(i) + '. My appetite is somewhat greater than usual (1 point)' + U;
                end;
              if Ans = 3 then
                begin
                  Score := Score + 2;
                  ResultString := ResultString + IntToStr(i) + '. My appetite is much less than before (2 points)' + U;
                end;
              if Ans = 4 then
                begin
                  Score := Score + 2;
                  ResultString := ResultString + IntToStr(i) + '. My appetite is much greater than usual (2 points)' + U;
                end;
              if Ans = 5 then
                begin
                  Score := Score + 3;
                  ResultString := ResultString + IntToStr(i) + '. I have no appetite at all (3 points)' + U;
                end;
              if Ans = 6 then
                begin
                  Score := Score + 3;
                  ResultString := ResultString + IntToStr(i) + '. I crave food all the time (3 points)' + U;
                end;
            end;

          if (i = 19) then
            begin
              Score := Score + Ans;
              if Ans = 0 then
                ResultString := ResultString + IntToStr(i) + '. I can concentrate as well as ever (0 point)' + U;
              if Ans = 1 then
                ResultString := ResultString + IntToStr(i) + '. I can''t concentrate as well as usual (1 point)' + U;
              if Ans = 2 then
                ResultString := ResultString + IntToStr(i) + '. It''s hard to keep my mind on anything for very long (2 points)' + U;
              if Ans = 3 then
                ResultString := ResultString + IntToStr(i) + '. I find I can''t concentrate on anything (3 points)' + U;
            end;

          if (i = 20) then
            begin
              Score := Score + Ans;
              if Ans = 0 then
                ResultString := ResultString + IntToStr(i) + '. I am no more tired or fatigued than usual (0 point)' + U;
              if Ans = 1 then
                ResultString := ResultString + IntToStr(i) + '. I get more tired or fatigued more easily than usual (1 point)' + U;
              if Ans = 2 then
                ResultString := ResultString + IntToStr(i) + '. I am too tired or fatigued to do a lot of the things I used to do (2 points)' + U;
              if Ans = 3 then
                ResultString := ResultString + IntToStr(i) + '. I am too tired or fatigued to do most of the things I used to do (3 points)' + U;
            end;

          if (i = 21) then
            begin
              Score := Score + Ans;
              if Ans = 0 then
                ResultString := ResultString + IntToStr(i) + '. I have not noticed any recent changes in my interest in sex (0 point)' + U;
              if Ans = 1 then
                ResultString := ResultString + IntToStr(i) + '. I am less interested in sex than I used to be (1 point)' + U;
              if Ans = 2 then
                ResultString := ResultString + IntToStr(i) + '. I am much less interested in sex now (2 points)' + U;
              if Ans = 3 then
                ResultString := ResultString + IntToStr(i) + '. I have lost interest in sex completely (3 points)' + U;
            end;

        end;
    end; // for i

  if (Skipped > 5) then
    Score := -1;

  Result := 'Total^^' + IntToStr(Score) + U + ResultString + RECORD_END;
end;


function ProgressNote_BDI2(
  ScoreString: string): string;
// The header, footer and questions and answers are handled else where; avoiding
// broker calls here. <.DLL_String.> provides location for this string in the
// report.

const
  Indent                      : string = ' ';
var
  BDI2_Score                  : string;
  Count, i,
    Score                     : integer;
begin
  Result := '';

  if (ScoreString = '') then
    Exit;

  BDI2_Score := Piece(ScoreString, U, 3);
  Score := StrToIntDef(BDI2_Score, -1);

  if (Score < 14) then
    Result := Result + Indent +
      'BDI2 Score: ' + BDI2_Score + ' indicates low depression. The overall range is 0 to 63 with low depression between 0 - 13.' + #13;

  if (Score = -1) then
    begin
      Result := '';
      Result := Result + Indent +
        'BDI2 score could not be determined.';
    end;

  if (Score < 20) and (Score > 13) then
    Result := Result + Indent +
      'BDI2 Score: ' + BDI2_Score + ' indicates mild depression. The overall range is 0 to 63 with mild depression between 14 - 19.' + #13;

  if (Score < 29) and (Score > 19) then
    Result := Result + Indent +
      'BDI2 Score: ' + BDI2_Score + ' indicates moderate depression. The overall range is 0 to 63 with moderate depression between 20 - 28.' + #13;


  if (Score > 28) then
    Result := Result + Indent +
      'BDI2 Score: ' + BDI2_Score + ' indicates severe depression. The overall range is 0 to 63 with severe depression above 29.' + #13;

  Result := Result + #13 + Indent +
    'Questions and answers ' + #13 + #13;

  Count := NumberOfPieces(ScoreString, U);

  for i := 4 to Count do
    begin
      if Piece(ScoreString, U, i) <> RECORD_END then
        Result := Result + Indent + Piece(ScoreString, U, i) + #13;
    end;

end;

end.

