unit uMHA_AUX_BSI;
// Beck Sucide Index
// 21 Questions, three possible answers scored 0 - 2.

interface

uses
  SysUtils,
  fIA_dialogs,
  uMHA_AUX_Const,
  uMHA_AUX_Utils;

function ProgressNote_BSI(
  ScoreString: string): string;

function ScoreAnswers_BSI(
  AnswerString: string): string;

implementation


function ScoreAnswers_BSI(
  AnswerString: string): string;

var
  Score, Ans,
    Skipped,
    Count, i                  : Integer;

  TempResult,
    TempAnswer,
    ResultString              : string;

begin
  // AnswerString Format:
  //      83=1^1330^234~
  //      84=0^214~...

  // '83' is QuestionID,
  // '1' is ItemIndex for radio group, but can be text from edit field
  // '1330' is ChoiceID (not used here)
  // '234' is AnswerID (not used here)

  // This is used by InstrumentResults to do a quick check on the
  // instrument as to its use of the dll
  if AnswerString = 'BSI' then
    begin
      Result := 'YES';
      exit;
    end;

  Result := '';
  Count := NumberOfPieces(AnswerString, RECORD_END);
  Skipped := 0;
  ResultString := '';

  if Count < 1 then
    Exit;

  Score := 0;

  for i := 1 to Count - 1 do
    begin
      TempResult := Piece(AnswerString, RECORD_END, i);
      TempAnswer := Piece(Piece(TempResult, U, 1), EQ, 2);

      // ignore (Blank) answers
      if (TempAnswer = '') or (TempAnswer = ' ') or
        (TempAnswer = '-1') or
        (TempAnswer = CHOICE_ID_OF_SKIPPED_MULTIPLE_CHOICE_QUESTION) or
        (TempAnswer = CHOICE_ID_OF_SKIPPED_BY_RULE_MC_QUESTION) or
        (TempAnswer = CHOICE_ID_OF_SKIPPED_REQUIRED_ANSWER) then
        begin
          Inc(Skipped);
          ResultString := ResultString + IntToStr(i) + '. Not answered.' + U;
        end // Skipped questions
      else
        begin
          Ans := StrToIntDef(TempAnswer, 0);
          Score := Score + Ans;

          if (i = 1) then
            begin
              if Ans = 0 then
                ResultString := ResultString + IntToStr(i) + '. I have a moderate to strong wish to live (0 point)' + U;
              if Ans = 1 then
                ResultString := ResultString + IntToStr(i) + '. I have a weak wish to live (1 point)' + U;
              if Ans = 2 then
                ResultString := ResultString + IntToStr(i) + '. I have no wish to live (2 points)' + U;
            end;

          if (i = 2) then
            begin
              if Ans = 0 then
                ResultString := ResultString + IntToStr(i) + '. I have no wish to die (0 point)' + U;
              if Ans = 1 then
                ResultString := ResultString + IntToStr(i) + '. I have a weak wish to die (1 point)' + U;
              if Ans = 2 then
                ResultString := ResultString + IntToStr(i) + '. I have a moderate to strong wish to die (2 points)' + U;
            end;

          if (i = 3) then
            begin
              if Ans = 0 then
                ResultString := ResultString + IntToStr(i) + '. My reasons for living outweigh my reasons for dying (0 point)' + U;
              if Ans = 1 then
                ResultString := ResultString + IntToStr(i) + '. My reasons for living or dying are about equal (1 point)' + U;
              if Ans = 2 then
                ResultString := ResultString + IntToStr(i) + '. My reasons for dying outweigh my reasons for living (2 points)' + U;
            end;

          if (i = 4) then
            begin
              if Ans = 0 then
                ResultString := ResultString + IntToStr(i) + '. I have no desire to kill myself (0 point)' + U;
              if Ans = 1 then
                ResultString := ResultString + IntToStr(i) + '. I have a weak desire to kill myself (1 point)' + U;
              if Ans = 2 then
                ResultString := ResultString + IntToStr(i) + '. I have a moderate to strong desire to kill myself (2 points)' + U;
            end;

          if (i = 5) then
            begin
              if Ans = 0 then
                ResultString := ResultString + IntToStr(i) + '. I would try to save my life if I found myself in a life-threatening situation (0 point)' + U;
              if Ans = 1 then
                ResultString := ResultString + IntToStr(i) + '. I would take a chance on life or death if I found myself in a life-threatening situation (1 point)' + U;
              if Ans = 2 then
                ResultString := ResultString + IntToStr(i) + '. I would not take the steps necessary to avoid death if I found myself in a life-threatening situation (2 points)' + U;
            end;

          if (i = 6) then
            begin
              if Ans = 0 then
                ResultString := ResultString + IntToStr(i) + '. I have brief periods of thinking about killing myself which pass quickly (0 point)' + U;
              if Ans = 1 then
                ResultString := ResultString + IntToStr(i) + '. I have periods of thinking about killing myself which last for moderate amounts of time (1 point)' + U;
              if Ans = 2 then
                ResultString := ResultString + IntToStr(i) + '. I have long periods of thinking about killing myself (2 points)' + U;
            end;

          if (i = 7) then
            begin
              if Ans = 0 then
                ResultString := ResultString + IntToStr(i) + '. I rarely or only occasionally think about killing myself (0 point)' + U;
              if Ans = 1 then
                ResultString := ResultString + IntToStr(i) + '. I have frequent thoughts about killing myself (1 point)' + U;
              if Ans = 2 then
                ResultString := ResultString + IntToStr(i) + '. I continuously think about killing myself (2 points)' + U;
            end;

          if (i = 8) then
            begin
              if Ans = 0 then
                ResultString := ResultString + IntToStr(i) + '. I do not accept the idea of killing myself (0 point)' + U;
              if Ans = 1 then
                ResultString := ResultString + IntToStr(i) + '. I neither accept nor reject the idea of killing myself (1 point)' + U;
              if Ans = 2 then
                ResultString := ResultString + IntToStr(i) + '. I accept the idea of killing myself (2 points)' + U;
            end;

          if (i = 9) then
            begin
              if Ans = 0 then
                ResultString := ResultString + IntToStr(i) + '. I can keep myself from committing suicide (0 point)' + U;
              if Ans = 1 then
                ResultString := ResultString + IntToStr(i) + '. I am unsure that I can keep myself from committing suicide (1 point)' + U;
              if Ans = 2 then
                ResultString := ResultString + IntToStr(i) + '. I cannot keep myself from committing suicide (2 points)' + U;
            end;

          if (i = 10) then
            begin
              if Ans = 0 then
                ResultString := ResultString + IntToStr(i) + '. I would not kill myself because of my family, friends, religion, possible injury from an unsuccessful attempt, etc. (0 point)' + U;
              if Ans = 1 then
                ResultString := ResultString + IntToStr(i) + '. I am somewhat concerned about killing myself because of my family, friends, religion, possible injury from an unsuccessful attempt, etc. (1 point)' + U;
              if Ans = 2 then
                ResultString := ResultString + IntToStr(i) + '. I am not or only a little concerned about killing myself because of family, friends, religion, possible injury from an unsuccessful attempt, etc. (2 points)' + U;
            end;

          if (i = 11) then
            begin
              if Ans = 0 then
                ResultString := ResultString + IntToStr(i) + '. My reasons for wanting to commit suicide are primarily aimed at influencing other people, such as getting even with people, making people pay attention to me, etc. (0 point)' + U;
              if Ans = 1 then
                ResultString := ResultString + IntToStr(i) + '. My reasons for wanting to commit suicide are not only aimed at influencing other people, but also represent a way of solving my problems (1 point)' + U;
              if Ans = 2 then
                ResultString := ResultString + IntToStr(i) + '. My reasons for wanting to commit suicide are primarily based upon escaping from my problems (2 points)' + U;
            end;

          if (i = 12) then
            begin
              if Ans = 0 then
                ResultString := ResultString + IntToStr(i) + '. I have no specific plan about how to kill myself (0 point)' + U;
              if Ans = 1 then
                ResultString := ResultString + IntToStr(i) + '. I have considered ways of killing myself, but have not worked out the details (1 point)' + U;
              if Ans = 2 then
                ResultString := ResultString + IntToStr(i) + '. I have a specific plan for killing myself (2 points)' + U;
            end;

          if (i = 13) then
            begin
              if Ans = 0 then
                ResultString := ResultString + IntToStr(i) + '. I do not have access to a method or an opportunity to kill myself (0 point)' + U;
              if Ans = 1 then
                ResultString := ResultString + IntToStr(i) + '. The method that I would use for committing suicide takes time, and I really do not have a good opportunity to use this method (1 point)' + U;
              if Ans = 2 then
                ResultString := ResultString + IntToStr(i) + '. I have access or anticipate having access to the method that I would choose for killing myself and also have opportunity to use it (2 points)' + U;
            end;

          if (i = 14) then
            begin
              if Ans = 0 then
                ResultString := ResultString + IntToStr(i) + '. I do not have the courage or the ability to commit suicide (0 point)' + U;
              if Ans = 1 then
                ResultString := ResultString + IntToStr(i) + '. I am unsure that I have the courage or the ability to commit suicide (1 point)' + U;
              if Ans = 2 then
                ResultString := ResultString + IntToStr(i) + '. I have the courage and the ability to commit suicide (2 points)' + U;
            end;

          if (i = 15) then
            begin
              if Ans = 0 then
                ResultString := ResultString + IntToStr(i) + '. I do not expect to make a suicide attempt (0 point)' + U;
              if Ans = 1 then
                ResultString := ResultString + IntToStr(i) + '. I am unsure that I shall make a suicide attempt (1 point)' + U;
              if Ans = 2 then
                ResultString := ResultString + IntToStr(i) + '. I am sure that I shall make a suicide attempt (2 points)' + U;
            end;

          if (i = 16) then
            begin
              if Ans = 0 then
                ResultString := ResultString + IntToStr(i) + '. I have made no preparations for committing suicide (0 point)' + U;
              if Ans = 1 then
                ResultString := ResultString + IntToStr(i) + '. I have made some preparations for committing suicide (1 point)' + U;
              if Ans = 2 then
                ResultString := ResultString + IntToStr(i) + '. I have almost finished or completed my preparations for committing suicide (2 points)' + U;
            end;

          if (i = 17) then
            begin
              if Ans = 0 then
                ResultString := ResultString + IntToStr(i) + '. I have not written a suicide note (0 point)' + U;
              if Ans = 1 then
                ResultString := ResultString + IntToStr(i) + '. I have thought about writing a suicide note or have started to write one, but have not completed it (1 point)' + U;
              if Ans = 2 then
                ResultString := ResultString + IntToStr(i) + '. I have completed a suicide note (2 points)' + U;
            end;

          if (i = 18) then
            begin
              if Ans = 0 then
                ResultString := ResultString + IntToStr(i) + '. I have made no arrangements for what will happen after I have committed suicide (0 point)' + U;
              if Ans = 1 then
                ResultString := ResultString + IntToStr(i) + '. I have thought about making some arrangements for what will happen after I have committed suicide (1 point)' + U;
              if Ans = 2 then
                ResultString := ResultString + IntToStr(i) + '. I have made definite arrangements for what will happen after I have committed suicide (2 points)' + U;
            end;

          if (i = 19) then
            begin
              if Ans = 0 then
                ResultString := ResultString + IntToStr(i) + '. I have not hidden my desire to kill myself from people (0 point)' + U;
              if Ans = 1 then
                ResultString := ResultString + IntToStr(i) + '. I have held back telling people about wanting to kill myself (1 point)' + U;
              if Ans = 2 then
                ResultString := ResultString + IntToStr(i) + '. I have attempted to hide, conceal, or lie about wanting to commit suicide (2 points)' + U;
            end;

          if (i = 20) then
            begin
              if Ans = 0 then
                ResultString := ResultString + IntToStr(i) + '. I have never attempted suicide (0 point)' + U;
              if Ans = 1 then
                ResultString := ResultString + IntToStr(i) + '. I have attempted suicide once (1 point)' + U;
              if Ans = 2 then
                ResultString := ResultString + IntToStr(i) + '. I have attempted suicide two or more times (2 points)' + U;
            end;

            if (i = 21) then
            begin
              if Ans = 0 then
                ResultString := ResultString + IntToStr(i) + '. My wish to die during the last suicide attempt was low (0 point)' + U;
              if Ans = 1 then
                ResultString := ResultString + IntToStr(i) + '. My wish to die during the last suicide attempt was moderate (1 point)' + U;
              if Ans = 2 then
                ResultString := ResultString + IntToStr(i) + '. My wish to die during the last suicide attempt was high (2 points)' + U;
            end;
        end;
    end; // for i

  if (Skipped > 5) then
    Score := -1;
  // changed Suicide Ideation to Score llh
  Result := 'Score^^' + IntToStr(Score) + U + ResultString + RECORD_END;
end;


function ProgressNote_BSI(
  ScoreString: string): string;
// The header, footer and questions and answers are handled else where; avoiding
// broker calls here. <.DLL_String.> provides location for this string in the
// report.

const
  Indent                      : string = ' ';
var
  BSI_Score                   : string;
  Count, i,
    Score                     : integer;
begin
  Result := '';

  if (ScoreString = '') then
    Exit;

  BSI_Score := Piece(ScoreString, U, 3);
  Score := StrToIntDef(BSI_Score, -1);

  if (Score < 22) then
    Result := Result + Indent +
      'BSI Score: ' + BSI_Score + ' indicates low suicidal risk. The overall range is 0 to 42 with low suicidal risk between 0 - 21.' + #13;

  if (Score = -1) then
    begin
      Result := '';
      Result := Result + Indent +
        'BSI score could not be determined.';
    end;

  if (Score < 36) and (Score > 21) then
    Result := Result + Indent +
      'BSI Score: ' + BSI_Score + ' indicates moderate suicidal risk. The overall range is 0 to 42 with moderate suicidal risk between 22 - 35.' + #13;

  if (Score > 35) then
    Result := Result + Indent +
      'BSI Score: ' + BSI_Score + ' indicates severe suicidal risk. The overall range is 0 to 42 with severe suicidal risk above 36.' + #13;

  Result := Result + #13 + Indent +
    'Questions and answers ' + #13 + #13;

  Count := NumberOfPieces(ScoreString, U);

  for i := 4 to Count do
    begin
      if Piece(ScoreString, U, i) <> RECORD_END then
        Result := Result + Indent + Piece(ScoreString, U, i) + #13;
    end;

end;

end.

