unit uMHA_AUX_BSL23;

interface

uses
  SysUtils;

function ScoreAnswers_BSL23(
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string): string;

function ProgressNote_BSL23(
  PatientName,
  PatientDOB,
  PatientSex,
  ScoreString,
  AnswerString: string): string;

implementation

uses
  uMHA_AUX_Const,
  fIA_dialogs,
  uMHA_AUX_Utils;


{*

 if more than 2 questions are skipped, the instrument cannot be scored.

*}


function ScoreAnswers_BSL23(
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string): string;

var
  total                     : integer ;
  i, Count,
  skipped                   : Integer;

  TempAnswer, TempResult    : string;

begin
  // AnswerString Format:
  //      83=1^1330~
  //      84=0^214~...
  //
  // '83' is QuestionID,
  // '1' is ItemIndex for radio group, but can be text from edit field
  // '1130' is ChoiceID (not used here)

  // This is used by InstrumentResults to do a quick check on the
  // instrument as to its use of the dll

  if AnswerString = 'BSL-23' then
    begin
      Result := 'YES';
      exit;
    end;

  skipped := 0 ;
  total := 0 ;
  TempAnswer := '' ;
  Result := '';
  Count := NumberOfPieces(AnswerString, RECORD_END);

  if Count < 1 then
    Exit;

  for i := 1 to Count - 1 do
    begin
      TempResult := Piece(AnswerString, RECORD_END, i);
      TempAnswer := Piece(Piece(TempResult, U, 1), EQ, 2);

      // if question has been skipped, can't add it to the score, so get next question response
      if (TempAnswer = '') or (TempAnswer = ' ') or (TempAnswer = '-1') or
        (TempAnswer = CHOICE_ID_OF_SKIPPED_MULTIPLE_CHOICE_QUESTION) or
        (TempAnswer = CHOICE_ID_OF_SKIPPED_BY_RULE_MC_QUESTION) or
        (TempAnswer = CHOICE_ID_OF_SKIPPED_REQUIRED_ANSWER) then skipped := skipped + 1
      else
      begin
        total := total + StrTointDef(TempAnswer, -1) ;
      end;
    end ;

  // Note: "BSL-23" is the name of the scale and it must be used here.
  // ~ScaleName^ScaleIEN^ScaleScore~
  if skipped > 1 then
    Result := 'BSL-23^^' + 'Missing data - cannot assess' + RECORD_END
  else
    Result := 'BSL-23^^' + inttostr(total) + RECORD_END ;
end;


function ProgressNote_BSL23(
  PatientName,
  PatientDOB,
  PatientSex,
  ScoreString,
  AnswerString: string): string;

const
  Indent                      : string = '   ';
var
  i                           : Integer;
  TempStr1,
    TempStr2                 : string;

begin
  Result := '';

  if (ScoreString = '') or (AnswerString = '') then
    Exit;

  for i := 1 to NumberOfPieces(ScoreString, RECORD_END) - 1 do
    begin
      TempStr1 := Piece(Piece(ScoreString, RECORD_END, i), U, 1);
      TempStr2 := Piece(Piece(ScoreString, RECORD_END, i), U, 3);

      Result := Result + Indent + TempStr1 + ': ' + TempStr2 + #13;
    end;
end;

end.

