unit uMHA_AUX_CAM;

interface

uses
  SysUtils;

function ScoreAnswers_CAM(
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string): string;

function ProgressNote_CAM(
  PatientName,
  PatientDOB,
  PatientSex,
  ScoreString,
  AnswerString: string): string;

implementation

uses
  uMHA_AUX_Const,
  fIA_dialogs,
  uMHA_AUX_Utils;


{*
  The format for output is used by Clinical reminders as their preferred method.
  The second field in the output is the Scale IEN, which is not used in IAT.

  //  CHANGE THIS..... make it pertinent to CAM

  Question 1 must be YES.
  For Question 1A through 1M, at least 7 have to be YES.
  Question 2 has to be YES.
  Question 3 has to be anything other than zero.

  If all these conditions are met, then a deliruim diagnosis is supported.


*}

function ScoreAnswers_CAM(
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string): string;

var
  Q1, Q2, Q3, Q4              : boolean;
  Count, i,
    TempAnswer                : Integer;

  CAM_Result,
    TempResult
                              : string;
begin
  // AnswerString Format:
  //      83=1^1330~
  //      84=0^214~...
  //
  // '83' is QuestionID,
  // '1' is ItemIndex for radio group, but can be text from edit field
  // '1130' is ChoiceID (not used here)

  // This is used by InstrumentResults to do a quick check on the
  // instrument as to its use of the dll

  if AnswerString = 'CAM' then
    begin
      Result := 'YES';
      exit;
    end;

  Result := '';
  Count := NumberOfPieces(AnswerString, RECORD_END);
  CAM_Result := 'Questions 1-4 not answered, no determination could be made.';
  Q1 := false;
  Q2 := false;
  Q3 := false;
  Q4 := false;

  // question 1 must = 1  and question 2 must = 1 or 2
  // AND
  // question 3 (i = 5) must = 1  or question 4 (i = 6) must > 0

  if Count < 1 then
    Exit;

  for i := 1 to Count - 1 do
    begin
      TempResult := Piece(AnswerString, RECORD_END, i);
      TempAnswer := StrToIntDef(Piece(Piece(TempResult, U, 1), EQ, 2), -1);

      if (TempAnswer = 1157) and ( (i = 1) or (i = 2) or (i = 5) or (i = 6) ) then
      begin
        CAM_Result := 'All questions needed to determine if deliruim is supported were ' +  
                      'not answered. A diagnosis of deliruim could not be determined.';
        Result := 'CAM^^' + CAM_Result + RECORD_END;
        exit;
      end;

      if (i = 1) and (TempAnswer = 1 ) then Q1 := true ; // Question 1

      if (i = 2) and ( (TempAnswer = 1) or (TempAnswer = 2) ) then Q2 := true ;

      if (i = 5) and (TempAnswer = 1) then Q3 := true ;

      if (i = 6) and (TempAnswer > 0) then Q4 := true ;

      if ( (Q1 = true) and (Q2 = true) and ((Q3 = true) or (Q4 = true)) ) then
        CAM_Result := 'The results of the CAM support the presence of deliruim.' 
      else
        CAM_Result := 'The results of the CAM do not suggest the presence of deliruim.' ;

    end ;

  // Note: "CDR" is the name of the scale and it must be used here.
  // ~ScaleName^ScaleIEN^ScaleScore~
  Result := 'CAM^^' + CAM_Result + RECORD_END;
end;


function ProgressNote_CAM(
  PatientName,
  PatientDOB,
  PatientSex,
  ScoreString,
  AnswerString: string): string;
// The header, footer and questions and answers are handled elsewhere; avoiding
// broker calls here.

const
  Indent                      : string = '   ';
var
  i                           : Integer;
  TempStr                     : string;
begin
  Result := '';

  if (ScoreString = '') or (AnswerString = '') then
    Exit;

  Result := Piece(Piece(ScoreString, RECORD_END, 1), U, 3);

end;

end.

