unit uMHA_AUX_SOCRATES_8A;

interface

uses
  SysUtils;

function ScoreAnswers_SOCRATES_8A(
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string): string;

function ProgressNote_SOCRATES_8A(
  PatientName,
  PatientDOB,
  PatientSex,
  ScoreString,
  AnswerString: string): string;

implementation

uses
  uMHA_AUX_Const,
  fIA_dialogs,
  uMHA_AUX_Utils;


{*
  The format for output is used by Clinical reminders as their preferred method.
  The second field in the output is the Scale IEN, which is not used in IAT.

  Questions 1, 3, 7, 10, 12, 15, 17 are used for the Recognition Scale. Add the
  results and the sum should be 7-35. The higher the scale the greater
  the person is presenting for alcohol treatment.

  Questions 2, 6 ,11, 16 are used for the Ambivalence Scale.  Add the results and the
  sum should be 4-20. The higher the scale the greater the person is presenting
  for alcohol treatment.

  Questions 4, 5, 8, 9, 13, 14, 18, 19 are used for the Taking Steps Scale.  Add
  the results and the sum should be 8-40. The higher the scale the greater the person
  is presenting for alcohol treatment.


*}

// using the scale name and raw score, return the decile score
function GetDScore(scaleName: string; rawScore: string): string;
begin
  Result := 'Insufficent information to determine' ;

  // get Decile score
  if scaleName = 'Recognition' then
    case strtoint(rawScore) of
       7..26: Result := '10 Very Low' ;
      27..28: Result := '20' ;
      29..30: Result := '30 Low' ;
          31: Result := '40' ;
      32..33: Result := '50 Medium' ;
          34: Result := '60' ;
          35: Result := '70 High' ;
    else
      Result := 'Recognition raw Score out of range to determine Decile Score' ;
    end;

  if scaleName = 'Ambivalence' then
    case strtoint(rawScore) of
        4..8: Result := '10 Very Low' ;
       9..11: Result := '20' ;
      12..13: Result := '30 Low' ;
          14: Result := '40' ;
          15: Result := '50 Medium' ;
          16: Result := '60' ;
          17: Result := '70 High' ;
          18: Result := '80' ;
          19: Result := '90 Very High' ;
    else
      Result := 'Ambivalence raw Score out of range to determine Decile Score' ;
    end;

  if scaleName = 'Taking steps' then
    case strtoint(rawScore) of
       8..25: Result := '10 Very Low' ;
      26..29: Result := '20' ;
          30: Result := '30 Low' ;
      31..32: Result := '40' ;
          33: Result := '50 Medium' ;
      34..35: Result := '60' ;
          36: Result := '70 High' ;
      37..38: Result := '80' ;
      39..40: Result := '90 Very High' ;
    else
      Result := 'Taking steps raw Score out of range to determine Decile Score' ;
    end;
end;



function ScoreAnswers_SOCRATES_8A(
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string): string;

var
  recog, amb,
    taks                      : double ;
  i, Count, TempAnswer        : Integer;

  TempResult                : string;

begin
  // AnswerString Format:
  //      83=1^1330~
  //      84=0^214~...
  //
  // '83' is QuestionID,
  // '1' is ItemIndex for radio group, but can be text from edit field
  // '1130' is ChoiceID (not used here)

  // This is used by InstrumentResults to do a quick check on the
  // instrument as to its use of the dll

  if AnswerString = 'SOCRATES 8A' then
    begin
      Result := 'YES';
      exit;
    end;

  recog := 0 ;
  amb := 0 ;
  taks := 0 ;
  TempAnswer := 0 ;
  Result := '';
  Count := NumberOfPieces(AnswerString, RECORD_END);

  if Count < 1 then
    Exit;

  for i := 1 to Count - 1 do
    begin
      TempResult := Piece(AnswerString, RECORD_END, i);
      TempAnswer := StrToIntDef(Piece(Piece(TempResult, U, 1), EQ, 2), -1);

      // if question has been skipped, can't add it to the score, so get next question response
      if (TempAnswer = 1155) then   // do nothing
      else
      begin
        // itemindex 0 = 1 so need to add 1 to TempAnswer
        if ( TempAnswer > -1 ) then
          TempAnswer := TempAnswer + 1 ;

        if ( (i = 1) or (i = 3) or (i = 7) or (i = 10) or
             (i = 12) or (i = 15) or (i = 17)) then
          recog := recog + TempAnswer ;

        if ( (i = 2) or (i = 6) or (i = 11) or (i = 16) ) then
          amb := amb + TempAnswer ;

        if ( (i = 4) or (i = 5) or (i = 8) or (i = 9) or
             (i = 13) or (i = 14) or (i = 18) or (i = 19) ) then
          taks := taks + TempAnswer ;
      end;
    end ;

  // Note: "Recognition" is the name of the scale and it must be used here.
  // ~ScaleName^ScaleIEN^ScaleScore~
  Result := 'Recognition^^' + floattostr(recog) + RECORD_END +
            'Ambivalence^^' + floattoStr(amb) + RECORD_END +
            'Taking steps^^' + floattostr(taks) + RECORD_END ;
end;


function ProgressNote_SOCRATES_8A(
  PatientName,
  PatientDOB,
  PatientSex,
  ScoreString,
  AnswerString: string): string;

const
  Indent                      : string = '   ';
var
  i                           : Integer;
  TempStr1, TempStr2,
    tempDecile,
      decilestr               : string;
begin
  Result := '';
  tempDecile := '' ;
  decilestr := '' ;

  if (ScoreString = '') or (AnswerString = '') then
    Exit;

  Result := #13 + 'Total Scale Scores:' + #13 ;

  for i := 1 to NumberOfPieces(ScoreString, RECORD_END) - 1 do
    begin
      TempStr1 := Piece(Piece(ScoreString, RECORD_END, i), U, 1);
      TempStr2 := Piece(Piece(ScoreString, RECORD_END, i), U, 3);

      Result := Result + Indent + TempStr1 + ': ' + TempStr2 + #13;
      tempDecile := GetDScore(TempStr1, TempStr2) ;
      decilestr := decilestr + Indent + TempStr1 + ': ' + tempDecile + #13;
    end;

  Result := Result + #13#13 + 'DECILE Scores:' ;
  Result := Result + #13 + decilestr;

end;

end.

