unit uMHA_AUX_SSTVOF;

// Introduced in patch 108 - llh

interface

uses
  SysUtils;

function ScoreAnswers_SSTVOF(
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string): string;

function ProgressNote_SSTVOF(
  PatientName,
  PatientDOB,
  PatientSex,
  ScoreString,
  AnswerString: string): string;

implementation

uses
  uMHA_AUX_Const,
  fIA_dialogs,
  uMHA_AUX_Utils;


{*
  The format for output is used by Clinical reminders as their preferred method.
  The second field in the output is the Scale IEN, which is not used in IAT.

  Questions 1, 3, 5 ,8 are used for the Satisfaction Scale.  Add the results and
  divide by 4 to get the score. Sum should be 4-28. The higher the scale the greater
  the satisfaction.

  Questions 2, 4, 6 ,9 are used for the Skill Scale.  Add the results and
  divide by 4 to get the score. Sum should be 4-28. The higher the scale the greater
  the skill.

  Questions 11,12,13,14,15 are used for the Skill Scale.  Add the results and
  divide by 4 to get the score. Sum should be 5-35. The higher the scale the greater
  the skill.


*}

function ScoreAnswers_SSTVOF(
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string): string;

var
  cnt1, cnt2, cnt3            : double ;
  i,Count,TempAnswer          : Integer;

    TempResult                : string;
begin
  // AnswerString Format:
  //      83=1^1330~
  //      84=0^214~...
  //
  // '83' is QuestionID,
  // '1' is ItemIndex for radio group, but can be text from edit field
  // '1130' is ChoiceID (not used here)

  // This is used by InstrumentResults to do a quick check on the
  // instrument as to its use of the dll

  if AnswerString = 'SST-VOF' then
    begin
      Result := 'YES';
      exit;
    end;

  cnt1 := 0 ;
  cnt2 := 0 ;
  cnt3 := 0 ;
  Result := '';
  Count := NumberOfPieces(AnswerString, RECORD_END);

  if Count < 1 then
    Exit;

  for i := 1 to Count - 1 do
    begin
      TempResult := Piece(AnswerString, RECORD_END, i);
      TempAnswer := StrToIntDef(Piece(Piece(TempResult, U, 1), EQ, 2), -1);

      // if question has been skipped, can't add it to the score, so get next question response
      if (TempAnswer = 1155) then   // do nothing
      else
      begin
        // itemindex 0 = 1 so need to add 1 to TempAnswer
        if ( TempAnswer > -1 ) then
          TempAnswer := TempAnswer + 1 ;

        if ( (i = 1) or (i = 3) or (i = 5) or (i = 8) ) then
          cnt1 := cnt1 + TempAnswer ;

        if ( (i = 2) or (i = 4) or (i = 6) or (i = 9) ) then
          cnt2 := cnt2 + TempAnswer ;

        // there is one un-numbered question and 1 unscored question that needs skipping.
        // >11 points to questions 11-15
        if (i > 11) then
          cnt3 := cnt3 + TempAnswer ;
      end;
    end ;

  if cnt1 > 0 then cnt1 := cnt1/4 ;

  if cnt2> 0 then cnt2 := cnt2/4 ;

  if cnt3 > 0 then cnt3 := cnt3/5 ;
    
  // Note: "Satisfaction" is the name of the scale and it must be used here.
  // ~ScaleName^ScaleIEN^ScaleScore~
  Result := 'Satisfaction^^' + floattostr(cnt1) + RECORD_END +
            'Skill^^' + floattoStr(cnt2) + RECORD_END +
            'Impact^^' + floattostr(cnt3) + RECORD_END ;
end;


function ProgressNote_SSTVOF(
  PatientName,
  PatientDOB,
  PatientSex,
  ScoreString,
  AnswerString: string): string;

// The header, footer and questions and answers are handled elsewhere; avoiding
// broker calls here.

const
  Indent                      : string = '   ';
var
  i                           : Integer;
  TempStr1,
    TempStr2                  : string;
begin
  Result := '';

  if (ScoreString = '') or (AnswerString = '') then
    Exit;

  Result := #13 + 'SST - Quality of Life Domains (Final Questionnaire)' + #13 + #13;

  for i := 1 to NumberOfPieces(ScoreString, RECORD_END) - 1 do
    begin
      TempStr1 := Piece(Piece(ScoreString, RECORD_END, i), U, 1);
      TempStr2 := Piece(Piece(ScoreString, RECORD_END, i), U, 3);

      Result := Result + Indent + TempStr1 + ': ' + TempStr2 + #13;
    end;

  Result := Result + #13 + Indent +
    'Range is 1 to 7 where a higher score indicates greater level of quality.';

end;

end.
