unit uMHA_AUX_WHODAS;

interface

uses
  SysUtils;

function ScoreAnswers_WHODAS(
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string): string;

function ProgressNote_WHODAS(
  PatientName,
  PatientDOB,
  PatientSex,
  ScoreString,
  AnswerString: string): string;


implementation

uses
  uMHA_AUX_Const,
  fIA_dialogs,
  uMHA_AUX_Utils;


{*
  The format for output is used by Clinical reminders as their preferred method.
  The second field in the output is the Scale IEN, which is not used in IAT.

  The WHODAS 2.0 has six domains and an overall score.


*}



// Scoring method is provided in WHODAS manual. There are two mehtods: simple
// and complex. We use the complex; it is more accurate.

function ScoreAnswers_WHODAS(
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string): string;

const
  Spacer                      : string = '      ';

var
  Count, i,
    Cog, Mobil, Self, Along, Life1, Life2, Part,
    CogM, MobilM, SelfM, AlongM, Life1M, Life2M, PartM: Integer;

  Working                     : boolean;
  CogStr, MobilStr, SelfStr, AlongStr, LifeStr1, LifeStr2, PartStr,
    SumStr, TempResult,
    TempStr1, TempStr2,
    TempAnswer                : string;

begin
  // AnswerString Format:
  //      83=1^1330~
  //      84=0^214~...
  //
  // '83' is QuestionID,
  // '1' is ItemIndex for radio group, but can be text from edit field
  // '1130' is ChoiceID (not used here)

  // This is used by InstrumentResults to do a quick check on the
  // instrument as to its use of the dll
  if AnswerString = 'WHODAS 2' then
    begin
      Result := 'YES';
      exit;
    end;

  Result := '';
  Count := NumberOfPieces(AnswerString, RECORD_END);

  if Count < 1 then
    Exit;

  Cog := 0;
  Mobil := 0;
  Self := 0;
  Along := 0;
  Life1 := 0;
  Life2 := 0;
  Part := 0;
  CogM := 0;
  MobilM := 0;
  SelfM := 0;
  AlongM := 0;
  Life1M := 0;
  Life2M := 0;
  PartM := 0;

  Working := false;
  TempStr1 := '';
  TempStr2 := '';
  CogStr := '';
  MobilStr := '';
  SelfStr := '';
  AlongStr := '';
  LifeStr1 := '';
  LifeStr2 := '';
  PartStr := '';
  SumStr := '';

  for i := 1 to Count - 1 do
    begin
      TempResult := Piece(AnswerString, RECORD_END, i);
      TempAnswer := Piece(Piece(TempResult, U, 1), EQ, 2);

      // Count the number of skipped (Blank) answers for each scale
      if (TempAnswer = '') or (TempAnswer = ' ') or
        (TempAnswer = '-1') or
        (TempAnswer = NOT_ASKED) or
        (TempAnswer = SKIPPED) or
        (TempAnswer = CHOICE_ID_OF_SKIPPED_MULTIPLE_CHOICE_QUESTION) or
        //(TempAnswer = CHOICE_ID_OF_SKIPPED_BY_RULE_MC_QUESTION) or
      (TempAnswer = CHOICE_ID_OF_SKIPPED_REQUIRED_ANSWER) then
        begin
          TempAnswer := '0';

          if (i >= 1) and (i < 7) then
            Inc(CogM);
          if (i >= 7) and (i < 12) then
            Inc(MobilM);
          if (i >= 12) and (i < 16) then
            Inc(SelfM);
          if (i >= 16) and (i < 21) then
            Inc(AlongM);
          if (i >= 21) and (i < 25) then
            Inc(Life1M);
          // Don't count #25: Are you working?
          if (i >= 26) and (i < 30) then
            Inc(Life2M);
          if (i >= 30) and (i < 38) then
            Inc(PartM);
        end // Skipped questions
      else
        begin
      // Cognition
          if (i = 1) then // D1.1
            begin
              Cog := Cog + StrToIntDef(TempAnswer, 0);
            end;
          if (i = 2) then // D1.2
            begin
              Cog := Cog + StrToIntDef(TempAnswer, 0);
            end;
          if (i = 3) then // D1.3
            begin
              Cog := Cog + StrToIntDef(TempAnswer, 0);
            end;
          if (i = 4) then // D1.4
            begin
              Cog := Cog + StrToIntDef(TempAnswer, 0);
            end;
          if (i = 5) then // D1.5
            begin
              if (TempAnswer = '1') or (TempAnswer = '2') then
                Cog := Cog + 1;
              if (TempAnswer = '3') or (TempAnswer = '4') then
                Cog := Cog + 2;
            end;
          if (i = 6) then // D1.6
            begin
              if (TempAnswer = '1') or (TempAnswer = '2') then
                Cog := Cog + 1;
              if (TempAnswer = '3') or (TempAnswer = '4') then
                Cog := Cog + 2;
            end;

        // Getting around - Mobility
          if (i = 7) then // D2.1
            begin
              Mobil := Mobil + StrToIntDef(TempAnswer, 0);
            end;
          if (i = 8) then // D2.2
            begin
              if (TempAnswer = '1') or (TempAnswer = '2') then
                Mobil := Mobil + 1;
              if (TempAnswer = '3') or (TempAnswer = '4') then
                Mobil := Mobil + 2;
            end;
          if (i = 9) then // D2.3
            begin
              if (TempAnswer = '1') or (TempAnswer = '2') then
                Mobil := Mobil + 1;
              if (TempAnswer = '3') or (TempAnswer = '4') then
                Mobil := Mobil + 2;
            end;
          if (i = 10) then // D2.4
            begin
              Mobil := Mobil + StrToIntDef(TempAnswer, 0);
            end;
          if (i = 11) then // D2.5
            begin
              Mobil := Mobil + StrToIntDef(TempAnswer, 0);
            end;

        // Self-Care
          if (i = 12) then // D3.1
            begin
              if (TempAnswer = '1') or (TempAnswer = '2') then
                Self := Self + 1;
              if (TempAnswer = '3') or (TempAnswer = '4') then
                Self := Self + 2;
            end;
          if (i = 13) then // D3.2
            begin
              Self := Self + StrToIntDef(TempAnswer, 0);
            end;
          if (i = 14) then // D3.3
            begin
              if (TempAnswer = '1') or (TempAnswer = '2') then
                Self := Self + 1;
              if (TempAnswer = '3') or (TempAnswer = '4') then
                Self := Self + 2;
            end;
          if (i = 15) then // D3.4
            begin
              if (TempAnswer = '1') or (TempAnswer = '2') then
                Self := Self + 1;
              if (TempAnswer = '3') or (TempAnswer = '4') then
                Self := Self + 2;
            end;

          // Getting Along
          if (i = 16) then // D4.1
            begin
              if (TempAnswer = '1') or (TempAnswer = '2') then
                Along := Along + 1;
              if (TempAnswer = '3') or (TempAnswer = '4') then
                Along := Along + 2;
            end;
          if (i = 17) then // D4.2
            begin
              if (TempAnswer = '1') or (TempAnswer = '2') then
                Along := Along + 1;
              if (TempAnswer = '3') or (TempAnswer = '4') then
                Along := Along + 2;
            end;
          if (i = 18) then // D4.3
            begin
              if (TempAnswer = '1') or (TempAnswer = '2') then
                Along := Along + 1;
              if (TempAnswer = '3') or (TempAnswer = '4') then
                Along := Along + 2;
            end;
          if (i = 19) then // D4.4
            begin
              Along := Along + StrToIntDef(TempAnswer, 0);
            end;
          if (i = 20) then // D4.5
            begin
              if (TempAnswer = '1') or (TempAnswer = '2') then
                Along := Along + 1;
              if (TempAnswer = '3') or (TempAnswer = '4') then
                Along := Along + 2;
            end;

          // Life activities: Household
          if (i = 21) then // D5.1
            begin
              if (TempAnswer = '1') or (TempAnswer = '2') then
                Life1 := Life1 + 1;
              if (TempAnswer = '3') or (TempAnswer = '4') then
                Life1 := Life1 + 2;
            end;
          if (i = 22) then // D5.2
            begin
              if (TempAnswer = '1') or (TempAnswer = '2') then
                Life1 := Life1 + 1;
              if (TempAnswer = '3') or (TempAnswer = '4') then
                Life1 := Life1 + 2;
            end;
          if (i = 23) then // D5.3
            begin
              Life1 := Life1 + StrToIntDef(TempAnswer, 0);
            end;
          if (i = 24) then // D5.4
            begin
              if (TempAnswer = '1') or (TempAnswer = '2') then
                Life1 := Life1 + 1;
              if (TempAnswer = '3') or (TempAnswer = '4') then
                Life1 := Life1 + 2;
            end;

          if (i = 25) then // Are you working
            begin
              if TempAnswer = '1' then // Yes
                Working := true;
            end;

          // Life activities: work/school
          if (i = 26) and Working then // D5.5
            begin
              if (TempAnswer = '1') or (TempAnswer = '2') then
                Life2 := Life2 + 1;
              if (TempAnswer = '3') or (TempAnswer = '4') then
                Life2 := Life2 + 2;
            end;
          if (i = 27) and Working then // D5.6
            begin
              Life2 := Life2 + StrToIntDef(TempAnswer, 0);
            end;
          if (i = 28) and Working then // D5.7
            begin
              Life2 := Life2 + StrToIntDef(TempAnswer, 0);
            end;
          if (i = 29) and Working then // D5.8
            begin
              Life2 := Life2 + StrToIntDef(TempAnswer, 0);
            end;

          // Participation in Society
          if (i = 30) then // D6.1
            begin
              if (TempAnswer = '1') or (TempAnswer = '2') then
                Part := Part + 1;
              if (TempAnswer = '3') or (TempAnswer = '4') then
                Part := Part + 2;
            end;
          if (i = 31) then // D6.2
            begin
              Part := Part + StrToIntDef(TempAnswer, 0);
            end;
          if (i = 32) then // D6.3
            begin
              if (TempAnswer = '1') or (TempAnswer = '2') then
                Part := Part + 1;
              if (TempAnswer = '3') or (TempAnswer = '4') then
                Part := Part + 2;
            end;
          if (i = 33) then // D6.4
            begin
              Part := Part + StrToIntDef(TempAnswer, 0);
            end;
          if (i = 34) then // D6.5
            begin
              Part := Part + StrToIntDef(TempAnswer, 0);
            end;
          if (i = 35) then // D6.6
            begin
              if (TempAnswer = '1') or (TempAnswer = '2') then
                Part := Part + 1;
              if (TempAnswer = '3') or (TempAnswer = '4') then
                Part := Part + 2;
            end;
          if (i = 36) then // D6.7
            begin
              Part := Part + StrToIntDef(TempAnswer, 0);
            end;
          if (i = 37) then // D6.8
            begin
              if (TempAnswer = '1') or (TempAnswer = '2') then
                Part := Part + 1;
              if (TempAnswer = '3') or (TempAnswer = '4') then
                Part := Part + 2;
            end;
        end; // for i
    end;

  // if more than two items are skipped, then bail out.
  if (CogM + MobilM + SelfM + AlongM + Life1M + Life2M + PartM) > 2 then
    begin
      Result := 'Too many missing answers. Max is 2.';
      exit;
    end;

  // One missing Cognitive score: Use the average of the other scores for missing
  // score.
  if (CogM = 1) then
    Cog := (Cog div 5) + Cog;
  CogStr := inttoStr(Cog * 100 div 20);

 // One missing Mobil score
  if (MobilM = 1) then
    Mobil := (Mobil div 5) + Mobil;
  MobilStr := inttoStr(Mobil * 100 div 16);

      // One missing Self score
  if (SelfM = 1) then
    Self := ((Self div 4) + Self);
  SelfStr := inttoStr(Self * 100 div 10);

      // One missing Getting Along score
  if (AlongM = 1) then
    Along := (Along div 5) + Along;
  AlongStr := inttoStr(Along * 100 div 12);

      // One missing Life score
  if (Life1M = 1) then
    Life1 := (Life1 div 4) + Life1;
  LifeStr1 := inttoStr(Life1 * 100 div 10);

  // One missing Life score for working folks
  if Working then
    begin
      if (Life2M = 1) then
        Life2 := (Life2 div 4) + Life2;
      LifeStr2 := inttoStr((Life2) * 100 div 14);
    end
    else
    LifeStr2 := 'N/A';

      // One missing participation score
  if (PartM = 1) then
    Part := (Part div 8) + Part;
  PartStr := inttoStr(Part * 100 div 24);

  if working then
    SumStr := inttoStr((Cog + Mobil + Self + Along + Life1 + Life2 + Part) * 100 div 106)
  else
    SumStr := inttoStr((Cog + Mobil + Self + Along + Life1 + Part) * 100 div 92);

  Result := 'Cognition^^' + CogStr + RECORD_END +
    'Mobility^^' + MobilStr + RECORD_END +
    'Self-care^^' + SelfStr + RECORD_END +
    'Getting along^^' + AlongStr + RECORD_END +
    'Life activities (household)^^' + LifeStr1 + RECORD_END +
    'Life activities (work/school)^^' + LifeStr2 + RECORD_END +
    'Participation^^' + PartStr + RECORD_END +
    'Summary^^' + SumStr;
end;


function ProgressNote_WHODAS(
  PatientName,
  PatientDOB,
  PatientSex,
  ScoreString,
  AnswerString: string): string;

// The header, footer, questions and answers are handled elsewhere; avoiding
// broker calls here. <.DLL_String.> provides location for this string in the
// report.

const
  Indent                      : string = '   ';
var
  i                           : Integer;
  TempStr1,
    TempStr2                  : string;
begin
  Result := '';

  if (ScoreString = '') or (AnswerString = '') then
    Exit;

  Result := #13 + #13 + ' WHO Disability Assessment Schedule Domains' + #13;

  for i := 1 to NumberOfPieces(ScoreString, RECORD_END) do
    begin
      TempStr1 := Piece(Piece(ScoreString, RECORD_END, i), U, 1);
      TempStr2 := Piece(Piece(ScoreString, RECORD_END, i), U, 3);
      Result := Result + Indent + TempStr1 + ':  ' + TempStr2 + #13;
    end;

  Result := Result + #13 + Indent +
    'Range is 0 to 100 where 0 indicates no disability and 100 means full disability.' + #13 + #13;
end;

end.

