unit uMHA_AUX_WHYMPI;

interface

uses
  SysUtils;

function ScoreAnswers_WHYMPI(
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string): string;

function ProgressNote_WHYMPI(
  PatientName,
  PatientDOB,
  PatientSex,
  ScoreString,
  AnswerString: string): string;

implementation

uses
  uMHA_AUX_Const,
  fIA_dialogs,
  uMHA_AUX_Utils;


{*
  The format for output is used by Clinical reminders as their preferred method.
  The second field in the output is the Scale IEN, which is not used in IAT.

  The WHYMPI has three groups of scales called Part I, II & III.
  A score is not reported if there are 25% questions not answered;
  this includes the overall score.

*}


// Averaging scoring method:

function ScoreAnswers_WHYMPI(
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string): string;

var
  {Part I}
  Inter, Sup, Pain, LC, Affect,
    InterSum, SupSum, PainSum, LCSum, AffectSum,
  {Part II}
  Neg, Sol, Dist,
    NegSum, SolSum, DistSum,
  {PART III}
  House, Work, Home, Soc, Gen,
    HouseSum, WorkSum, HomeSum, SocSum, GenSum,
    Count, i                  : Integer;

  InterSc, SupSc, PainSc, LCSc, AffectSc,
    NegSc, SolSc, DistSc,
    HouseSc, WorkSc, HomeSc, SocSc, GenSc
                              : Extended;
  TempResult,
    TempAnswer                : string;
  PChr                        : PChar;
begin
  // AnswerString Format:
  //      83=1^1330~
  //      84=0^214~...
  //
  // '83' is QuestionID,
  // '1' is ItemIndex for radio group, but can be text from edit field
  // '1130' is ChoiceID (not used here)

  // This is used by InstrumentResults to do a quick check on the
  // instrument as to its use of the dll
  if AnswerString = 'WHYMPI' then
  begin
     Result := 'YES';
     exit;
  end;

  Result := '';
  Count := NumberOfPieces(AnswerString, RECORD_END);

  if Count < 1 then
    Exit;

  Inter := 0;
  Sup := 0;
  Pain := 0;
  LC := 0;
  Affect := 0;
  InterSum := 0;
  SupSum := 0;
  PainSum := 0;
  LCSum := 0;
  AffectSum := 0;
  {Part II}
  Neg := 0;
  Sol := 0;
  Dist := 0;
  NegSum := 0;
  SolSum := 0;
  DistSum := 0;
  {PART III}
  House := 0;
  Work := 0;
  Home := 0;
  Soc := 0;
  Gen := 0;
  HouseSum := 0;
  WorkSum := 0;
  HomeSum := 0;
  SocSum := 0;
  GenSum := 0;

  InterSc := 0;
  SupSc := 0;
  PainSc := 0;
  LCSc := 0;
  AffectSc := 0;
  NegSc := 0;
  SolSc := 0;
  DistSc := 0;
  HouseSc := 0;
  WorkSc := 0;
  HomeSc := 0;
  SocSc := 0;
  GenSc := 0;

  for i := 1 to Count - 1 do
    begin

      TempResult := Piece(AnswerString, RECORD_END, i);
      TempAnswer := Piece(Piece(TempResult, U, 1), EQ, 2);

      // Count the number of skipped (Blank) answers for each scale
      if (TempAnswer = '') or (TempAnswer = ' ') or
        (TempAnswer = '-1') or
        (TempAnswer = CHOICE_ID_OF_SKIPPED_MULTIPLE_CHOICE_QUESTION) or
        (TempAnswer = CHOICE_ID_OF_SKIPPED_BY_RULE_MC_QUESTION) or
        (TempAnswer = CHOICE_ID_OF_SKIPPED_REQUIRED_ANSWER) then
        begin
          // Interference
          if (i = 5) or (i = 7) or (i = 8) or (i = 12) or
            (i = 13) or (i = 17) or (i = 19) or (i = 22) or (i = 24) then
            Inc(Inter);
          // Support
          if (i = 9) or (i = 14) or (i = 20) then
            Inc(Sup);
          // Pain Severity
          if (i = 4) or (i = 11) or (i = 16) then
            Inc(Pain);
          // Life-Control
          if (i = 15) or (i = 21) then
            Inc(LC);
          // Affective Distress
          if (i = 10) or (i = 23) or (i = 25) then
            Inc(Affect);
          // Negative Response
          if (i = 26) or (i = 29) or (i = 32) or (i = 35) then
            Inc(Neg);
            // Solicitous Response
          if (i = 27) or (i = 30) or (i = 33) or (i = 36) or (i = 38) or (i = 39) then
            Inc(Sol);
            //Distracting Response
          if (i = 28) or (i = 31) or (i = 34) or (i = 37) then
            Inc(Dist);
            // Household Chores
          if (i = 40) or (i = 44) or (i = 48) or (i = 52) or (i = 56) then
            Inc(House);
            // Outdoor Work
          if (i = 41) or (i = 45) or (i = 49) or (i = 53) or (i = 57) then
            Inc(Work);
            // Activities away from home
          if (i = 42) or (i = 46) or (i = 50) or (i = 54) then
            Inc(Home);
            // Social Activities
          if (i = 43) or (i = 47) or (i = 51) or (i = 55) then
            Inc(Soc);
            // General Activity
          Gen := House + Work + Home + Soc;
        end // Skipped questions
      else
        // Tally the scale's raw sum
        begin
          // Interference
          if (i = 5) or (i = 7) or (i = 8) or (i = 12) or
            (i = 13) or (i = 17) or (i = 19) or (i = 22) or (i = 24) then
            InterSum := InterSum + StrToIntDef(TempAnswer, 0);

          // Support
          if (i = 9) or (i = 14) or (i = 20) then
            SupSum := SupSum + StrToIntDef(TempAnswer, 0);

          // Pain Severity
          if (i = 4) or (i = 11) or (i = 16) then
            PainSum := PainSum + StrToIntDef(TempAnswer, 0);

          // Life-Control
          if (i = 15) or (i = 21) then
            LCSum := LCSum + StrToIntDef(TempAnswer, 0);

          // Affective Distress
          if (i = 9) or (i = 21) or (i = 23) then
            AffectSum := AffectSum + StrToIntDef(TempAnswer, 0);

          // Negative Response
          if (i = 26) or (i = 29) or (i = 32) or (i = 35) then
            NegSum := NegSum + StrToIntDef(TempAnswer, 0);

            // Solicitous Response
          if (i = 27) or (i = 30) or (i = 33) or (i = 36) or (i = 38) or (i = 39) then
            SOLSum := SolSum + StrToIntDef(TempAnswer, 0);

            //Distracting Response
          if (i = 28) or (i = 31) or (i = 34) or (i = 37) then
            DistSum := DistSum + StrToIntDef(TempAnswer, 0);

            // Household Chores
          if (i = 40) or (i = 44) or (i = 48) or (i = 52) or (i = 56) then
            HouseSum := HouseSum + StrToIntDef(TempAnswer, 0);

            // Outdoor Work
          if (i = 41) or (i = 45) or (i = 49) or (i = 53) or (i = 57) then
            WorkSum := WorkSum + StrToIntDef(TempAnswer, 0);

            // Activities away from home
          if (i = 42) or (i = 46) or (i = 50) or (i = 54) then
            HomeSum := HomeSum + StrToIntDef(TempAnswer, 0);

            // Social Activities
          if (i = 43) or (i = 47) or (i = 51) or (i = 55) then
            SocSum := SocSum + StrToIntDef(TempAnswer, 0);

            // General Activity
          Gen := HouseSum + WorkSum + HomeSum + SocSum;

        end; // Tally scale's raw sum
    end; // for i

  // Calculate the average for each scale, given that there are
  // more than 25% questions for that scale.

  if Inter < 3 then // Number of skipped questions
    begin
      InterSc := InterSum / (9 - Inter);
      Result := 'Interference^^' + FloatToStrF(InterSc, ffFixed, 8, 2) + RECORD_END;
    end
  else
    Result := 'Interference^^Too many skipped questions' + RECORD_END;

  if Sup < 2 then
    begin
      SupSc := SupSum / (3 - Sup);
      Result := Result + 'Support^^' + FloatToStrF(SupSc, ffFixed, 8, 2) + RECORD_END;
    end
  else
    Result := Result + 'Support^^Too many skipped questions' + RECORD_END;

  if Pain < 2 then
    begin
      PainSc := PainSum / (3 - Pain);
      Result := Result + 'Pain Severity^^' + FloatToStrF(PainSc, ffFixed, 8, 2) + RECORD_END;
    end
  else
    Result := Result + 'Pain Severity^^Too many skipped questions' + RECORD_END;

  if LC < 2 then
    begin
      LCSc := LCSum / (2 - LC);
      Result := Result + 'Life-Control^^' + FloatToStrF(LCSc, ffFixed, 8, 2) + RECORD_END;
    end
  else
    Result := Result + 'Life-Control^^Too many skipped questions' + RECORD_END;

  if Affect < 2 then
    begin
      AffectSc := AffectSum / (3 - Affect);
      Result := Result + 'Affective Distress^^' + FloatToStrF(AffectSc, ffFixed, 8, 2) + RECORD_END;
    end
  else
    Result := Result + 'Affective Distress^^Too many skipped questions' + RECORD_END;

  if Neg < 2 then
    begin
      NegSc := NegSum / (4 - Neg);
      Result := Result + 'Negative Responses^^' + FloatToStrF(NegSc, ffFixed, 8, 2) + RECORD_END;
    end
  else
    Result := Result + 'Negative Responses^^Too many skipped questions' + RECORD_END;

  if Sol < 2 then
    begin
      SolSc := SolSum / (6 - Sol);
      Result := Result + 'Solicitous Responses^^' + FloatToStrF(SolSc, ffFixed, 8, 2) + RECORD_END;
    end
  else
    Result := Result + 'Solicitous Responses^^Too many skipped questions' + RECORD_END;

  if Dist < 2 then
    begin
      DistSc := DistSum / (4 - Dist);
      Result := Result + 'Distracting Responses^^' + FloatToStrF(DistSc, ffFixed, 8, 2) + RECORD_END;
    end
  else
    Result := Result + 'Distracting Responses^^Too many skipped questions' + RECORD_END;

  if House < 2 then
    begin
      HouseSc := HouseSum / (5 - House);
      Result := Result + 'Household Chores^^' + FloatToStrF(HouseSc, ffFixed, 8, 2) + RECORD_END;
    end
  else
    Result := Result + 'Household Chores^^Too many skipped questions' + RECORD_END;

  if Work < 2 then
    begin
      WorkSc := WorkSum / (5 - Work);
      Result := Result + 'Outdoor Work^^' + FloatToStrF(WorkSc, ffFixed, 8, 2) + RECORD_END;
    end
  else
    Result := Result + 'Outdoor Work^^Too many skipped questions' + RECORD_END;

  if Home < 2 then
    begin
      HomeSc := HomeSum / (4 - Home);
      Result := Result + 'Activities Away from Home^^' + FloatToStrF(HomeSc, ffFixed, 8, 2) + RECORD_END;
    end
  else
    Result := Result + 'Activities Away from Home^^Too many skipped questions' + RECORD_END;

  if Soc < 2 then
    begin
      SocSc := SocSum / (4 - Soc);
      Result := Result + 'Social Activities^^' + FloatToStrF(SocSc, ffFixed, 8, 2) + RECORD_END;
    end
  else
    Result := Result + 'Social Activities^^Too many skipped questions' + RECORD_END;

  Gen := House + Work + Home + Soc;
  if Gen < 5 then
    begin
      GenSc := (HouseSum + WorkSum + HomeSum + SocSum) /
        (18 - Gen);
      Result := Result + 'General Activity^^' + FloatToStrF(GenSc, ffFixed, 8, 2) + RECORD_END;
    end
  else
    Result := Result + 'General Activity^^Too many skipped questions' + RECORD_END;
end;


function ProgressNote_WHYMPI(
  PatientName,
  PatientDOB,
  PatientSex,
  ScoreString,
  AnswerString: string): string;

// The header, footer, questions and answers are handled elsewhere; avoiding
// broker calls here.  <.DLL_String.> provides location for this string in the
// report.

const
  Indent                      : string = '   ';
var
  i                           : Integer;
  TempStr                     : string;
begin
  Result := '';

  if (ScoreString = '') or (AnswerString = '') then
    Exit;

  Result := 'WHYMPI Scales' + #13;

  for i := 1 to NumberOfPieces(ScoreString, RECORD_END) - 1 do
    begin
      if i = 1 then
        Result := Result + #13 + ' Part I'+ #13;
      if i = 6 then
        Result := Result + #13 + ' Part II'+ #13;
      if i = 9 then
        Result := Result + #13 + ' Part III'+ #13;

      TempStr := Piece(Piece(ScoreString, RECORD_END, i), U, 1);
      while Length(TempStr) < 35 do
        TempStr := TempStr + ' ';
      Result := Result + Indent + TempStr +
        Piece(Piece(ScoreString, RECORD_END, i), U, 3) + #13;
    end;

//  Result := Result + #13 + Indent +
//    'Note: Lower scores indicate less frequent symptoms or difficulty' + #13 +
//    Indent +
//    '      whereas higher scores reflect more serious symptoms or difficulty.';

//  Result := Result + #13;// + #13 +
  //  Indent + '------------ WHYMPI Responses ----------' + #13 + #13;

end;


end.

