unit fIA_Graph;

(*

This form is used by Display Instrument and
Instrument Authoring Tool to display the graph
of scale results.

*)

interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  JvExControls,
  JvComponent,
  JvXPCore,
  JvXPButtons,
  ExtCtrls,
  TeEngine,
  Series,
  TeeProcs,
  Chart,
  Grids,
  BaseGrid,
  AdvGrid,
  RzPanel;

type
  TfrmIA_Graph = class(TForm)
    pnlTop: TPanel;
    Splitter1: TSplitter;
    pnlBottom: TPanel;
    pnlButtons: TPanel;
    btnClose: TJvXPButton;
    grpScores: TRzGroupBox;
    grdScores: TAdvStringGrid;
    chrtScores: TChart;
    Series3: TLineSeries;
    Series2: TLineSeries;
    Series1: TLineSeries;
    LineSeries1: TLineSeries;
    Series4: TLineSeries;
    Series5: TLineSeries;
    Series9: TLineSeries;
    Series10: TLineSeries;
    Series6: TLineSeries;
    Series7: TLineSeries;
    Series8: TLineSeries;
    Series11: TLineSeries;
    Series12: TLineSeries;
    pnlHeader: TPanel;
    procedure FormShow(Sender: TObject);

    function GetScaleScore(AScaleName: string): string;
    procedure GraphScores(IsMultiScale: Boolean;
      aTitle: string);
    procedure InitChart(grp: Integer);
    procedure BuildGrid;

    function DetermineGroup(const aTitle: string): Integer;
    procedure GraphSingleScale(aTitle: string);
    procedure GraphSeveralScales(aTitle: string);
    procedure btnCloseClick(Sender: TObject);

    procedure grdScoresClickCell(Sender: TObject; ARow, ACol: Integer);
    function GetXLabel(aScaleName: string): string;
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  frmIA_Graph                 : TfrmIA_Graph;

implementation

{$R *.dfm}

uses
  uIA_Const,
  uIA_GUIControl,
  uIA_Utils,
  fIA_Dialogs,
  uIA_Instrument,
  uIA_Answer,
  uIA_Scale,
  uIA_ScaleGroup,
  uIA_Key,
  uIA_Scales_IO;

var
  lvDateColWidth              : Integer;
  lvIsMultiple                : Boolean;


procedure TfrmIA_Graph.FormShow(Sender: TObject);
var
  TempStr                     : string;
begin
  with GUI_Control do
    begin
      Caption := Instrument.Name + ': Graph of scales';
      lvDateColWidth := TextWidth('XXX/XX/XXXXX', gvFont);
      pnlButtons.Height := ButtonHeight * 2;
      pnlHeader.Height := pnlHeaderHeight;

 (*     btnPrior.Height := ButtonHeight;
      btnPrior.Width := ButtonWidth(btnPrior.Caption);
      btnPrior.Top := (pnlButtons.Height - btnClose.Height) div 2;
      // left set by anchors

      btnNext.Height := ButtonHeight;
      btnNext.Width := ButtonWidth(btnNext.Caption);
      btnNext.Top := (pnlButtons.Height - btnClose.Height) div 2;
      btnNext.Left := (2 * btnPrior.Left) + btnPrior.Width;*)

      btnClose.Height := ButtonHeight;
      btnClose.Width := ButtonWidth(btnClose.Caption);
      btnClose.Top := (pnlButtons.Height - btnClose.Height) div 2;
      // left set by anchors
      //btnNext.Visible := False;
      //btnPrior.Visible := False;
      lvIsMultiple := False;
    end;

  InitChart(0);
  BuildGrid;

  if Instrument.ListOfScaleGroups.Count > 1 then
    begin
      //btnNext.Visible := True;
      //btnPrior.Visible := True;
      lvIsMultiple := True;
      GraphScores(False, grdscores.cells[0, 2]);
    end
  else
    begin
      GraphScores(False, grdscores.cells[0, 1]);
    end;
end;


function TfrmIA_Graph.GetScaleScore(AScaleName: string): string;
var
  Score,
    i, j, k                   : Integer;

begin
  Result := 'X';
  Score := 0;

  if Instrument.ListOfScales.Count = 0 then
    Exit;

  for i := 0 to Instrument.ListOfScales.Count - 1 do
    begin
      if AScaleName = TScale(Instrument.ListOfScales[i]).Name then
        begin
          for j := 0 to Instrument.ListOfKeys.Count - 1 do
            begin
              if TScale(Instrument.ListOfScales[i]).IEN =
                TKey(Instrument.ListOfKeys[j]).ScaleID then
                begin
                  for k := 0 to Instrument.ListOfAnswers.Count - 1 do
                    begin
                      if ((TAnswer(Instrument.ListOfAnswers[k]).QuestionID) =
                        (TKey(Instrument.ListOfKeys[j]).QuestionID)) and
                        ((TAnswer(Instrument.ListOfAnswers[k]).AnswerText) =
                        (TKey(Instrument.ListOfKeys[j]).AnswerText)) then
                        Score := Score + StrToInt(TKey(Instrument.ListOfKeys[j]).Value);

                    end; // for k
                end; // if
            end; // for j
        end; // if
    end; // for i
  Result := IntToStr(Score);
end;


procedure TfrmIA_Graph.GraphScores(IsMultiScale: Boolean;
  aTitle: string);
begin

  if IsMultiScale then
    GraphSeveralScales(aTitle)
  else
    GraphSingleScale(aTitle);

end;


procedure TfrmIA_Graph.InitChart(grp: Integer);
var
  i                           : integer;
begin

  //clear the chart
  with chrtScores do
    begin
      Legend.Visible := False;
      Legend.FirstValue := 0;
      with LeftAxis do
        begin
          AutomaticMaximum := False;
          Minimum := -1;
          Maximum := 1;

          for i := 0 to 12 do
            begin
              SeriesList.Series[i].Clear;
              SeriesList.Series[i].Active := False;
            end;

          if Instrument.ListOfScaleGroups.Count = 0 then
            Exit;

          Maximum := TScaleGroup(Instrument.ListOfScaleGroups[grp]).OrdinateMax;
          if Maximum = -1 then
            AutomaticMaximum := True;
          if TScaleGroup(Instrument.ListOfScaleGroups[grp]).OrdinateMin <= Maximum then
            Minimum := TScaleGroup(Instrument.ListOfScaleGroups[grp]).OrdinateMin;
          Title.Caption := TScaleGroup(Instrument.ListOfScaleGroups[grp]).OrdinateTitle;
          if TScaleGroup(Instrument.ListOfScaleGroups[grp]).OrdinateIncrement = -1 then
            increment := 1
          else
            Increment := TScaleGroup(Instrument.ListOfScaleGroups[grp]).OrdinateIncrement;
        end; //with LeftAxis

    end; //with chrtTestGraph

end;

procedure TfrmIA_Graph.BuildGrid;
// Screen readers will have a different table configuration.
var
  acol, arow, i, j            : integer;
  tmpString                   : string;
  TempDate                    : TDateTime;

const
  indent                      = '   ';

begin
  arow := 0;

  with grdScores do
    begin
      Clear;
      Font.Assign(gvFont);

      DefaultRowHeight := 3 * (GUI_Control.TextHeight('Yy', gvFont))
        div 2;
      grpScores.Caption := ' Scores ';

      SelectionRectangle := False;
      ShowSelection := True;
      Options := [goFixedVertLine, goFixedHorzLine, goVertLine, goHorzLine,
        goDrawFocusSelected, goRowSelect]; //goColSizing,
      FixedFont.Assign(gvFont);
      FixedRowAlways := True;
      FixedColAlways := True;
      FixedRows := 1;
      FixedCols := 1;
      FixedRowHeight := DefaultRowHeight;
      FixedColWidth := lvDateColWidth;
      SelectionColor := clHighlight;
      SelectionTextColor := clHighlightText;
      ShowHint := True;
      // This number is too large, it will be lowered later when the actual
      // number of dates are known.
      ColCount := 2; //Result_OPS.ListOfResults.Count;
      ColWidths[0] := (5 * lvDateColWidth) div 2;

      RowCount := Instrument.ListOfScales.Count + Instrument.ListOfScaleGroups.Count;
      SelectionColor := clWindow;
      SelectionTextColor := clWindowText;
      arow := 1;
      Cells[1, 0] := DateToStr(Date);

      if Instrument.ListOfScaleGroups.Count > 1 then
        begin
          FontStyles[0, 1] := [fsBold];
          for i := 0 to Instrument.ListOfScaleGroups.Count - 1 do
            begin

              Cells[0, arow] := TScaleGroup(Instrument.ListOfScaleGroups[i]).GroupName;
              inc(arow);
              for j := 0 to Instrument.ListOfScales.Count - 1 do
                begin
                  if (TScale(Instrument.ListOfScales[j]).GroupIEN =
                    TScaleGroup(Instrument.ListOfScaleGroups[i]).IEN) then
                    begin
                      Cells[0, arow] := indent + TScale(Instrument.ListOfScales[j]).Name;
                      Cells[1, arow] := GetScaleScore(TScale(Instrument.ListOfScales[j]).Name);
                      inc(arow);
                    end;
                end;
            end;
        end
      else
        begin
          for j := 0 to Instrument.ListOfScales.Count - 1 do
            begin
              Cells[0, arow] := ' ' + TScale(Instrument.ListOfScales[j]).Name;
              Cells[1, arow] := GetScaleScore(TScale(Instrument.ListOfScales[j]).Name);
              inc(arow);
            end;
        end;
      RowCount := arow;

      ColumnSize.Stretch := False;
    end; //with grdScores
end;


function TfrmIA_Graph.DetermineGroup(const aTitle: string): Integer;
var
  i, j                        : Integer;
  TempTitle,
    TempIEN                   : string;
begin
  result := -1;

  if not lvIsMultiple then
    begin
      result := 0;
      Exit;
    end;

  TempTitle := Trim(aTitle);
  for i := 0 to Instrument.ListOfScaleGroups.Count - 1 do
    if TempTitle = TScaleGroup(Instrument.ListOfScaleGroups[i]).GroupName then
      begin
        result := i;
        Exit;
      end;

  for i := 0 to Instrument.ListOfScales.Count - 1 do
    if CompareStr(TempTitle, TScale(Instrument.ListOfScales[i]).Name) = 0 then
      begin
        TempIEN := TScale(Instrument.ListOfScales[i]).GroupIEN;
        for j := 0 to Instrument.ListOfScaleGroups.Count - 1 do
          if TempIEN = TScaleGroup(Instrument.ListOfScaleGroups[j]).IEN then
            begin
              result := j;
              Exit;
            end;
      end;
end;

function TfrmIA_Graph.GetXLabel(aScaleName: string): string;
var
  i                           : Integer;
begin
  Result := '';

  for i := 0 to Instrument.ListOfScales.Count - 1 do

    if TScale(Instrument.ListOfScales[i]).Name = aScaleName then
      begin
        result := TScale(Instrument.ListOfScales[i]).xLabel;
        if result = '' then
          result := Copy(aScaleName, 1, 3);
        break;
      end;
end;


procedure TfrmIA_Graph.GraphSeveralScales(aTitle: string);
var
  grp, i, StartHere, c,
    icol, irow, iseries, ordinate: integer;
  MyColor                     : TColor;
  TempStr, TempStr1,
    xlabel, CellValue         : string;

  procedure DrawGridLines(XValue: integer; XLabel: string);
  var
    DrawGrid                  : BOOLEAN;
    GridOrdinate              : array[0..2] of integer;
    ii                        : integer;
  begin

    GridOrdinate[0] := TScaleGroup(Instrument.ListOfScaleGroups[grp]).Grid1Ordinate;
    GridOrdinate[1] := TScaleGroup(Instrument.ListOfScaleGroups[grp]).Grid2Ordinate;
    GridOrdinate[2] := TScaleGroup(Instrument.ListOfScaleGroups[grp]).Grid3Ordinate;

    for ii := 0 to 2 do
      begin
        DrawGrid := (GridOrdinate[ii] > -1);
        chrtScores.SeriesList.Series[ii].Active := DrawGrid;

        if DrawGrid then
          chrtScores.SeriesList.Series[ii].AddXY(XValue, GridOrdinate[ii],
            Xlabel, clBlack);
      end;
  end;


begin

  with chrtScores do
    begin
      if aTitle = 'ByDate' then
        begin
          if grdScores.ColCount < 2 then
            Exit;
          // Get first group-scale info
          TempStr := '';
          for i := 1 to grdScores.RowCount do
            begin
              if grdScores.FontStyles[0, i] = [fsBold] then
                begin
                  TempStr := grdScores.Cells[0, i];

                  if TempStr[1] <> ' ' then
                    begin
                      Title.Text.Text := TempStr;
                      grdScores.FontStyles[0, i] := [fsBold];
                      StartHere := i;
                      Break;
                    end;
                end;
            end; // for i

          if TempStr <> '' then
            begin
              grp := DetermineGroup(TempStr);
              if grp = -1 then
                begin
                  ShowMessage('Unable to determine the Scale-Group.');
                  Exit;
                end;

              iseries := 3; // Grid lines are 0,1,2
              InitChart(grp);
              Legend.Visible := True;

              for c := 1 to grdScores.ColCount do
                begin
                  if grdScores.FontStyles[c, 0] = [fsBold] then
                    begin
                      SeriesList.Series[iseries].Active := True;
                      MyColor := SeriesList.Series[iseries].SeriesColor;
                      // Date
                      SeriesList.Series[iseries].Title :=
                        grdScores.Cells[c {Col}, 0 {Row}];

                      for i := StartHere + 1 to grdScores.RowCount do
                        begin
                          TempStr1 := grdScores.Cells[0, i];
                          if Length(Tempstr1) > 1 then
                            if TempStr1[1] = ' ' then
                              begin
                                CellValue := Trim(grdScores.Cells[c, i]);

                              //be sure the cell contains an integer
                                if (StrToInt64Def(CellValue, -1) > -1) then
                                  begin
                                    ordinate := StrToInt(CellValue);
                                  // Scale name to XLabel
                                    xlabel := GetXLabel(Trim(TempStr1));

                                    SeriesList.Series[iseries].AddXy(i, ordinate,
                                      xLabel, MyColor);
                                  end; //if cell contains an integer
                              end // If a scale
                            else
                              break;
                        end; // for i
                    end; // bold

                  Inc(iseries);
                  if iseries > 13 then
                    Exit;
                end; // for c
            end; //  TempStr not blank
        end; // ByDate

    end; // with ChartScores
end;


procedure TfrmIA_Graph.GraphSingleScale(aTitle: string);
var
  i, grp                      : Integer;

  procedure DrawGridLines;
  var
    GridOrdinate              : array[0..2] of integer;
    ii, jj                    : integer;
  begin

    GridOrdinate[0] := TScaleGroup(Instrument.ListOfScaleGroups[grp]).Grid1Ordinate;
    GridOrdinate[1] := TScaleGroup(Instrument.ListOfScaleGroups[grp]).Grid2Ordinate;
    GridOrdinate[2] := TScaleGroup(Instrument.ListOfScaleGroups[grp]).Grid3Ordinate;

    for ii := 0 to 2 do
      begin
        if (GridOrdinate[ii] > -1) then
          begin
            chrtScores.SeriesList.Series[ii].Active := true;

            for jj := 1 to grdScores.ColCount - 1 do
              chrtScores.SeriesList.Series[ii].AddXY(
                StrToDateTime(grdScores.Cells[jj, 0]),
                GridOrdinate[ii],
                grdScores.Cells[jj, 0],
                clBlack);
          end
        else
          chrtScores.SeriesList.Series[ii].Active := False;
      end;
  end;

begin
  grp := DetermineGroup(aTitle);

  if grp = -1 then
    begin
      ShowMessage('Unable to determine the Scale-Group.');
      Exit;
    end;

  InitChart(grp);

  with chrtScores do
    begin
      Legend.Visible := False;
      Title.Text.Text := aTitle;
      SeriesList.Series[3].Active := True;

      for i := 1 to grdScores.ColCount - 1 do
        begin
          if (grdScores.Cells[i, 0] <> '') then
            SeriesList.Series[3].AddXY(StrToDateTime(grdScores.Cells[i, 0]),
              StrToInt64Def(grdScores.Cells[i, grdScores.row], 0),
              grdScores.Cells[i, 0], clred);
        end;
    end; //with chrtScores

  DrawGridLines;
end;

procedure TfrmIA_Graph.grdScoresClickCell(Sender: TObject;
  ARow, ACol: Integer);
var
  i                           : Integer;
  TempStr                     : string;
begin

  grdScores.SelectionColor := clHighlight;
  grdScores.SelectionTextColor := clHighlightText;

  if (ARow = 0) then // Date selected
    begin
      if grdScores.FontStyles[aCol, 0] = [fsBold] then
        grdScores.FontStyles[aCol, 0] := []
      else
        begin
          grdScores.FontStyles[aCol, 0] := [fsBold];
          grdScores.SelectCols(aCol, 1);
        end;
      GraphScores(true, 'ByDate');
    end
  else // Scale or Scale-Group selected
    begin
      for i := 0 to grdScores.ColCount - 1 do
        grdScores.FontStyles[i, 0] := [];
      grdScores.row := ARow;

      TempStr := grdScores.Cells[0, grdScores.row];
      if TempStr <> '' then
        if TempStr[1] = ' ' then // scale name
          GraphScores(false, TempStr)
        else
          begin
            for i := 0 to grdScores.RowCount - 1 do
              grdScores.FontStyles[0, i] := [];
            grdScores.FontStyles[0, aRow] := [fsBold];
            GraphScores(true, TempStr);
          end;
    end;

end;


procedure TfrmIA_Graph.btnCloseClick(Sender: TObject);
begin
  Close;
end;


end.

