unit uIA_GUIControl;

interface

uses
  SysUtils,
  WinProcs,
  WinTypes,
  Windows,
  Messages,
  Classes,
  Graphics,
  Controls,
  Forms,
  fIA_Dialogs,
  ComCtrls,
  StdCtrls,
  ExtCtrls,
  RzPanel,
  RzLabel,
  Buttons;

type

  TGUI_Control = class(TObject)
  private
    { Private declarations }
    FQuestionHeight, // Height of Item
      FpnlHeaderHeight,
      FlblHeaderHeight,
      FlblHeaderFontSize,
      FlblPatientFontSize: Integer;
    FNeedGUIUpdate: boolean;

  protected
  public
    { Public declarations }
    constructor Create;
    destructor Destroy; override;
    property NeedGUIUpdate: Boolean read FNeedGUIUpdate write FNeedGUIUpdate;

    function GetTop(const aHeight: integer): Integer;
    function GetLeft(const aWidth: integer): Integer;
    function GetHeight(const aHeight: integer): Integer;
    function GetWidth(const aWidth: integer): Integer;
    function CheckHeight(aHeight: Integer): Boolean;
    function CheckWidth(aWidth: Integer): Boolean;
    //function CheckHeight(const Ideal, aHeight: integer): Integer;
    //function CheckWidth(const Ideal, aWidth: integer): Integer;
    property QuestionHeight: Integer read FQuestionHeight write FQuestionHeight;
    property pnlHeaderHeight: Integer read FpnlHeaderheight write FpnlHeaderHeight;
    property lblHeaderHeight: Integer read FlblHeaderheight write FlblHeaderHeight;
    property lblHeaderFontSize: Integer read FlblHeaderFontSize write FlblHeaderFontSize;
    property lblPatientFontSize: Integer read FlblPatientFontSize write FlblPatientFontSize;
    function ButtonWidth(CaptionIn: string): integer;
    function ButtonWidthNormal: Integer;
    function ButtonHeight: Integer;
    function TextWidth(TextIn: string; FontIn: TFont): Integer;
    function TextHeight(TextIn: string; FontIn: TFont): Integer;
    //function TopCenterOverParent(const HeightIn: Integer): integer;
    //function LeftCenterOverParent(const WidthIn: Integer): Integer;
    //function BestFormWidth(const Size8: Integer): Integer;
    //function BestFormHeight(const Size8: Integer): Integer;
    function SetLabelHeight(const FontSizeIn: Integer): Integer;
    function LeftForFirstButton(const Width: Integer): Integer;
    function LeftForSecondButton(const Width: Integer): Integer;
    function LeftForThirdButton(const Width: Integer): Integer;
    function HeightListBox(Width, Col: Integer; FontIn: TFont;
      Items: TStrings): Integer;

    procedure ConfigureGUI(FormFont: TFont);
    function SetItemHeight(const FontSizeIn: Integer): integer;
    function ReDrawForm(FontIn: TFont): Boolean;
    procedure ConfigureRadioGroup(Caption: string; Items: TStrings;
      Font, ItemFont: TFont; aWidth, StartXPos: Integer;
      var Columns, ItemHeight, aHeight, StartYPos: integer);
    function ExpandTrackLabels
      (WidthIn: integer; Text: string; Font: TFont): string;
    function FontSizeMenuSelections: integer;

    function GetStandardHeight(FontIn: TFont): integer;
    function GetItemHeight(FontIn: TFont): Integer;
    function GetWrapHeight(CaptionIn: string;
      WidthIn: Integer; FontIn: TFont): Integer;
    function GetMemoHeight(FontIn: TFont): Integer;
    function GetTrackHeight(FontIn: TFont): Integer;
    function GetTrackBarLabel2Height(FontIn: TFont): integer;
  published
  end;

var
  GUI_Control                 : TGUI_Control;

const
  cLabel                      : string[3] = 'lbl';
  cTitle                      : string[3] = 'ttl';
  cSection                    : string[3] = 'sec';
  cIntroduction               : string[3] = 'int';
  cEdit                       : string[3] = 'edt';
  cMaskEdit                   : string[3] = 'msk';
  cSpinEdit                   : string[3] = 'spn';
  cDate                       : string[3] = 'dpt';
  cCurrency                   : string[3] = 'cur';
  cReal                       : string[3] = 'rel';
  cListBox                    : string[3] = 'lst';
  cCheckList                  : string[3] = 'chk';
  cRadioGroup                 : string[3] = 'rgp';
  cComboBox                   : string[3] = 'cbx';
  cComboBoxStaff              : string[3] = 'cbs';
  cComboBoxConsult            : string[3] = 'cbc';
  cTrackBar                   : string[3] = 'trk';
  cTrackBarLabel1             : string[3] = 'tr1';
  cTrackBarLabel2             : string[3] = 'tr2';
  cMemo                       : string[3] = 'mem';
  cGrpBox                     : string[3] = 'grp';

  //  clSkyBlue         = TColor($F0CAA6);
  //  //clInfoBk          = TColor($F0FBFF);
  //  clCream           = TColor($F0FBFF);
  //  clMoneyGreen      = TColor($C0DCC0);
  //  clInfoBk = TColor($02E1FFFF);
  //  clNone = TColor($02FFFFFF);
  clCream                     = TColor($A6CAF0);
  clMoneyGreen                = TColor($C0DCC0);
  clSkyBlue                   = TColor($FFFBF0);
  clWorkspace                 = TColor($F0FBDA);
  clMedGray                   = TColor($A4A0A0);


implementation

uses
  uIA_Const,
  uIA_Utils;


constructor TGUI_Control.Create;
begin
  inherited Create;
end;


procedure TGUI_Control.ConfigureGUI(FormFont: TFont);
var
  HeaderSize                  : Integer;
begin

  HeaderSize := gvFont.Size + 2;
  if HeaderSize <= 14 then
    HeaderSize := 14;
  if HeaderSize > 14 then
    HeaderSize := 18;
  FlblHeaderFontSize := HeaderSize;
  FpnlHeaderHeight := 2 * ButtonHeight;
  FlblPatientFontSize := gvFont.Size;
  FQuestionHeight := 3 * TextHeightByFont(gvFont.Handle, 'Yy') div 2;

  gvDefaultEditWidth := GUI_Control.TextWidth('XXXXXXXXXXXXXXX', gvFont); // 15 x's
  //gvColumn80Width := GUI_Control.TextWidth('XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX', gvFont); // 80 x's

  if gvHighlightForeground = gvHighlightBackground then
    begin
      gvHighlightForeground := clBlack;
      gvHighlightBackground := clYellow;
    end;
end;

function TGUI_Control.FontSizeMenuSelections: integer;
begin
  // Most fonts have a size of 22 except for the following:
  Result := 22;
  if (gvFont.Name = 'Terminal') then
    Result := 14;
  if (gvFont.Name = 'Courier') then
    Result := 15;
  if (gvFont.Name = 'MS Sans Serif') or
    (gvFont.Name = 'MS Serif') then
    Result := 18;
end;

function TGUI_Control.ReDrawForm(FontIn: TFont): Boolean;
begin
  Result := False;
  if ((FontIn.Size <> gvFont.Size) or
    (FontIn.Name <> gvFont.Name) or
    (FontIn.Color <> gvFont.Color) or
    (FontIn.Style <> gvFont.Style)) then
    Result := True;
end;

(*
procedure TGUI_Control.ConfigureForm(WhichForm: TForm;
  const FormName, PatientInfo: string;
  BestHeight, BestWidth: Integer);
var
  btnPanelHeight,
    btnPanelWidth,
    i                         : Integer;
begin
  ConfigureGUI(WhichForm.Font); // Reset all the values
  with WhichForm do
    begin
      Caption := ID_CAPTION; //+ ' in use by ' + User.Name;

      // Set the parent-font, size and position of form
      Font.Assign(gvFont);

      if (gvFont.Size > 12) and (BestHeight > 300) then
        WindowState := wsMaximized
      else
        begin
          Top := CheckTop(Top);
          Left := CheckLeft(Left);
          Width := BestFormWidth(BestWidth);
          if BestHeight = gvHeight then
            Height := gvHeight
          else
            Height := CheckHeight(BestFormHeight(BestHeight));

          if Left + Width > Screen.Width then
            Left := Screen.Width - Width;
          if Top + Height > Screen.Height then
            Top := Screen.Height - Height;
        end;

      btnPanelHeight := (3 * ButtonHeight) div 2;
      btnPanelWidth := TextWidth('XXXXXX', gvFont);

      for i := 0 to ComponentCount - 1 do
        begin
          if (Components[i] is TPanel) then
            if TPanel(Components[i]).Name = 'pnlButton' then
              begin
                TPanel(Components[i]).Height := btnPanelHeight;
                btnPanelWidth := TPanel(Components[i]).Width;
              end;
          if (Components[i] is TBitBtn) then
            begin
              TBitBtn(Components[i]).Font.Assign(gvFont);
              TBitBtn(Components[i]).Font.Style := [];
              TBitBtn(Components[i]).Spacing := gvFont.size;
              TBitBtn(Components[i]).Height := ButtonHeight;
              if ((TBitBtn(Components[i]).Caption = '&Cancel') or
                (TBitBtn(Components[i]).Caption = '&OK') or
                (TBitBtn(Components[i]).Caption = '&Done') or
                (TBitBtn(Components[i]).Caption = '&Save') or
                (TBitBtn(Components[i]).Caption = '&Help') or
                (TBitBtn(Components[i]).Caption = 'Cancel') or
                (TBitBtn(Components[i]).Caption = 'OK') or
                (TBitBtn(Components[i]).Caption = 'Done') or
                (TBitBtn(Components[i]).Caption = 'Save') or
                (TBitBtn(Components[i]).Caption = 'Help')) then
                begin
                  TBitBtn(Components[i]).Width := ButtonWidthNormal;
                  TBitBtn(Components[i]).Top := (btnPanelHeight - ButtonHeight) div 2;
                end
              else
                TBitBtn(Components[i]).Width :=
                  ButtonWidth(TBitBtn(Components[i]).Caption);
              // No caption occurs with arrow buttons
              if TBitBtn(Components[i]).Caption = '' then
                begin
                  TBitBtn(Components[i]).Width := 25;
                  TBitBtn(Components[i]).Height := 25;
                end;
              if ((TBitBtn(Components[i]).Caption = 'Cancel') or
                (TBitBtn(Components[i]).Caption = '&Cancel')) then
                TBitBtn(Components[i]).Left := LeftForFirstButton(btnPanelWidth);
              if ((TBitBtn(Components[i]).Caption = 'OK') or
                (TBitBtn(Components[i]).Caption = '&OK') or
                (TBitBtn(Components[i]).Caption = 'Done') or
                (TBitBtn(Components[i]).Caption = 'Save') or
                (TBitBtn(Components[i]).Caption = '&Save') or
                (TBitBtn(Components[i]).Caption = '&Done')) then
                TBitBtn(Components[i]).Left := LeftForSecondButton(btnPanelWidth);
              if ((TBitBtn(Components[i]).Caption = 'Help') or
                (TBitBtn(Components[i]).Caption = '&Help')) then
                TBitBtn(Components[i]).Left := LeftForThirdButton(btnPanelWidth);
            end;
          if (Components[i] is TButton) then
            begin
              TButton(Components[i]).Font.Assign(gvFont);
              TButton(Components[i]).Font.Style := [];
              TButton(Components[i]).Height := ButtonHeight;
              if ((TButton(Components[i]).Caption = '&Cancel') or
                (TButton(Components[i]).Caption = '&OK') or
                (TButton(Components[i]).Caption = '&Done') or
                (TButton(Components[i]).Caption = '&Save') or
                (TButton(Components[i]).Caption = '&Help') or
                (TButton(Components[i]).Caption = 'Cancel') or
                (TButton(Components[i]).Caption = 'OK') or
                (TButton(Components[i]).Caption = 'Done') or
                (TButton(Components[i]).Caption = 'Save') or
                (TButton(Components[i]).Caption = 'Help')) then
                begin
                  TButton(Components[i]).Width := ButtonWidthNormal;
                  TButton(Components[i]).Top := (btnPanelHeight - ButtonHeight) div 2;
                end
              else
                TButton(Components[i]).Width :=
                  ButtonWidth(TButton(Components[i]).Caption);
              // No caption occurs with arrow buttons. Note eventhough TButton does
              // not support gylphs, this will catch TBitBtn in this "if" statement.
              if TButton(Components[i]).Caption = '' then
                begin
                  TButton(Components[i]).Width := 25;
                  TButton(Components[i]).Height := 25;
                end;
              if ((TButton(Components[i]).Caption = 'Cancel') or
                (TButton(Components[i]).Caption = '&Cancel')) then
                TButton(Components[i]).Left := LeftForFirstButton(btnPanelWidth);
              if ((TButton(Components[i]).Caption = 'OK') or
                (TButton(Components[i]).Caption = '&OK') or
                (TButton(Components[i]).Caption = 'Done') or
                (TButton(Components[i]).Caption = 'Save') or
                (TButton(Components[i]).Caption = '&Save') or
                (TButton(Components[i]).Caption = '&Done')) then
                TButton(Components[i]).Left := LeftForSecondButton(btnPanelWidth);
              if ((TButton(Components[i]).Caption = 'Help') or
                (TButton(Components[i]).Caption = '&Help')) then
                TButton(Components[i]).Left := LeftForThirdButton(btnPanelWidth);
            end;
          // Set the header panel
          if (Components[i] is TRzPanel) then
            begin
              if TRzPanel(Components[i]).Name = 'pnlHeader' then
                begin
                  TRzPanel(Components[i]).Height := pnlHeaderHeight;
                  TRzPanel(Components[i]).Color := clWhite;
                end;
    //          if TRzPanel(Components[i]).Name = 'pnlHeaderSpacer' then
//                begin
//                  TRzPanel(Components[i]).Align := alRight;
//                  TRzPanel(Components[i]).Color := clWhite;
//                end;
            end;
          // Set the form's title on white panel
          if (Components[i] is TRzLabel) then
            begin
              if TRzLabel(Components[i]).Name = 'lblHeader' then
                begin
                  TRzLabel(Components[i]).Caption := '   ' + FormName;
                  TRzLabel(Components[i]).Font.Assign(gvFont);
                  TRzLabel(Components[i]).Font.Size := lblHeaderFontSize;
                  TRzLabel(Components[i]).Font.Color := clBlack;
                  TRzLabel(Components[i]).Height := FlblHeaderHeight;
                  TRzLabel(Components[i]).AutoSize := True;
                  TRzLabel(Components[i]).Layout := tlCenter;
                  TRzLabel(Components[i]).Align := alLeft;
                  TRzLabel(Components[i]).Color := clWhite;
                  TRzLabel(Components[i]).Width := 4 * Width div 5;
                  if TextWidth(TRzLabel(Components[i]).Caption,
                    TRzLabel(Components[i]).Font) > TRzLabel(Components[i]).Width then
                    begin
                      TRzLabel(Components[i]).WordWrap := true;
                      //TRzLabel(Components[i]).Layout := tlTop;
                    end;
                end;

              if TRzLabel(Components[i]).Name = 'lblPatient' then
                begin
                  TRzLabel(Components[i]).Font.Assign(gvFont);
                  TRzLabel(Components[i]).Font.Size := lblPatientFontSize;
                  TRzLabel(Components[i]).Font.Color := clBlack;
                  TRzLabel(Components[i]).Color := clWhite;
                  TRzLabel(Components[i]).Layout := tlCenter;
                  TRzLabel(Components[i]).Align := alRight;
                  TRzLabel(Components[i]).AutoSize := false;
                  TRzLabel(Components[i]).Caption := PatientInfo;
                  TRzLabel(Components[i]).Left := 3 * Width div 5;
                  TRzLabel(Components[i]).Width := Width - TRzLabel(Components[i]).Left -
                    gvLeftMargin;
                  //if TextWidth(TRzLabel(Components[i]).Caption,
                  //  TRzLabel(Components[i]).Font) > TRzLabel(Components[i]).Width then
                  //  begin
                  //    TRzLabel(Components[i]).Layout := tlTop;
                  //    TRzLabel(Components[i]).WordWrap := true;
                  //  end;
                end;
            end;
        end;
      Hide;
    end;
end;*)


function TGUI_Control.SetLabelHeight(const FontSizeIn: Integer): integer;
var
  FontSize                    : Integer;
begin
  Result := 14;

  FontSize := FontSizeIn;
  if FontSize < 8 then
    FontSize := 8;
  if FontSize > 18 then
    FontSize := 18;

  case FontSize of
    8: Result := 14;
    9: Result := 15;
    10: Result := 16;
    11: Result := 17;
    12: Result := 18;
    13: Result := 19;
    14: Result := 22;
    15: Result := 23;
    16: Result := 24;
    17: Result := 26;
    18: Result := 27;
    19: Result := 28;
    20: Result := 32;
  end;
end;

function TextWidthByFont(AFontHandle: THandle; x: string): Integer;
{ returns the width of a string in pixels, given a FONT handle and string }
var
  DC                          : HDC;
  SaveFont                    : HFont;
  TextSize                    : TSize;
begin
  DC := GetDC(0);
  SaveFont := SelectObject(DC, AFontHandle);
  GetTextExtentPoint32(DC, PChar(x), Length(x), TextSize);
  Result := TextSize.cx;
  SelectObject(DC, SaveFont);
  ReleaseDC(0, DC);
end;

function TextHeightByFont(AFontHandle: THandle; x: string): Integer;
var
  DC                          : HDC;
  SaveFont                    : HFont;
  TextSize                    : TSize;
begin
  DC := GetDC(0);
  SaveFont := SelectObject(DC, AFontHandle);
  GetTextExtentPoint32(DC, PChar(x), Length(x), TextSize);
  Result := TextSize.cy;
  SelectObject(DC, SaveFont);
  ReleaseDC(0, DC);
end;

function TGUI_Control.TextWidth(TextIn: string; FontIn: TFont): Integer;
begin
  Result := TextWidthByFont(FontIn.Handle, TextIn);
end;

function TGUI_Control.TextHeight(TextIn: string; FontIn: TFont): Integer;
begin
  Result := TextHeightByFont(FontIn.Handle, TextIn);
end;

(*function TGUI_Control.TopCenterOverParent(const HeightIn: Integer): Integer;
begin
  Result := gvTop + (gvHeight - HeightIn) div 2;
  if Result < 0 then
    Result := 0;
end;

function TGUI_Control.LeftCenterOverParent(const WidthIn: Integer): Integer;
begin
  Result := gvLeft + (gvWidth - WidthIn) div 2;
  if Result < 0 then
    Result := 0;
end; *)

function TGUI_Control.ButtonWidth(CaptionIn: string): Integer;
// This is for buttons with text longer than 6 characters, no gif's
begin
  Result := TextWidthByFont(gvFont.Handle, 'XX' + CaptionIn);
end;

function TGUI_Control.ButtonWidthNormal: Integer;
// This is for Cancel, Ok buttons which also contain gif's
var
  FontSize                    : Integer;
begin
  Result := TextWidthByFont(gvFont.Handle, 'XXXXXXXXXXX'); // 11 Char
  FontSize := gvFont.Size;
  if Odd(FontSize) then
    Inc(FontSize);
  if FontSize <= 18 then
    Result := 126;
  if FontSize <= 14 then
    Result := 108;
  if FontSize <= 12 then
    Result := 92;
  if FontSize <= 10 then
    Result := 83;
  if FontSize <= 8 then
    Result := 75;
end;

function TGUI_Control.ButtonHeight: Integer;
// Button height is one and a half that of font size (except 8 which is double
// so that it handles the gif).
begin
  if gvFont.size = 8 then
    Result := 2 * TextHeightByFont(gvFont.Handle, 'Yy')
  else
    Result := (3 * TextHeightByFont(gvFont.Handle, 'Yy')) div 2;
end;

function TGUI_Control.LeftForFirstButton(const Width: Integer): Integer;
begin
  Result := Width - ButtonWidthNormal - ButtonHeight div 4;
end;

function TGUI_Control.LeftForSecondButton(const Width: Integer): Integer;
begin
  Result := Width - (2 * ButtonWidthNormal) - ButtonHeight div 2;
end;

function TGUI_Control.LeftForThirdButton(const Width: Integer): Integer;
begin
  Result := Width - (3 * ButtonWidthNormal) - (3 * ButtonHeight div 4);
end;

function TGUI_Control.GetTop(const aHeight: integer): integer;
begin
  Result := gvTop;

  if Result < 1 then
    Result := 1;

  // if at wsNormal, this will be set to wsMaximized in CheckHeight
  if aHeight >= Screen.WorkAreaHeight then
    Result := 1;
end;

function TGUI_Control.GetLeft(const aWidth: integer): integer;
begin
  Result := gvLeft;

  // Avoid having form off screen
  if Result < 1 then
    Result := 1;

  // if at wsNormal, this will be set to wsMaximized in CheckWidth
  if aWidth >= Screen.WorkAreaWidth then
    Result := 1;

  // Avoid having form off screen
  if Result + aWidth > Screen.WorkAreaWidth then
    Result := Screen.WorkAreaWidth - aWidth;
end;

function TGUI_Control.GetWidth(const aWidth: integer): integer;
begin
  Result := aWidth;

  if Result > Screen.WorkAreaWidth then
    Result := Screen.WorkAreaWidth;
end;

function TGUI_Control.GetHeight(const aHeight: integer): integer;
begin
  Result := aHeight;

  if Result > Screen.WorkAreaHeight then
    Result := Screen.WorkAreaHeight;
end;


function TGUI_Control.CheckHeight(aHeight: Integer): Boolean;
begin
  Result := False;
  if aHeight >= Screen.WorkAreaHeight then
    Result := True;
end;


function TGUI_Control.CheckWidth(aWidth: Integer): Boolean;
begin
  Result := False;

  if aWidth >= Screen.WorkAreaWidth then
    Result := True;
end;

(*function TGUI_Control.CheckHeight(const Ideal, aHeight: integer): integer;
var
  TempHeight                  : Integer;
begin
  if gvDPI > 96 then
    TempHeight := 30
  else
    TempHeight := 0;

  result := Ideal + ((gvFont.Size - 8) * 21) + TempHeight;

  if Result < aHeight then
    Result := aHeight;

  if Result > Screen.WorkAreaHeight then
    Result := Screen.WorkAreaHeight;
end;

function TGUI_Control.CheckWidth(const Ideal, aWidth: integer): integer;
// Ideal is the design width when Font.Size 8 is used.
var
  TempWidth                   : Integer;
begin
  Result := Ideal;

  if gvDPI > 96 then
    TempWidth := 30
  else
    TempWidth := 0;

  Result := Ideal + ((gvFont.Size - 8) * 25 {72}) + TempWidth;

  if Result < 200 then // Minimum Size
    Result := 300;

  if Result < aWidth then
    Result := aWidth;

  if Result > Screen.WorkAreaWidth then
    Result := Screen.WorkAreaWidth;
end; *)

(*function TGUI_Control.BestFormWidth(const Size8: Integer): Integer;
// Size8 is the design width when Font.Size 8 is used.
begin
  Result := Size8 + ((gvFont.Size - 8) * 25 {72});
end;

function TGUI_Control.BestFormHeight(const Size8: Integer): Integer;
begin
  result := Size8 + ((gvFont.Size - 8) * 21);
end;*)

function TGUI_Control.HeightListBox(Width, Col: Integer; FontIn: TFont;
  Items: TStrings): Integer;
var
  i, k,
    Max,
    FWidth,
    FCol,
    ItemHeight,
    row                       : Integer;
begin
  //Result := 1;
  FWidth := Width;
  // Standard, unwrapped height of item
  ItemHeight := 5 * GUI_Control.TextHeight('Yy', FontIn) div 4;
  // Safety move
  FCol := Col;
  if Fcol <= 0 then
    Fcol := 1;
  if FWidth <= 0 then
    FWidth := 1;
  // Figure out how many rows; add one if not even
  row := Items.Count div Fcol;
  if (Items.Count mod Fcol) > 0 then
    Inc(row);
  // Check for wrapped items
  Max := 0;
  for i := 0 to Items.Count - 1 do
    begin
      k := GUI_Control.TextWidth(Items[i] + 'XX', FontIn)
        div (FWidth div FCol);
      if k > Max then
        Max := k;
    end;
  if Max > 0 then
    ItemHeight := (Max + 1) * ItemHeight;
  result := (ItemHeight * row) +
    GUI_Control.TextHeight('Yy', FontIn) div 2;
end;

(*
function TGUI_Control.HeightRadioGrp(Width, Col, VerticalSpacing: Integer;
FontIn: TFont; lItems: TStrings): Integer;
var
i, k,
  Max,
  TempWidth,
  lItemHeight,
  lCol,
  row             : Integer;
begin
lCol := Col;
//lItemHeight := 5 * GUI_Control.TextHeight('Yy', FontIn) div 4;
lItemHeight := 3 * GUI_Control.TextHeight('Yy', FontIn) div 2;
// No zero
if lCol <= 0 then
  lCol := 1;

row := lItems.Count div lCol;
if (lItems.Count mod lCol) > 0 then
  Inc(row);
// Check for wrapped item
Max := 0;

TempWidth := Width - (VerticalSpacing * lCol);
// Avoid zero
if TempWidth <= 0 then
  TempWidth := 1;

for i := 0 to lItems.Count - 1 do
  begin
    k := GUI_Control.TextWidth(lItems[i] + 'XX', FontIn)
      div TempWidth div lCol;
    if k > Max then
      Max := k;
  end;
if Max > 0 then
  lItemHeight := (Max + 1) * lItemHeight;

result := lItemHeight * (row + 2);
end;*)
(*
function TGUI_Control.HeightRadioCaption(Width: Integer; Caption: string;
FontIn: TFont): Integer;
// Use for Radio Group captions only
var
FHeight,
  FWidth,
  row             : Integer;
begin
FWidth := Width;
// Check for wrapped caption
if FWidth <= 0 then
  FWidth := 1;
row := GUI_Control.TextWidth(Trim(Caption) + 'XXX', FontIn) div FWidth;
FHeight := GUI_Control.TextHeight('Yy', FontIn);
if row > 0 then
  result := (row * FHeight)
else
  result := FHeight div 4;
end;*)

(*
procedure TGUI_Control.ConfigureRadioGroup(RadioGroup: TRzRadioGroup);
var
i, k, wide, Max,
  TempWidth,
  TempHeight,
  row             : Integer;
begin
with RadioGroup do
  begin
    if (ItemFont.Size >= 7) and
      (ItemFont.Size <= 8) then
      ItemHeight := 16; // Font.size 8
    if (ItemFont.Size >= 9) and
      (ItemFont.Size <= 10) then
      ItemHeight := 18; // Font.size 10
    if (ItemFont.Size >= 11) and
      (ItemFont.Size <= 12) then
      ItemHeight := 20; // Font.size 12
    if (ItemFont.Size >= 13) and
      (ItemFont.Size <= 14) then
      ItemHeight := 24; // Font.size 14
    if (ItemFont.Size >= 15) and
      (ItemFont.Size <= 16) then
      ItemHeight := 28; // Font.size 16
    if (ItemFont.Size >= 17) and
      (ItemFont.Size <= 18) then
      ItemHeight := 30; // Font.size 18
    VerticalSpacing := GUI_Control.TextHeight('Yy', ItemFont) div 4;
    Wide := Width - StartXPos - 13; //GlyphWidth := 13
    if Columns <= 0 then
      Columns := 1;
    row := Items.Count div Columns;
    if row > 6 then
      Inc(row);
    if row > 9 then
      Inc(row);
    if (Items.Count mod Columns) > 0 then
      Inc(row);
    // Set Item height: check for wrapped item(s)
    Max := 0;
    TempWidth := Wide - (VerticalSpacing * Columns);
    // Avoid zero
    if TempWidth <= 0 then
      TempWidth := 1;
    // Find longest item
    for i := 0 to Items.Count - 1 do
      begin
        k := GUI_Control.TextWidth(Items[i] + 'XX', ItemFont)
          div (TempWidth div Columns);
        if k > Max then
          Max := k;
      end;
    if Max > 0 then
      ItemHeight := (Max + 1) * ItemHeight;

    // Set overall height: based on caption, rows and columns
    Height := (row + 1) * ItemHeight;
    // Check for wrapped caption
    row := GUI_Control.TextWidth(Trim(Caption), Font) div Wide;
    TempHeight := GUI_Control.TextHeight('Yy', Font);
    if row > 0 then
      StartYPos := row * TempHeight
    else
      StartYPos := 0;
    if StartYPos = 0 then
      Height := Height + GUI_Control.TextHeight('Yy', Font)
    else
      Height := Height + (2 * StartYPos);
  end;

end; *)



function TGUI_Control.GetTrackHeight(FontIn: TFont): integer;
begin
  Result := 30 + // LabelsLeftTop
    GUI_Control.TextHeight('Yy', FontIn);
end;

function TGUI_Control.SetItemHeight(const FontSizeIn: Integer): integer;
var
  FontSize                    : Integer;
begin
  Result := 14;
  FontSize := FontSizeIn;
  if FontSize < 8 then
    FontSize := 8;
  if FontSize > 18 then
    FontSize := 18;

  case FontSize of
    8: Result := 14;
    9: Result := 15;
    10: Result := 16;
    11: Result := 17;
    12: Result := 18;
    13: Result := 19;
    14: Result := 22;
    15: Result := 23;
    16: Result := 24;
    17: Result := 26;
    18: Result := 27;
    19: Result := 28;
    20: Result := 32;
  end;
end;

procedure TGUI_Control.ConfigureRadioGroup(Caption: string; Items: TStrings;
  Font, ItemFont: TFont; aWidth, StartXPos: Integer;
  var Columns, ItemHeight, aHeight, StartYPos: integer);
var
  i, k, wide, Max,
    VertSpacing,
    TempWidth,
    row                       : Integer;
begin
  // Catch division by zero errors
  if aWidth = 0 then
    aWidth := 200;
  if Columns = 0 then
    Columns := 1;
  // Default setting: Does okay with DPI of 120
  ItemHeight := 5 * GUI_Control.TextHeight('Yy', ItemFont) div 4;

  // Fine tuned setting
  if gvDPI = 96 then
    begin
      if (ItemFont.Size >= 7) and
        (ItemFont.Size <= 8) then
        ItemHeight := 16; // Font.size 8
      if (ItemFont.Size >= 9) and
        (ItemFont.Size <= 10) then
        ItemHeight := 18; // Font.size 10
      if (ItemFont.Size >= 11) and
        (ItemFont.Size <= 12) then
        ItemHeight := 20; // Font.size 12
      if (ItemFont.Size >= 13) and
        (ItemFont.Size <= 14) then
        ItemHeight := 24; // Font.size 14
      if (ItemFont.Size >= 15) and
        (ItemFont.Size <= 16) then
        ItemHeight := 28; // Font.size 16
      if (ItemFont.Size >= 17) and
        (ItemFont.Size <= 18) then
        ItemHeight := 30; // Font.size 18
    end;

  VertSpacing := GUI_Control.TextHeight('Yy', ItemFont) div 4;

  Wide := aWidth - StartXPos - 13; //GlyphWidth := 13
  // Large fonts need more room
  if Columns <= 0 then
    Columns := 1;
//  if ItemFont.Size > 12 then
//        begin
//          if Columns > 2 then
//            Columns := Columns - 1;
//        end;
//  if ItemFont.Size > 16 then
//    Columns := 1;
  row := Items.Count div Columns;
  if row > 6 then
    Inc(row);
  if row > 9 then
    Inc(row);
  if (Items.Count mod Columns) > 0 then
    Inc(row);
  // Set Item height: check for wrapped item(s)
  Max := 0;

  TempWidth := Wide - (VertSpacing * Columns);
  // Avoid zeros
  if TempWidth <= 0 then
    TempWidth := 1;

  // Find longest item
  for i := 0 to Items.Count - 1 do
    begin
      k := GUI_Control.TextWidth(Items[i] + 'XX', ItemFont)
        div (TempWidth div Columns);
      if k > Max then
        Max := k;
    end;

  // Set ItemHeight to height of longest (perhaps wrapped) item.
  if Max > 0 then
    ItemHeight := (Max + 1) * ItemHeight;

  // Set overall height: based on caption, rows and columns
  if row = 1 then
    aHeight := row * ItemHeight
  else
    begin
      aHeight := (row * ItemHeight) + (row * (ItemHeight div 4));
      if Max > 0 then // wrapped item(s)
        aHeight := ((row + 1) * ItemHeight);
    end;

  // Check for wrapped caption. "Row" is being used for a different purpose here.
  if Pos(#13, Caption) = 0 then
    row := GUI_Control.TextWidth(Trim(Caption), Font) div aWidth
  else
    row := NumberOfPieces(Caption, #13) - 1;

  if row > 0 then // StartYPos is where the first item will appear below the
                  // caption.
    StartYPos := (row * GUI_Control.TextHeight('Yy', Font)) +
      (GUI_Control.TextHeight('Yy', Font) div 4)
  else
    StartYPos := 0;

  if StartYPos = 0 then
    begin
      if Max = 0 then
        aHeight := aHeight + 4 * GUI_Control.TextHeight('Yy', Font) div 3
    end
  else
    begin
      if Max = 0 then
        aHeight := aHeight + (2 * StartYPos);
    end;
end;

function TGUI_Control.ExpandTrackLabels
  (WidthIn: integer; Text: string; Font: TFont): string;
var
  x                           : Integer;
  TempStr,
    TempMid,
    Left, Right               : string;
begin
  Result := '';
  Left := '';
  Right := '';
  TempMid := '';

  if length(text) = 0 then
    Exit;
  if Text[Length(Text)] <> DELIMITER then
    Text := Text + '  |';
  x := NumberOfPieces(Text, DELIMITER); // This starts at 1

  if x = 3 then
    begin
      TempStr := '';
      while TextWidthByFont(Font.Handle, TempStr) <= WidthIn do
        begin
          TempStr := piece(text, DELIMITER, 1) + Left + piece(text, DELIMITER, 2) + 'X';
          Left := Left + ' ';
        end;
      Result := piece(text, DELIMITER, 1) + Left + piece(text, DELIMITER, 2);
    end;

  if x = 4 then
    begin
      TempMid := piece(text, DELIMITER, 2);
      TempMid := copy(TempMid, 0, length(TempMid) div 2);
      TempStr := '';
      while TextWidthByFont(Font.Handle, TempStr) <= (WidthIn div 2) do
        begin
          TempStr := piece(text, DELIMITER, 1) + Left + TempMid + 'X';
          Left := Left + ' ';
        end;
      TempStr := '';

      while TextWidthByFont(Font.Handle, TempStr) <= (WidthIn div 2) do
        begin
          TempStr := TempMid + 'X' + Right + piece(text, DELIMITER, 3);
          Right := Right + ' ';
        end;
      Result := piece(text, DELIMITER, 1) + Left + piece(text, DELIMITER, 2) +
        Right + piece(text, DELIMITER, 3);
    end;

end;

function TGUI_Control.GetStandardHeight(FontIn: TFont): integer;
// In Editor: Used for labels, spin edits, edit boxes in lower panels. Not used in
// work space except as default.
begin
  result := 5 * GUI_Control.TextHeight('Yy', FontIn) div 3;
end;

function TGUI_Control.GetItemHeight(FontIn: TFont): integer;
// In Editor: Used for combo boxes, radio groups, list boxes in lower panels. Not used in
// work space except as default.
begin
  result := 5 * GUI_Control.TextHeight('Yy', FontIn) div 4;
end;


function TGUI_Control.GetMemoHeight(FontIn: TFont): integer;
// In Editor: Used for memo in lower panels. Also used in Building Forms.
begin
  result := 9 * GUI_Control.TextHeight('Yy', FontIn) div 2;
end;


function TGUI_Control.GetTrackBarLabel2Height(FontIn: TFont): integer;
// In Editor: Used for track bar description label in lower panels.
// Also used in Building Forms.
begin
  result := 3 * GUI_Control.TextHeight('Yy', FontIn) div 2;
end;

function TGUI_Control.GetWrapHeight(CaptionIn: string; WidthIn: Integer;
  FontIn: TFont): integer;
// Height of label which may have wrapped caption.
var
  i                           : Integer;
begin
  // Set default
  Result := GetStandardHeight(FontIn);

  if WidthIn < 1 then // avoid dividing by zero
    Exit;

  // How many lines of text?
  i := GUI_Control.TextWidth(CaptionIn, FontIn) div WidthIn;

  if i > 0 then
    Result := GetItemHeight(FontIn) * (i + 1);

  // Add a line when a carriage return appears, ignore the first one
  Result := Result + (NumberofPieces(CaptionIn, #13) - 1) *
    GUI_Control.TextHeight('Yy', FontIn);
end;

destructor TGUI_Control.Destroy;
begin
  inherited Destroy;
end;


initialization
  GUI_Control := TGUI_Control.Create;

finalization
  GUI_Control.Free;

end.

