unit uIA_Patient;

interface

uses
  SysUtils,
  TRPCb;

type

  TPatient = class
  private
    FName,
      FDFN,
      FSex,
      FSSN,
      FFormattedSSN,
      FDOB,
      FDOD,
      FFirstNameLastName,
      FCaptionInfo: string;
    FIsDead,
      FIsServiceConnected,
      FIsSelected: Boolean;
    FPctSC,
      FAge: Integer;
  protected
  public
    destructor Destroy; override;
    constructor Create;
    procedure Clear;
    function Load(ADFN: string; Broker: TRPCBroker): string;
    procedure SetCaptionInfo;

    property Name: string read FName write FName;
    property DFN: string read FDFN write FDFN;
    property Sex: string read FSex write FSex;
    property SSN: string read FSSN write FSSN;
    property DOB: string read FDOB write FDOB;
    property Age: Integer read FAge write FAge;
    property PctSC: integer read FPctSC write FPctSC;
    property IsSelected: BOOLEAN read FIsSelected write FIsSelected;
    property FormattedSSN: string read FFormattedSSN write FFormattedSSN;
    property CaptionInfo: string read FCaptionInfo write FCaptionInfo;
    property IsServiceConnected: BOOLEAN read FIsServiceConnected write
      FIsServiceConnected;
    property IsDead: BOOLEAN read FIsDead write FIsDead;
    property DOD: string read FDOD write FDOD;
    property FirstNameLastName: string read FFirstNameLastName write
      FFirstNameLastName;
  published
  end; { TPatient }

var
  Patient                     : TPatient;


implementation

uses
  uIA_Const,
  uIA_Utils,
  uIA_Patient_IO;

constructor TPatient.Create;
begin
  inherited Create;
  Clear;
end;

destructor TPatient.Destroy;
begin
  inherited Destroy;
end;

function TPatient.Load(aDFN: string; Broker: TRPCBroker): string;
var
  TempPt, TempPiece           : string;
begin
  Result := OK;

  TempPt := ReadPatientProperties(aDFN, Broker);

  if (TempPt = BAD_DATA) or (TempPt = '') then
    begin
      Result := TempPt;
      Exit;
    end;

  FDFN := aDFN;
  IsSelected := not (FDFN = '');

  //1--GREEN,FROG^name
  //2--123456789^123-45-6789^ssn
  //3--2470420^APR 20,1947^dob
  //4--57^age
  //5--M^MALE^sex
  //6--3020820.1748^AUG 20,2002@1748^date of death
  //7--0^service connected

  //Name
  FName := Piece(TempPt, U, 1);
  //SSN
  TempPiece := Piece(TempPt, RECORD_END, 2);
  FSSN := Piece(TempPiece, U, 1);
  FFormattedSSN := Piece(TempPiece, U, 2);
  if length(FFormattedSSN) = 11 then
    FFormattedSSN := 'xxx-xx-' + Copy(FFormattedSSN, 8, 4)
    else
    FFormattedSSN := 'xxx-xx-xxxx';
  //DOB
  TempPiece := Piece(TempPt, RECORD_END, 3);
  FDOB := Piece(TempPiece, U, 2);
  //Age
  TempPiece := Piece(TempPt, RECORD_END, 4);
  FAge := StrToIntDef(Piece(TempPiece, U, 1), 0);
  //Sex
  TempPiece := Piece(TempPt, RECORD_END, 5);
  if Length(Piece(TempPiece, U, 1)) > 0 then
    FSex := Piece(TempPiece, U, 1)
  else
    FSex := 'U';

  //DOD      Date of Death
  TempPiece := Piece(TempPt, RECORD_END, 6);
  //FIsDead := (StrToIntDef(Piece(TempPiece, U, 1), 0) > 0); Don't use
  FIsDead := Length(Piece(TempPiece, U, 1)) > 1;
  if IsDead then
    FDOD := FMDateStrToDateStr(Piece(TempPiece, U, 1));

  //Service Connected
  TempPiece := Piece(TempPt, RECORD_END, 7);
  FIsServiceConnected := (Piece(TempPiece, U, 1) = '1');
  if IsServiceConnected then
    FPctSC := StrToIntDef(Piece(TempPiece, U, 2), 0);

//  FFirstNameLastName := Trim(Piece(Self.Name, ',', 2)) + ' ' +
//    Trim(Piece(Self.Name, ',', 1));
  SetCaptionInfo;

end;


procedure TPatient.Clear;
begin
  FDFN := '';
  FIsSelected := False;
  FSex := 'U';
  FSSN := '';
  FFormattedSSN := '';
  FName := '';
  FDOB := DateToStr(Date);
  FAge := 0;
  FPctSC := 0;
  FCaptionInfo := '';
  FDOD := '';
  FIsDead := False;
  FIsServiceConnected := False;
end;

procedure TPatient.SetCaptionInfo;
var
  tmpName                     : string;
begin
  tmpName := Trim(Piece(FName, ',', 1)) + ', ' + Trim(Piece(FName, ',', 2));
  CaptionInfo := MixedCase(tmpName) + '   ' + CRLF + 'SSN: ' + FFormattedSSN +
    '   ';

end;

initialization
  Patient := TPatient.Create;

finalization
  Patient.Free;

end.

