unit uMHA_AUX_MMPI2RF;

interface

uses
  Classes,
  SysUtils,
  fIA_dialogs,
  uMHA_AUX_Const,
  uMHA_AUX_Utils;

function ProgressNote_MMPI2RF(
  PatientName,
  PatientDOB,
  PatientSex,
  ScoreString: string): string;

function ScoreAnswers_MMPI2RF(
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string): string;

implementation

function GetTScore(ScaleIn: string; RawScoreIn: integer): integer;
// To avoid typos and copy errors, the tables of t scores in the MMPI-2-RF
// manual were copied and arranged as case procedures.
begin
  result := -1;

  if ScaleIn = 'VRIN_r' then
    begin
      result := 120; //18-46	120
      case RawScoreIn of
        0: result := 34;
        1: result := 39;
        2: result := 43;
        3: result := 48;
        4: result := 53;
        5: result := 58;
        6: result := 63;
        7: result := 68;
        8: result := 73;
        9: result := 77;
        10: result := 82;
        11: result := 87;
        12: result := 92;
        13: result := 97;
        14: result := 102;
        15: result := 106;
        16: result := 111;
        17: result := 116;
        // 18-46	120
      end; // case
    end; // if

  if ScaleIn = 'TRIN_r' then
    begin
      result := 120; // 0, 1, 21-26	120
      case RawScoreIn of
        //    0:   120
        //    1:	120
        2: result := 118;
        3: result := 110;
        4: result := 103;
        5: result := 95;
        6: result := 88;
        7: result := 80;
        8: result := 73;
        9: result := 65;
        10: result := 57;
        11: result := 50;
        12: result := 57;
        13: result := 65;
        14: result := 73;
        15: result := 80;
        16: result := 88;
        17: result := 95;
        18: result := 103;
        19: result := 110;
        20: result := 118;
        // 21-26	120
      end;
    end;

  if ScaleIn = 'F_r' then
    begin
      result := 120; // 17-32	120
      case RawScoreIn of
        0: result := 42;
        1: result := 47;
        2: result := 51;
        3: result := 56;
        4: result := 61;
        5: result := 65;
        6: result := 70;
        7: result := 74;
        8: result := 79;
        9: result := 83;
        10: result := 88;
        11: result := 92;
        12: result := 97;
        13: result := 101;
        14: result := 106;
        15: result := 111;
        16: result := 115;
      end;
    end;

  if ScaleIn = 'Fp_r' then
    begin
      result := 120; // 10-21	120
      case RawScoreIn of
        0: result := 42;
        1: result := 51;
        2: result := 59;
        3: result := 68;
        4: result := 77;
        5: result := 85;
        6: result := 94;
        7: result := 102;
        8: result := 111;
        9: result := 119;
        //10-21	120
      end;
    end;

  if ScaleIn = 'Fs' then
    begin
      result := 120; // 10-16	120
      case RawScoreIn of
        0: result := 42;
        1: result := 50;
        2: result := 58;
        3: result := 66;
        4: result := 74;
        5: result := 83;
        6: result := 91;
        7: result := 99;
        8: result := 107;
        9: result := 115;
        //10-16	120
      end;
    end;

  if ScaleIn = 'FBS_r' then
    begin
      case RawScoreIn of
        0: result := 26;
        1: result := 29;
        2: result := 32;
        3: result := 35;
        4: result := 39;
        5: result := 42;
        6: result := 45;
        7: result := 48;
        8: result := 51;
        9: result := 54;
        10: result := 58;
        11: result := 61;
        12: result := 64;
        13: result := 67;
        14: result := 70;
        15: result := 73;
        16: result := 77;
        17: result := 80;
        18: result := 83;
        19: result := 86;
        20: result := 89;
        21: result := 92;
        22: result := 96;
        23: result := 99;
        24: result := 102;
        25: result := 105;
        26: result := 108;
        27: result := 111;
        28: result := 115;
        29: result := 118;
        30: result := 120;
      end;
    end;

  if ScaleIn = 'RBS' then
    begin
      result := 120; // 22-28
      case RawScoreIn of
        0: result := 29;
        1: result := 33;
        2: result := 38;
        3: result := 42;
        4: result := 46;
        5: result := 50;
        6: result := 54;
        7: result := 59;
        8: result := 63;
        9: result := 67;
        10: result := 71;
        11: result := 76;
        12: result := 80;
        13: result := 84;
        14: result := 88;
        15: result := 92;
        16: result := 97;
        17: result := 101;
        18: result := 105;
        19: result := 109;
        20: result := 114;
        21: result := 118;
      end;
    end;

  if ScaleIn = 'L_r' then
    begin
      case RawScoreIn of
        0: result := 37;
        1: result := 42;
        2: result := 47;
        3: result := 52;
        4: result := 57;
        5: result := 62;
        6: result := 66;
        7: result := 71;
        8: result := 76;
        9: result := 81;
        10: result := 86;
        11: result := 91;
        12: result := 95;
        13: result := 100;
        14: result := 105;
      end;
    end;

  if ScaleIn = 'K_r' then
    begin
      case RawScoreIn of
        0: result := 24;
        1: result := 28;
        2: result := 31;
        3: result := 35;
        4: result := 38;
        5: result := 42;
        6: result := 45;
        7: result := 48;
        8: result := 52;
        9: result := 55;
        10: result := 59;
        11: result := 62;
        12: result := 66;
        13: result := 69;
        14: result := 72;
      end;
    end;

  if ScaleIn = 'EID' then
    begin
      case RawScoreIn of
        0: result := 30;
        1: result := 33;
        2: result := 36;
        3: result := 38;
        4: result := 41;
        5: result := 43;
        6: result := 45;
        7: result := 46;
        8: result := 48;
        9: result := 49;
        10: result := 51;
        11: result := 52;
        12: result := 53;
        13: result := 54;
        14: result := 56;
        15: result := 57;
        16: result := 58;
        17: result := 59;
        18: result := 61;
        19: result := 62;
        20: result := 64;
        21: result := 65;
        22: result := 66;
        23: result := 68;
        24: result := 69;
        25: result := 71;
        26: result := 72;
        27: result := 73;
        28: result := 75;
        29: result := 76;
        30: result := 78;
        31: result := 79;
        32: result := 80;
        33: result := 82;
        34: result := 83;
        35: result := 85;
        36: result := 86;
        37: result := 87;
        38: result := 89;
        39: result := 90;
        40: result := 92;
        41: result := 93;
      end;
    end;

  if ScaleIn = 'THD' then
    begin
      result := 100;
      case RawScoreIn of
        0: result := 39;
        1: result := 48;
        2: result := 53;
        3: result := 57;
        4: result := 60;
        5: result := 63;
        6: result := 67;
        7: result := 70;
        8: result := 74;
        9: result := 77;
        10: result := 81;
        11: result := 84;
        12: result := 88;
        13: result := 91;
        14: result := 94;
        15: result := 98;
        //16-26	100
      end;
    end;

  if ScaleIn = 'BXD' then
    begin
      case RawScoreIn of
        0: result := 32;
        1: result := 36;
        2: result := 40;
        3: result := 43;
        4: result := 46;
        5: result := 48;
        6: result := 50;
        7: result := 53;
        8: result := 55;
        9: result := 57;
        10: result := 60;
        11: result := 63;
        12: result := 65;
        13: result := 68;
        14: result := 70;
        15: result := 73;
        16: result := 76;
        17: result := 78;
        18: result := 81;
        19: result := 84;
        20: result := 86;
        21: result := 89;
        22: result := 92;
        23: result := 94;
      end;
    end;

  if ScaleIn = 'RCd' then
    begin
      case RawScoreIn of
        0: result := 37;
        1: result := 42;
        2: result := 46;
        3: result := 49;
        4: result := 51;
        5: result := 53;
        6: result := 54;
        7: result := 55;
        8: result := 57;
        9: result := 58;
        10: result := 60;
        11: result := 62;
        12: result := 64;
        13: result := 66;
        14: result := 67;
        15: result := 69;
        16: result := 71;
        17: result := 73;
        18: result := 75;
        19: result := 77;
        20: result := 79;
        21: result := 81;
        22: result := 83;
        23: result := 85;
        24: result := 86;
      end;
    end;

  if ScaleIn = 'RC1' then
    begin
      result := 100;
      case RawScoreIn of
        0: result := 36;
        1: result := 42;
        2: result := 47;
        3: result := 51;
        4: result := 54;
        5: result := 56;
        6: result := 59;
        7: result := 61;
        8: result := 63;
        9: result := 65;
        10: result := 68;
        11: result := 70;
        12: result := 72;
        13: result := 74;
        14: result := 77;
        15: result := 79;
        16: result := 81;
        17: result := 84;
        18: result := 86;
        19: result := 88;
        20: result := 90;
        21: result := 93;
        22: result := 95;
        23: result := 97;
        24: result := 99;
          // 25-27	100
      end;
    end;

  if ScaleIn = 'RC2' then
    begin
      case RawScoreIn of
        0: result := 34;
        1: result := 38;
        2: result := 42;
        3: result := 46;
        4: result := 50;
        5: result := 54;
        6: result := 58;
        7: result := 61;
        8: result := 65;
        9: result := 69;
        10: result := 73;
        11: result := 76;
        12: result := 80;
        13: result := 84;
        14: result := 88;
        15: result := 92;
        16: result := 95;
        17: result := 99;
      end;
    end;

  if ScaleIn = 'RC3' then
    begin
      case RawScoreIn of
        0: result := 34;
        1: result := 38;
        2: result := 41;
        3: result := 43;
        4: result := 46;
        5: result := 47;
        6: result := 49;
        7: result := 51;
        8: result := 54;
        9: result := 57;
        10: result := 61;
        11: result := 65;
        12: result := 70;
        13: result := 74;
        14: result := 79;
        15: result := 83;
      end;
    end;

  if ScaleIn = 'RC4' then
    begin
      case RawScoreIn of
        0: result := 34;
        1: result := 39;
        2: result := 43;
        3: result := 46;
        4: result := 49;
        5: result := 52;
        6: result := 54;
        7: result := 57;
        8: result := 59;
        9: result := 62;
        10: result := 65;
        11: result := 68;
        12: result := 71;
        13: result := 73;
        14: result := 76;
        15: result := 79;
        16: result := 82;
        17: result := 85;
        18: result := 87;
        19: result := 90;
        20: result := 93;
        21: result := 96;
        22: result := 99;
      end;
    end;

  if ScaleIn = 'RC6' then
    begin
      result := 100;
      case RawScoreIn of
        0: result := 43;
        1: result := 56;
        2: result := 61;
        3: result := 66;
        4: result := 70;
        5: result := 75;
        6: result := 80;
        7: result := 84;
        8: result := 89;
        9: result := 94;
        10: result := 98;
        // 11-17	100
      end;
    end;

  if ScaleIn = 'RC7' then
    begin
      case RawScoreIn of
        0: result := 34;
        1: result := 38;
        2: result := 41;
        3: result := 44;
        4: result := 46;
        5: result := 48;
        6: result := 50;
        7: result := 52;
        8: result := 53;
        9: result := 55;
        10: result := 57;
        11: result := 60;
        12: result := 62;
        13: result := 65;
        14: result := 68;
        15: result := 70;
        16: result := 73;
        17: result := 75;
        18: result := 78;
        19: result := 81;
        20: result := 83;
        21: result := 86;
        22: result := 88;
        23: result := 91;
        24: result := 94;
      end;
    end;

  if ScaleIn = 'RC8' then
    begin
      result := 100;
      case RawScoreIn of
        0: result := 39;
        1: result := 47;
        2: result := 52;
        3: result := 56;
        4: result := 59;
        5: result := 63;
        6: result := 66;
        7: result := 70;
        8: result := 73;
        9: result := 76;
        10: result := 80;
        11: result := 83;
        12: result := 86;
        13: result := 90;
        14: result := 93;
        15: result := 96;
        // 16-18	100
      end;
    end;

  if ScaleIn = 'RC9' then
    begin
      case RawScoreIn of
        0: result := 25;
        1: result := 28;
        2: result := 31;
        3: result := 33;
        4: result := 36;
        5: result := 38;
        6: result := 40;
        7: result := 42;
        8: result := 43;
        9: result := 45;
        10: result := 46;
        11: result := 48;
        12: result := 50;
        13: result := 51;
        14: result := 53;
        15: result := 56;
        16: result := 58;
        17: result := 61;
        18: result := 63;
        19: result := 66;
        20: result := 69;
        21: result := 72;
        22: result := 74;
        23: result := 77;
        24: result := 80;
        25: result := 83;
        26: result := 85;
        27: result := 88;
        28: result := 91;
      end;
    end;

  if ScaleIn = 'MLS' then
    begin
      case RawScoreIn of
        0: result := 38;
        1: result := 46;
        2: result := 52;
        3: result := 57;
        4: result := 63;
        5: result := 69;
        6: result := 75;
        7: result := 81;
        8: result := 87;
      end;
    end;

  if ScaleIn = 'HPC' then
    begin
      case RawScoreIn of
        0: result := 42;
        1: result := 53;
        2: result := 59;
        3: result := 65;
        4: result := 72;
        5: result := 78;
        6: result := 85;
      end;
    end;

  if ScaleIn = 'NUC' then
    begin
      case RawScoreIn of
        0: result := 41;
        1: result := 53;
        2: result := 59;
        3: result := 65;
        4: result := 70;
        5: result := 75;
        6: result := 80;
        7: result := 86;
        8: result := 91;
        9: result := 96;
        10: result := 100;
      end;
    end;

  if ScaleIn = 'GIC' then
    begin
      case RawScoreIn of
        0: result := 46;
        1: result := 64;
        2: result := 72;
        3: result := 80;
        4: result := 88;
        5: result := 96;
      end;
    end;

  if ScaleIn = 'SUI' then
    begin
      result := 100;
      case RawScoreIn of
        0: result := 45;
        1: result := 66;
        2: result := 79;
        3: result := 91;
        // 4-5	100
      end;
    end;

  if ScaleIn = 'HLP' then
    begin
      case RawScoreIn of
        0: result := 40;
        1: result := 52;
        2: result := 60;
        3: result := 69;
        4: result := 79;
        5: result := 88;
      end;
    end;

  if ScaleIn = 'SFD' then
    begin
      case RawScoreIn of
        0: result := 42;
        1: result := 52;
        2: result := 56;
        3: result := 65;
        4: result := 76;
      end;
    end;

  if ScaleIn = 'NFC' then
    begin
      case RawScoreIn of
        0: result := 36;
        1: result := 43;
        2: result := 48;
        3: result := 51;
        4: result := 54;
        5: result := 58;
        6: result := 64;
        7: result := 69;
        8: result := 75;
        9: result := 80;
      end;
    end;

  if ScaleIn = 'COG' then
    begin
      case RawScoreIn of
        0: result := 40;
        1: result := 50;
        2: result := 54;
        3: result := 58;
        4: result := 64;
        5: result := 69;
        6: result := 75;
        7: result := 80;
        8: result := 86;
        9: result := 91;
        10: result := 96;
      end;
    end;

  if ScaleIn = 'STW' then
    begin
      case RawScoreIn of
        0: result := 36;
        1: result := 43;
        2: result := 47;
        3: result := 52;
        4: result := 57;
        5: result := 65;
        6: result := 73;
        7: result := 81;
      end;
    end;

  if ScaleIn = 'AXY' then
    begin
      case RawScoreIn of
        0: result := 44;
        1: result := 59;
        2: result := 70;
        3: result := 80;
        4: result := 91;
        5: result := 100;
      end;
    end;

  if ScaleIn = 'ANP' then
    begin
      case RawScoreIn of
        0: result := 39;
        1: result := 47;
        2: result := 51;
        3: result := 54;
        4: result := 59;
        5: result := 66;
        6: result := 73;
        7: result := 80;
      end;
    end;

  if ScaleIn = 'BRF' then
    begin
      result := 100;
      case RawScoreIn of
        0: result := 43;
        1: result := 56;
        2: result := 63;
        3: result := 71;
        4: result := 79;
        5: result := 86;
        6: result := 94;
        // 7-9	100
      end;
    end;

  if ScaleIn = 'MSF' then
    begin
      case RawScoreIn of
        0: result := 36;
        1: result := 42;
        2: result := 46;
        3: result := 48;
        4: result := 51;
        5: result := 54;
        6: result := 59;
        7: result := 65;
        8: result := 71;
        9: result := 78;
      end;
    end;

  if ScaleIn = 'JCP' then
    begin
      case RawScoreIn of
        0: result := 40;
        1: result := 50;
        2: result := 57;
        3: result := 63;
        4: result := 70;
        5: result := 77;
        6: result := 84;
      end;
    end;

  if ScaleIn = 'SUB' then
    begin
      case RawScoreIn of
        0: result := 41;
        1: result := 50;
        2: result := 55;
        3: result := 61;
        4: result := 69;
        5: result := 77;
        6: result := 85;
        7: result := 93;
      end;
    end;

  if ScaleIn = 'AGG' then
    begin
      case RawScoreIn of
        0: result := 37;
        1: result := 45;
        2: result := 51;
        3: result := 56;
        4: result := 61;
        5: result := 67;
        6: result := 73;
        7: result := 79;
        8: result := 86;
        9: result := 92;
      end;
    end;

  if ScaleIn = 'ACT' then
    begin
      case RawScoreIn of
        0: result := 33;
        1: result := 39;
        2: result := 44;
        3: result := 48;
        4: result := 53;
        5: result := 59;
        6: result := 67;
        7: result := 75;
        8: result := 83;
      end;
    end;

  if ScaleIn = 'FML' then
    begin
      case RawScoreIn of
        0: result := 37;
        1: result := 44;
        2: result := 49;
        3: result := 53;
        4: result := 58;
        5: result := 63;
        6: result := 68;
        7: result := 74;
        8: result := 79;
        9: result := 84;
        10: result := 90;
      end;
    end;

  if ScaleIn = 'IPP' then
    begin
      case RawScoreIn of
        0: result := 34;
        1: result := 39;
        2: result := 43;
        3: result := 46;
        4: result := 49;
        5: result := 52;
        6: result := 56;
        7: result := 62;
        8: result := 68;
        9: result := 74;
        10: result := 81;
      end;
    end;

  if ScaleIn = 'SAV' then
    begin
      case RawScoreIn of
        0: result := 36;
        1: result := 43;
        2: result := 47;
        3: result := 50;
        4: result := 52;
        5: result := 55;
        6: result := 59;
        7: result := 65;
        8: result := 70;
        9: result := 75;
        10: result := 80;
      end;
    end;


  if ScaleIn = 'SHY' then
    begin
      case RawScoreIn of
        0: result := 37;
        1: result := 44;
        2: result := 47;
        3: result := 50;
        4: result := 52;
        5: result := 57;
        6: result := 66;
        7: result := 75;
      end;
    end;

  if ScaleIn = 'DSF' then
    begin
      case RawScoreIn of
        0: result := 44;
        1: result := 58;
        2: result := 68;
        3: result := 78;
        4: result := 88;
        5: result := 98;
        6: result := 100;
      end;
    end;

  if ScaleIn = 'AES' then
    begin
      case RawScoreIn of
        0: result := 33;
        1: result := 39;
        2: result := 45;
        3: result := 50;
        4: result := 56;
        5: result := 62;
        6: result := 68;
        7: result := 73;
      end;
    end;

  if ScaleIn = 'MEC' then
    begin
      case RawScoreIn of
        0: result := 38;
        1: result := 43;
        2: result := 47;
        3: result := 52;
        4: result := 56;
        5: result := 61;
        6: result := 65;
        7: result := 69;
        8: result := 74;
        9: result := 78;
      end;
    end;

  if ScaleIn = 'AGGR_r' then
    begin
      case RawScoreIn of
        0: result := 28;
        1: result := 32;
        2: result := 35;
        3: result := 37;
        4: result := 39;
        5: result := 41;
        6: result := 43;
        7: result := 45;
        8: result := 47;
        9: result := 50;
        10: result := 53;
        11: result := 56;
        12: result := 60;
        13: result := 65;
        14: result := 69;
        15: result := 74;
        16: result := 78;
        17: result := 83;
        18: result := 88;
      end;
    end;

  if ScaleIn = 'PSYC_r' then
    begin
      result := 100;
      case RawScoreIn of
        0: result := 38;
        1: result := 47;
        2: result := 52;
        3: result := 56;
        4: result := 59;
        5: result := 63;
        6: result := 66;
        7: result := 69;
        8: result := 73;
        9: result := 76;
        10: result := 79;
        11: result := 83;
        12: result := 86;
        13: result := 89;
        14: result := 93;
        15: result := 96;
        // 16-26	100
      end;
    end;

  if ScaleIn = 'DISC_r' then
    begin
      case RawScoreIn of
        0: result := 31;
        1: result := 35;
        2: result := 38;
        3: result := 41;
        4: result := 44;
        5: result := 47;
        6: result := 49;
        7: result := 51;
        8: result := 54;
        9: result := 56;
        10: result := 59;
        11: result := 63;
        12: result := 66;
        13: result := 69;
        14: result := 72;
        15: result := 75;
        16: result := 79;
        17: result := 82;
        18: result := 85;
        19: result := 88;
        20: result := 92;
      end;
    end;

  if ScaleIn = 'NEGE_r' then
    begin
      case RawScoreIn of
        0: result := 32;
        1: result := 36;
        2: result := 40;
        3: result := 42;
        4: result := 45;
        5: result := 47;
        6: result := 49;
        7: result := 51;
        8: result := 53;
        9: result := 56;
        10: result := 59;
        11: result := 62;
        12: result := 66;
        13: result := 69;
        14: result := 73;
        15: result := 77;
        16: result := 80;
        17: result := 84;
        18: result := 88;
        19: result := 91;
        20: result := 95;
      end;
    end;

  if ScaleIn = 'INTR_r' then
    begin
      case RawScoreIn of
        0: result := 32;
        1: result := 36;
        2: result := 39;
        3: result := 42;
        4: result := 45;
        5: result := 47;
        6: result := 49;
        7: result := 52;
        8: result := 54;
        9: result := 57;
        10: result := 60;
        11: result := 64;
        12: result := 67;
        13: result := 70;
        14: result := 74;
        15: result := 77;
        16: result := 80;
        17: result := 83;
        18: result := 87;
        19: result := 90;
        20: result := 93;
      end;
    end;
end;

function ScoreAnswers_MMPI2RF(
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string): string;

// MMPI-2-RF has 338 items. 51 scales.
// This function will step through each answer and score it for any scale that
// uses it. This is the raw score for each scale. Later, the T-score will be
// calculated for each raw score.
//
// The two validity scales VRIN_r and TRIN_r compare the results of two questions.
// If the response to the first question is significant, then a boolean is set to
// true. When the second question is considered, the boolean is compared to the
// results of the second question in the pair.
// Things become tricky when a question is skipped for the VRIN-r and TRIN-r
// scales. See page 27 of the manual for a confusing description.
var
  Count,
    CannotSay, // Number of skipped questions
    TrueCount, // Number of responses that are true

// Names for Raw scores
//	Validity Scales
  VRIN_r, //Variable Response Inconsistency
    TRIN_r, //True Response Inconsistency
    F_r, //Infrequent Responses
    Fp_r, //	Infrequent Psychopathology Responses
    Fs, //	Infrequent Somatic Responses
    FBS_r, //	Symptom Validity
    RBS, //	Response Bias Scale
    L_r, //	Uncommon Virtues
    K_r, //	Adjustment Validity

//	Higher-Order (H-O) Scales
  EID, //	Emotional/Internalizing Dysfunction
    THD, //	Thought Dysfunction
    BXD, //	Behavioral/Externalizing Dysfunction

//	Restructured Clinical (RC) Scales
  RCd, //	Demoralization (DEM)
    RC1, //	Somatic Complaints (SOM)
    RC2, //	Low Positive Emotions (LPE)
    RC3, //	Cynicism (CYN)
    RC4, //	Antisocial Behavior (ASB)
    RC6, //	Ideas of Persecution (PER)
    RC7, //	Dysfunctional Negative Emotions (DNE)
    RC8, //	Aberrant Experiences (ABX)
    RC9, //	Hypomanic Activation (HPM)

//	Somatic/Cognitive Scales
  MLS, //	Malaise
    GIC, //	Gastrointestinal Complaints
    HPC, //	Head Pain Complaints
    NUC, //	Neurological Complaints
    COG, //	Cognitive Complaints

//	Internalizing Scales
  SUI, //	Suicidal/Death Ideation
    HLP, //	Helplessness/Hopelessness
    SFD, //	Self_Doubt
    NFC, //	Inefficacy
    STW, //	Stress/Worry
    AXY, //	Anxiety
    ANP, //	Anger Proneness
    BRF, //	Behavior_Restricting Fears
    MSF, //	Multiple Specific Fears

//	Externalizing Scales
  JCP, //	Juvenile Conduct Problems
    SUB, //	Substance Abuse
    AGG, //	Aggression
    ACT, //	Activation

//	Interpersonal Scales
  FML, //	Family Problems
    IPP, //	Interpersonal Passivity
    SAV, //	Social Avoidance
    SHY, //	Shyness
    DSF, //	Disaffiliativeness

//	Interest Scales
  AES, //	Aesthetic_Literary Interests
    MEC, //	Mechanical_Physical Interests

//	Personality Psychopathology Five (PSY_5) Scales
  AGGR_r, //	Aggressiveness_Revised
    PSYC_r, //	Psychoticism_Revised
    DISC_r, //	Disconstraint_Revised
    NEGE_r, //	Negative Emotionality/Neuroticism_Revised
    INTR_r, //	Introversion/Low Positive Emotionality_Revised


//	Validity Scales
  VRIN_r_raw, //Variable Response Inconsistency
    TRIN_r_raw, //True Response Inconsistency
    F_r_raw, //Infrequent Responses
    Fp_r_raw, //	Infrequent Psychopathology Responses
    Fs_raw, //	Infrequent Somatic Responses
    FBS_r_raw, //	Symptom Validity
    RBS_raw, //	Response Bias Scale
    L_r_raw, //	Uncommon Virtues
    K_r_raw //	Adjustment Validity
(*
//	Higher-Order (H-O) Scales
  EID_raw, //	Emotional/Internalizing Dysfunction
    THD_raw, //	Thought Dysfunction
    BXD_raw, //	Behavioral/Externalizing Dysfunction

//	Restructured Clinical (RC) Scales
  RCd_raw, //	Demoralization (DEM)
    RC1_raw, //	Somatic Complaints (SOM)
    RC2_raw, //	Low Positive Emotions (LPE)
    RC3_raw, //	Cynicism (CYN)
    RC4_raw, //	Antisocial Behavior (ASB)
    RC6_raw, //	Ideas of Persecution (PER)
    RC7_raw, //	Dysfunctional Negative Emotions (DNE)
    RC8_raw, //	Aberrant Experiences (ABX)
    RC9_raw, //	Hypomanic Activation (HPM)

//	Somatic/Cognitive Scales
  MLS_raw, //	Malaise
    GIC_raw, //	Gastrointestinal Complaints
    HPC_raw, //	Head Pain Complaints
    NUC_raw, //	Neurological Complaints
    COG_raw, //	Cognitive Complaints

//	Internalizing Scales
  SUI_raw, //	Suicidal/Death Ideation
    HLP_raw, //	Helplessness/Hopelessness
    SFD_raw, //	Self_Doubt
    NFC_raw, //	Inefficacy
    STW_raw, //	Stress/Worry
    AXY_raw, //	Anxiety
    ANP_raw, //	Anger Proneness
    BRF_raw, //	Behavior_Restricting Fears
    MSF_raw, //	Multiple Specific Fears

//	Externalizing Scales
  JCP_raw, //	Juvenile Conduct Problems
    SUB_raw, //	Substance Abuse
    AGG_raw, //	Aggression
    ACT_raw, //	Activation

//	Interpersonal Scales
  FML_raw, //	Family Problems
    IPP_raw, //	Interpersonal Passivity
    SAV_raw, //	Social Avoidance
    SHY_raw, //	Shyness
    DSF_raw, //	Disaffiliativeness

//	Interest Scales
  AES_raw, //	Aesthetic_Literary Interests
    MEC_raw, //	Mechanical_Physical Interests

//	Personality Psychopathology Five (PSY_5) Scales
  AGGR_r_raw, //	Aggressiveness_Revised
    PSYC_r_raw, //	Psychoticism_Revised
    DISC_r_raw, //	Disconstraint_Revised
    NEGE_r_raw, //	Negative Emotionality/Neuroticism_Revised
    INTR_r_raw //	Introversion/Low Positive Emotionality_Revised
*)

                              : integer;

// Report on the number of scale items answered (response percent).
//	Validity Scales
  VRIN_r_Count, //Variable Response Inconsistency
    TRIN_r_Count, //True Response Inconsistency
    F_r_Count, //Infrequent Responses
    Fp_r_Count, //	Infrequent Psychopathology Responses
    Fs_Count, //	Infrequent Somatic Responses
    FBS_r_Count, //	Symptom Validity
    RBS_Count, //	Response Bias Scale
    L_r_Count, //	Uncommon Virtues
    K_r_Count, //	Adjustment Validity

//	Higher_Order (H_O) Scales
  EID_Count, //	Emotional/Internalizing Dysfunction
    THD_Count, //	Thought Dysfunction
    BXD_Count, //	Behavioral/Externalizing Dysfunction

//	Restructured Clinical (RC) Scales
  RCd_Count, //	Demoralization (DEM)
    RC1_Count, //	Somatic Complaints (SOM)
    RC2_Count, //	Low Positive Emotions (LPE)
    RC3_Count, //	Cynicism (CYN)
    RC4_Count, //	Antisocial Behavior (ASB)
    RC6_Count, //_Count,	//	Ideas of Persecution (PER)
    RC7_Count, //	Dysfunctional Negative Emotions (DNE)
    RC8_Count, //	Aberrant Experiences (ABX)
    RC9_Count, //	Hypomanic Activation (HPM)

//	Somatic/Cognitive Scales
  MLS_Count, //	Malaise
    GIC_Count, //	Gastrointestinal Complaints
    HPC_Count, //	Head Pain Complaints
    NUC_Count, //	Neurological Complaints
    COG_Count, //	Cognitive Complaints

//	Internalizing Scales
  SUI_Count, //	Suicidal/Death Ideation
    HLP_Count, //	Helplessness/Hopelessness
    SFD_Count, //	Self_Doubt
    NFC_Count, //	Inefficacy
    STW_Count, //	Stress/Worry
    AXY_Count, //	Anxiety
    ANP_Count, //	Anger Proneness
    BRF_Count, //	Behavior_Restricting Fears
    MSF_Count, //	Multiple Specific Fears

//	Externalizing Scales
  JCP_Count, //	Juvenile Conduct Problems
    SUB_Count, //	Substance Abuse
    AGG_Count, //	Aggression
    ACT_Count, //	Activation

//	Interpersonal Scales
  FML_Count, //	Family Problems
    IPP_Count, //	Interpersonal Passivity
    SAV_Count, //	Social Avoidance
    SHY_Count, //	Shyness
    DSF_Count, //	Disaffiliativeness

//	Interest Scales
  AES_Count, //	Aesthetic_Literary Interests
    MEC_Count, //	Mechanical_Physical Interests

//	Personality Psychopathology Five (PSY_5) Scales
  AGGR_r_Count, //	Aggressiveness_Revised
    PSYC_r_Count, //	Psychoticism_Revised
    DISC_r_Count, //	Disconstraint_Revised
    NEGE_r_Count, //	Negative Emotionality/Neuroticism_Revised
    INTR_r_Count //	Introversion/Low Positive Emotionality_Revised

                              : integer;

  VRIN_r_Adjust, //Variable Response Inconsistency
    TRIN_r_Adjust_AboveEleven,
    TRIN_r_Adjust_BelowEleven,
    F_r_Adjust, //Infrequent Responses
    Fp_r_Adjust, //	Infrequent Psychopathology Responses
    Fs_Adjust, //	Infrequent Somatic Responses
    FBS_r_Adjust, //	Symptom Validity
    RBS_Adjust, //	Response Bias Scale
    L_r_Adjust, //	Uncommon Virtues
    K_r_Adjust, //	Adjustment Validity

  // VRIN_r pairs
  V6F_136T,
    V67T_278F,
    V158F_228T,
    V8F_323T,
    V71T_264F,
    V158T_261F,
    V17T_94F,
    V79T_289F,
    V165F_317T,
    V24T_319F,
    V83F_105T,
    V166T_207F,
    V25T_174F,
    V88F_265T,
    V175T_291F,
    V29T_116F,
    V89F_232T,
    V175F_291T,
    V32F_257T,
    V93T_164F,
    V187F_331T,
    V35T_114F,
    V93F_164T,
    V194F_233T,
    V40T_144F,
    V95F_144T,
    V196F_245T,
    V44T_177F,
    V108T_152F,
    V213F_260T,
    V44F_177T,
    V108F_152T,
    V219F_267T,
    V47T_201F,
    V119F_303T,
    V239F_302T,
    V55T_99F,
    V123T_338F,
    V246F_282T,
    V55F_171T,
    V134F_293T,
    V252T_287F,
    V58F_180T,
    V141F_266T,
    V299T_324F,
    V60F_327T,
    V146T_275F,
    V316F_329T,
    V62F_74T,
    V147F_276T,
    V318T_337F,
    V66F_223T,
    V156F_205T,

    // VRIN-r pairs that are used in reversed pairs
  V44_177_Skip,
    V93_164_Skip,
    V108_152_Skip,
    V175_291_Skip,


  // VRIN-r pairs that have a common question
  V95_144_Skip,
    V40_144_Skip,
    V55_Skip,
    V55_99_Skip,
    V55_171_Skip,
    V158_Skip,
    V158_228_Skip,
    V158_261_Skip,


    // TRIN_r pairs; add one point
  T17T_44T,
    T83T_120T,
    T158T_202T,
    T30T_217T,
    T101T_189T,
    T169T_282T,
    T37T_322T,
    T109T_278T,
    T212T_264T,
    T48T_182T,
    T134T_248T,
    T234T_261T,
    T57T_67T,
    T146T_293T,
    T269T_314T,

    // TRIN_r pairs; subtract one point
  T4F_22F,
    T83F_158F,
    T178F_221F,
    T44F_94F,
    T108F_246F,
    T201F_278F,
    T59F_306F,
    T119F_134F,
    T261F_293F,
    T73F_338F,
    T147F_319F
                              : Boolean;
  SkippedItems,
    CriticalItems,
    Responses,
    TempResult,
    TempAnswer                : string;

  procedure ResetVariables;
  begin
     // Reset the string that shows the summary of responses
    Responses := '';

    // Reset string that holds the critical questions
    CriticalItems := '';

    // Reset string that is populated with the question-number of a skipped question.
    SkippedItems := '';

    // Count the number of skipped questions
    CannotSay := 0;

    // Count the number of true responses
    TrueCount := 0;

    // Validity Scales
    VRIN_r := 0; //  Variable Response Inconsistency
    TRIN_r := 11; //  True Response Inconsistency: Start at 11, not zero
    F_r := 0; //  Infrequent Responses
    Fp_r := 0; //	Infrequent Psychopathology Responses
    Fs := 0; //	Infrequent Somatic Responses
    FBS_r := 0; //	Symptom Validity
    RBS := 0; //	Response Bias Scale
    L_r := 0; //	Uncommon Virtues
    K_r := 0; //	Adjustment Validity

//	Higher-Order (H-O) Scales
    EID := 0; //	Emotional/Internalizing Dysfunction
    THD := 0; //	Thought Dysfunction
    BXD := 0; //	Behavioral/Externalizing Dysfunction

//	Restructured Clinical (RC) Scales
    RCd := 0; //	Demoralization (DEM)
    RC1 := 0; //	Somatic Complaints (SOM)
    RC2 := 0; //	Low Positive Emotions (LPE)
    RC3 := 0; //	Cynicism (CYN)
    RC4 := 0; //	Antisocial Behavior (ASB)
    RC6 := 0; // 	Ideas of Persecution (PER)
    RC7 := 0; //	Dysfunctional Negative Emotions (DNE)
    RC8 := 0; //	Aberrant Experiences (ABX)
    RC9 := 0; //	Hypomanic Activation (HPM)

//	Somatic/Cognitive Scales
    MLS := 0; //	Malaise
    GIC := 0; //	Gastrointestinal Complaints
    HPC := 0; //	Head Pain Complaints
    NUC := 0; //	Neurological Complaints
    COG := 0; //	Cognitive Complaints

//	Internalizing Scales
    SUI := 0; //	Suicidal/Death Ideation
    HLP := 0; //	Helplessness/Hopelessness
    SFD := 0; //	Self_Doubt
    NFC := 0; //	Inefficacy
    STW := 0; //	Stress/Worry
    AXY := 0; //	Anxiety
    ANP := 0; //	Anger Proneness
    BRF := 0; //	Behavior_Restricting Fears
    MSF := 0; //	Multiple Specific Fears

//	Externalizing Scales
    JCP := 0; //	Juvenile Conduct Problems
    SUB := 0; //	Substance Abuse
    AGG := 0; //	Aggression
    ACT := 0; //	Activation

//	Interpersonal Scales
    FML := 0; //	Family Problems
    IPP := 0; //	Interpersonal Passivity
    SAV := 0; //	Social Avoidance
    SHY := 0; //	Shyness
    DSF := 0; //	Disaffiliativeness

//	Interest Scales
    AES := 0; //	Aesthetic_Literary Interests
    MEC := 0; //	Mechanical_Physical Interests

//	Personality Psychopathology Five (PSY_5) Scales
    AGGR_r := 0; //	Aggressiveness_Revised
    PSYC_r := 0; //	Psychoticism_Revised
    DISC_r := 0; //	Disconstraint_Revised
    NEGE_r := 0; //	Negative Emotionality/Neuroticism_Revised
    INTR_r := 0;

    // Reset count used for response percent
    VRIN_r_Count := 0; //Variable Response Inconsistency
    TRIN_r_Count := 0; //True Response Inconsistency
    F_r_Count := 0; //Infrequent Responses
    Fp_r_Count := 0; //	Infrequent Psychopathology Responses
    Fs_Count := 0; //	Infrequent Somatic Responses
    FBS_r_Count := 0; //	Symptom Validity
    RBS_Count := 0; //	Response Bias Scale
    L_r_Count := 0; //	Uncommon Virtues
    K_r_Count := 0; //	Adjustment Validity

//	Higher_Order (H_O) Scales
    EID_Count := 0; //	Emotional/Internalizing Dysfunction
    THD_Count := 0; //	Thought Dysfunction
    BXD_Count := 0; //	Behavioral/Externalizing Dysfunction

//	Restructured Clinical (RC) Scales
    RCd_Count := 0; //	Demoralization (DEM)
    RC1_Count := 0; //	Somatic Complaints (SOM)
    RC2_Count := 0; //	Low Positive Emotions (LPE)
    RC3_Count := 0; //	Cynicism (CYN)
    RC4_Count := 0; //	Antisocial Behavior (ASB)
    RC6_Count := 0; //_Count,	//	Ideas of Persecution (PER)
    RC7_Count := 0; //	Dysfunctional Negative Emotions (DNE)
    RC8_Count := 0; //	Aberrant Experiences (ABX)
    RC9_Count := 0; //	Hypomanic Activation (HPM)

//	Somatic/Cognitive Scales
    MLS_Count := 0; //	Malaise
    GIC_Count := 0; //	Gastrointestinal Complaints
    HPC_Count := 0; //	Head Pain Complaints
    NUC_Count := 0; //	Neurological Complaints
    COG_Count := 0; //	Cognitive Complaints

//	Internalizing Scales
    SUI_Count := 0; //	Suicidal/Death Ideation
    HLP_Count := 0; //	Helplessness/Hopelessness
    SFD_Count := 0; //	Self_Doubt
    NFC_Count := 0; //	Inefficacy
    STW_Count := 0; //	Stress/Worry
    AXY_Count := 0; //	Anxiety
    ANP_Count := 0; //	Anger Proneness
    BRF_Count := 0; //	Behavior_Restricting Fears
    MSF_Count := 0; //	Multiple Specific Fears

//	Externalizing Scales
    JCP_Count := 0; //	Juvenile Conduct Problems
    SUB_Count := 0; //	Substance Abuse
    AGG_Count := 0; //	Aggression
    ACT_Count := 0; //	Activation

//	Interpersonal Scales
    FML_Count := 0; //	Family Problems
    IPP_Count := 0; //	Interpersonal Passivity
    SAV_Count := 0; //	Social Avoidance
    SHY_Count := 0; //	Shyness
    DSF_Count := 0; //	Disaffiliativeness

//	Interest Scales
    AES_Count := 0; //	Aesthetic_Literary Interests
    MEC_Count := 0; //	Mechanical_Physical Interests

//	Personality Psychopathology Five (PSY_5) Scales
    AGGR_r_Count := 0; //	Aggressiveness_Revised
    PSYC_r_Count := 0; //	Psychoticism_Revised
    DISC_r_Count := 0; //	Disconstraint_Revised
    NEGE_r_Count := 0; //	Negative Emotionality/Neuroticism_Revised
    INTR_r_Count := 0; //


    V6F_136T := False;
    V67T_278F := False;
    V158F_228T := False;
    V8F_323T := False;
    V71T_264F := False;
    V158T_261F := False;
    V17T_94F := False;
    V79T_289F := False;
    V165F_317T := False;
    V24T_319F := False;
    V83F_105T := False;
    V166T_207F := False;
    V25T_174F := False;
    V88F_265T := False;
    V175T_291F := False;
    V29T_116F := False;
    V89F_232T := False;
    V175F_291T := False;
    V32F_257T := False;
    V93T_164F := False;
    V187F_331T := False;
    V35T_114F := False;
    V93F_164T := False;
    V194F_233T := False;
    V40T_144F := False;
    V95F_144T := False;
    V196F_245T := False;
    V44T_177F := False;
    V108T_152F := False;
    V213F_260T := False;
    V44F_177T := False;
    V108F_152T := False;
    V219F_267T := False;
    V47T_201F := False;
    V119F_303T := False;
    V239F_302T := False;
    V55T_99F := False;
    V123T_338F := False;
    V246F_282T := False;
    V55F_171T := False;
    V134F_293T := False;
    V252T_287F := False;
    V58F_180T := False;
    V141F_266T := False;
    V299T_324F := False;
    V60F_327T := False;
    V146T_275F := False;
    V316F_329T := False;
    V62F_74T := False;
    V147F_276T := False;
    V318T_337F := False;
    V66F_223T := False;
    V156F_205T := False;

    V44_177_Skip := False;
    V93_164_Skip := False;
    V108_152_Skip := False;
    V175_291_Skip := False;
    V95_144_Skip := False;
    V40_144_Skip := False;
    V55_Skip := False;
    V55_99_Skip := False;
    V55_171_Skip := False;
    V158_Skip := False;
    V158_228_Skip := False;
    V158_261_Skip := False;

    T17T_44T := False;
    T83T_120T := False;
    T158T_202T := False;
    T30T_217T := False;
    T101T_189T := False;
    T169T_282T := False;
    T37T_322T := False;
    T109T_278T := False;
    T212T_264T := False;
    T48T_182T := False;
    T134T_248T := False;
    T234T_261T := False;
    T57T_67T := False;
    T146T_293T := False;
    T269T_314T := False;

    T4F_22F := False;
    T83F_158F := False;
    T178F_221F := False;
    T44F_94F := False;
    T108F_246F := False;
    T201F_278F := False;
    T59F_306F := False;
    T119F_134F := False;
    T261F_293F := False;
    T73F_338F := False;
    T147F_319F := False;
  end;

  procedure ScanTheAnswers(HowMany: integer; FirstRun: boolean);
  var
    i                         : integer;
  begin

    for i := 1 to HowMany - 1 do
      begin
  // AnswerString Format:
  //      83=1^1330^234~
  //      84=0^214~...

  // '83' is QuestionID,
  // '1' is ItemIndex for radio group, but can be text from edit field
  // '1330' is ChoiceID (not used here)
  // '234' is AnswerID (not used here)

        TempResult := Piece(AnswerString, RECORD_END, i);
        TempAnswer := Piece(Piece(TempResult, EQ, 2), '^', 1);

        if TempAnswer = '0' then
          begin
            Inc(TrueCount);
            Responses := Responses + InttoStr(i) + '. T  ';
          end;

        if TempAnswer = '1' then
          Responses := Responses + InttoStr(i) + '. F  ';

      // Score skipped answers in special way
        if (TempAnswer <> '0') and (TempAnswer <> '1') then
          begin
            CannotSay := CannotSay + 1;
            SkippedItems := SkippedItems + IntToStr(i) + ', ';
            Responses := Responses + InttoStr(i) + '. X  ';

            if not FirstRun then
              begin
//                CannotSay := CannotSay + 1;
//                SkippedItems := SkippedItems + IntToStr(i) + ', ';
//                Responses := Responses + InttoStr(i) + '. X  ';

                if VRIN_r_Adjust then
                  begin

          // VRIN-r will count skipped items as answered in the direction of
          // satisfying the rule. See page 27 of the manual.

          // These four pairs of items are scored positive when one or both of
          // the pairs is skipped.
          //  44  - 177
          //  93  - 164
          //  108 - 152
          //  175 - 291

                    if i = 44 then
                      begin
                        V44_177_Skip := true;
                        VRIN_r := VRIN_r + 1;
                      end;
                    if i = 93 then
                      begin
                        V93_164_Skip := true;
                        VRIN_r := VRIN_r + 1;
                      end;
                    if i = 108 then
                      begin
                        V108_152_Skip := true;
                        VRIN_r := VRIN_r + 1;
                      end;
                    if i = 175 then
                      begin
                        V175_291_Skip := true;
                        VRIN_r := VRIN_r + 1;
                      end;

                    if (i = 177) and not V44_177_Skip then
                      VRIN_r := VRIN_r + 1;
                    if (i = 164) and not V93_164_Skip then
                      VRIN_r := VRIN_r + 1;
                    if (i = 152) and not V108_152_Skip then
                      VRIN_r := VRIN_r + 1;
                    if (i = 291) and V175_291_Skip then
                      VRIN_r := VRIN_r + 1;

          // These three pairs of items share one question. If the common item
          // is skipped and one of the other pair is skipped or answered in
          // keyed direction then increase VRIN-r OR
          // If both of the non-common items are skipped, then VIRN-r is increased
          // by one.
          //    40  - 144   and   95  - 144
          //    55  - 99    and   55  - 171
          //   158  - 228   and  158  - 261


                    if i = 40 then // non-common item
                      V40_144_Skip := true;

                    if i = 95 then // non-common item
                      begin
                        V95_144_Skip := true;
                        if V95_144_Skip then
                          VRIN_r := VRIN_r + 1;
                      end;

          // Common item
                    if (i = 144) and not (V40_144_Skip and V95_144_Skip) then
                      begin
                        if V40_144_Skip and not V95_144_Skip then
                          VRIN_r := VRIN_r + 1;
                        if not V40_144_Skip and V95_144_Skip then
                          VRIN_r := VRIN_r + 1;
                        if (V40T_144F or V95F_144T) then
                          VRIN_r := VRIN_r + 1;
                      end;

                    if i = 55 then // Common item
                      V55_Skip := true;
                    if (i = 99) then // non-common item
                      begin
                        V55_99_Skip := true;
                        if V55_Skip then
                          VRIN_r := VRIN_r + 1;
              // also see i = 99 if answered in keyed direction
                      end;
                    if (i = 171) then // non-common item
                      begin
                        V55_171_Skip := true;
              // Don't add to VRIN-r twice
                        if not V55_99_Skip and V55_Skip then
                          VRIN_r := VRIN_r + 1;
              // if both non-common questions are skipped, then Increase VRIN-r
                        if not V55_Skip and V55_99_Skip then
                          VRIN_r := VRIN_r + 1;
              // also see i = 171 if answered in keyed direction
                      end;

                    if i = 158 then // Common item
                      V158_Skip := true;
                    if (i = 228) then // non-common item
                      begin
                        V158_228_Skip := true;
                        if V158_Skip then
                          VRIN_r := VRIN_r + 1;
              // also see i = 228 if answered in keyed direction
                      end;
                    if (i = 261) then // non-common item
                      begin
                        V158_261_Skip := true;
              // Don't add to VRIN-r twice
                        if not V158_228_Skip and V158_Skip then
                          VRIN_r := VRIN_r + 1;
              // if both non-common questions are skipped, then increase VRIN-r
                        if not V158_Skip and V158_228_Skip then
                          VRIN_r := VRIN_r + 1;
                      end;

          // Standard item pairs: Treat a skipped question as if it were
          // answered in the keyed direction.
          // First member of the pair.
                    if i = 6 then
                      V6F_136T := true;
                    if i = 67 then
                      V67T_278F := true;
                    if i = 8 then
                      V8F_323T := true;
                    if i = 71 then
                      V71T_264F := true;
                    if i = 17 then
                      V17T_94F := true;
                    if i = 79 then
                      V79T_289F := true;
                    if i = 165 then
                      V165F_317T := true;
                    if i = 24 then
                      V24T_319F := true;
                    if i = 83 then
                      V83F_105T := true;
                    if i = 166 then
                      V166T_207F := true;
                    if i = 25 then
                      V25T_174F := true;
                    if i = 88 then
                      V88F_265T := true;
                    if i = 175 then
                      V175T_291F := true;
                    if i = 29 then
                      V29T_116F := true;
                    if i = 89 then
                      V89F_232T := true;
                    if i = 32 then
                      V32F_257T := true;
                    if i = 187 then
                      V187F_331T := true;
                    if i = 35 then
                      V35T_114F := true;
                    if i = 93 then
                      V93F_164T := true;
                    if i = 194 then
                      V194F_233T := true;
                    if i = 196 then
                      V196F_245T := true;
                    if i = 213 then
                      V213F_260T := true;
                    if i = 219 then
                      V219F_267T := true;
                    if i = 47 then
                      V47T_201F := true;
                    if i = 119 then
                      V119F_303T := true;
                    if i = 239 then
                      V239F_302T := true;
                    if i = 123 then
                      V123T_338F := true;
                    if i = 246 then
                      V246F_282T := true;
                    if i = 134 then
                      V134F_293T := true;
                    if i = 252 then
                      V252T_287F := true;
                    if i = 58 then
                      V58F_180T := true;
                    if i = 141 then
                      V141F_266T := true;
                    if i = 299 then
                      V299T_324F := true;
                    if i = 60 then
                      V60F_327T := true;
                    if i = 146 then
                      V146T_275F := true;
                    if i = 316 then
                      V316F_329T := true;
                    if i = 62 then
                      V62F_74T := true;
                    if i = 147 then
                      V147F_276T := true;
                    if i = 318 then
                      V318T_337F := true;
                    if i = 66 then
                      V66F_223T := true;
                    if i = 156 then
                      V156F_205T := true;

          // Standard item pairs: Treat a skipped question as if it were
          // answered in the keyed direction.
          // Second member of the pair. The first member is already keyed by
          // either a true/false response or by being skipped.
                    if (i = 136) and V6F_136T then
                      VRIN_r := VRIN_r + 1;
                    if (i = 278) and V67T_278F then
                      VRIN_r := VRIN_r + 1;
                    if (i = 323) and V8F_323T then
                      VRIN_r := VRIN_r + 1;
                    if (i = 264) and V71T_264F then
                      VRIN_r := VRIN_r + 1;
                    if (i = 94) and V17T_94F then
                      VRIN_r := VRIN_r + 1;
                    if (i = 289) and V79T_289F then
                      VRIN_r := VRIN_r + 1;
                    if (i = 317) and V165F_317T then
                      VRIN_r := VRIN_r + 1;
                    if (i = 319) and V24T_319F then
                      VRIN_r := VRIN_r + 1;
                    if (i = 105) and V83F_105T then
                      VRIN_r := VRIN_r + 1;
                    if (i = 207) and V166T_207F then
                      VRIN_r := VRIN_r + 1;
                    if (i = 174) and V25T_174F then
                      VRIN_r := VRIN_r + 1;
                    if (i = 265) and V88F_265T then
                      VRIN_r := VRIN_r + 1;
                    if (i = 116) and V29T_116F then
                      VRIN_r := VRIN_r + 1;
                    if (i = 232) and V89F_232T then
                      VRIN_r := VRIN_r + 1;
                    if (i = 291) and V175F_291T then
                      VRIN_r := VRIN_r + 1;
                    if (i = 257) and V32F_257T then
                      VRIN_r := VRIN_r + 1;
                    if (i = 331) and V187F_331T then
                      VRIN_r := VRIN_r + 1;
                    if (i = 114) and V35T_114F then
                      VRIN_r := VRIN_r + 1;
                    if (i = 164) and V93F_164T then
                      VRIN_r := VRIN_r + 1;
                    if (i = 233) and V194F_233T then
                      VRIN_r := VRIN_r + 1;
                    if (i = 245) and V196F_245T then
                      VRIN_r := VRIN_r + 1;
                    if (i = 177) and V44T_177F then
                      VRIN_r := VRIN_r + 1;
                    if (i = 152) and V108T_152F then
                      VRIN_r := VRIN_r + 1;
                    if (i = 260) and V213F_260T then
                      VRIN_r := VRIN_r + 1;
                    if (i = 267) and V219F_267T then
                      VRIN_r := VRIN_r + 1;
                    if (i = 201) and V47T_201F then
                      VRIN_r := VRIN_r + 1;
                    if (i = 303) and V119F_303T then
                      VRIN_r := VRIN_r + 1;
                    if (i = 302) and V239F_302T then
                      VRIN_r := VRIN_r + 1;
                    if (i = 338) and V123T_338F then
                      VRIN_r := VRIN_r + 1;
                    if (i = 282) and V246F_282T then
                      VRIN_r := VRIN_r + 1;
                    if (i = 293) and V134F_293T then
                      VRIN_r := VRIN_r + 1;
                    if (i = 287) and V252T_287F then
                      VRIN_r := VRIN_r + 1;
                    if (i = 180) and V58F_180T then
                      VRIN_r := VRIN_r + 1;
                    if (i = 266) and V141F_266T then
                      VRIN_r := VRIN_r + 1;
                    if (i = 324) and V299T_324F then
                      VRIN_r := VRIN_r + 1;
                    if (i = 327) and V60F_327T then
                      VRIN_r := VRIN_r + 1;
                    if (i = 275) and V146T_275F then
                      VRIN_r := VRIN_r + 1;
                    if (i = 329) and V316F_329T then
                      VRIN_r := VRIN_r + 1;
                    if (i = 74) and V62F_74T then
                      VRIN_r := VRIN_r + 1;
                    if (i = 276) and V147F_276T then
                      VRIN_r := VRIN_r + 1;
                    if (i = 337) and V318T_337F then
                      VRIN_r := VRIN_r + 1;
                    if (i = 223) and V66F_223T then
                      VRIN_r := VRIN_r + 1;
                    if (i = 205) and V156F_205T then
                      VRIN_r := VRIN_r + 1;
                  end;


                if TRIN_r_Adjust_AboveEleven then
                  begin
                    if i = 17 then
                      T17T_44T := true;
                    if i = 30 then
                      T30T_217T := true;
                    if i = 37 then
                      T37T_322T := true;
                    if i = 48 then
                      T48T_182T := true;
                    if i = 57 then
                      T57T_67T := true;
                    if i = 83 then
                      T83T_120T := true;
                    if i = 101 then
                      T101T_189T := true;
                    if i = 109 then
                      T109T_278T := true;
                    if i = 134 then
                      T134T_248T := true;
                    if i = 146 then
                      T146T_293T := true;
                    if i = 158 then
                      T158T_202T := true;
                    if i = 169 then
                      T169T_282T := true;
                    if i = 212 then
                      T212T_264T := true;
                    if i = 234 then
                      T234T_261T := true;
                    if i = 269 then
                      T269T_314T := true;

                    if (i = 44) and T17T_44T then
                      Inc(TRIN_r);
                    if (i = 217) and T30T_217T then
                      Inc(TRIN_r);
                    if (i = 322) and T37T_322T then
                      Inc(TRIN_r);
                    if (i = 182) and T48T_182T then
                      Inc(TRIN_r);
                    if (i = 67) and T57T_67T then
                      Inc(TRIN_r);
                    if (i = 120) and T83T_120T then
                      Inc(TRIN_r);
                    if (i = 189) and T101T_189T then
                      Inc(TRIN_r);
                    if (i = 278) and T109T_278T then
                      Inc(TRIN_r);
                    if (i = 248) and T134T_248T then
                      Inc(TRIN_r);
                    if (i = 293) and T146T_293T then
                      Inc(TRIN_r);
                    if (i = 202) and T158T_202T then
                      Inc(TRIN_r);
                    if (i = 282) and T169T_282T then
                      Inc(TRIN_r);
                    if (i = 264) and T212T_264T then
                      Inc(TRIN_r);
                    if (i = 261) and T234T_261T then
                      Inc(TRIN_r);
                    if (i = 314) and T269T_314T then
                      Inc(TRIN_r);
                  end;

                if TRIN_r_Adjust_BelowEleven then
                  begin
                    if i = 4 then
                      T4F_22F := true;
                    if i = 44 then
                      T44F_94F := true;
                    if i = 59 then
                      T59F_306F := true;
                    if i = 73 then
                      T73F_338F := true;
                    if i = 83 then
                      T83F_158F := true;
                    if i = 108 then
                      T108F_246F := true;
                    if i = 119 then
                      T119F_134F := true;
                    if i = 147 then
                      T147F_319F := true;
                    if i = 178 then
                      T178F_221F := true;
                    if i = 201 then
                      T201F_278F := true;
                    if i = 261 then
                      T261F_293F := true;

                    if (i = 22) and T4F_22F then
                      Dec(TRIN_r);
                    if (i = 94) and T44F_94F then
                      Dec(TRIN_r);
                    if (i = 306) and T59F_306F then
                      Dec(TRIN_r);
                    if (i = 338) and T73F_338F then
                      Dec(TRIN_r);
                    if (i = 158) and T83F_158F then
                      Dec(TRIN_r);
                    if (i = 246) and T108F_246F then
                      Dec(TRIN_r);
                    if (i = 134) and T119F_134F then
                      Dec(TRIN_r);
                    if (i = 319) and T147F_319F then
                      Dec(TRIN_r);
                    if (i = 221) and T178F_221F then
                      Dec(TRIN_r);
                    if (i = 278) and T201F_278F then
                      Dec(TRIN_r);
                    if (i = 293) and T261F_293F then
                      Dec(TRIN_r);
                  end;

                if F_r_Adjust then
                  begin
                    if i = 14 then
                      Inc(F_r);
                    if i = 30 then
                      Inc(F_r);
                    if i = 46 then
                      Inc(F_r);
                    if i = 56 then
                      Inc(F_r);
                    if i = 59 then
                      Inc(F_r);
                    if i = 67 then
                      Inc(F_r);
                    if i = 71 then
                      Inc(F_r);
                    if i = 74 then
                      Inc(F_r);
                    if i = 83 then
                      Inc(F_r);
                    if i = 102 then
                      Inc(F_r);
                    if i = 106 then
                      Inc(F_r);
                    if i = 117 then
                      Inc(F_r);
                    if i = 120 then
                      Inc(F_r);
                    if i = 139 then
                      Inc(F_r);
                    if i = 146 then
                      Inc(F_r);
                    if i = 164 then
                      Inc(F_r);
                    if i = 174 then
                      Inc(F_r);
                    if i = 203 then
                      Inc(F_r);
                    if i = 218 then
                      Inc(F_r);
                    if i = 227 then
                      Inc(F_r);
                    if i = 231 then
                      Inc(F_r);
                    if i = 240 then
                      Inc(F_r);
                    if i = 253 then
                      Inc(F_r);
                    if i = 264 then
                      Inc(F_r);
                    if i = 275 then
                      Inc(F_r);
                    if i = 277 then
                      Inc(F_r);
                    if i = 281 then
                      Inc(F_r);
                    if i = 294 then
                      Inc(F_r);
                    if i = 301 then
                      Inc(F_r);
                    if i = 310 then
                      Inc(F_r);
                    if i = 312 then
                      Inc(F_r);
                    if i = 332 then
                      Inc(F_r);
                  end;

                if Fp_r_Adjust then
                  begin
                    if i = 7 then
                      Inc(Fp_r);
                    if i = 40 then
                      Inc(Fp_r);
                    if i = 41 then
                      Inc(Fp_r);
                    if i = 79 then
                      Inc(Fp_r);
                    if i = 86 then
                      Inc(Fp_r);
                    if i = 98 then
                      Inc(Fp_r);
                    if i = 124 then
                      Inc(Fp_r);
                    if i = 129 then
                      Inc(Fp_r);
                    if i = 150 then
                      Inc(Fp_r);
                    if i = 157 then
                      Inc(Fp_r);
                    if i = 168 then
                      Inc(Fp_r);
                    if i = 178 then
                      Inc(Fp_r);
                    if i = 191 then
                      Inc(Fp_r);
                    if i = 208 then
                      Inc(Fp_r);
                    if i = 221 then
                      Inc(Fp_r);
                    if i = 252 then
                      Inc(Fp_r);
                    if i = 255 then
                      Inc(Fp_r);
                    if i = 270 then
                      Inc(Fp_r);
                    if i = 283 then
                      Inc(Fp_r);
                    if i = 314 then
                      Inc(Fp_r);
                    if i = 317 then
                      Inc(Fp_r);
                  end;

                if Fs_Adjust then
                  begin
                    if i = 2 then
                      Inc(Fs);
                    if i = 15 then
                      Inc(Fs);
                    if i = 33 then
                      Inc(Fs);
                    if i = 43 then
                      Inc(Fs);
                    if i = 78 then
                      Inc(Fs);
                    if i = 122 then
                      Inc(Fs);
                    if i = 133 then
                      Inc(Fs);
                    if i = 137 then
                      Inc(Fs);
                    if i = 159 then
                      Inc(Fs);
                    if i = 170 then
                      Inc(Fs);
                    if i = 186 then
                      Inc(Fs);
                    if i = 199 then
                      Inc(Fs);
                    if i = 216 then
                      Inc(Fs);
                    if i = 225 then
                      Inc(Fs);
                    if i = 272 then
                      Inc(Fs);
                    if i = 308 then
                      Inc(Fs);
                  end;

                if FBS_r_Adjust then
                  begin
                    if i = 6 then
                      Inc(FBS_r);
                    if i = 15 then
                      Inc(FBS_r);
                    if i = 36 then
                      Inc(FBS_r);
                    if i = 43 then
                      Inc(FBS_r);
                    if i = 45 then
                      Inc(FBS_r);
                    if i = 55 then
                      Inc(FBS_r);
                    if i = 76 then
                      Inc(FBS_r);
                    if i = 77 then
                      Inc(FBS_r);
                    if i = 79 then
                      Inc(FBS_r);
                    if i = 88 then
                      Inc(FBS_r);
                    if i = 93 then
                      Inc(FBS_r);
                    if i = 99 then
                      Inc(FBS_r);
                    if i = 101 then
                      Inc(FBS_r);
                    if i = 133 then
                      Inc(FBS_r);
                    if i = 141 then
                      Inc(FBS_r);
                    if i = 156 then
                      Inc(FBS_r);
                    if i = 162 then
                      Inc(FBS_r);
                    if i = 171 then
                      Inc(FBS_r);
                    if i = 187 then
                      Inc(FBS_r);
                    if i = 189 then
                      Inc(FBS_r);
                    if i = 193 then
                      Inc(FBS_r);
                    if i = 200 then
                      Inc(FBS_r);
                    if i = 210 then
                      Inc(FBS_r);
                    if i = 230 then
                      Inc(FBS_r);
                    if i = 234 then
                      Inc(FBS_r);
                    if i = 247 then
                      Inc(FBS_r);
                    if i = 261 then
                      Inc(FBS_r);
                    if i = 265 then
                      Inc(FBS_r);
                    if i = 290 then
                      Inc(FBS_r);
                    if i = 315 then
                      Inc(FBS_r);
                  end;

                if RBS_Adjust then
                  begin
                    if i = 6 then
                      Inc(RBS);
                    if i = 11 then
                      Inc(RBS);
                    if i = 21 then
                      Inc(RBS);
                    if i = 24 then
                      Inc(RBS);
                    if i = 26 then
                      Inc(RBS);
                    if i = 31 then
                      Inc(RBS);
                    if i = 53 then
                      Inc(RBS);
                    if i = 59 then
                      Inc(RBS);
                    if i = 68 then
                      Inc(RBS);
                    if i = 74 then
                      Inc(RBS);
                    if i = 79 then
                      Inc(RBS);
                    if i = 92 then
                      Inc(RBS);
                    if i = 101 then
                      Inc(RBS);
                    if i = 106 then
                      Inc(RBS);
                    if i = 120 then
                      Inc(RBS);
                    if i = 125 then
                      Inc(RBS);
                    if i = 131 then
                      Inc(RBS);
                    if i = 132 then
                      Inc(RBS);
                    if i = 136 then
                      Inc(RBS);
                    if i = 137 then
                      Inc(RBS);
                    if i = 156 then
                      Inc(RBS);
                    if i = 159 then
                      Inc(RBS);
                    if i = 219 then
                      Inc(RBS);
                    if i = 242 then
                      Inc(RBS);
                    if i = 252 then
                      Inc(RBS);
                    if i = 268 then
                      Inc(RBS);
                    if i = 273 then
                      Inc(RBS);
                    if i = 325 then
                      Inc(RBS);
                  end;

                if L_r_Adjust then
                  begin
                    if i = 16 then
                      Inc(L_r);
                    if i = 45 then
                      Inc(L_r);
                    if i = 61 then
                      Inc(L_r);
                    if i = 70 then
                      Inc(L_r);
                    if i = 95 then
                      Inc(L_r);
                    if i = 127 then
                      Inc(L_r);
                    if i = 154 then
                      Inc(L_r);
                    if i = 182 then
                      Inc(L_r);
                    if i = 183 then
                      Inc(L_r);
                    if i = 211 then
                      Inc(L_r);
                    if i = 241 then
                      Inc(L_r);
                    if i = 268 then
                      Inc(L_r);
                    if i = 298 then
                      Inc(L_r);
                    if i = 325 then
                      Inc(L_r);
                  end;

                if K_r_Adjust then
                  begin
                    if i = 10 then
                      Inc(K_r);
                    if i = 23 then
                      Inc(K_r);
                    if i = 36 then
                      Inc(K_r);
                    if i = 44 then
                      Inc(K_r);
                    if i = 72 then
                      Inc(K_r);
                    if i = 80 then
                      Inc(K_r);
                    if i = 89 then
                      Inc(K_r);
                    if i = 99 then
                      Inc(K_r);
                    if i = 155 then
                      Inc(K_r);
                    if i = 171 then
                      Inc(K_r);
                    if i = 187 then
                      Inc(K_r);
                    if i = 202 then
                      Inc(K_r);
                    if i = 322 then
                      Inc(K_r);
                    if i = 338 then
                      Inc(K_r);
                  end;

              end; // Skipped questions
          end // Not First Run
        else

       // Questions that are answered 0 - true or 1 - false
          begin
            if i = 1 then
              begin
                Inc(MEC_Count);
                if TempAnswer = '0' then
                  MEC := MEC + 1;
              end;
            if i = 2 then
              begin
                Inc(Fs_Count);
                Inc(RC1_Count);
                Inc(GIC_Count);
                if TempAnswer = '1' then
                  begin
                    Fs := Fs + 1;
                    RC1 := RC1 + 1;
                    GIC := GIC + 1;
                  end;
              end;
            if i = 3 then
              begin
                Inc(AES_Count);
                if TempAnswer = '0' then
                  AES := AES + 1;
              end;
            if i = 4 then
              begin
                Inc(EID_Count);
                Inc(RC2_Count);
                Inc(MLS_Count);
                Inc(INTR_r_Count);
                Inc(TRIN_r_Count);

                if TempAnswer = '1' then
                  begin
                    EID := EID + 1;
                    RC2 := RC2 + 1;
                    MLS := MLS + 1;
                    INTR_r := INTR_r + 1;
                    T4F_22F := true;
                  end;
              end;
            if i = 5 then
              begin
                Inc(RC4_Count);
                if TempAnswer = '0' then
                  RC4 := RC4 + 1;
              end;
            if i = 6 then
              begin
                Inc(FBS_r_Count);
                Inc(RBS_Count);
                Inc(RCd_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    FBS_r := FBS_r + 1;
                    RBS := RBS + 1;
                    RCd := RCd + 1;
                  end;

                if TempAnswer = '1' then
                  V6F_136T := true;
              end;
            if i = 7 then
              begin
                Inc(Fp_r_Count);
                if TempAnswer = '1' then
                  Fp_r := Fp_r + 1;
              end;
            if i = 8 then
              begin
                Inc(DSF_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '1' then
                  begin
                    DSF := DSF + 1;
                    V8F_323T := true;
                  end;
              end;
            if i = 9 then
              begin
                Inc(RC7_Count);
                Inc(NEGE_r_Count);
                if TempAnswer = '0' then
                  begin
                    RC7 := RC7 + 1;
                    NEGE_r := NEGE_r + 1;
                  end;
              end;
            if i = 10 then
              begin
                Inc(RC3_Count);
                Inc(K_r_Count);
                if TempAnswer = '0' then
                  RC3 := RC3 + 1;
                if TempAnswer = '1' then
                  K_r := K_r + 1;
              end;
            if i = 11 then
              begin
                Inc(INTR_r_Count);
                Inc(RBS_Count);
                Inc(SAV_Count);
                if TempAnswer = '1' then
                  begin
                    RBS := RBS + 1;
                    SAV := SAV + 1;
                    INTR_r := INTR_r + 1;
                  end;
              end;
            if i = 12 then
              begin
                Inc(THD_Count);
                Inc(RC8_Count);
                Inc(PSYC_r_Count);
                if TempAnswer = '0' then
                  begin
                    THD := THD + 1;
                    RC8 := RC8 + 1;
                    PSYC_r := PSYC_r + 1;
                    CriticalItems := CriticalItems +
                      'I often feel I can read other people''s minds. TRUE^';
                  end;
              end;
            if i = 13 then
              begin
                Inc(RC9_Count);
                if TempAnswer = '0' then
                  begin
                    RC9 := RC9 + 1;
                  end;
              end;
            if i = 14 then
              begin
                Inc(THD_Count);
                Inc(PSYC_r_Count);
                Inc(F_r_Count);
                Inc(RC6_Count);
                if TempAnswer = '0' then
                  begin
                    F_r := F_r + 1;
                    THD := THD + 1;
                    RC6 := RC6 + 1;
                    PSYC_r := PSYC_r + 1;
                    CriticalItems := CriticalItems +
                      'Evil spirits possess me at times. TRUE^';
                  end;
              end;
            if i = 15 then
              begin
                Inc(Fs_Count);
                Inc(RC1_Count);
                Inc(FBS_r_Count);
                if TempAnswer = '0' then
                  begin
                    Fs := Fs + 1;
                    FBS_r := FBS_r + 1;
                    RC1 := RC1 + 1;
                  end;
              end;
            if i = 16 then
              begin
                Inc(L_r_Count);
                if TempAnswer = '1' then
                  L_r := L_r + 1;
              end;
            if i = 17 then
              begin
                Inc(EID_Count);
                Inc(RC2_Count);
                Inc(INTR_r_Count);
                Inc(SAV_Count);
                Inc(TRIN_r_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    V17T_94F := true;
                    T17T_44T := true;
                  end;

                if TempAnswer = '1' then
                  begin
                    EID := EID + 1;
                    RC2 := RC2 + 1;
                    SAV := SAV + 1;
                    INTR_r := INTR_r + 1;
                  end;
              end;
            if i = 18 then
              begin
                Inc(MLS_Count);
                if TempAnswer = '0' then
                  MLS := MLS + 1;
              end;
            if i = 19 then
              begin
                Inc(RC4_Count);
                Inc(FML_Count);
                if TempAnswer = '1' then
                  begin
                    RC4 := RC4 + 1;
                    FML := FML + 1;
                  end;
              end;
            if i = 20 then
              begin
                Inc(BRF_Count);
                if TempAnswer = '0' then
                  BRF := BRF + 1;
              end;
            if i = 21 then
              begin
                Inc(RC4_Count);
                Inc(RBS_Count);
                Inc(BXD_Count);
                Inc(JCP_Count);
                Inc(DISC_r_Count);
                if TempAnswer = '0' then
                  begin
                    BXD := BXD + 1;
                    RC4 := RC4 + 1;
                    JCP := JCP + 1;
                    DISC_r := DISC_r + 1;
                  end;

                if TempAnswer = '1' then
                  RBS := RBS + 1;
              end;
            if i = 22 then
              begin
                Inc(EID_Count);
                Inc(RCd_Count);
                Inc(TRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    EID := EID + 1;
                    RCd := RCd + 1;
                  end;

                if (TempAnswer = '1') and T4F_22F then
                  TRIN_r := TRIN_r - 1;
              end;
            if i = 23 then
              begin
                Inc(AGG_Count);
                Inc(RC7_Count);
                Inc(NEGE_r_Count);
                Inc(K_r_Count);
                if TempAnswer = '0' then
                  begin
                    RC7 := RC7 + 1;
                    AGG := AGG + 1;
                    NEGE_r := NEGE_r + 1;
                    CriticalItems := CriticalItems +
                      'At times I feel like smashing things. TRUE^';
                  end;

                if TempAnswer = '1' then
                  K_r := K_r + 1;
              end;
            if i = 24 then
              begin
                Inc(RBS_Count);
                Inc(IPP_Count);
                Inc(AGGR_r_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    RBS := RBS + 1;
                    IPP := IPP + 1;
                    V24T_319F := true;
                  end;

                if TempAnswer = '1' then
                  AGGR_r := AGGR_r + 1;
              end;
            if i = 25 then
              begin
                Inc(EID_Count);
                Inc(RC2_Count);
                Inc(MLS_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  V25T_174F := TRUE;

                if TempAnswer = '1' then
                  begin
                    EID := EID + 1;
                    RC2 := RC2 + 1;
                    MLS := MLS + 1;
                  end;
              end;
            if i = 26 then
              begin
                Inc(RBS_Count);
                Inc(RC9_Count);
                Inc(AGG_Count);
                Inc(AGGR_r_Count);
                if TempAnswer = '0' then
                  begin
                    RBS := RBS + 1;
                    RC9 := RC9 + 1;
                    AGG := AGG + 1;
                    AGGR_r := AGGR_r + 1;
                    CriticalItems := CriticalItems +
                      'When people do me a wrong, I feel I should pay them ' +
                      'back if I can, just for the principle of the thing. TRUE^';
                  end;
              end;
            if i = 27 then
              begin
                Inc(NFC_Count);
                if TempAnswer = '0' then
                  NFC := NFC + 1;
              end;
            if i = 28 then
              begin
                Inc(RC1_Count);
                if TempAnswer = '1' then
                  RC1 := RC1 + 1;
              end;
            if i = 29 then
              begin
                Inc(NEGE_r_Count);
                Inc(STW_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    STW := STW + 1;
                    NEGE_r := NEGE_r + 1;
                    V29T_116F := TRUE;
                  end;
              end;
            if i = 30 then
              begin
                Inc(EID_Count);
                Inc(RCd_Count);
                Inc(F_r_Count);
                Inc(TRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    F_r := F_r + 1;
                    EID := EID + 1;
                    RCd := RCd + 1;
                    T30T_217T := true;
                  end;
              end;
            if i = 31 then
              begin
                Inc(RBS_Count);
                Inc(COG_Count);
                if TempAnswer = '0' then
                  begin
                    RBS := RBS + 1;
                    COG := COG + 1;
                  end;
              end;
            if i = 32 then
              begin
                Inc(RC8_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    RC8 := RC8 + 1;
                    CriticalItems := CriticalItems +
                      'I have had very peculiar and strange experiences. TRUE^';
                  end;

                if TempAnswer = '1' then
                  V32F_257T := TRUE;
              end;
            if i = 33 then
              begin
                Inc(Fs_Count);
                if TempAnswer = '0' then
                  Fs := Fs + 1;
              end;
            if i = 34 then
              begin
                Inc(PSYC_r_Count);
                Inc(RC6_Count);
                if TempAnswer = '0' then
                  begin
                    RC6 := RC6 + 1;
                    PSYC_r := PSYC_r + 1;
                    CriticalItems := CriticalItems +
                      'Ghosts or spirits can influence people for good or bad. TRUE^';
                  end;
              end;
            if i = 35 then
              begin
                Inc(EID_Count);
                Inc(RC7_Count);
                Inc(SHY_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    EID := EID + 1;
                    RC7 := RC7 + 1;
                    SHY := SHY + 1;
                    V35T_114f := TRUE;
                  end;
              end;
            if i = 36 then
              begin
                Inc(FBS_r_Count);
                Inc(RC3_Count);
                Inc(K_r_Count);
                if TempAnswer = '0' then
                  RC3 := RC3 + 1;

                if TempAnswer = '1' then
                  begin
                    FBS_r := FBS_r + 1;
                    K_r := K_r + 1;
                  end;
              end;
            if i = 37 then
              begin
                Inc(EID_Count);
                Inc(NEGE_r_Count);
                Inc(TRIN_r_Count);

                if TempAnswer = '0' then
                  T37T_322T := true;

                if TempAnswer = '1' then
                  begin
                    EID := EID + 1;
                    NEGE_r := NEGE_r + 1;
                  end;
              end;
            if i = 38 then
              begin
                Inc(RC4_Count);
                if TempAnswer = '1' then
                  RC4 := RC4 + 1;
              end;
            if i = 39 then
              begin
                Inc(RC9_Count);
                Inc(AGGR_r_Count);
                if TempAnswer = '0' then
                  begin
                    RC9 := RC9 + 1;
                    AGGR_r := AGGR_r + 1;
                  end;
              end;
            if i = 40 then
              begin
                Inc(Fp_r_Count);
                Inc(PSYC_r_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    Fp_r := Fp_r + 1;
                    PSYC_r := PSYC_r + 1;
                    V40T_144F := TRUE;
                  end;
              end;
            if i = 41 then
              begin
                Inc(Fp_r_Count);
                Inc(AGG_Count);
                if TempAnswer = '0' then
                  begin
                    Fp_r := Fp_r + 1;
                    AGG := AGG + 1;
                    CriticalItems := CriticalItems +
                      'Sometimes I enjoy hurting persons I love. TRUE^';
                  end;
              end;
            if i = 42 then
              begin
                Inc(MEC_Count);
                Inc(DISC_r_Count);
                if TempAnswer = '0' then
                  begin
                    MEC := MEC + 1;
                    DISC_r := DISC_r + 1;
                  end;
              end;
            if i = 43 then
              begin
                Inc(Fs_Count);
                Inc(RC1_Count);
                Inc(GIC_Count);
                Inc(FBS_r_Count);
                if TempAnswer = '0' then
                  begin
                    Fs := Fs + 1;
                    FBS_r := FBS_r + 1;
                    RC1 := RC1 + 1;
                    GIC := GIC + 1;
                  end;
              end;
            if i = 44 then
              begin
                Inc(K_r_Count);
                Inc(SHY_Count);
                Inc(TRIN_r_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    SHY := SHY + 1;
                    V44T_177F := TRUE;
                    if T17T_44T then
                      TRIN_r := TRIN_r + 1;
                  end;

                if TempAnswer = '1' then
                  begin
                    K_r := K_r + 1;
                    V44F_177T := TRUE;
                    T44F_94F := true;
                  end;
              end;
            if i = 45 then
              begin
                Inc(FBS_r_Count);
                Inc(L_r_Count);
                if TempAnswer = '1' then
                  begin
                    FBS_r := FBS_r + 1;
                    L_r := L_r + 1;
                  end;
              end;
            if i = 46 then
              begin
                Inc(THD_Count);
                Inc(PSYC_r_Count);
                Inc(RC8_Count);
                Inc(F_r_Count);
                if TempAnswer = '0' then
                  begin
                    F_r := F_r + 1;
                    THD := THD + 1;
                    RC8 := RC8 + 1;
                    PSYC_r := PSYC_r + 1;
                    CriticalItems := CriticalItems +
                      'When I am with people, I am bothered by hearing very strange things. TRUE^';
                  end;
              end;
            if i = 47 then
              begin
                Inc(INTR_r_Count);
                Inc(SAV_Count);
                Inc(RC9_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    RC9 := RC9 + 1;
                    V47T_201F := true;
                  end;

                if TempAnswer = '1' then
                  begin
                    SAV := SAV + 1;
                    INTR_r := INTR_r + 1;
                  end;
              end;
            if i = 48 then
              begin
                Inc(EID_Count);
                Inc(RCd_Count);
                Inc(SFD_Count);
                Inc(TRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    EID := EID + 1;
                    RCd := RCd + 1;
                    SFD := SFD + 1;
                    T48T_182T := true;
                  end;
              end;
            if i = 49 then
              begin
                Inc(RC4_Count);
                Inc(BXD_Count);
                Inc(DISC_r_Count);
                Inc(SUB_Count);
                if TempAnswer = '0' then
                  begin
                    BXD := BXD + 1;
                    RC4 := RC4 + 1;
                    SUB := SUB + 1;
                    DISC_r := DISC_r + 1;
                    CriticalItems := CriticalItems +
                      'I have enjoyed using marijuana. TRUE^';
                  end;
              end;
            if i = 50 then
              begin
                Inc(AES_Count);
                if TempAnswer = '0' then
                  AES := AES + 1;
              end;
            if i = 51 then
              begin
                Inc(RC7_Count);
                if TempAnswer = '0' then
                  RC7 := RC7 + 1;
              end;
            if i = 52 then
              begin
                Inc(RC1_Count);
                if TempAnswer = '1' then
                  RC1 := RC1 + 1;
              end;
            if i = 53 then
              begin
                Inc(RC2_Count);
                Inc(INTR_r_Count);
                Inc(RBS_Count);
                if TempAnswer = '1' then
                  begin
                    RBS := RBS + 1;
                    RC2 := RC2 + 1;
                    INTR_r := INTR_r + 1;
                  end;
              end;
            if i = 54 then
              begin
                Inc(MSF_Count);
                if TempAnswer = '0' then
                  MSF := MSF + 1;
              end;
            if i = 55 then
              begin
                Inc(FBS_r_Count);
                Inc(RC3_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    RC3 := RC3 + 1;
                    V55T_99F := true;
                  end;
                if TempAnswer = '1' then
                  begin
                    FBS_r := FBS_r + 1;
                    V55F_171T := true;
                  end;
              end;
            if i = 56 then
              begin
                Inc(NEGE_r_Count);
                Inc(F_r_Count);
                Inc(BRF_Count);
                if TempAnswer = '0' then
                  begin
                    F_r := F_r + 1;
                    BRF := BRF + 1;
                    NEGE_r := NEGE_r + 1;
                  end;
              end;
            if i = 57 then
              begin
                Inc(EID_Count);
                Inc(INTR_r_Count);
                Inc(SAV_Count);
                Inc(TRIN_r_Count);

                if TempAnswer = '0' then
                  T57T_67T := true;

                if TempAnswer = '1' then
                  begin
                    EID := EID + 1;
                    SAV := SAV + 1;
                    INTR_r := INTR_r + 1;
                  end;
              end;
            if i = 58 then
              begin
                Inc(FML_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  FML := FML + 1;
                if TempAnswer = '1' then
                  V58F_180T := true;
              end;
            if i = 59 then
              begin
                Inc(RBS_Count);
                Inc(F_r_Count);
                Inc(COG_Count);
                Inc(TRIN_r_Count);

                if TempAnswer = '1' then
                  begin
                    F_r := F_r + 1;
                    RBS := RBS + 1;
                    COG := COG + 1;
                    T59F_306F := true;
                  end;
              end;
            if i = 60 then
              begin
                Inc(IPP_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '1' then
                  begin
                    IPP := IPP + 1;
                    V60F_327T := true;
                  end;
              end;
            if i = 61 then
              begin
                Inc(RC9_Count);
                Inc(L_r_Count);
                Inc(BXD_Count);
                Inc(DISC_r_Count);
                if TempAnswer = '0' then
                  L_r := L_r + 1;

                if TempAnswer = '1' then
                  begin
                    BXD := BXD + 1;
                    RC9 := RC9 + 1;
                    DISC_r := DISC_r + 1;
                  end;
              end;
            if i = 62 then
              begin
                Inc(RCd_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  RCd := RCd + 1;
                if TempAnswer = '1' then
                  V62F_74T := true;
              end;
            if i = 63 then
              begin
                Inc(RC7_Count);
                if TempAnswer = '0' then
                  RC7 := RC7 + 1;
              end;
            if i = 64 then
              begin
                Inc(EID_Count);
                Inc(RC2_Count);
                Inc(INTR_r_Count);
                if TempAnswer = '1' then
                  begin
                    EID := EID + 1;
                    RC2 := RC2 + 1;
                    INTR_r := INTR_r + 1;
                  end;
              end;
            if i = 65 then
              begin
                Inc(RC1_Count);
                if TempAnswer = '1' then
                  RC1 := RC1 + 1;
              end;
            if i = 66 then
              begin
                Inc(RC4_Count);
                Inc(BXD_Count);
                Inc(JCP_Count);
                Inc(DISC_r_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    BXD := BXD + 1;
                    RC4 := RC4 + 1;
                    JCP := JCP + 1;
                    DISC_r := DISC_r + 1;
                  end;
                if TempAnswer = '1' then
                  V66F_223T := true;
              end;
            if i = 67 then
              begin
                Inc(DSF_Count);
                Inc(F_r_Count);
                Inc(TRIN_r_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    F_r := F_r + 1;
                    DSF := DSF + 1;
                    V67T_278F := true;
                    if T57T_67T then
                      TRIN_r := TRIN_r + 1;
                  end;
              end;
            if i = 68 then
              begin
                Inc(RBS_Count);
                Inc(NFC_Count);
                if TempAnswer = '0' then
                  begin
                    RBS := RBS + 1;
                    NFC := NFC + 1;
                  end;
              end;
            if i = 69 then
              begin
                Inc(RC1_Count);
                Inc(NUC_Count);
                if TempAnswer = '1' then
                  begin
                    RC1 := RC1 + 1;
                    NUC := NUC + 1;
                  end;
              end;
            if i = 70 then
              begin
                Inc(L_r_Count);
                if TempAnswer = '1' then
                  L_r := L_r + 1;
              end;
            if i = 71 then
              begin
                Inc(THD_Count);
                Inc(PSYC_r_Count);
                Inc(F_r_Count);
                Inc(RC6_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    F_r := F_r + 1;
                    THD := THD + 1;
                    RC6 := RC6 + 1;
                    PSYC_r := PSYC_r + 1;
                    V71T_264F := true;
                    CriticalItems := CriticalItems +
                      'I believe I am being plotted against. TRUE^';
                  end;
              end;
            if i = 72 then
              begin
                Inc(K_r_Count);
                Inc(RC9_Count);
                Inc(ACT_Count);
                if TempAnswer = '0' then
                  begin
                    RC9 := RC9 + 1;
                    ACT := ACT + 1;
                  end;

                if TempAnswer = '1' then
                  K_r := K_r + 1;
              end;
            if i = 73 then
              begin
                Inc(EID_Count);
                Inc(NEGE_r_Count);
                Inc(STW_Count);
                Inc(TRIN_r_Count);

                if TempAnswer = '1' then
                  begin
                    EID := EID + 1;
                    STW := STW + 1;
                    NEGE_r := NEGE_r + 1;
                    T73F_338F := true;
                  end;
              end;
            if i = 74 then
              begin
                Inc(RCd_Count);
                Inc(RBS_Count);
                Inc(F_r_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    F_r := F_r + 1;
                    RBS := RBS + 1;
                    RCd := RCd + 1;
                    if V62F_74T then
                      VRIN_r := VRIN_r + 1;
                  end;
              end;
            if i = 75 then
              begin
                Inc(MEC_Count);
                Inc(DISC_r_Count);
                if TempAnswer = '0' then
                  begin
                    MEC := MEC + 1;
                    DISC_r := DISC_r + 1;
                  end;
              end;
            if i = 76 then
              begin
                Inc(RC1_Count);
                Inc(GIC_Count);
                Inc(FBS_r_Count);
                if TempAnswer = '0' then
                  begin
                    FBS_r := FBS_r + 1;
                    RC1 := RC1 + 1;
                    GIC := GIC + 1;
                  end;
              end;
            if i = 77 then
              begin
                Inc(FBS_r_Count);
                Inc(NEGE_r_Count);
                Inc(RC7_Count);
                if TempAnswer = '0' then
                  begin
                    FBS_r := FBS_r + 1;
                    RC7 := RC7 + 1;
                    NEGE_r := NEGE_r + 1;
                  end;
              end;
            if i = 78 then
              begin
                Inc(Fs_Count);
                if TempAnswer = '1' then
                  Fs := Fs + 1;
              end;
            if i = 79 then
              begin
                Inc(FBS_r_Count);
                Inc(RBS_Count);
                Inc(Fp_r_Count);
                Inc(AXY_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    Fp_r := Fp_r + 1;
                    FBS_r := FBS_r + 1;
                    RBS := RBS + 1;
                    AXY := AXY + 1;
                    V79T_289F := true;
                    CriticalItems := CriticalItems +
                      'I have nightmares every few nights. TRUE^';
                  end;
              end;
            if i = 80 then
              begin
                Inc(RC4_Count);
                Inc(K_r_Count);
                Inc(FML_Count);
                if TempAnswer = '0' then
                  K_r := K_r + 1;

                if TempAnswer = '1' then
                  begin
                    RC4 := RC4 + 1;
                    FML := FML + 1;
                  end;
              end;
            if i = 81 then
              begin
                Inc(ACT_Count);
                if TempAnswer = '0' then
                  ACT := ACT + 1;
              end;
            if i = 82 then
              begin
                Inc(MSF_Count);
                if TempAnswer = '1' then
                  MSF := MSF + 1;
              end;
            if i = 83 then
              begin
                Inc(EID_Count);
                Inc(RC2_Count);
                Inc(F_r_Count);
                Inc(TRIN_r_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  T83T_120T := true;

                if TempAnswer = '1' then
                  begin
                    F_r := F_r + 1;
                    EID := EID + 1;
                    RC2 := RC2 + 1;
                    V83F_105T := true;
                    T83F_158F := true;
                  end;
              end;
            if i = 84 then
              begin
                Inc(RC9_Count);
                Inc(BXD_Count);
                Inc(AGG_Count);
                Inc(AGGR_r_Count);
                if TempAnswer = '0' then
                  begin
                    BXD := BXD + 1;
                    RC9 := RC9 + 1;
                    AGG := AGG + 1;
                    AGGR_r := AGGR_r + 1;
                    CriticalItems := CriticalItems +
                      'At times I feel like picking a fist fight with someone. TRUE^';
                  end;
              end;
            if i = 85 then
              begin
                Inc(THD_Count);
                Inc(RC8_Count);
                Inc(PSYC_r_Count);
                if TempAnswer = '1' then
                  begin
                    THD := THD + 1;
                    RC8 := RC8 + 1;
                    PSYC_r := PSYC_r + 1;
                    CriticalItems := CriticalItems +
                      'I have never seen a vision. FALSE^';
                  end;
              end;
            if i = 86 then
              begin
                Inc(Fp_r_Count);
                Inc(SUB_Count);
                if TempAnswer = '0' then
                  begin
                    Fp_r := Fp_r + 1;
                    SUB := SUB + 1;
                    CriticalItems := CriticalItems +
                      'I can express my true feelings only when I drink. TRUE^';
                  end;
              end;
            if i = 87 then
              begin
                Inc(RC3_Count);
                if TempAnswer = '0' then
                  RC3 := RC3 + 1;
              end;
            if i = 88 then
              begin
                Inc(RC1_Count);
                Inc(FBS_r_Count);
                Inc(HPC_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '1' then
                  begin
                    FBS_r := FBS_r + 1;
                    RC1 := RC1 + 1;
                    HPC := HPC + 1;
                    V88F_265T := true;
                  end;
              end;
            if i = 89 then
              begin
                Inc(RCd_Count);
                Inc(EID_Count);
                Inc(K_r_Count);
                Inc(SFD_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    EID := EID + 1;
                    RCd := RCd + 1;
                    SFD := SFD + 1;
                  end;

                if TempAnswer = '1' then
                  begin
                    K_r := K_r + 1;
                    V89F_232T := true;
                  end;
              end;
            if i = 90 then
              begin
                Inc(BRF_Count);
                if TempAnswer = '0' then
                  BRF := BRF + 1;
              end;
            if i = 91 then
              begin
                Inc(EID_Count);
                Inc(RC7_Count);
                Inc(SHY_Count);
                if TempAnswer = '0' then
                  begin
                    EID := EID + 1;
                    RC7 := RC7 + 1;
                    SHY := SHY + 1;
                  end;
              end;
            if i = 92 then
              begin
                Inc(RBS_Count);
                Inc(THD_Count);
                Inc(PSYC_r_Count);
                Inc(RC6_Count);
                if TempAnswer = '0' then
                  begin
                    RBS := RBS + 1;
                    THD := THD + 1;
                    RC6 := RC6 + 1;
                    PSYC_r := PSYC_r + 1;
                    CriticalItems := CriticalItems +
                      'I believe I am being followed. TRUE^';
                  end;
              end;
            if i = 93 then
              begin
                Inc(FBS_r_Count);
                Inc(SUI_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    FBS_r := FBS_r + 1;
                    SUI := SUI + 1;
                    V93T_164F := true;
                    CriticalItems := CriticalItems +
                      'I have recently considered killing myself. TRUE ^';
                  end;

                if TempAnswer = '1' then
                  V93F_164T := true;
              end;
            if i = 94 then
              begin
                Inc(SAV_Count);
                Inc(VRIN_r_Count);
                Inc(TRIN_r_Count);

                if TempAnswer = '1' then
                  begin
                    SAV := SAV + 1;
                    if V17T_94F then
                      VRIN_r := VRIN_r + 1;
                    if T44F_94F then
                      TRIN_r := TRIN_r - 1;
                  end;
              end;
            if i = 95 then
              begin
                Inc(L_r_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '1' then
                  begin
                    L_r := L_r + 1;
                    V95F_144T := true;
                  end;
              end;
            if i = 96 then
              begin
                Inc(RC4_Count);
                Inc(BXD_Count);
                Inc(JCP_Count);
                if TempAnswer = '0' then
                  begin
                    BXD := BXD + 1;
                    RC4 := RC4 + 1;
                    JCP := JCP + 1;
                  end;
              end;
            if i = 97 then
              begin
                Inc(RC9_Count);
                if TempAnswer = '0' then
                  RC9 := RC9 + 1;
              end;
            if i = 98 then
              begin
                Inc(Fp_r_Count);
                if TempAnswer = '1' then
                  Fp_r := Fp_r + 1;
              end;
            if i = 99 then
              begin
                Inc(FBS_r_Count);
                Inc(RC3_Count);
                Inc(K_r_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  RC3 := RC3 + 1;

                if TempAnswer = '1' then
                  begin
                    FBS_r := FBS_r + 1;
                    K_r := K_r + 1;
                    if (V55T_99F or V55_Skip) then
                      VRIN_r := VRIN_r + 1;
                  end;
              end;
            if i = 100 then
              begin
                Inc(AES_Count);
                if TempAnswer = '0' then
                  AES := AES + 1;
              end;
            if i = 101 then
              begin
                Inc(RC1_Count);
                Inc(FBS_r_Count);
                Inc(RBS_Count);
                Inc(HPC_Count);
                Inc(TRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    FBS_r := FBS_r + 1;
                    RBS := RBS + 1;
                    RC1 := RC1 + 1;
                    HPC := HPC + 1;
                    T101T_189T := true;
                  end;
              end;
            if i = 102 then
              begin
                Inc(EID_Count);
                Inc(RC2_Count);
                Inc(INTR_r_Count);
                Inc(F_r_Count);
                Inc(COG_Count);
                if TempAnswer = '1' then
                  begin
                    F_r := F_r + 1;
                    EID := EID + 1;
                    RC2 := RC2 + 1;
                    COG := COG + 1;
                    INTR_r := INTR_r + 1;
                  end;
              end;
            if i = 103 then
              begin
                Inc(FML_Count);
                if TempAnswer = '0' then
                  FML := FML + 1;
              end;
            if i = 104 then
              begin
                Inc(IPP_Count);
                Inc(AGGR_r_Count);

                if TempAnswer = '0' then
                  AGGR_r := AGGR_r + 1;

                if TempAnswer = '1' then
                  IPP := IPP + 1;
              end;
            if i = 105 then
              begin
                Inc(EID_Count);
                Inc(RCd_Count);
                Inc(VRIN_r_Count);

                if (TempAnswer = '0') and V83F_105T then
                  VRIN_r := VRIN_r + 1;

                if TempAnswer = '1' then
                  begin
                    EID := EID + 1;
                    RCd := RCd + 1;
                  end;
              end;
            if i = 106 then
              begin
                Inc(RBS_Count);
                Inc(RC8_Count);
                Inc(F_r_Count);
                if TempAnswer = '0' then
                  begin
                    F_r := F_r + 1;
                    RBS := RBS + 1;
                    RC8 := RC8 + 1;
                    CriticalItems := CriticalItems +
                      'I have had periods in which I carried on activities without ' +
                      'knowing later what I had been doing. TRUE^';
                  end;
              end;
            if i = 107 then
              begin
                Inc(RC9_Count);
                Inc(BXD_Count);
                Inc(DISC_r_Count);
                if TempAnswer = '0' then
                  begin
                    BXD := BXD + 1;
                    RC9 := RC9 + 1;
                    DISC_r := DISC_r + 1;
                  end;
              end;
            if i = 108 then
              begin
                Inc(NFC_Count);
                Inc(TRIN_r_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    NFC := NFC + 1;
                    V108T_152F := true;
                  end;

                if TempAnswer = '1' then
                  begin
                    V108F_152T := true;
                    T108F_246F := true;
                  end;
              end;
            if i = 109 then
              begin
                Inc(INTR_r_Count);
                Inc(SAV_Count);
                Inc(TRIN_r_Count);

                if TempAnswer = '0' then
                  T109T_278T := true;

                if TempAnswer = '1' then
                  begin
                    SAV := SAV + 1;
                    INTR_r := INTR_r + 1;
                  end;
              end;
            if i = 110 then
              begin
                Inc(THD_Count);
                Inc(RC6_Count);
                if TempAnswer = '0' then
                  begin
                    THD := THD + 1;
                    RC6 := RC6 + 1;
                    CriticalItems := CriticalItems +
                      'I feel that I have often been punished without cause. TRUE^';
                  end;
              end;
            if i = 111 then
              begin
                Inc(MEC_Count);
                if TempAnswer = '0' then
                  MEC := MEC + 1;
              end;
            if i = 112 then
              begin
                Inc(RC7_Count);
                if TempAnswer = '0' then
                  RC7 := RC7 + 1;
              end;
            if i = 113 then
              begin
                Inc(RC1_Count);
                Inc(NUC_Count);
                if TempAnswer = '1' then
                  begin
                    RC1 := RC1 + 1;
                    NUC := NUC + 1;
                  end;
              end;
            if i = 114 then
              begin
                Inc(EID_Count);
                Inc(SHY_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    EID := EID + 1;
                    SHY := SHY + 1;
                  end;

                if (TempAnswer = '1') and V35T_114F then
                  VRIN_r := VRIN_r + 1;
              end;
            if i = 115 then
              begin
                Inc(DISC_r_Count);
                Inc(MSF_Count);
                if TempAnswer = '0' then
                  DISC_r := DISC_r + 1;

                if TempAnswer = '1' then
                  MSF := MSF + 1;
              end;
            if i = 116 then
              begin
                Inc(NEGE_r_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  NEGE_r := NEGE_r + 1;

                if (TempAnswer = '1') and V29T_116F then
                  VRIN_r := VRIN_r + 1;
              end;
            if i = 117 then
              begin
                Inc(RCd_Count);
                Inc(F_r_Count);
                if TempAnswer = '0' then
                  begin
                    F_r := F_r + 1;
                    RCd := RCd + 1;
                  end;
              end;
            if i = 118 then
              begin
                Inc(INTR_r_Count);
                Inc(RC9_Count);
                if TempAnswer = '0' then
                  RC9 := RC9 + 1;

                if TempAnswer = '1' then
                  INTR_r := INTR_r + 1;
              end;
            if i = 119 then
              begin
                Inc(EID_Count);
                Inc(RC7_Count);
                Inc(TRIN_r_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    EID := EID + 1;
                    RC7 := RC7 + 1;
                    ANP := ANP + 1;
                  end;

                if TempAnswer = '1' then
                  begin
                    V119F_303T := true;
                    T119F_134F := true;
                  end;
              end;
            if i = 120 then
              begin
                Inc(EID_Count);
                Inc(RBS_Count);
                Inc(F_r_Count);
                Inc(SUI_Count);
                Inc(TRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    F_r := F_r + 1;
                    RBS := RBS + 1;
                    EID := EID + 1;
                    SUI := SUI + 1;
                    if T83T_120T then
                      TRIN_r := TRIN_r + 1;
                    CriticalItems := CriticalItems +
                      'Most of the time I wish I were dead. TRUE^';
                  end;
              end;

            if i = 121 then
              begin
                Inc(RC3_Count);
                if TempAnswer = '0' then
                  RC3 := RC3 + 1;
              end;
            if i = 122 then
              begin
                Inc(Fs_Count);
                Inc(THD_Count);
                Inc(RC8_Count);
                Inc(NUC_Count);
                if TempAnswer = '0' then
                  begin
                    Fs := Fs + 1;
                    THD := THD + 1;
                    RC8 := RC8 + 1;
                    NUC := NUC + 1;
                    CriticalItems := CriticalItems +
                      'I have had attacks in which I could not control my ' +
                      'movements or speech but in which I knew what was going on around me. TRUE^';
                  end;
              end;
            if i = 123 then
              begin
                Inc(NEGE_r_Count);
                Inc(STW_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    STW := STW + 1;
                    NEGE_r := NEGE_r + 1;
                    V123T_338F := true;
                  end;
              end;
            if i = 124 then
              begin
                Inc(Fp_r_Count);
                Inc(DSF_Count);

                if TempAnswer = '0' then
                  begin
                    Fp_r := Fp_r + 1;
                    DSF := DSF + 1;
                  end;
              end;
            if i = 125 then
              begin
                Inc(RC1_Count);
                Inc(RBS_Count);
                Inc(NUC_Count);
                if TempAnswer = '1' then
                  begin
                    RBS := RBS + 1;
                    RC1 := RC1 + 1;
                    NUC := NUC + 1;
                  end;
              end;
            if i = 126 then
              begin
                Inc(RC4_Count);
                if TempAnswer = '1' then
                  RC4 := RC4 + 1;
              end;
            if i = 127 then
              begin
                Inc(L_r_Count);
                if TempAnswer = '1' then
                  L_r := L_r + 1;
              end;
            if i = 128 then
              begin
                Inc(BRF_Count);
                if TempAnswer = '1' then
                  BRF := BRF + 1;
              end;
            if i = 129 then
              begin
                Inc(Fp_r_Count);
                Inc(THD_Count);
                Inc(PSYC_r_Count);
                Inc(RC6_Count);
                if TempAnswer = '0' then
                  begin
                    Fp_r := Fp_r + 1;
                    THD := THD + 1;
                    RC6 := RC6 + 1;
                    PSYC_r := PSYC_r + 1;
                    CriticalItems := CriticalItems +
                      'Someone has been trying to poison me. TRUE^';
                  end;
              end;
            if i = 130 then
              begin
                Inc(RCd_Count);
                if TempAnswer = '0' then
                  RCd := RCd + 1;
              end;
            if i = 131 then
              begin
                Inc(RBS_Count);
                Inc(RC9_Count);
                Inc(BXD_Count);
                Inc(DISC_r_Count);
                if TempAnswer = '0' then
                  begin
                    BXD := BXD + 1;
                    RC9 := RC9 + 1;
                    DISC_r := DISC_r + 1;
                  end;

                if TempAnswer = '1' then
                  RBS := RBS + 1;
              end;
            if i = 132 then
              begin
                Inc(RC7_Count);
                Inc(RBS_Count);
                if TempAnswer = '0' then
                  begin
                    RBS := RBS + 1;
                    RC7 := RC7 + 1;
                  end;
              end;
            if i = 133 then
              begin
                Inc(Fs_Count);
                Inc(FBS_r_Count);
                if TempAnswer = '0' then
                  begin
                    Fs := Fs + 1;
                    FBS_r := FBS_r + 1;
                  end;
              end;
            if i = 134 then
              begin
                Inc(NEGE_r_Count);
                Inc(ANP_Count);
                Inc(ANP_Count);
                Inc(TRIN_r_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  T134T_248T := true;

                if TempAnswer = '1' then
                  begin
                    ANP := ANP + 1;
                    NEGE_r := NEGE_r + 1;
                    V134F_293T := true;
                    if T119F_134F then
                      TRIN_r := TRIN_r - 1;
                  end;
              end;
            if i = 135 then
              begin
                Inc(HLP_Count);
                if TempAnswer = '0' then
                  begin
                    HLP := HLP + 1;
                    CriticalItems := CriticalItems +
                      'It bothers me greatly to think of making changes in my life. TRUE^';
                  end;
              end;
            if i = 136 then
              begin
                Inc(COG_Count);
                Inc(RBS_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    RBS := RBS + 1;
                    COG := COG + 1;
                    if V6F_136T then
                      VRIN_r := VRIN_r + 1;
                  end;
              end;
            if i = 137 then
              begin
                Inc(Fs_Count);
                Inc(RC1_Count);
                Inc(RBS_Count);
                Inc(PSYC_r_Count);
                if TempAnswer = '0' then
                  begin
                    Fs := Fs + 1;
                    RBS := RBS + 1;
                    RC1 := RC1 + 1;
                    PSYC_r := PSYC_r + 1;
                  end;
              end;
            if i = 138 then
              begin
                Inc(FML_Count);
                if TempAnswer = '0' then
                  FML := FML + 1;
              end;
            if i = 139 then
              begin
                Inc(THD_Count);
                Inc(RC8_Count);
                Inc(PSYC_r_Count);
                Inc(F_r_Count);
                if TempAnswer = '0' then
                  begin
                    F_r := F_r + 1;
                    THD := THD + 1;
                    RC8 := RC8 + 1;
                    PSYC_r := PSYC_r + 1;
                    CriticalItems := CriticalItems +
                      'I often hear voices without knowing where they come from. TRUE^';
                  end;
              end;
            if i = 140 then
              begin
                Inc(EID_Count);
                Inc(RC2_Count);
                Inc(INTR_r_Count);
                if TempAnswer = '1' then
                  begin
                    EID := EID + 1;
                    RC2 := RC2 + 1;
                    INTR_r := INTR_r + 1;
                  end;
              end;
            if i = 141 then
              begin
                Inc(RC4_Count);
                Inc(FBS_r_Count);
                Inc(SUB_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    RC4 := RC4 + 1;
                    SUB := SUB + 1;
                    CriticalItems := CriticalItems +
                      'I have used alcohol excessively. TRUE^';
                  end;

                if TempAnswer = '1' then
                  begin
                    FBS_r := FBS_r + 1;
                    V141F_266T := true;
                  end;
              end;
            if i = 142 then
              begin
                Inc(RC3_Count);
                if TempAnswer = '0' then
                  RC3 := RC3 + 1;
              end;
            if i = 143 then
              begin
                Inc(RC9_Count);
                if TempAnswer = '0' then
                  RC9 := RC9 + 1;
              end;
            if i = 144 then
              begin
                Inc(RCd_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    RCd := RCd + 1;
                    if V95F_144T then
                      VRIN_r := VRIN_r + 1;
                  end;

                if (TempAnswer = '1') and V40T_144F then
                  VRIN_r := VRIN_r + 1;
              end;
            if i = 145 then
              begin
                Inc(AES_Count);
                if TempAnswer = '0' then
                  AES := AES + 1;
              end;
            if i = 146 then
              begin
                Inc(RC7_Count);
                Inc(NEGE_r_Count);
                Inc(F_r_Count);
                Inc(AXY_Count);
                Inc(TRIN_r_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    F_r := F_r + 1;
                    RC7 := RC7 + 1;
                    AXY := AXY + 1;
                    NEGE_r := NEGE_r + 1;
                    V146T_275F := true;
                    T146T_293T := true;
                    CriticalItems := CriticalItems +
                      'Almost every day something happens to frighten me. TRUE^';
                  end;
              end;
            if i = 147 then
              begin
                Inc(IPP_Count);
                Inc(AGGR_r_Count);
                Inc(TRIN_r_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  AGGR_r := AGGR_r + 1;

                if TempAnswer = '1' then
                  begin
                    IPP := IPP + 1;
                    V147F_276T := true;
                    T147F_319F := true;
                  end;
              end;
            if i = 148 then
              begin
                Inc(MEC_Count);
                if TempAnswer = '0' then
                  MEC := MEC + 1;
              end;
            if i = 149 then
              begin
                Inc(RC7_Count);
                if TempAnswer = '0' then
                  RC7 := RC7 + 1;
              end;
            if i = 150 then
              begin
                Inc(Fp_r_Count);
                Inc(THD_Count);
                Inc(PSYC_r_Count);
                Inc(RC6_Count);
                if TempAnswer = '0' then
                  begin
                    Fp_r := Fp_r + 1;
                    THD := THD + 1;
                    RC6 := RC6 + 1;
                    PSYC_r := PSYC_r + 1;
                    CriticalItems := CriticalItems +
                      'Someone has been trying to rob me. TRUE^';
                  end;
              end;
            if i = 151 then
              begin
                Inc(MSF_Count);
                if TempAnswer = '0' then
                  MSF := MSF + 1;
              end;
            if i = 152 then
              begin
                Inc(NFC_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    NFC := NFC + 1;
                    if V108F_152T then
                      VRIN_r := VRIN_r + 1;
                  end;

                if (TempAnswer = '1') and V108T_152F then
                  VRIN_r := VRIN_r + 1;
              end;
            if i = 153 then
              begin
                Inc(INTR_r_Count);
                Inc(SAV_Count);
                if TempAnswer = '1' then
                  begin
                    SAV := SAV + 1;
                    INTR_r := INTR_r + 1;
                  end;
              end;
            if i = 154 then
              begin
                Inc(L_r_Count);
                if TempAnswer = '1' then
                  L_r := L_r + 1;
              end;
            if i = 155 then
              begin
                Inc(NEGE_r_Count);
                Inc(K_r_Count);
                Inc(RC9_Count);
                Inc(ANP_Count);
                if TempAnswer = '0' then
                  begin
                    RC9 := RC9 + 1;
                    ANP := ANP + 1;
                    NEGE_r := NEGE_r + 1;
                  end;

                if TempAnswer = '1' then
                  K_r := K_r + 1;
              end;
            if i = 156 then
              begin
                Inc(RC4_Count);
                Inc(FBS_r_Count);
                Inc(RBS_Count);
                Inc(BXD_Count);
                Inc(DISC_r_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    BXD := BXD + 1;
                    RC4 := RC4 + 1;
                    DISC_r := DISC_r + 1;
                  end;

                if TempAnswer = '1' then
                  begin
                    FBS_r := FBS_r + 1;
                    RBS := RBS + 1;
                    V156F_205T := true;
                  end;
              end;
            if i = 157 then
              begin
                Inc(Fp_r_Count);
                if TempAnswer = '1' then
                  Fp_r := Fp_r + 1;
              end;
            if i = 158 then
              begin
                Inc(EID_Count);
                Inc(RCd_Count);
                Inc(TRIN_r_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    EID := EID + 1;
                    RCd := RCd + 1;
                    V158T_261F := true;
                    T158T_202T := true;
                  end;

                if TempAnswer = '1' then
                  begin
                    V158F_228T := true;
                    if T83F_158F then
                      TRIN_r := TRIN_r - 1;
                  end;
              end;
            if i = 159 then
              begin
                Inc(Fs_Count);
                Inc(RC8_Count);
                Inc(COG_Count);
                Inc(RBS_Count);
                if TempAnswer = '0' then
                  begin
                    Fs := Fs + 1;
                    RC8 := RC8 + 1;
                    COG := COG + 1;
                    RBS := RBS + 1;
                    CriticalItems := CriticalItems +
                      'I have had blank spells in which my activities were ' +
                      'interrupted and I did not know what was going on around me. TRUE^';
                  end;
              end;
            if i = 160 then
              begin
                Inc(RC2_Count);
                if TempAnswer = '1' then
                  RC2 := RC2 + 1;
              end;
            if i = 161 then
              begin
                Inc(RC7_Count);
                if TempAnswer = '0' then
                  RC7 := RC7 + 1;
              end;
            if i = 162 then
              begin
                Inc(RC1_Count);
                Inc(FBS_r_Count);
                Inc(NUC_Count);
                if TempAnswer = '1' then
                  begin
                    FBS_r := FBS_r + 1;
                    RC1 := RC1 + 1;
                    NUC := NUC + 1;
                  end;
              end;
            if i = 163 then
              begin
                Inc(MLS_Count);
                if TempAnswer = '1' then
                  MLS := MLS + 1;
              end;
            if i = 164 then
              begin
                Inc(F_r_Count);
                Inc(SUI_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    F_r := F_r + 1;
                    SUI := SUI + 1;
                    if V93F_164T then
                      VRIN_r := VRIN_r + 1;
                    CriticalItems := CriticalItems +
                      'Lately I have thought a lot about killing myself. TRUE^';
                  end;

                if (TempAnswer = '1') and V93T_164F then
                  VRIN_r := VRIN_r + 1;
              end;
            if i = 165 then
              begin
                Inc(BRF_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  BRF := BRF + 1;

                if TempAnswer = '1' then
                  V165F_317T := true;
              end;
            if i = 166 then
              begin
                Inc(INTR_r_Count);
                Inc(RC9_Count);
                Inc(ACT_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    RC9 := RC9 + 1;
                    ACT := ACT + 1;
                    V166T_207F := true;
                  end;

                if TempAnswer = '1' then
                  INTR_r := INTR_r + 1;
              end;
            if i = 167 then
              begin
                Inc(EID_Count);
                Inc(NEGE_r_Count);
                Inc(STW_Count);
                if TempAnswer = '0' then
                  begin
                    EID := EID + 1;
                    STW := STW + 1;
                    NEGE_r := NEGE_r + 1;
                  end;
              end;
            if i = 168 then
              begin
                Inc(Fp_r_Count);
                Inc(THD_Count);
                Inc(PSYC_r_Count);
                Inc(RC6_Count);
                if TempAnswer = '0' then
                  begin
                    Fp_r := Fp_r + 1;
                    THD := THD + 1;
                    RC6 := RC6 + 1;
                    PSYC_r := PSYC_r + 1;
                    CriticalItems := CriticalItems +
                      'There are persons who are trying to steal my thoughts and ideas. TRUE^';
                  end;
              end;
            if i = 169 then
              begin
                Inc(EID_Count);
                Inc(HLP_Count);
                Inc(TRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    EID := EID + 1;
                    HLP := HLP + 1;
                    T169T_282T := true;
                    CriticalItems := CriticalItems +
                      'The future seems hopeless to me. TRUE^';
                  end;
              end;
            if i = 170 then
              begin
                Inc(Fs_Count);
                if TempAnswer = '0' then
                  Fs := Fs + 1;
              end;
            if i = 171 then
              begin
                Inc(FBS_r_Count);
                Inc(RC3_Count);
                Inc(K_r_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    RC3 := RC3 + 1;
                    if (V55F_171T or V55_Skip) then
                      VRIN_r := VRIN_r + 1;
                  end;

                if TempAnswer = '1' then
                  begin
                    FBS_r := FBS_r + 1;
                    K_r := K_r + 1;
                  end;
              end;
            if i = 172 then
              begin
                Inc(EID_Count);
                Inc(RCd_Count);
                if TempAnswer = '0' then
                  begin
                    EID := EID + 1;
                    RCd := RCd + 1;
                  end;
              end;
            if i = 173 then
              begin
                Inc(RC4_Count);
                if TempAnswer = '0' then
                  RC4 := RC4 + 1;
              end;
            if i = 174 then
              begin
                Inc(RC1_Count);
                Inc(MLS_Count);
                Inc(F_r_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '1' then
                  begin
                    F_r := F_r + 1;
                    RC1 := RC1 + 1;
                    MLS := MLS + 1;
                    if V25T_174F then
                      VRIN_r := VRIN_r + 1;
                  end;
              end;
            if i = 175 then
              begin
                Inc(DSF_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    DSF := DSF + 1;
                    V175T_291F := true;
                  end;

                if TempAnswer = '1' then
                  V175F_291T := true;
              end;
            if i = 176 then
              begin
                Inc(RC1_Count);
                Inc(HPC_Count);
                if TempAnswer = '0' then
                  begin
                    RC1 := RC1 + 1;
                    HPC := HPC + 1;
                  end;
              end;
            if i = 177 then
              begin
                Inc(SHY_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    SHY := SHY + 1;
                    if V44F_177T then
                      VRIN_r := VRIN_r + 1;
                  end;

                if (TempAnswer = '1') and V44T_177F then
                  VRIN_r := VRIN_r + 1;
              end;
            if i = 178 then
              begin
                Inc(Fp_r_Count);
                Inc(TRIN_r_Count);

                if TempAnswer = '0' then
                  Fp_r := Fp_r + 1;

                if TempAnswer = '1' then
                  T178F_221F := true;
              end;
            if i = 179 then
              begin
                Inc(THD_Count);
                Inc(RC8_Count);
                Inc(PSYC_r_Count);
                if TempAnswer = '0' then
                  begin
                    THD := THD + 1;
                    RC8 := RC8 + 1;
                    PSYC_r := PSYC_r + 1;
                    CriticalItems := CriticalItems +
                      'Sometimes my voice leaves me or changes even though I have no cold. TRUE^';
                  end;
              end;
            if i = 180 then
              begin
                Inc(FML_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    FML := FML + 1;
                    if V58F_180T then
                      VRIN_r := VRIN_r + 1;
                  end;
              end;
            if i = 181 then
              begin
                Inc(INTR_r_Count);
                Inc(RC9_Count);
                Inc(ACT_Count);
                if TempAnswer = '0' then
                  begin
                    RC9 := RC9 + 1;
                    ACT := ACT + 1;
                  end;

                if TempAnswer = '1' then
                  INTR_r := INTR_r + 1;
              end;
            if i = 182 then
              begin
                Inc(RC2_Count);
                Inc(L_r_Count);
                Inc(AGGR_r_Count);
                Inc(TRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    L_r := L_r + 1;
                    AGGR_r := AGGR_r + 1;
                    if T48T_182T then
                      TRIN_r := TRIN_r + 1;
                  end;

                if TempAnswer = '1' then
                  RC2 := RC2 + 1;
              end;
            if i = 183 then
              begin
                Inc(L_r_Count);
                if TempAnswer = '1' then
                  L_r := L_r + 1;
              end;
            if i = 184 then
              begin
                Inc(MSF_Count);
                if TempAnswer = '1' then
                  MSF := MSF + 1;
              end;
            if i = 185 then
              begin
                Inc(RC3_Count);
                if TempAnswer = '0' then
                  RC3 := RC3 + 1;
              end;
            if i = 186 then
              begin
                Inc(Fs_Count);
                Inc(NUC_Count);
                if TempAnswer = '1' then
                  begin
                    Fs := Fs + 1;
                    NUC := NUC + 1;
                  end;
              end;
            if i = 187 then
              begin
                Inc(EID_Count);
                Inc(FBS_r_Count);
                Inc(RCd_Count);
                Inc(K_r_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    FBS_r := FBS_r + 1;
                    EID := EID + 1;
                    RCd := RCd + 1;
                  end;

                if TempAnswer = '1' then
                  begin
                    K_r := K_r + 1;
                    V187F_331T := true;
                  end;
              end;
            if i = 188 then
              begin
                Inc(MEC_Count);
                if TempAnswer = '0' then
                  MEC := MEC + 1;
              end;
            if i = 189 then
              begin
                Inc(RC1_Count);
                Inc(FBS_r_Count);
                Inc(HPC_Count);
                Inc(TRIN_r_Count);

                if (TempAnswer = '0') and T101T_189T then
                  TRIN_r := TRIN_r + 1;

                if TempAnswer = '1' then
                  begin
                    FBS_r := FBS_r + 1;
                    RC1 := RC1 + 1;
                    HPC := HPC + 1;
                  end;
              end;
            if i = 190 then
              begin
                Inc(RC4_Count);
                Inc(BXD_Count);
                Inc(DISC_r_Count);
                if TempAnswer = '1' then
                  begin
                    BXD := BXD + 1;
                    RC4 := RC4 + 1;
                    DISC_r := DISC_r + 1;
                  end;
              end;
            if i = 191 then
              begin
                Inc(Fp_r_Count);
                if TempAnswer = '0' then
                  Fp_r := Fp_r + 1;
              end;
            if i = 192 then
              begin
                Inc(SUB_Count);
                if TempAnswer = '0' then
                  begin
                    SUB := SUB + 1;
                    CriticalItems := CriticalItems +
                      'After a bad day, I usually need a few drinks to relax. TRUE^';
                  end;
              end;
            if i = 193 then
              begin
                Inc(FBS_r_Count);
                Inc(RC9_Count);
                Inc(BXD_Count);
                Inc(DISC_r_Count);
                if TempAnswer = '0' then
                  begin
                    BXD := BXD + 1;
                    RC9 := RC9 + 1;
                    DISC_r := DISC_r + 1;
                  end;

                if TempAnswer = '1' then
                  FBS_r := FBS_r + 1;
              end;
            if i = 194 then
              begin
                Inc(RC6_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    RC6 := RC6 + 1;
                    CriticalItems := CriticalItems +
                      'I am sure I am being talked about. TRUE^';
                  end;

                if TempAnswer = '1' then
                  V194F_233T := true;
              end;
            if i = 195 then
              begin
                Inc(RC2_Count);
                Inc(INTR_r_Count);
                if TempAnswer = '1' then
                  begin
                    RC2 := RC2 + 1;
                    INTR_r := INTR_r + 1;
                  end;
              end;
            if i = 196 then
              begin
                Inc(AES_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  AES := AES + 1;

                if TempAnswer = '1' then
                  V196F_245T := true;
              end;
            if i = 197 then
              begin
                Inc(IPP_Count);
                Inc(AGGR_r_Count);
                if TempAnswer = '0' then
                  AGGR_r := AGGR_r + 1;

                if TempAnswer = '1' then
                  IPP := IPP + 1;
              end;
            if i = 198 then
              begin
                Inc(NFC_Count);
                if TempAnswer = '0' then
                  NFC := NFC + 1;
              end;
            if i = 199 then
              begin
                Inc(THD_Count);
                Inc(Fs_Count);
                Inc(RC8_Count);
                Inc(PSYC_r_Count);
                if TempAnswer = '0' then
                  begin
                    Fs := Fs + 1;
                    THD := THD + 1;
                    RC8 := RC8 + 1;
                    PSYC_r := PSYC_r + 1;
                    CriticalItems := CriticalItems +
                      'Peculiar odors come to me at times. TRUE^';
                  end;
              end;
            if i = 200 then
              begin
                Inc(FBS_r_Count);
                Inc(COG_Count);
                if TempAnswer = '0' then
                  begin
                    FBS_r := FBS_r + 1;
                    COG := COG + 1;
                  end;
              end;
            if i = 201 then
              begin
                Inc(INTR_r_Count);
                Inc(SAV_Count);
                Inc(TRIN_r_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '1' then
                  begin
                    SAV := SAV + 1;
                    INTR_r := INTR_r + 1;
                    if V47T_201F then
                      VRIN_r := VRIN_r + 1;
                    T201F_278F := true;
                  end;
              end;
            if i = 202 then
              begin
                Inc(EID_Count);
                Inc(RC2_Count);
                Inc(MLS_Count);
                Inc(K_r_Count);
                Inc(TRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    K_r := K_r + 1;
                    if T158T_202T then
                      TRIN_r := TRIN_r + 1;
                  end;

                if TempAnswer = '1' then
                  begin
                    EID := EID + 1;
                    RC2 := RC2 + 1;
                    MLS := MLS + 1;
                  end;
              end;
            if i = 203 then
              begin
                Inc(THD_Count);
                Inc(RC8_Count);
                Inc(PSYC_r_Count);
                Inc(F_r_Count);
                if TempAnswer = '0' then
                  begin
                    F_r := F_r + 1;
                    THD := THD + 1;
                    RC8 := RC8 + 1;
                    PSYC_r := PSYC_r + 1;
                    CriticalItems := CriticalItems +
                      'My soul sometimes leaves my body. TRUE^';
                  end;
              end;
            if i = 204 then
              begin
                Inc(EID_Count);
                Inc(RCd_Count);
                if TempAnswer = '0' then
                  begin
                    EID := EID + 1;
                    RCd := RCd + 1;
                  end;
              end;
            if i = 205 then
              begin
                Inc(RC4_Count);
                Inc(BXD_Count);
                Inc(JCP_Count);
                Inc(DISC_r_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    BXD := BXD + 1;
                    RC4 := RC4 + 1;
                    JCP := JCP + 1;
                    DISC_r := DISC_r + 1;
                    if V156F_205T then
                      VRIN_r := VRIN_r + 1;
                  end;
              end;
            if i = 206 then
              begin
                Inc(RC7_Count);
                Inc(NEGE_r_Count);
                if TempAnswer = '0' then
                  begin
                    RC7 := RC7 + 1;
                    NEGE_r := NEGE_r + 1;
                  end;
              end;
            if i = 207 then
              begin
                Inc(INTR_r_Count);
                Inc(RC9_Count);
                Inc(ACT_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    RC9 := RC9 + 1;
                    ACT := ACT + 1;
                  end;

                if TempAnswer = '1' then
                  begin
                    INTR_r := INTR_r + 1;
                    if V166T_207F then
                      VRIN_r := VRIN_r + 1;
                  end;
              end;
            if i = 208 then
              begin
                Inc(Fp_r_Count);
                Inc(BRF_Count);
                if TempAnswer = '0' then
                  begin
                    Fp_r := Fp_r + 1;
                    BRF := BRF + 1;
                  end;
              end;
            if i = 209 then
              begin
                Inc(NEGE_r_Count);
                if TempAnswer = '0' then
                  NEGE_r := NEGE_r + 1;
              end;
            if i = 210 then
              begin
                Inc(GIC_Count);
                Inc(FBS_r_Count);
                if TempAnswer = '0' then
                  begin
                    FBS_r := FBS_r + 1;
                    GIC := GIC + 1;
                  end;
              end;
            if i = 211 then
              begin
                Inc(L_r_Count);
                if TempAnswer = '1' then
                  L_r := L_r + 1;
              end;
            if i = 212 then
              begin
                Inc(THD_Count);
                Inc(RC6_Count);
                Inc(TRIN_r_Count);

                if TempAnswer = '0' then
                  T212T_264T := true;

                if TempAnswer = '1' then
                  begin
                    THD := THD + 1;
                    RC6 := RC6 + 1;
                    CriticalItems := CriticalItems +
                      'I have no enemies who really wish to harm me. FALSE^';
                  end;
              end;
            if i = 213 then
              begin
                Inc(RC3_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  RC3 := RC3 + 1;

                if TempAnswer = '1' then
                  V213F_260T := true;
              end;
            if i = 214 then
              begin
                Inc(HLP_Count);
                if TempAnswer = '0' then
                  begin
                    HLP := HLP + 1;
                    CriticalItems := CriticalItems +
                      'Although I am not happy with my life, there is nothing I can do about it now. TRUE^';
                  end;
              end;
            if i = 215 then
              begin
                Inc(FML_Count);
                if TempAnswer = '0' then
                  FML := FML + 1;
              end;
            if i = 216 then
              begin
                Inc(Fs_Count);
                Inc(THD_Count);
                Inc(RC8_Count);
                Inc(PSYC_r_Count);
                if TempAnswer = '0' then
                  begin
                    Fs := Fs + 1;
                    THD := THD + 1;
                    RC8 := RC8 + 1;
                    PSYC_r := PSYC_r + 1;
                    CriticalItems := CriticalItems +
                      'At times I hear so well it bothers me. TRUE^';
                  end;
              end;
            if i = 217 then
              begin
                Inc(EID_Count);
                Inc(RCd_Count);
                Inc(TRIN_r_Count);

                if (TempAnswer = '0') and T30T_217T then
                  TRIN_r := TRIN_r + 1;

                if TempAnswer = '1' then
                  begin
                    EID := EID + 1;
                    RCd := RCd + 1;
                  end;
              end;
            if i = 218 then
              begin
                Inc(RC4_Count);
                Inc(F_r_Count);
                if TempAnswer = '0' then
                  begin
                    F_r := F_r + 1;
                    RC4 := RC4 + 1;
                  end;
              end;
            if i = 219 then
              begin
                Inc(RBS_Count);
                Inc(RC9_Count);
                Inc(ACT_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    RC9 := RC9 + 1;
                    ACT := ACT + 1;
                  end;

                if TempAnswer = '1' then
                  begin
                    RBS := RBS + 1;
                    V219F_267T := true;
                  end;
              end;
            if i = 220 then
              begin
                Inc(MSF_Count);
                if TempAnswer = '1' then
                  MSF := MSF + 1;
              end;
            if i = 221 then
              begin
                Inc(Fp_r_Count);
                Inc(TRIN_r_Count);

                if TempAnswer = '1' then
                  begin
                    Fp_r := Fp_r + 1;
                    if T178F_221F then
                      TRIN_r := TRIN_r - 1;
                  end;
              end;
            if i = 222 then
              begin
                Inc(EID_Count);
                Inc(RC2_Count);
                Inc(INTR_r_Count);
                Inc(SAV_Count);
                if TempAnswer = '1' then
                  begin
                    EID := EID + 1;
                    RC2 := RC2 + 1;
                    SAV := SAV + 1;
                    INTR_r := INTR_r + 1;
                  end;
              end;
            if i = 223 then
              begin
                Inc(RC4_Count);
                Inc(BXD_Count);
                Inc(JCP_Count);
                Inc(DISC_r_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    BXD := BXD + 1;
                    RC4 := RC4 + 1;
                    JCP := JCP + 1;
                    DISC_r := DISC_r + 1;
                    if V66F_223T then
                      VRIN_r := VRIN_r + 1;
                  end;
              end;
            if i = 224 then
              begin
                Inc(STW_Count);
                if TempAnswer = '0' then
                  STW := STW + 1;
              end;
            if i = 225 then
              begin
                Inc(Fs_Count);
                if TempAnswer = '0' then
                  Fs := Fs + 1;
              end;
            if i = 226 then
              begin
                Inc(MEC_Count);
                Inc(BXD_Count);
                Inc(DISC_r_Count);
                if TempAnswer = '0' then
                  begin
                    BXD := BXD + 1;
                    MEC := MEC + 1;
                    DISC_r := DISC_r + 1;
                  end;
              end;
            if i = 227 then
              begin
                Inc(RC1_Count);
                Inc(F_r_Count);
                Inc(NUC_Count);
                if TempAnswer = '1' then
                  begin
                    F_r := F_r + 1;
                    RC1 := RC1 + 1;
                    NUC := NUC + 1;
                  end;
              end;
            if i = 228 then
              begin
                Inc(EID_Count);
                Inc(RC7_Count);
                Inc(AXY_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    EID := EID + 1;
                    RC7 := RC7 + 1;
                    AXY := AXY + 1;
                    if (V158F_228T or V158_Skip) then
                      VRIN_r := VRIN_r + 1;
                    CriticalItems := CriticalItems +
                      'I feel anxiety about something or someone almost all the time. TRUE^';
                  end;
              end;
            if i = 229 then
              begin
                Inc(NFC_Count);
                if TempAnswer = '0' then
                  NFC := NFC + 1;
              end;
            if i = 230 then
              begin
                Inc(RC1_Count);
                Inc(GIC_Count);
                Inc(FBS_r_Count);
                if TempAnswer = '0' then
                  begin
                    FBS_r := FBS_r + 1;
                    RC1 := RC1 + 1;
                    GIC := GIC + 1;
                  end;
              end;
            if i = 231 then
              begin
                Inc(F_r_Count);
                Inc(RC9_Count);
                Inc(BXD_Count);
                Inc(AGG_Count);
                Inc(AGGR_r_Count);
                if TempAnswer = '0' then
                  begin
                    F_r := F_r + 1;
                    BXD := BXD + 1;
                    RC9 := RC9 + 1;
                    AGG := AGG + 1;
                    AGGR_r := AGGR_r + 1;
                    CriticalItems := CriticalItems +
                      'I can easily make other people afraid of me, and sometimes do for the fun of it. TRUE^';
                  end;
              end;
            if i = 232 then
              begin
                Inc(EID_Count);
                Inc(RCd_Count);
                Inc(SFD_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    EID := EID + 1;
                    RCd := RCd + 1;
                    SFD := SFD + 1;
                    if V89F_232T then
                      VRIN_r := VRIN_r + 1;
                  end;
              end;
            if i = 233 then
              begin
                Inc(RC6_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    RC6 := RC6 + 1;
                    if V194F_233T then
                      VRIN_r := VRIN_r + 1;
                    CriticalItems := CriticalItems +
                      'People say insulting and vulgar things about me. TRUE^';
                  end;
              end;
            if i = 234 then
              begin
                Inc(EID_Count);
                Inc(FBS_r_Count);
                Inc(NEGE_r_Count);
                Inc(STW_Count);
                Inc(TRIN_r_Count);

                if TempAnswer = '0' then
                  T234T_261T := true;

                if TempAnswer = '1' then
                  begin
                    FBS_r := FBS_r + 1;
                    EID := EID + 1;
                    STW := STW + 1;
                    NEGE_r := NEGE_r + 1;
                  end;
              end;
            if i = 235 then
              begin
                Inc(RC7_Count);
                if TempAnswer = '0' then
                  RC7 := RC7 + 1;
              end;
            if i = 236 then
              begin
                Inc(DSF_Count);
                if TempAnswer = '0' then
                  DSF := DSF + 1;
              end;
            if i = 237 then
              begin
                Inc(RC4_Count);
                Inc(BXD_Count);
                Inc(DISC_r_Count);
                Inc(SUB_Count);
                if TempAnswer = '1' then
                  begin
                    BXD := BXD + 1;
                    RC4 := RC4 + 1;
                    SUB := SUB + 1;
                    DISC_r := DISC_r + 1;
                    CriticalItems := CriticalItems +
                      'Except by doctor''s orders I never take drugs or sleeping pills. FALSE^';
                  end;
              end;
            if i = 238 then
              begin
                Inc(RC3_Count);
                if TempAnswer = '0' then
                  RC3 := RC3 + 1;
              end;
            if i = 239 then
              begin
                Inc(IPP_Count);
                Inc(AGGR_r_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  AGGR_r := AGGR_r + 1;

                if TempAnswer = '1' then
                  begin
                    IPP := IPP + 1;
                    V239F_302T := true;
                  end;
              end;
            if i = 240 then
              begin
                Inc(RC8_Count);
                Inc(PSYC_r_Count);
                Inc(F_r_Count);
                Inc(COG_Count);
                if TempAnswer = '0' then
                  begin
                    F_r := F_r + 1;
                    RC8 := RC8 + 1;
                    COG := COG + 1;
                    PSYC_r := PSYC_r + 1;
                    CriticalItems := CriticalItems +
                      'I often feel as if things are not real. TRUE^';
                  end;
              end;
            if i = 241 then
              begin
                Inc(L_r_Count);
                if TempAnswer = '1' then
                  L_r := L_r + 1;
              end;
            if i = 242 then
              begin
                Inc(RC1_Count);
                Inc(RBS_Count);
                if TempAnswer = '0' then
                  begin
                    RBS := RBS + 1;
                    RC1 := RC1 + 1;
                  end;
              end;
            if i = 243 then
              begin
                Inc(BRF_Count);
                if TempAnswer = '0' then
                  BRF := BRF + 1;
              end;
            if i = 244 then
              begin
                Inc(RC9_Count);
                if TempAnswer = '0' then
                  RC9 := RC9 + 1;
              end;
            if i = 245 then
              begin
                Inc(AES_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    AES := AES + 1;
                    if V196F_245T then
                      VRIN_r := VRIN_r + 1;
                  end;
              end;
            if i = 246 then
              begin
                Inc(EID_Count);
                Inc(RC2_Count);
                Inc(INTR_r_Count);
                Inc(TRIN_r_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '1' then
                  begin
                    EID := EID + 1;
                    RC2 := RC2 + 1;
                    INTR_r := INTR_r + 1;
                    V246F_282T := true;
                    if T108F_246F then
                      TRIN_r := TRIN_r - 1;
                  end;
              end;
            if i = 247 then
              begin
                Inc(FBS_r_Count);
                Inc(RCd_Count);
                if TempAnswer = '0' then
                  begin
                    FBS_r := FBS_r + 1;
                    RCd := RCd + 1;
                  end;
              end;
            if i = 248 then
              begin
                Inc(RC9_Count);
                Inc(BXD_Count);
                Inc(ANP_Count);
                Inc(TRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    BXD := BXD + 1;
                    RC9 := RC9 + 1;
                    ANP := ANP + 1;
                    if T134T_248T then
                      TRIN_r := TRIN_r + 1;
                  end;
              end;
            if i = 249 then
              begin
                Inc(SHY_Count);
                if TempAnswer = '0' then
                  SHY := SHY + 1;
              end;
            if i = 250 then
              begin
                Inc(EID_Count);
                Inc(RC7_Count);
                if TempAnswer = '0' then
                  begin
                    EID := EID + 1;
                    RC7 := RC7 + 1;
                  end;
              end;
            if i = 251 then
              begin
                Inc(SUI_Count);
                if TempAnswer = '0' then
                  begin
                    SUI := SUI + 1;
                    CriticalItems := CriticalItems +
                      'No one knows it but I have tried to kill myself. TRUE^';
                  end;
              end;
            if i = 252 then
              begin
                Inc(RBS_Count);
                Inc(Fp_r_Count);
                Inc(THD_Count);
                Inc(PSYC_r_Count);
                Inc(RC6_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    Fp_r := Fp_r + 1;
                    RBS := RBS + 1;
                    THD := THD + 1;
                    RC6 := RC6 + 1;
                    PSYC_r := PSYC_r + 1;
                    V252T_287F := true;
                    CriticalItems := CriticalItems +
                      'Someone has control over my mind. TRUE^';
                  end;
              end;
            if i = 253 then
              begin
                Inc(RC4_Count);
                Inc(F_r_Count);
                Inc(BXD_Count);
                Inc(JCP_Count);
                Inc(DISC_r_Count);
                if TempAnswer = '0' then
                  begin
                    F_r := F_r + 1;
                    BXD := BXD + 1;
                    RC4 := RC4 + 1;
                    JCP := JCP + 1;
                    DISC_r := DISC_r + 1;
                  end;
              end;
            if i = 254 then
              begin
                Inc(RC1_Count);
                if TempAnswer = '1' then
                  RC1 := RC1 + 1;
              end;
            if i = 255 then
              begin
                Inc(Fp_r_Count);
                if TempAnswer = '0' then
                  Fp_r := Fp_r + 1;
              end;
            if i = 256 then
              begin
                Inc(RC9_Count);
                Inc(AGGR_r_Count);
                if TempAnswer = '0' then
                  begin
                    RC9 := RC9 + 1;
                    AGGR_r := AGGR_r + 1;
                  end;
              end;
            if i = 257 then
              begin
                Inc(RC8_Count);
                Inc(COG_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    RC8 := RC8 + 1;
                    COG := COG + 1;
                    if V32F_257T then
                      VRIN_r := VRIN_r + 1;
                    CriticalItems := CriticalItems +
                      'I have strange and peculiar thoughts. TRUE^';
                  end;
              end;
            if i = 258 then
              begin
                Inc(MSF_Count);
                if TempAnswer = '0' then
                  MSF := MSF + 1;
              end;
            if i = 259 then
              begin
                Inc(MEC_Count);
                if TempAnswer = '0' then
                  MEC := MEC + 1;
              end;
            if i = 260 then
              begin
                Inc(RC3_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    RC3 := RC3 + 1;
                    if V213F_260T then
                      VRIN_r := VRIN_r + 1;
                  end;
              end;
            if i = 261 then
              begin
                Inc(EID_Count);
                Inc(FBS_r_Count);
                Inc(RCd_Count);
                Inc(VRIN_r_Count);
                Inc(TRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    FBS_r := FBS_r + 1;
                    EID := EID + 1;
                    RCd := RCd + 1;
                    if T234T_261T then
                      TRIN_r := TRIN_r + 1;
                  end;

                if TempAnswer = '1' then
                  begin
                    if V158T_261F then
                      VRIN_r := VRIN_r + 1;
                    T261F_293F := true;
                  end;
              end;
            if i = 262 then
              begin
                Inc(MLS_Count);
                if TempAnswer = '1' then
                  MLS := MLS + 1;
              end;
            if i = 263 then
              begin
                Inc(RC7_Count);
                Inc(NEGE_r_Count);
                if TempAnswer = '0' then
                  begin
                    RC7 := RC7 + 1;
                    NEGE_r := NEGE_r + 1;
                  end;
              end;
            if i = 264 then
              begin
                Inc(THD_Count);
                Inc(PSYC_r_Count);
                Inc(F_r_Count);
                Inc(RC6_Count);
                Inc(VRIN_r_Count);
                Inc(TRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    F_r := F_r + 1;
                    THD := THD + 1;
                    RC6 := RC6 + 1;
                    PSYC_r := PSYC_r + 1;
                    if T212T_264T then
                      TRIN_r := TRIN_r + 1;
                    CriticalItems := CriticalItems +
                      'Someone has it in for me. TRUE^';
                  end;

                if (TempAnswer = '1') and V71T_264F then
                  VRIN_r := VRIN_r + 1;
              end;
            if i = 265 then
              begin
                Inc(RC1_Count);
                Inc(FBS_r_Count);
                Inc(HPC_Count);
                Inc(VRIN_r_Count);

                if (TempAnswer = '0') and V88F_265T then
                  VRIN_r := VRIN_r + 1;

                if TempAnswer = '1' then
                  begin
                    FBS_r := FBS_r + 1;
                    RC1 := RC1 + 1;
                    HPC := HPC + 1;
                  end;
              end;
            if i = 266 then
              begin
                Inc(RC4_Count);
                Inc(BXD_Count);
                Inc(SUB_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    BXD := BXD + 1;
                    RC4 := RC4 + 1;
                    SUB := SUB + 1;
                    if V141F_266T then
                      VRIN_r := VRIN_r + 1;
                    CriticalItems := CriticalItems +
                      'I have a drug or alcohol problem. TRUE^';
                  end;
              end;
            if i = 267 then
              begin
                Inc(RC9_Count);
                Inc(ACT_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    RC9 := RC9 + 1;
                    ACT := ACT + 1;
                    if V219F_267T then
                      VRIN_r := VRIN_r + 1;
                  end;
              end;
            if i = 268 then
              begin
                Inc(RBS_Count);
                Inc(L_r_Count);
                if TempAnswer = '0' then
                  begin
                    RBS := RBS + 1;
                    L_r := L_r + 1;
                  end;
              end;
            if i = 269 then
              begin
                Inc(FML_Count);
                Inc(TRIN_r_Count);

                if TempAnswer = '0' then
                  T269T_314T := true;

                if TempAnswer = '1' then
                  FML := FML + 1;
              end;
            if i = 270 then
              begin
                Inc(Fp_r_Count);
                Inc(THD_Count);
                Inc(PSYC_r_Count);
                Inc(RC6_Count);
                if TempAnswer = '0' then
                  begin
                    Fp_r := Fp_r + 1;
                    THD := THD + 1;
                    RC6 := RC6 + 1;
                    PSYC_r := PSYC_r + 1;
                    CriticalItems := CriticalItems +
                      'At one or more times in my life I felt that someone was making me do things by hypnotizing me. TRUE^';
                  end;
              end;
            if i = 271 then
              begin
                Inc(NFC_Count);
                if TempAnswer = '0' then
                  NFC := NFC + 1;
              end;
            if i = 272 then
              begin
                Inc(Fs_Count);
                if TempAnswer = '1' then
                  Fs := Fs + 1;
              end;
            if i = 273 then
              begin
                Inc(RBS_Count);
                Inc(THD_Count);
                Inc(RC8_Count);
                if TempAnswer = '0' then
                  begin
                    RBS := RBS + 1;
                    THD := THD + 1;
                    RC8 := RC8 + 1;
                    CriticalItems := CriticalItems +
                      'I hear strange things when I am alone. TRUE^';
                  end;
              end;
            if i = 274 then
              begin
                Inc(RCd_Count);
                Inc(NFC_Count);
                if TempAnswer = '0' then
                  begin
                    RCd := RCd + 1;
                    NFC := NFC + 1;
                  end;
              end;
            if i = 275 then
              begin
                Inc(RC7_Count);
                Inc(F_r_Count);
                Inc(AXY_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    F_r := F_r + 1;
                    RC7 := RC7 + 1;
                    AXY := AXY + 1;
                    CriticalItems := CriticalItems +
                      'Several times a week I feel as if something dreadful is about to happen. TRUE^';
                  end;

                if (TempAnswer = '1') and V146T_275F then
                  VRIN_r := VRIN_r + 1;
              end;
            if i = 276 then
              begin
                Inc(IPP_Count);
                Inc(AGGR_r_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '1' then
                  IPP := IPP + 1;

                if TempAnswer = '0' then
                  begin
                    AGGR_r := AGGR_r + 1;
                    if V147F_276T then
                      VRIN_r := VRIN_r + 1;
                  end;
              end;
            if i = 277 then
              begin
                Inc(RC1_Count);
                Inc(NEGE_r_Count);
                Inc(F_r_Count);
                Inc(NUC_Count);
                if TempAnswer = '0' then
                  begin
                    F_r := F_r + 1;
                    RC1 := RC1 + 1;
                    NUC := NUC + 1;
                    NEGE_r := NEGE_r + 1;
                  end;
              end;
            if i = 278 then
              begin
                Inc(SAV_Count);
                Inc(VRIN_r_Count);
                Inc(TRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    SAV := SAV + 1;
                    if T109T_278T then
                      TRIN_r := TRIN_r + 1;
                  end;

                if TempAnswer = '1' then
                  begin
                    if V67T_278F then
                      VRIN_r := VRIN_r + 1;
                    if T201F_278F then
                      TRIN_r := TRIN_r - 1;
                  end;
              end;
            if i = 279 then
              begin
                Inc(RC3_Count);
                if TempAnswer = '0' then
                  RC3 := RC3 + 1;
              end;
            if i = 280 then
              begin
                Inc(COG_Count);
                if TempAnswer = '0' then
                  COG := COG + 1;
              end;
            if i = 281 then
              begin
                Inc(F_r_Count);
                Inc(FML_Count);
                if TempAnswer = '0' then
                  begin
                    F_r := F_r + 1;
                    FML := FML + 1;
                  end;
              end;
            if i = 282 then
              begin
                Inc(EID_Count);
                Inc(RC2_Count);
                Inc(HLP_Count);
                Inc(VRIN_r_Count);
                Inc(TRIN_r_Count);

                if (TempAnswer = '0') then
                  begin
                    if V246F_282T then
                      VRIN_r := VRIN_r + 1;
                    if T169T_282T then
                      TRIN_r := TRIN_r + 1;
                  end;

                if TempAnswer = '1' then
                  begin
                    EID := EID + 1;
                    RC2 := RC2 + 1;
                    HLP := HLP + 1;
                    CriticalItems := CriticalItems +
                      'My main goals in life are within my reach. FALSE^';
                  end;
              end;
            if i = 283 then
              begin
                Inc(Fp_r_Count);
                if TempAnswer = '1' then
                  Fp_r := Fp_r + 1;
              end;
            if i = 284 then
              begin
                Inc(BRF_Count);
                if TempAnswer = '0' then
                  BRF := BRF + 1;
              end;
            if i = 285 then
              begin
                Inc(ACT_Count);
                if TempAnswer = '0' then
                  ACT := ACT + 1;
              end;
            if i = 286 then
              begin
                Inc(MSF_Count);
                if TempAnswer = '1' then
                  MSF := MSF + 1;
              end;
            if i = 287 then
              begin
                Inc(THD_Count);
                Inc(PSYC_r_Count);
                Inc(RC6_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    THD := THD + 1;
                    RC6 := RC6 + 1;
                    PSYC_r := PSYC_r + 1;
                    CriticalItems := CriticalItems +
                      'Someone has been trying to influence my mind. TRUE^';
                  end;

                if (TempAnswer = '1') and V252T_287F then
                  VRIN_r := VRIN_r + 1;
              end;
            if i = 288 then
              begin
                Inc(EID_Count);
                Inc(RCd_Count);
                Inc(SFD_Count);
                if TempAnswer = '0' then
                  begin
                    EID := EID + 1;
                    RCd := RCd + 1;
                    SFD := SFD + 1;
                  end;
              end;
            if i = 289 then
              begin
                Inc(RC7_Count);
                Inc(AXY_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    RC7 := RC7 + 1;
                    AXY := AXY + 1;
                    CriticalItems := CriticalItems +
                      'I have often been frightened in the middle of the night. TRUE^';
                  end;

                if (TempAnswer = '1') and V79T_289F then
                  VRIN_r := VRIN_r + 1;
              end;
            if i = 290 then
              begin
                Inc(RC1_Count);
                Inc(FBS_r_Count);
                if TempAnswer = '1' then
                  begin
                    FBS_r := FBS_r + 1;
                    RC1 := RC1 + 1;
                  end;
              end;
            if i = 291 then
              begin
                Inc(DSF_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    DSF := DSF + 1;
                    if V175F_291T then
                      VRIN_r := VRIN_r + 1;
                  end;

                if (TempAnswer = '1') and V175T_291F then
                  VRIN_r := VRIN_r + 1;
              end;
            if i = 292 then
              begin
                Inc(RC9_Count);
                Inc(BXD_Count);
                Inc(DISC_r_Count);
                if TempAnswer = '0' then
                  begin
                    BXD := BXD + 1;
                    RC9 := RC9 + 1;
                    DISC_r := DISC_r + 1;
                  end;
              end;
            if i = 293 then
              begin
                Inc(EID_Count);
                Inc(NEGE_r_Count);
                Inc(ANP_Count);
                Inc(VRIN_r_Count);
                Inc(TRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    if V134F_293T then
                      VRIN_r := VRIN_r + 1;
                    if T146T_293T then
                      TRIN_r := TRIN_r + 1;
                  end;

                if TempAnswer = '1' then
                  begin
                    EID := EID + 1;
                    ANP := ANP + 1;
                    NEGE_r := NEGE_r + 1;
                    if T261F_293F then
                      TRIN_r := TRIN_r - 1;
                  end;
              end;
            if i = 294 then
              begin
                Inc(THD_Count);
                Inc(RC8_Count);
                Inc(PSYC_r_Count);
                Inc(F_r_Count);
                if TempAnswer = '0' then
                  begin
                    F_r := F_r + 1;
                    THD := THD + 1;
                    RC8 := RC8 + 1;
                    PSYC_r := PSYC_r + 1;
                    CriticalItems := CriticalItems +
                      'I see things or animals or people around me that others do not see. TRUE^';
                  end;
              end;
            if i = 295 then
              begin
                Inc(SHY_Count);
                if TempAnswer = '1' then
                  SHY := SHY + 1;
              end;
            if i = 296 then
              begin
                Inc(AES_Count);
                if TempAnswer = '0' then
                  AES := AES + 1;
              end;
            if i = 297 then
              begin
                Inc(RC4_Count);
                Inc(SUB_Count);
                Inc(DISC_r_Count);
                if TempAnswer = '0' then
                  begin
                    RC4 := RC4 + 1;
                    SUB := SUB + 1;
                    DISC_r := DISC_r + 1;
                    CriticalItems := CriticalItems +
                      'Once a week or more I get high or drunk. TRUE^';
                  end;
              end;
            if i = 298 then
              begin
                Inc(L_r_Count);
                if TempAnswer = '1' then
                  L_r := L_r + 1;
              end;
            if i = 299 then
              begin
                Inc(RCd_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    RCd := RCd + 1;
                    V299T_324F := true;
                  end;
              end;
            if i = 300 then
              begin
                Inc(MEC_Count);
                Inc(DISC_r_Count);
                if TempAnswer = '0' then
                  begin
                    MEC := MEC + 1;
                    DISC_r := DISC_r + 1;
                  end;
              end;
            if i = 301 then
              begin
                Inc(RC1_Count);
                Inc(F_r_Count);
                Inc(NUC_Count);
                if TempAnswer = '0' then
                  begin
                    F_r := F_r + 1;
                    RC1 := RC1 + 1;
                    NUC := NUC + 1;
                  end;
              end;
            if i = 302 then
              begin
                Inc(RC2_Count);
                Inc(IPP_Count);
                Inc(AGGR_r_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '1' then
                  begin
                    RC2 := RC2 + 1;
                    IPP := IPP + 1;
                  end;

                if TempAnswer = '0' then
                  begin
                    AGGR_r := AGGR_r + 1;
                    if V239F_302T then
                      VRIN_r := VRIN_r + 1;
                  end;
              end;
            if i = 303 then
              begin
                Inc(RC7_Count);
                Inc(ANP_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    RC7 := RC7 + 1;
                    ANP := ANP + 1;
                    if V119F_303T then
                      VRIN_r := VRIN_r + 1;
                  end;
              end;
            if i = 304 then
              begin
                Inc(RC3_Count);
                if TempAnswer = '0' then
                  RC3 := RC3 + 1;
              end;
            if i = 305 then
              begin
                Inc(RC9_Count);
                if TempAnswer = '0' then
                  RC9 := RC9 + 1;
              end;
            if i = 306 then
              begin
                Inc(COG_Count);
                Inc(TRIN_r_Count);

                if TempAnswer = '0' then
                  COG := COG + 1;

                if (TempAnswer = '1') and T59F_306F then
                  TRIN_r := TRIN_r - 1;
              end;
            if i = 307 then
              begin
                Inc(FML_Count);
                if TempAnswer = '0' then
                  FML := FML + 1;
              end;
            if i = 308 then
              begin
                Inc(Fs_Count);
                if TempAnswer = '0' then
                  Fs := Fs + 1;
              end;
            if i = 309 then
              begin
                Inc(NEGE_r_Count);
                Inc(STW_Count);
                if TempAnswer = '0' then
                  begin
                    STW := STW + 1;
                    NEGE_r := NEGE_r + 1;
                  end;
              end;
            if i = 310 then
              begin
                Inc(F_r_Count);
                Inc(RC6_Count);
                if TempAnswer = '0' then
                  begin
                    F_r := F_r + 1;
                    RC6 := RC6 + 1;
                    CriticalItems := CriticalItems +
                      'People are not very kind to me. TRUE^';
                  end;
              end;
            if i = 311 then
              begin
                Inc(THD_Count);
                Inc(RC8_Count);
                Inc(PSYC_r_Count);
                if TempAnswer = '0' then
                  begin
                    THD := THD + 1;
                    RC8 := RC8 + 1;
                    PSYC_r := PSYC_r + 1;
                    CriticalItems := CriticalItems +
                      'Sometimes I am sure that other people can tell what I am thinking. TRUE^';
                  end;
              end;
            if i = 312 then
              begin
                Inc(RC4_Count);
                Inc(F_r_Count);
                Inc(BXD_Count);
                Inc(AGG_Count);
                if TempAnswer = '0' then
                  begin
                    F_r := F_r + 1;
                    BXD := BXD + 1;
                    RC4 := RC4 + 1;
                    AGG := AGG + 1;
                    CriticalItems := CriticalItems +
                      'I have gotten angry and broken furniture or dishes when I was drinking. TRUE^';
                  end;
              end;
            if i = 313 then
              begin
                Inc(RC1_Count);
                Inc(NUC_Count);
                if TempAnswer = '1' then
                  begin
                    RC1 := RC1 + 1;
                    NUC := NUC + 1;
                  end;
              end;
            if i = 314 then
              begin
                Inc(Fp_r_Count);
                Inc(TRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    Fp_r := Fp_r + 1;
                    if T269T_314T then
                      TRIN_r := TRIN_r + 1;
                  end;
              end;
            if i = 315 then
              begin
                Inc(FBS_r_Count);
                Inc(RCd_Count);
                if TempAnswer = '0' then
                  begin
                    FBS_r := FBS_r + 1;
                    RCd := RCd + 1;
                  end;
              end;
            if i = 316 then
              begin
                Inc(RC9_Count);
                Inc(BXD_Count);
                Inc(AGG_Count);
                Inc(AGGR_r_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    BXD := BXD + 1;
                    RC9 := RC9 + 1;
                    AGG := AGG + 1;
                    AGGR_r := AGGR_r + 1;
                    CriticalItems := CriticalItems +
                      'I have at times had to be rough with people who were rude or annoying. TRUE^';
                  end;

                if TempAnswer = '1' then
                  V316F_329T := true;
              end;
            if i = 317 then
              begin
                Inc(Fp_r_Count);
                Inc(BRF_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    Fp_r := Fp_r + 1;
                    BRF := BRF + 1;
                    if V165F_317T then
                      VRIN_r := VRIN_r + 1;
                  end;
              end;
            if i = 318 then
              begin
                Inc(RC7_Count);
                Inc(ANP_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    RC7 := RC7 + 1;
                    ANP := ANP + 1;
                    V318T_337F := true;
                  end;
              end;
            if i = 319 then
              begin
                Inc(AGGR_r_Count);
                Inc(VRIN_r_Count);
                Inc(TRIN_r_Count);

                if TempAnswer = '1' then
                  begin
                    AGGR_r := AGGR_r + 1;
                    if V24T_319F then
                      VRIN_r := VRIN_r + 1;
                    if T147F_319F then
                      TRIN_r := TRIN_r - 1;
                  end;
              end;
            if i = 320 then
              begin
                Inc(MSF_Count);
                if TempAnswer = '0' then
                  MSF := MSF + 1;
              end;
            if i = 321 then
              begin
                Inc(IPP_Count);
                Inc(AGGR_r_Count);
                if TempAnswer = '1' then
                  IPP := IPP + 1;

                if TempAnswer = '0' then
                  AGGR_r := AGGR_r + 1;
              end;
            if i = 322 then
              begin
                Inc(EID_Count);
                Inc(RC7_Count);
                Inc(K_r_Count);
                Inc(TRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    EID := EID + 1;
                    RC7 := RC7 + 1;
                    if T37T_322T then
                      TRIN_r := TRIN_r + 1;
                  end;

                if TempAnswer = '1' then
                  K_r := K_r + 1;
              end;
            if i = 323 then
              begin
                Inc(RC2_Count);
                Inc(INTR_r_Count);
                Inc(VRIN_r_Count);

                if (TempAnswer = '0') and V8F_323T then
                  VRIN_r := VRIN_r + 1;

                if TempAnswer = '1' then
                  begin
                    RC2 := RC2 + 1;
                    INTR_r := INTR_r + 1;
                  end;
              end;
            if i = 324 then
              begin
                Inc(NFC_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  NFC := NFC + 1;

                if (TempAnswer = '1') and V299T_324F then
                  VRIN_r := VRIN_r + 1;
              end;
            if i = 325 then
              begin
                Inc(RBS_Count);
                Inc(L_r_Count);
                if TempAnswer = '1' then
                  begin
                    RBS := RBS + 1;
                    L_r := L_r + 1;
                  end;
              end;
            if i = 326 then
              begin
                Inc(RC3_Count);
                if TempAnswer = '0' then
                  RC3 := RC3 + 1;
              end;
            if i = 327 then
              begin
                Inc(RC9_Count);
                Inc(IPP_Count);
                Inc(AGGR_r_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    RC9 := RC9 + 1;
                    AGGR_r := AGGR_r + 1;
                    if V60F_327T then
                      VRIN_r := VRIN_r + 1;
                  end;

                if TempAnswer = '1' then
                  IPP := IPP + 1;
              end;
            if i = 328 then
              begin
                Inc(RC1_Count);
                Inc(HPC_Count);
                if TempAnswer = '0' then
                  begin
                    RC1 := RC1 + 1;
                    HPC := HPC + 1;
                  end;
              end;
            if i = 329 then
              begin
                Inc(RC4_Count);
                Inc(BXD_Count);
                Inc(AGG_Count);
                Inc(AGGR_r_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    BXD := BXD + 1;
                    RC4 := RC4 + 1;
                    AGG := AGG + 1;
                    AGGR_r := AGGR_r + 1;
                    if V316F_329T then
                      VRIN_r := VRIN_r + 1;
                    CriticalItems := CriticalItems +
                      'I''ve been so angry at times that I''ve hurt someone in a physical fight. TRUE^';
                  end;
              end;
            if i = 330 then
              begin
                Inc(THD_Count);
                Inc(RC8_Count);
                Inc(PSYC_r_Count);
                if TempAnswer = '0' then
                  begin
                    THD := THD + 1;
                    RC8 := RC8 + 1;
                    PSYC_r := PSYC_r + 1;
                    CriticalItems := CriticalItems +
                      'I sometimes seem to hear my thoughts being spoken out loud. TRUE^';
                  end;
              end;
            if i = 331 then
              begin
                Inc(EID_Count);
                Inc(RCd_Count);
                Inc(VRIN_r_Count);

                if TempAnswer = '0' then
                  begin
                    EID := EID + 1;
                    RCd := RCd + 1;
                    if V187F_331T then
                      VRIN_r := VRIN_r + 1;
                  end;
              end;
            if i = 332 then
              begin
                Inc(THD_Count);
                Inc(PSYC_r_Count);
                Inc(F_r_Count);
                Inc(RC6_Count);
                if TempAnswer = '0' then
                  begin
                    F_r := F_r + 1;
                    THD := THD + 1;
                    RC6 := RC6 + 1;
                    PSYC_r := PSYC_r + 1;
                    CriticalItems := CriticalItems +
                      'If people had not had it in for me, I would have been much more successful. TRUE^';
                  end;
              end;
            if i = 333 then
              begin
                Inc(MLS_Count);
                if TempAnswer = '1' then
                  MLS := MLS + 1;
              end;
            if i = 334 then
              begin
                Inc(SUI_Count);
                if TempAnswer = '0' then
                  begin
                    SUI := SUI + 1;
                    CriticalItems := CriticalItems +
                      'My thoughts these days turn more and more to death and the life hereafter. TRUE^';
                  end;
              end;
            if i = 335 then
              begin
                Inc(EID_Count);
                Inc(RC7_Count);
                if TempAnswer = '0' then
                  begin
                    EID := EID + 1;
                    RC7 := RC7 + 1;
                  end;
              end;
            if i = 336 then
              begin
                Inc(HLP_Count);
                if TempAnswer = '0' then
                  begin
                    HLP := HLP + 1;
                    CriticalItems := CriticalItems +
                      'I recognize several faults in myself that I will not be able to change. TRUE^';
                  end;
              end;
            if i = 337 then
              begin
                Inc(RC9_Count);
                Inc(AGG_Count);
                Inc(VRIN_r_Count);
                if TempAnswer = '0' then
                  begin
                    RC9 := RC9 + 1;
                    AGG := AGG + 1;
                    CriticalItems := CriticalItems +
                      'I have become so angry with someone that I have felt as if I would explode. TRUE^';
                  end;

                if (TempAnswer = '1') and V318T_337F then
                  VRIN_r := VRIN_r + 1;
              end;
            if i = 338 then
              begin
                Inc(K_r_Count);
                Inc(VRIN_r_Count);
                Inc(TRIN_r_Count);

                if TempAnswer = '1' then
                  begin
                    K_r := K_r + 1;
                    if V123T_338F then
                      VRIN_r := VRIN_r + 1;
                    if T73F_338F then
                      TRIN_r := TRIN_r - 1;
                  end;
              end;

          end;
      end; // for i
  end;

// Remove this after testing, note llh - leave commented out
(*
  procedure SaveRawScoresToFile;
  var
    RawScores                 : TStringList;
  begin
    RawScores := TStringList.create;
    RawScores.Add('Cannot Say = ' + IntToStr(CannotSay));
    RawScores.Add('VRIN_r = ' + IntToStr(VRIN_r)); //Variable Response Inconsistency
    RawScores.Add('TRIN_r = ' + IntToStr(TRIN_r)); //True Response Inconsistency
    RawScores.Add('F_r = ' + IntToStr(F_r)); //Infrequent Responses
    RawScores.Add('Fp_r = ' + IntToStr(Fp_r)); //	Infrequent Psychopathology Responses
    RawScores.Add('Fs = ' + IntToStr(Fs)); //	Infrequent Somatic Responses
    RawScores.Add('FBS_r = ' + IntToStr(FBS_r)); //	Symptom Validity
    RawScores.Add('RBS = ' + IntToStr(RBS)); //	Response Bias Scale
    RawScores.Add('L_r = ' + IntToStr(L_r)); //	Uncommon Virtues
    RawScores.Add('K_r = ' + IntToStr(K_r)); //	Adjustment Validity
//	Higher_Order (H_O) Scales
    RawScores.Add('EID = ' + IntToStr(EID)); //	Emotional/Internalizing Dysfunction
    RawScores.Add('THD = ' + IntToStr(THD)); //	Thought Dysfunction
    RawScores.Add('BXD = ' + IntToStr(BXD)); //	Behavioral/Externalizing Dysfunction
//	Restructured Clinical (RC) Scales
    RawScores.Add('RCd = ' + IntToStr(RCd)); //	Demoralization (DEM)
    RawScores.Add('RC1 = ' + IntToStr(RC1)); //	Somatic Complaints (SOM)
    RawScores.Add('RC2 = ' + IntToStr(RC2)); //	Low Positive Emotions (LPE)
    RawScores.Add('RC3 = ' + IntToStr(RC3)); //	Cynicism (CYN)
    RawScores.Add('RC4 = ' + IntToStr(RC4)); //	Antisocial Behavior (ASB)
    RawScores.Add('RC6 = ' + IntToStr(RC6)); // = ' + IntToStr( )); 	//	Ideas of Persecution (PER)
    RawScores.Add('RC7 = ' + IntToStr(RC7)); //	Dysfunctional Negative Emotions (DNE)
    RawScores.Add('RC8 = ' + IntToStr(RC8)); //	Aberrant Experiences (ABX)
    RawScores.Add('RC9 = ' + IntToStr(RC9)); //	Hypomanic Activation (HPM)
//	Somatic/Cognitive Scales
    RawScores.Add('MLS = ' + IntToStr(MLS)); //	Malaise
    RawScores.Add('GIC = ' + IntToStr(GIC)); //	Gastrointestinal Complaints
    RawScores.Add('HPC = ' + IntToStr(HPC)); //	Head Pain Complaints
    RawScores.Add('NUC = ' + IntToStr(NUC)); //	Neurological Complaints
    RawScores.Add('COG = ' + IntToStr(COG)); //	Cognitive Complaints
//	Internalizing Scales
    RawScores.Add('SUI = ' + IntToStr(SUI)); //	Suicidal/Death Ideation
    RawScores.Add('HLP = ' + IntToStr(HLP)); //	Helplessness/Hopelessness
    RawScores.Add('SFD = ' + IntToStr(SFD)); //	Self_Doubt
    RawScores.Add('NFC = ' + IntToStr(NFC)); //	Inefficacy
    RawScores.Add('STW = ' + IntToStr(STW)); //	Stress/Worry
    RawScores.Add('AXY = ' + IntToStr(AXY)); //	Anxiety
    RawScores.Add('ANP = ' + IntToStr(ANP)); //	Anger Proneness
    RawScores.Add('BRF = ' + IntToStr(BRF)); //	Behavior_Restricting Fears
    RawScores.Add('MSF = ' + IntToStr(MSF)); //	Multiple Specific Fears
//	Externalizing Scales
    RawScores.Add('JCP = ' + IntToStr(JCP)); //	Juvenile Conduct Problems
    RawScores.Add('SUB = ' + IntToStr(SUB)); //	Substance Abuse
    RawScores.Add('AGG = ' + IntToStr(AGG)); //	Aggression
    RawScores.Add('ACT = ' + IntToStr(ACT)); //	Activation
//	Interpersonal Scales
    RawScores.Add('FML = ' + IntToStr(FML)); //	Family Problems
    RawScores.Add('IPP = ' + IntToStr(IPP)); //	Interpersonal Passivity
    RawScores.Add('SAV = ' + IntToStr(SAV)); //	Social Avoidance
    RawScores.Add('SHY = ' + IntToStr(SHY)); //	Shyness
    RawScores.Add('DSF = ' + IntToStr(DSF)); //	Disaffiliativeness
//	Interest Scales
    RawScores.Add('AES = ' + IntToStr(AES)); //	Aesthetic_Literary Interests
    RawScores.Add('MEC = ' + IntToStr(MEC)); //	Mechanical_Physical Interests
//	Personality Psychopathology Five (PSY_5) Scales
    RawScores.Add('AGGR_r = ' + IntToStr(AGGR_r)); //	Aggressiveness_Revised
    RawScores.Add('PSYC_r = ' + IntToStr(PSYC_r)); //	Psychoticism_Revised
    RawScores.Add('DISC_r = ' + IntToStr(DISC_r)); //	Disconstraint_Revised
    RawScores.Add('NEGE_r = ' + IntToStr(NEGE_r)); //	Negative Emotionality/Neuroticism_Revised
    RawScores.Add('INTR_r = ' + IntToStr(INTR_r));

    RawScores.SaveToFile('MMPI-2-RF Raw Scores.txt');

    RawScores.free;
  end; *)

begin
  // This is used by InstrumentResults to do a quick check on the
  // instrument as to its use of the dll
  if AnswerString = 'MMPI-2-RF' then
    begin
      Result := 'YES';
      exit;
    end;

  Result := '';

  Count := NumberOfPieces(AnswerString, RECORD_END);

  if Count < 1 then
    Exit;

  ResetVariables;

  // Reset these only once. Used for adjusted T-scores
  VRIN_r_Adjust := false;
  TRIN_r_Adjust_AboveEleven := false;
  TRIN_r_Adjust_BelowEleven := false;
  F_r_Adjust := false;
  Fp_r_Adjust := false;
  Fs_Adjust := false;
  FBS_r_Adjust := false;
  RBS_Adjust := false;
  L_r_Adjust := false;
  K_r_Adjust := false;

 // Run through all the answers for the raw scores and T-scores.
  ScanTheAnswers(Count, True {First run});

 // Save the raw scores
 // Validity Scales
  VRIN_r_raw := VRIN_r;
  TRIN_r_raw := TRIN_r;
  F_r_raw := F_r;
  Fp_r_raw := Fp_r;
  Fs_raw := Fs;
  FBS_r_raw := FBS_r;
  RBS_raw := RBS;
  L_r_raw := L_r;
  K_r_raw := K_r;

  // Check if any of the Validity Scales have response-percentage
  // under 90%.
  if Round(VRIN_r_Count / 95 * 100) < 90 then
    VRIN_r_Adjust := true;
  if Round(TRIN_r_Count / 45 * 100) < 90 then
    begin
      if TRIN_r > 11 then
        TRIN_r_Adjust_AboveEleven := true;
      if TRIN_r < 11 then
        TRIN_r_Adjust_BelowEleven := true;
    end;
  if Round(F_r_Count / 32 * 100) < 90 then
    F_r_Adjust := true;
  if Round(Fp_r_Count / 21 * 100) < 90 then
    Fp_r_Adjust := true;
  if Round(Fs_Count / 16 * 100) < 90 then
    Fs_Adjust := true;
  if Round(FBS_r_Count / 30 * 100) < 90 then
    FBS_r_Adjust := true;
  if Round(RBS_Count / 28 * 100) < 90 then
    RBS_Adjust := true;
  if Round(L_r_Count / 14 * 100) < 90 then
    L_r_Adjust := true;
  if Round(K_r_Count / 14 * 100) < 90 then
    K_r_Adjust := true;

  // Run through all the answers for the adjusted T-scores only if one
  // or more validity scales are below 90% responses.
  if VRIN_r_Adjust or TRIN_r_Adjust_AboveEleven or TRIN_r_Adjust_BelowEleven or
    F_r_Adjust or Fp_r_Adjust or Fs_Adjust or FBS_r_Adjust or RBS_Adjust or
    L_r_Adjust or K_r_Adjust then
    begin
      ResetVariables;
      ScanTheAnswers(Count, False {Second run});
    end;

  TrueCount := Round(TrueCount / (338 - CannotSay) * 100);
  VRIN_r_Count := Round(VRIN_r_Count / 95 * 100);
  TRIN_r_Count := Round(TRIN_r_Count / 45 * 100);
  F_r_Count := Round(F_r_Count / 32 * 100);
  Fp_r_Count := Round(Fp_r_Count / 21 * 100);
  Fs_Count := Round(Fs_Count / 16 * 100); //	Infrequent Somatic Responses
  FBS_r_Count := Round(FBS_r_Count / 30 * 100); //	Symptom Validity
  RBS_Count := Round(RBS_Count / 28 * 100); //	Response Bias Scale
  L_r_Count := Round(L_r_Count / 14 * 100); //	Uncommon Virtues
  K_r_Count := Round(K_r_Count / 14 * 100); //	Adjustment Validity
  EID_Count := Round(EID_Count / 41 * 100); //	Emotional/Internalizing Dysfunction
  THD_Count := Round(THD_Count / 26 * 100); //	Thought Dysfunction
  BXD_Count := Round(BXD_Count / 23 * 100); //	Behavioral/Externalizing Dysfunction
  RCd_Count := Round(RCd_Count / 24 * 100); //	Demoralization (DEM)
  RC1_Count := Round(RC1_Count / 27 * 100); //	Somatic Complaints (SOM)
  RC2_Count := Round(RC2_Count / 17 * 100); //	Low Positive Emotions (LPE)
  RC3_Count := Round(RC3_Count / 15 * 100); //	Cynicism (CYN)
  RC4_Count := Round(RC4_Count / 22 * 100); //	Antisocial Behavior (ASB)
  RC6_Count := Round(RC6_Count / 17 * 100); //_Count,	//	Ideas of Persecution (PER)
  RC7_Count := Round(RC7_Count / 24 * 100); //	Dysfunctional Negative Emotions (DNE)
  RC8_Count := Round(RC8_Count / 18 * 100); //	Aberrant Experiences (ABX)
  RC9_Count := Round(RC9_Count / 28 * 100); //	Hypomanic Activation (HPM)
  MLS_Count := Round(MLS_Count / 8 * 100); //	Malaise
  GIC_Count := Round(GIC_Count / 5 * 100); //	Gastrointestinal Complaints
  HPC_Count := Round(HPC_Count / 6 * 100); //	Head Pain Complaints
  NUC_Count := Round(NUC_Count / 10 * 100); //	Neurological Complaints
  COG_Count := Round(COG_Count / 10 * 100); //	Cognitive Complaints
  SUI_Count := Round(SUI_Count / 5 * 100); //	Suicidal/Death Ideation
  HLP_Count := Round(HLP_Count / 5 * 100); //	Helplessness/Hopelessness
  SFD_Count := Round(SFD_Count / 4 * 100); //	Self_Doubt
  NFC_Count := Round(NFC_Count / 9 * 100); //	Inefficacy
  STW_Count := Round(STW_Count / 7 * 100); //	Stress/Worry
  AXY_Count := Round(AXY_Count / 5 * 100); //	Anxiety
  ANP_Count := Round(ANP_Count / 7 * 100); //	Anger Proneness
  BRF_Count := Round(BRF_Count / 9 * 100); //	Behavior_Restricting Fears
  MSF_Count := Round(MSF_Count / 9 * 100); //	Multiple Specific Fears
  JCP_Count := Round(JCP_Count / 6 * 100); //	Juvenile Conduct Problems
  SUB_Count := Round(SUB_Count / 7 * 100); //	Substance Abuse
  AGG_Count := Round(AGG_Count / 9 * 100); //	Aggression
  ACT_Count := Round(ACT_Count / 8 * 100); //	Activation
  FML_Count := Round(FML_Count / 10 * 100); //	Family Problems
  IPP_Count := Round(IPP_Count / 10 * 100); //	Interpersonal Passivity
  SAV_Count := Round(SAV_Count / 10 * 100); //	Social Avoidance
  SHY_Count := Round(SHY_Count / 7 * 100); //	Shyness
  DSF_Count := Round(DSF_Count / 6 * 100); //	Disaffiliativeness
  AES_Count := Round(AES_Count / 7 * 100); //	Aesthetic_Literary Interests
  MEC_Count := Round(MEC_Count / 9 * 100); //	Mechanical_Physical Interests
  AGGR_r_Count := Round(AGGR_r_Count / 18 * 100); //	Aggressiveness_Revised
  PSYC_r_Count := Round(PSYC_r_Count / 26 * 100); //	Psychoticism_Revised
  DISC_r_Count := Round(DISC_r_Count / 20 * 100); //	Disconstraint_Revised
  NEGE_r_Count := Round(NEGE_r_Count / 20 * 100); //	Negative Emotionality/Neuroticism_Revised
  INTR_r_Count := Round(INTR_r_Count / 20 * 100); //

 // Remember that the names of the scales has to be the same as the names used
 // in the editor (IAT).

  Result :=

  'Cannot Say^' + IntToStr(CannotSay) + '^' + SkippedItems + '^' + RECORD_END +
    'True count^' + IntToStr(TrueCount) + RECORD_END +
    'VRIN-r^' + IntToStr(VRIN_r_raw) + '^' +
    IntToStr(GetTScore('VRIN_r', VRIN_r)) + '^' + IntToStr(VRIN_r_Count) + RECORD_END +
    'TRIN-r^' + IntToStr(TRIN_r_raw) + '^' +
    IntToStr(GetTScore('TRIN_r', TRIN_r)) + '^' + IntToStr(TRIN_r_Count) + RECORD_END +
    'F-r^' + IntToStr(F_r_raw) + '^' +
    IntToStr(GetTScore('F_r', F_r)) + '^' + IntToStr(F_r_Count) + RECORD_END +
    'Fp-r^' + IntToStr(Fp_r_raw) + '^' +
    IntToStr(GetTScore('Fp_r', Fp_r)) + '^' + IntToStr(Fp_r_Count) + RECORD_END +
    'Fs^' + IntToStr(Fs_raw) + '^' +
    IntToStr(GetTScore('Fs', Fs)) + '^' + IntToStr(Fs_Count) + RECORD_END +
    'FBS-r^' + IntToStr(FBS_r_raw) + '^' +
    IntToStr(GetTScore('FBS_r', FBS_r)) + '^' + IntToStr(FBS_r_Count) + RECORD_END +
    // patch 105 llh - next 2 lines commented out, missed in testing until 5/10, uncommented
    'RBS^' + IntToStr(RBS_raw) + '^' +
    IntToStr(GetTScore('RBS', RBS)) + '^' + IntToStr(RBS_Count) + RECORD_END +
    'L-r^' + IntToStr(L_r_raw) + '^' +
    IntToStr(GetTScore('L_r', L_r)) + '^' + IntToStr(L_r_Count) + RECORD_END +
    'K-r^' + IntToStr(K_r_raw) + '^' +
    IntToStr(GetTScore('K_r', K_r)) + '^' + IntToStr(K_r_Count) + RECORD_END +

  'EID^' + IntToStr(EID) + '^' +
    IntToStr(GetTScore('EID', EID)) + '^' + IntToStr(EID_Count) + RECORD_END +
    'THD^' + IntToStr(THD) + '^' +
    IntToStr(GetTScore('THD', THD)) + '^' + IntToStr(THD_Count) + RECORD_END +
    'BXD^' + IntToStr(BXD) + '^' +
    IntToStr(GetTScore('BXD', BXD)) + '^' + IntToStr(BXD_Count) + RECORD_END +
    'RCd^' + IntToStr(RCd) + '^' +
    IntToStr(GetTScore('RCd', RCd)) + '^' + IntToStr(RCd_Count) + RECORD_END +
    'RC1^' + IntToStr(RC1) + '^' +
    IntToStr(GetTScore('RC1', RC1)) + '^' + IntToStr(RC1_Count) + RECORD_END +
    'RC2^' + IntToStr(RC2) + '^' +
    IntToStr(GetTScore('RC2', RC2)) + '^' + IntToStr(RC2_Count) + RECORD_END +
    'RC3^' + IntToStr(RC3) + '^' +
    IntToStr(GetTScore('RC3', RC3)) + '^' + IntToStr(RC3_Count) + RECORD_END +
    'RC4^' + IntToStr(RC4) + '^' +
    IntToStr(GetTScore('RC4', RC4)) + '^' + IntToStr(RC4_Count) + RECORD_END +
    'RC6^' + IntToStr(RC6) + '^' +
    IntToStr(GetTScore('RC6', RC6)) + '^' + IntToStr(RC6_Count) + RECORD_END +
    'RC7^' + IntToStr(RC7) + '^' +
    IntToStr(GetTScore('RC7', RC7)) + '^' + IntToStr(RC7_Count) + RECORD_END +
    'RC8^' + IntToStr(RC8) + '^' +
    IntToStr(GetTScore('RC8', RC8)) + '^' + IntToStr(RC8_Count) + RECORD_END +
    'RC9^' + IntToStr(RC9) + '^' +
    IntToStr(GetTScore('RC9', RC9)) + '^' + IntToStr(RC9_Count) + RECORD_END +

  'MLS^' + IntToStr(MLS) + '^' +
    IntToStr(GetTScore('MLS', MLS)) + '^' + IntToStr(MLS_Count) + RECORD_END +
    'GIC^' + IntToStr(GIC) + '^' +
    IntToStr(GetTScore('GIC', GIC)) + '^' + IntToStr(GIC_Count) + RECORD_END +
    'HPC^' + IntToStr(HPC) + '^' +
    IntToStr(GetTScore('HPC', HPC)) + '^' + IntToStr(HPC_Count) + RECORD_END +
    'NUC^' + IntToStr(NUC) + '^' +
    IntToStr(GetTScore('NUC', NUC)) + '^' + IntToStr(NUC_Count) + RECORD_END +
    'COG^' + IntToStr(COG) + '^' +
    IntToStr(GetTScore('COG', COG)) + '^' + IntToStr(COG_Count) + RECORD_END +
    'SUI^' + IntToStr(SUI) + '^' +
    IntToStr(GetTScore('SUI', SUI)) + '^' + IntToStr(SUI_Count) + RECORD_END +
    'HLP^' + IntToStr(HLP) + '^' +
    IntToStr(GetTScore('HLP', HLP)) + '^' + IntToStr(HLP_Count) + RECORD_END +
    'SFD^' + IntToStr(SFD) + '^' +
    IntToStr(GetTScore('SFD', SFD)) + '^' + IntToStr(SFD_Count) + RECORD_END +
    'NFC^' + IntToStr(NFC) + '^' +
    IntToStr(GetTScore('NFC', NFC)) + '^' + IntToStr(NFC_Count) + RECORD_END +
    'STW^' + IntToStr(STW) + '^' +
    IntToStr(GetTScore('STW', STW)) + '^' + IntToStr(STW_Count) + RECORD_END +
    'AXY^' + IntToStr(AXY) + '^' +
    IntToStr(GetTScore('AXY', AXY)) + '^' + IntToStr(AXY_Count) + RECORD_END +
    'ANP^' + IntToStr(ANP) + '^' +
    IntToStr(GetTScore('ANP', ANP)) + '^' + IntToStr(ANP_Count) + RECORD_END +
    'BRF^' + IntToStr(BRF) + '^' +
    IntToStr(GetTScore('BRF', BRF)) + '^' + IntToStr(BRF_Count) + RECORD_END +
    'MSF^' + IntToStr(MSF) + '^' +
    IntToStr(GetTScore('MSF', MSF)) + '^' + IntToStr(MSF_Count) + RECORD_END +

  'JCP^' + IntToStr(JCP) + '^' +
    IntToStr(GetTScore('JCP', JCP)) + '^' + IntToStr(JCP_Count) + RECORD_END +
    'SUB^' + IntToStr(SUB) + '^' +
    IntToStr(GetTScore('SUB', SUB)) + '^' + IntToStr(SUB_Count) + RECORD_END +
    'AGG^' + IntToStr(AGG) + '^' +
    IntToStr(GetTScore('AGG', AGG)) + '^' + IntToStr(AGG_Count) + RECORD_END +
    'ACT^' + IntToStr(ACT) + '^' +
    IntToStr(GetTScore('ACT', ACT)) + '^' + IntToStr(ACT_Count) + RECORD_END +
    'FML^' + IntToStr(FML) + '^' +
    IntToStr(GetTScore('FML', FML)) + '^' + IntToStr(FML_Count) + RECORD_END +
    'IPP^' + IntToStr(IPP) + '^' +
    IntToStr(GetTScore('IPP', IPP)) + '^' + IntToStr(IPP_Count) + RECORD_END +
    'SAV^' + IntToStr(SAV) + '^' +
    IntToStr(GetTScore('SAV', SAV)) + '^' + IntToStr(SAV_Count) + RECORD_END +
    'SHY^' + IntToStr(SHY) + '^' +
    IntToStr(GetTScore('SHY', SHY)) + '^' + IntToStr(SHY_Count) + RECORD_END +
    'DSF^' + IntToStr(DSF) + '^' +
    IntToStr(GetTScore('DSF', DSF)) + '^' + IntToStr(DSF_Count) + RECORD_END +
    'AES^' + IntToStr(AES) + '^' +
    IntToStr(GetTScore('AES', AES)) + '^' + IntToStr(AES_Count) + RECORD_END +
    'MEC^' + IntToStr(MEC) + '^' +
    IntToStr(GetTScore('MEC', MEC)) + '^' + IntToStr(MEC_Count) + RECORD_END +

  'AGGR-r^' + IntToStr(AGGR_r) + '^' +
    IntToStr(GetTScore('AGGR_r', AGGR_r)) + '^' + IntToStr(AGGR_r_Count) + RECORD_END +
    'PSYC-r^' + IntToStr(PSYC_r) + '^' +
    IntToStr(GetTScore('PSYC_r', PSYC_r)) + '^' + IntToStr(PSYC_r_Count) + RECORD_END +
    'DISC-r^' + IntToStr(DISC_r) + '^' +
    IntToStr(GetTScore('DISC_r', DISC_r)) + '^' + IntToStr(DISC_r_Count) + RECORD_END +
    'NEGE-r^' + IntToStr(NEGE_r) + '^' +
    IntToStr(GetTScore('NEGE_r', NEGE_r)) + '^' + IntToStr(NEGE_r_Count) + RECORD_END +
    'INTR-r^' + IntToStr(INTR_r) + '^' +
    IntToStr(GetTScore('INTR_r', INTR_r)) + '^' + IntToStr(INTR_r_Count) + RECORD_END +

  CriticalItems + RECORD_END +
    Responses + RECORD_END;
end;


function AddSpace(StringIn: string; Position: integer): string;
begin
  result := result + StringIn;
  while Length(result) < Position do
    result := result + ' ';
end;

function ProgressNote_MMPI2RF(
  PatientName,
  PatientDOB,
  PatientSex,
  ScoreString: string): string;

const
  Indent1                     : string = '   ';
  Indent2                     : string = '      ';
  Header                      : string =
    '   Scale    Raw Score    T Score    Response %';
  EOP                         : string = '***eop***';
var
  i                           : integer;
  TempStr,
    TempStr1,
    TempStr2                  : string;

  ValidScale,
    Invalid                   : boolean;

  procedure DisplayValidityScales;
  var
    ii, jj, kk                : integer;
    // patch 105 llh, added RBS (RB) scale in 5/20/13
    V, T, F, Fp, Fs, FB, RB, L, K,
      Number,
      TempLine                : string;

    procedure DisplayMinMax;
    var
      kk                      : integer;
    begin
      for kk := 0 to 3 do
        begin
          if IntToStr(ii * 4 + 20 - kk) = V then // VRIN-r
            begin
              TempLine[9] := '-';
              TempLine[10] := '-';
              TempLine[11] := '-';
            end;
          if IntToStr(ii * 4 + 20 - kk) = T then // TRIN-r
            begin
              TempLine[17] := '-';
              TempLine[18] := '-';
              TempLine[19] := '-';
            end;
          if IntToStr(ii * 4 + 20 - kk) = F then // F-r
            begin
              TempLine[25] := '-';
              TempLine[26] := '-';
              TempLine[27] := '-';
            end;
          if IntToStr(ii * 4 + 20 - kk) = Fp then // Fp-r
            begin
              TempLine[33] := '-';
              TempLine[34] := '-';
              TempLine[35] := '-';
            end;
          if IntToStr(ii * 4 + 20 - kk) = Fs then // Fs
            begin
              TempLine[41] := '-';
              TempLine[42] := '-';
              TempLine[43] := '-';
            end;
          if IntToStr(ii * 4 + 20 - kk) = FB then // FBS-r
            begin
              TempLine[49] := '-';
              TempLine[50] := '-';
              TempLine[51] := '-';
            end;

          // patch 105 llh - need to accomodate for RBS scale,  added RBS scale
          if IntToStr(ii * 4 + 20 - kk) = RB then // RBS
            begin
              TempLine[57] := '-';
              TempLine[58] := '-';
              TempLine[59] := '-';
            end;

          if IntToStr(ii * 4 + 20 - kk) = L then // L-r
            begin
              TempLine[65] := '-';
              TempLine[66] := '-';
              TempLine[67] := '-';
            end;
          if IntToStr(ii * 4 + 20 - kk) = K then // K-r
            begin
              TempLine[73] := '-';
              TempLine[74] := '-';
              TempLine[75] := '-';
            end;
        end; // kk
    end;

  begin
    result := result + EOP + #13 +
      '            MMPI-2-RF  VALIDITY SCALES' + #13 + #13;

    for ii := 25 downto 0 do
      begin
        TempLine := '';

        // Set t-score percentages, pips and line for y-axis.
        if (ii mod 5 = 0) then
          begin
            Number := IntToStr(ii * 4 + 20);
            if Length(Number) = 2 then
              Number := ' ' + Number + ' -|'
            else
              Number := Number + ' -|';
          end
        else
          Number := '     |';
        TempLine := Number;

        if (ii = 8) or (ii = 0) then
          // patch 105 llh - need to accomodate for RBS scale, changed 70 to 76
          for jj := 6 to 76 do
            TempLine := TempLine + '_'
        else
          // patch 105 llh - need to accomodate for RBS scale, changed 70 to 76
          for jj := 6 to 76 do
            TempLine := TempLine + ' ';

        // Max score
        V := '120';
        T := '120';
        F := '120';
        Fp := '120';
        Fs := '120';
        FB := '120';
        // patch 105 llh - need to accomodate for RBS scale, added RBS in
        // add here in a minute
        RB := '120' ;
        L := '105';
        K := '72';
        DisplayMinMax;

        // Min Score
        V := '34';
        T := '50';
        F := '42';
        Fp := '42';
        Fs := '42';
        FB := '26';
        // patch 105 llh - need to accomodate for RBS scale, added RBS in
        // add here in a minute
        RB := '29' ;
        L := '37';
        K := '24';
        DisplayMinMax;

        V := Piece(Piece(ScoreString, RECORD_END, 3), '^', 3);
        T := Piece(Piece(ScoreString, RECORD_END, 4), '^', 3);
        F := Piece(Piece(ScoreString, RECORD_END, 5), '^', 3);
        Fp := Piece(Piece(ScoreString, RECORD_END, 6), '^', 3);
        Fs := Piece(Piece(ScoreString, RECORD_END, 7), '^', 3);
        FB := Piece(Piece(ScoreString, RECORD_END, 8), '^', 3);
        // patch 105 llh - had to add RBS scale back in 5/20/13
        RB := Piece(Piece(ScoreString, RECORD_END, 9), '^', 3);
        L := Piece(Piece(ScoreString, RECORD_END, 10), '^', 3);
        K := Piece(Piece(ScoreString, RECORD_END, 11), '^', 3);

        for kk := 0 to 3 do
          begin
            if IntToStr(ii * 4 + (20 - kk)) = V then // VRIN-r
              TempLine[10] := '*';
            if IntToStr(ii * 4 + (20 - kk)) = T then // TRIN-r
              TempLine[18] := '*';
            if IntToStr(ii * 4 + (20 - kk)) = F then // F-r
              TempLine[26] := '*';
            if IntToStr(ii * 4 + (20 - kk)) = Fp then // Fp-r
              TempLine[34] := '*';
            if IntToStr(ii * 4 + (20 - kk)) = Fs then // Fs
              TempLine[42] := '*';
            if IntToStr(ii * 4 + (20 - kk)) = FB then // FBS-r
              TempLine[50] := '*';
            // patch 105 llh - need to accomodate for RBS scale, added RBS in
            if IntToStr(ii * 4 + (20 - kk)) = RB then // RBS
              TempLine[58] := '*';
            if IntToStr(ii * 4 + (20 - kk)) = L then // L-r
              TempLine[66] := '*';
            if IntToStr(ii * 4 + (20 - kk)) = K then // K-r
              TempLine[74] := '*';
          end; // kk
        result := result + TempLine + #13;
      end;

    result := result +
      '         |' +
      '       |' +
      '       |' +
      '       |' +
      '       |' +
      '       |' +
      '       |' +
      //patch 105 llh, to accomodate putting RBS scale in
      '       |' +
      '       |' + #13;

    TempStr1 := '';
    if StrToIntDef(Piece(Piece(ScoreString, RECORD_END, 3), '^', 4), 0) < 90 then
      TempStr1 := '      *VRIN-r'
    else
      TempStr1 := '       VRIN-r';
    if StrToIntDef(Piece(Piece(ScoreString, RECORD_END, 4), '^', 4), 0) < 90 then
      TempStr1 := TempStr1 + ' *TRIN-r'
    else
      TempStr1 := TempStr1 + '  TRIN-r';
    if StrToIntDef(Piece(Piece(ScoreString, RECORD_END, 5), '^', 4), 0) < 90 then
      TempStr1 := TempStr1 + '  *F-r'
    else
      TempStr1 := TempStr1 + '   F-r';
    if StrToIntDef(Piece(Piece(ScoreString, RECORD_END, 6), '^', 4), 0) < 90 then
      TempStr1 := TempStr1 + '   *Fp-r'
    else
      TempStr1 := TempStr1 + '    Fp-r';
    if StrToIntDef(Piece(Piece(ScoreString, RECORD_END, 7), '^', 4), 0) < 90 then
      TempStr1 := TempStr1 + '     *Fs'
    else
      TempStr1 := TempStr1 + '      Fs';
    if StrToIntDef(Piece(Piece(ScoreString, RECORD_END, 8), '^', 4), 0) < 90 then
      TempStr1 := TempStr1 + '   *FBS-r'
    else
      TempStr1 := TempStr1 + '    FBS-r';
    // patch 105 llh - RBS scale omitted, needed to be added in at end of testing 5/20/2013
    if StrToIntDef(Piece(Piece(ScoreString, RECORD_END, 9), '^', 4), 0) < 90 then
      TempStr1 := TempStr1 + '   *RBS'
    else
      TempStr1 := TempStr1 + '    RBS';
    if StrToIntDef(Piece(Piece(ScoreString, RECORD_END, 10), '^', 4), 0) < 90 then
      TempStr1 := TempStr1 + '    *L-r'
    else
      TempStr1 := TempStr1 + '     L-r';
    if StrToIntDef(Piece(Piece(ScoreString, RECORD_END, 11), '^', 4), 0) < 90 then
      TempStr1 := TempStr1 + '    *K-r'
    else
      TempStr1 := TempStr1 + '     K-r';

    result := result + TempStr1 + #13 + #13;

    TempLine := '';
    TempLine := AddSpace('Raw:', 9) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 3), '^', 2), 8) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 4), '^', 2), 8) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 5), '^', 2), 8) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 6), '^', 2), 8) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 7), '^', 2), 8) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 8), '^', 2), 8) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 9), '^', 2), 8) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 10), '^', 2), 8) +
      // patch 105 llh - need to add RBS scale in, now 11 pieces
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 11), '^', 2), 8);
    result := result + TempLine + #13;

    TempLine := '';
    TempLine := AddSpace('T score:', 9) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 3), '^', 3), 8) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 4), '^', 3), 8) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 5), '^', 3), 8) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 6), '^', 3), 8) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 7), '^', 3), 8) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 8), '^', 3), 8) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 9), '^', 3), 8) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 10), '^', 3), 8) +
      // patch 105 llh - need to add RBS scale in, now 11 pieces
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 11), '^', 3), 8);
    result := result + TempLine + #13;

    TempLine := '';
    TempLine := AddSpace('Resp %:', 9) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 3), '^', 4), 8) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 4), '^', 4), 8) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 5), '^', 4), 8) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 6), '^', 4), 8) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 7), '^', 4), 8) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 8), '^', 4), 8) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 9), '^', 4), 8) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 10), '^', 4), 8) +
      // patch 105 llh - need to add RBS scale in, now 11 pieces
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 11), '^', 4), 8);

    result := result + TempLine + #13 + #13;

    TempLine := '';
    TempLine := AddSpace('Cannot Say (Raw) ' +
      Piece(Piece(ScoreString, RECORD_END, 1), '^', 2), 33) +
      'Percent True of items answered: ' +
      Piece(Piece(ScoreString, RECORD_END, 2), '^', 2);
    result := result + TempLine + #13 + #13;

    result := result +
      'The highest and lowest T scores possible on each scale are ' +
      'indicated by a "---"; T scores are non-gendered.' + #13;

    result := result + 'Scales marked with an asterisk have response percentages below 90%.' + #13 + #13;

    result := result +
      AddSpace('VRIN-r', 8) + AddSpace('Variable Response Inconsistency', 38) +
      AddSpace('Fs', 6) + 'Infrequent Somatic Responses' + #13 +
      AddSpace('TRIN-r', 8) + AddSpace('True Response Inconsistency', 38) +
      AddSpace('FBS-r', 6) + 'Symptom Validity' + #13 +

      AddSpace('F-r', 8) + AddSpace('Infrequent Responses', 38) +
      // patch 105 llh - missing originally, added 5/20/2013
      AddSpace('RBS', 6) + 'Response Bias Scale' + #13 +

      AddSpace('L-r', 8) + AddSpace('Uncommon Virtues',38) +
      AddSpace('Fp-r', 6) + 'Infrequent Psychopathology Responses' + #13 +
      AddSpace('K-r', 8) + AddSpace('Adjustment Validity',38) + #13 + #13;

  end;

  procedure DisplayHOandRCScales;
  var
    ii, jj, kk                : integer;
    e, t, b, rD, r1, r2, r3, r4, r6, r7, r8, r9,
      Number,
      TempLine                : string;

    procedure DisplayMinMax;
    var
      kk                      : integer;
    begin
      for kk := 0 to 3 do
        begin
          if IntToStr(ii * 4 + 20 - kk) = E then // EID
            begin
              TempLine[9] := '-';
              TempLine[10] := '-';
              TempLine[11] := '-';
            end;
          if IntToStr(ii * 4 + 20 - kk) = T then // THD
            begin
              TempLine[15] := '-';
              TempLine[16] := '-';
              TempLine[17] := '-';
            end;
          if IntToStr(ii * 4 + 20 - kk) = B then // BXD
            begin
              TempLine[21] := '-';
              TempLine[22] := '-';
              TempLine[23] := '-';
            end;
          if IntToStr(ii * 4 + 20 - kk) = Rd then // RCd
            begin
              TempLine[27] := '-';
              TempLine[28] := '-';
              TempLine[29] := '-';
            end;
          if IntToStr(ii * 4 + 20 - kk) = R1 then // RC1
            begin
              TempLine[33] := '-';
              TempLine[34] := '-';
              TempLine[35] := '-';
            end;
          if IntToStr(ii * 4 + 20 - kk) = R2 then // RC2
            begin
              TempLine[39] := '-';
              TempLine[40] := '-';
              TempLine[41] := '-';
            end;
          if IntToStr(ii * 4 + 20 - kk) = R3 then // RC3
            begin
              TempLine[45] := '-';
              TempLine[46] := '-';
              TempLine[47] := '-';
            end;
          if IntToStr(ii * 4 + 20 - kk) = R4 then // RC4
            begin
              TempLine[51] := '-';
              TempLine[52] := '-';
              TempLine[53] := '-';
            end;
          if IntToStr(ii * 4 + 20 - kk) = R6 then // RC6
            begin
              TempLine[57] := '-';
              TempLine[58] := '-';
              TempLine[59] := '-';
            end;
          if IntToStr(ii * 4 + 20 - kk) = R7 then // RC7
            begin
              TempLine[63] := '-';
              TempLine[64] := '-';
              TempLine[65] := '-';
            end;
          if IntToStr(ii * 4 + 20 - kk) = R8 then // RC8
            begin
              TempLine[69] := '-';
              TempLine[70] := '-';
              TempLine[71] := '-';
            end;
          if IntToStr(ii * 4 + 20 - kk) = R9 then // RC9
            begin
              TempLine[75] := '-';
              TempLine[76] := '-';
              TempLine[77] := '-';
            end;
        end; // kk
    end;

  begin

    result := result + EOP + #13 +
      '            MMPI-2-RF  HIGHER-ORDER (H-O) and RESTRUCTURED CLINICAL (RC) SCALES' + #13 + #13;
    result := result +
      '         Higher Order                   Restructured Clinical' + #13;
    for ii := 25 downto 0 do
      begin
        TempLine := '';

        // Set t-score percentages, pips and line for y-axis.
        if (ii mod 5 = 0) then
          begin
            Number := IntToStr(ii * 4 + 20);
            if Length(Number) = 2 then
              Number := ' ' + Number + ' -|'
            else
              Number := Number + ' -|';
          end
        else
          Number := '     |';
        TempLine := Number;

        // patch 105 llh - 5/20/13 adding lines missed in original coding
        //if (ii = 8) or (ii = 0) then
        if (ii = 12) or (ii = 8) or (ii = 5) or (ii = 0) then
          for jj := 6 to 78 do
            TempLine := TempLine + '_'
        else
          for jj := 6 to 78 do
            TempLine := TempLine + ' ';

        TempLine[25] := '|';

        // Max score
        E := '93';
        T := '100';
        B := '94';
        Rd := '86';
        R1 := '100';
        R2 := '99';
        R3 := '83';
        R4 := '99';
        R6 := '100';
        R7 := '94';
        R8 := '100';
        R9 := '91';
        DisplayMinMax;

        // Min Score
        E := '30';
        T := '39';
        B := '32';
        Rd := '37';
        R1 := '36';
        R2 := '34';
        R3 := '34';
        R4 := '34';
        R6 := '43';
        R7 := '34';
        R8 := '39';
        R9 := '25';
        DisplayMinMax;


        E := Piece(Piece(ScoreString, RECORD_END, 11), '^', 3);
        T := Piece(Piece(ScoreString, RECORD_END, 12), '^', 3);
        B := Piece(Piece(ScoreString, RECORD_END, 13), '^', 3);
        Rd := Piece(Piece(ScoreString, RECORD_END, 14), '^', 3);
        R1 := Piece(Piece(ScoreString, RECORD_END, 15), '^', 3);
        R2 := Piece(Piece(ScoreString, RECORD_END, 16), '^', 3);
        R3 := Piece(Piece(ScoreString, RECORD_END, 17), '^', 3);
        R4 := Piece(Piece(ScoreString, RECORD_END, 18), '^', 3);
        R6 := Piece(Piece(ScoreString, RECORD_END, 19), '^', 3);
        R7 := Piece(Piece(ScoreString, RECORD_END, 20), '^', 3);
        R8 := Piece(Piece(ScoreString, RECORD_END, 21), '^', 3);
        R9 := Piece(Piece(ScoreString, RECORD_END, 22), '^', 3);

        for kk := 0 to 3 do
          begin
            if IntToStr(ii * 4 + 20 - kk) = E then // EID
              TempLine[10] := '*';
            if IntToStr(ii * 4 + 20 - kk) = T then // THD
              TempLine[16] := '*';
            if IntToStr(ii * 4 + 20 - kk) = B then // BXD
              TempLine[22] := '*';
            if IntToStr(ii * 4 + 20 - kk) = Rd then // RCd
              TempLine[28] := '*';
            if IntToStr(ii * 4 + 20 - kk) = R1 then // RC1
              TempLine[34] := '*';
            if IntToStr(ii * 4 + 20 - kk) = R2 then // RC2
              TempLine[40] := '*';
            if IntToStr(ii * 4 + 20 - kk) = R3 then // RC3
              TempLine[46] := '*';
            if IntToStr(ii * 4 + 20 - kk) = R4 then // RC4
              TempLine[52] := '*';
            if IntToStr(ii * 4 + 20 - kk) = R6 then // RC6
              TempLine[58] := '*';
            if IntToStr(ii * 4 + 20 - kk) = R7 then // RC7
              TempLine[64] := '*';
            if IntToStr(ii * 4 + 20 - kk) = R8 then // RC8
              TempLine[70] := '*';
            if IntToStr(ii * 4 + 20 - kk) = R9 then // RC9
              TempLine[76] := '*';
          end; // kk
        result := result + TempLine + #13;
      end;

    result := result +
      '         |' +
      '     |' +
      '     |' +
      '     |' +
      '     |' +
      '     |' +
      '     |' +
      '     |' +
      '     |' +
      '     |' +
      '     |' +
      '     |' + #13;

    TempStr1 := '';
    if StrToIntDef(Piece(Piece(ScoreString, RECORD_END, 11), '^', 4), 0) < 90 then
      TempStr1 := '       *EID'
    else
      TempStr1 := '        EID';
    if StrToIntDef(Piece(Piece(ScoreString, RECORD_END, 12), '^', 4), 0) < 90 then
      TempStr1 := TempStr1 + '  *THD'
    else
      TempStr1 := TempStr1 + '   THD';
    if StrToIntDef(Piece(Piece(ScoreString, RECORD_END, 13), '^', 4), 0) < 90 then
      TempStr1 := TempStr1 + '  *BXD'
    else
      TempStr1 := TempStr1 + '   BXD';
    if StrToIntDef(Piece(Piece(ScoreString, RECORD_END, 14), '^', 4), 0) < 90 then
      TempStr1 := TempStr1 + '  *RCd'
    else
      TempStr1 := TempStr1 + '   RCd';
    if StrToIntDef(Piece(Piece(ScoreString, RECORD_END, 15), '^', 4), 0) < 90 then
      TempStr1 := TempStr1 + '  *RC1'
    else
      TempStr1 := TempStr1 + '   RC1';
    if StrToIntDef(Piece(Piece(ScoreString, RECORD_END, 16), '^', 4), 0) < 90 then
      TempStr1 := TempStr1 + '  *RC2'
    else
      TempStr1 := TempStr1 + '   RC2';
    if StrToIntDef(Piece(Piece(ScoreString, RECORD_END, 17), '^', 4), 0) < 90 then
      TempStr1 := TempStr1 + '  *RC3'
    else
      TempStr1 := TempStr1 + '   RC3';
    if StrToIntDef(Piece(Piece(ScoreString, RECORD_END, 18), '^', 4), 0) < 90 then
      TempStr1 := TempStr1 + '  *RC4'
    else
      TempStr1 := TempStr1 + '   RC4';
    if StrToIntDef(Piece(Piece(ScoreString, RECORD_END, 19), '^', 4), 0) < 90 then
      TempStr1 := TempStr1 + '  *RC6'
    else
      TempStr1 := TempStr1 + '   RC6';
    if StrToIntDef(Piece(Piece(ScoreString, RECORD_END, 20), '^', 4), 0) < 90 then
      TempStr1 := TempStr1 + '  *RC7'
    else
      TempStr1 := TempStr1 + '   RC7';
    if StrToIntDef(Piece(Piece(ScoreString, RECORD_END, 21), '^', 4), 0) < 90 then
      TempStr1 := TempStr1 + '  *RC8'
    else
      TempStr1 := TempStr1 + '   RC8';
    if StrToIntDef(Piece(Piece(ScoreString, RECORD_END, 22), '^', 4), 0) < 90 then
      TempStr1 := TempStr1 + '  *RC9'
    else
      TempStr1 := TempStr1 + '   RC9';

    result := result + TempStr1 + #13 + #13;

    TempLine := '';
    TempLine := AddSpace('Raw', 8) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 11), '^', 2), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 12), '^', 2), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 13), '^', 2), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 14), '^', 2), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 15), '^', 2), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 16), '^', 2), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 17), '^', 2), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 18), '^', 2), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 19), '^', 2), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 20), '^', 2), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 21), '^', 2), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 22), '^', 2), 6);
    result := result + TempLine + #13;

    TempLine := '';
    TempLine := AddSpace('T score', 8) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 11), '^', 3), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 12), '^', 3), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 13), '^', 3), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 14), '^', 3), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 15), '^', 3), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 16), '^', 3), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 17), '^', 3), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 18), '^', 3), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 19), '^', 3), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 20), '^', 3), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 21), '^', 3), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 22), '^', 3), 6);
    result := result + TempLine + #13;

    TempLine := '';
    TempLine := AddSpace('Resp %', 8) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 11), '^', 4), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 12), '^', 4), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 13), '^', 4), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 14), '^', 4), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 15), '^', 4), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 16), '^', 4), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 17), '^', 4), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 18), '^', 4), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 19), '^', 4), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 20), '^', 4), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 21), '^', 4), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 22), '^', 4), 6);
    result := result + TempLine + #13 + #13;

    result := result +
      'The highest and lowest T scores possible on each scale are ' +
      'indicated by a "---"; T scores are non-gendered.' + #13;
    result := result + 'Scales marked with an asterisk have response percentages below 90%.' + #13 + #13;

    result := result +
      AddSpace('EID', 4) + AddSpace('Emotional/Internalizing Dysfunction', 37) +
      AddSpace('RCd', 4) + AddSpace('Demoralization', 22) +
      AddSpace('RC6', 4) + 'Ideas of Persecution' + #13 +

    AddSpace('THD', 4) + AddSpace('Thought Dysfunction', 37) +
      AddSpace('RC1', 4) + AddSpace('Somatic Complaints', 22) +
      AddSpace('RC7', 4) + 'Dysfunctional Negative Emotions' + #13 +

    AddSpace('BXD', 4) + AddSpace('Behavioral/Externalizing Dysfunction', 37) +
      AddSpace('RC2', 4) + AddSpace('Low Positive Emotions', 22) +
      AddSpace('RC8', 4) + 'Aberrant Experiences' + #13 +

    AddSpace(' ', 41) + AddSpace('RC3', 4) + AddSpace('Cynicism', 22) +
      AddSpace('RC9', 4) + 'Hypomanic Activation' + #13 +
      AddSpace(' ', 41) + AddSpace('RC4', 4) + 'Antisocial Behavior' + #13 + #13;
  end;


  procedure DisplaySCIScales;
  var
    ii, jj, kk                : integer;
    M, G, H, N, C, S, HLP, SFD, NFC, STW, A, ANP, B, MSF,
      Number,
      TempLine                : string;

    procedure DisplayMinMax;
    var
      kk                      : integer;
    begin
      for kk := 0 to 3 do
        begin
          if IntToStr(ii * 4 + 20 - kk) = M then // MLS
            begin
              TempLine[9] := '-';
              TempLine[10] := '-';
              TempLine[11] := '-';
            end;
          if IntToStr(ii * 4 + 20 - kk) = G then // GIC
            begin
              TempLine[15] := '-';
              TempLine[16] := '-';
              TempLine[17] := '-';
            end;
          if IntToStr(ii * 4 + 20 - kk) = H then // HPC
            begin
              TempLine[21] := '-';
              TempLine[22] := '-';
              TempLine[23] := '-';
            end;
          if IntToStr(ii * 4 + 20 - kk) = N then // NUC
            begin
              TempLine[27] := '-';
              TempLine[28] := '-';
              TempLine[29] := '-';
            end;
          if IntToStr(ii * 4 + 20 - kk) = C then // COG
            begin
              TempLine[33] := '-';
              TempLine[34] := '-';
              TempLine[35] := '-';
            end;
          if IntToStr(ii * 4 + 20 - kk) = S then // SUI
            begin
              TempLine[39] := '-';
              TempLine[40] := '-';
              TempLine[41] := '-';
            end;
          if IntToStr(ii * 4 + 20 - kk) = HLP then
            begin
              TempLine[45] := '-';
              TempLine[46] := '-';
              TempLine[47] := '-';
            end;
          if IntToStr(ii * 4 + 20 - kk) = SFD then
            begin
              TempLine[51] := '-';
              TempLine[52] := '-';
              TempLine[53] := '-';
            end;
          if IntToStr(ii * 4 + 20 - kk) = NFC then
            begin
              TempLine[57] := '-';
              TempLine[58] := '-';
              TempLine[59] := '-';
            end;
          if IntToStr(ii * 4 + 20 - kk) = STW then
            begin
              TempLine[63] := '-';
              TempLine[64] := '-';
              TempLine[65] := '-';
            end;
          if IntToStr(ii * 4 + 20 - kk) = A then // AYX
            begin
              TempLine[69] := '-';
              TempLine[70] := '-';
              TempLine[71] := '-';
            end;
          if IntToStr(ii * 4 + 20 - kk) = ANP then // ANP
            begin
              TempLine[75] := '-';
              TempLine[76] := '-';
              TempLine[77] := '-';
            end;
          if IntToStr(ii * 4 + 20 - kk) = B then // BRF
            begin
              TempLine[81] := '-';
              TempLine[82] := '-';
              TempLine[83] := '-';
            end;
          if IntToStr(ii * 4 + 20 - kk) = MSF then
            begin
              TempLine[87] := '-';
              TempLine[88] := '-';
              TempLine[89] := '-';
            end;
        end; // kk
    end;

  begin

    result := result + EOP + #13 +
      '            MMPI-2-RF  SOMATIC/COGNITIVE and INTERNALIZING SCALES' + #13 + #13;
    result := result +
      '           Somatic/Cognitive                     Internalizing' + #13;
    for ii := 25 downto 0 do
      begin
        TempLine := '';

        // Set t-score percentages, pips and line for y-axis.
        if (ii mod 5 = 0) then
          begin
            Number := IntToStr(ii * 4 + 20);
            if Length(Number) = 2 then
              Number := ' ' + Number + ' -|'
            else
              Number := Number + ' -|';
          end
        else
          Number := '     |';
        TempLine := Number;

        // patch 105 llh - 5/20/13 adding lines missed in original coding
        //if (ii = 8) or (ii = 0) then
        if (ii = 12) or (ii = 8) or (ii = 5) or (ii = 0) then
          for jj := 6 to 89 do
            TempLine := TempLine + '_'
        else
          for jj := 6 to 89 do
            TempLine := TempLine + ' ';

        TempLine[37] := '|';

        // Max score
        M := '87';
        G := '96';
        H := '85';
        N := '100';
        C := '96';
        S := '100';
        HLP := '88';
        SFD := '76';
        NFC := '80';
        STW := '81';
        A := '100';
        ANP := '80';
        B := '100';
        MSF := '78';
        DisplayMinMax;

        // Min Score
        M := '38';
        G := '46';
        H := '42';
        N := '41';
        C := '40';
        S := '45';
        HLP := '40';
        SFD := '42';
        NFC := '36';
        STW := '36';
        A := '44';
        ANP := '39';
        B := '43';
        MSF := '36';
        DisplayMinMax;

        M := Piece(Piece(ScoreString, RECORD_END, 23), '^', 3);
        G := Piece(Piece(ScoreString, RECORD_END, 24), '^', 3);
        H := Piece(Piece(ScoreString, RECORD_END, 25), '^', 3);
        N := Piece(Piece(ScoreString, RECORD_END, 26), '^', 3);
        C := Piece(Piece(ScoreString, RECORD_END, 27), '^', 3);
        S := Piece(Piece(ScoreString, RECORD_END, 28), '^', 3);
        HLP := Piece(Piece(ScoreString, RECORD_END, 29), '^', 3);
        SFD := Piece(Piece(ScoreString, RECORD_END, 30), '^', 3);
        NFC := Piece(Piece(ScoreString, RECORD_END, 31), '^', 3);
        STW := Piece(Piece(ScoreString, RECORD_END, 32), '^', 3);
        A := Piece(Piece(ScoreString, RECORD_END, 33), '^', 3);
        ANP := Piece(Piece(ScoreString, RECORD_END, 34), '^', 3);
        B := Piece(Piece(ScoreString, RECORD_END, 35), '^', 3);
        MSF := Piece(Piece(ScoreString, RECORD_END, 36), '^', 3);

        for kk := 0 to 3 do
          begin
            if IntToStr(ii * 4 + 20 - kk) = M then //
              TempLine[10] := '*';
            if IntToStr(ii * 4 + 20 - kk) = G then //
              TempLine[16] := '*';
            if IntToStr(ii * 4 + 20 - kk) = H then //
              TempLine[22] := '*';
            if IntToStr(ii * 4 + 20 - kk) = N then //
              TempLine[28] := '*';
            if IntToStr(ii * 4 + 20 - kk) = C then //
              TempLine[34] := '*';
            if IntToStr(ii * 4 + 20 - kk) = S then //
              TempLine[40] := '*';
            if IntToStr(ii * 4 + 20 - kk) = HLP then //
              TempLine[46] := '*';
            if IntToStr(ii * 4 + 20 - kk) = SFD then //
              TempLine[52] := '*';
            if IntToStr(ii * 4 + 20 - kk) = NFC then //
              TempLine[58] := '*';
            if IntToStr(ii * 4 + 20 - kk) = STW then //
              TempLine[64] := '*';
            if IntToStr(ii * 4 + 20 - kk) = A then //
              TempLine[70] := '*';
            if IntToStr(ii * 4 + 20 - kk) = ANP then //
              TempLine[76] := '*';
            if IntToStr(ii * 4 + 20 - kk) = B then //
              TempLine[82] := '*';
            if IntToStr(ii * 4 + 20 - kk) = MSF then //
              TempLine[88] := '*';
          end; // kk

        result := result + TempLine + #13;
      end;

    result := result +
      '         |' +
      '     |' +
      '     |' +
      '     |' +
      '     |' +
      '     |' +
      '     |' +
      '     |' +
      '     |' +
      '     |' +
      '     |' +
      '     |' +
      '     |' +
      '     |' + #13;

    TempStr1 := '';
    if StrToIntDef(Piece(Piece(ScoreString, RECORD_END, 23), '^', 4), 0) < 90 then
      TempStr1 := '       *MLS'
    else
      TempStr1 := '        MLS';
    if StrToIntDef(Piece(Piece(ScoreString, RECORD_END, 24), '^', 4), 0) < 90 then
      TempStr1 := TempStr1 + '  *GIC'
    else
      TempStr1 := TempStr1 + '   GIC';
    if StrToIntDef(Piece(Piece(ScoreString, RECORD_END, 25), '^', 4), 0) < 90 then
      TempStr1 := TempStr1 + '  *HPC'
    else
      TempStr1 := TempStr1 + '   HPC';
    if StrToIntDef(Piece(Piece(ScoreString, RECORD_END, 26), '^', 4), 0) < 90 then
      TempStr1 := TempStr1 + '  *NUC'
    else
      TempStr1 := TempStr1 + '   NUC';
    if StrToIntDef(Piece(Piece(ScoreString, RECORD_END, 27), '^', 4), 0) < 90 then
      TempStr1 := TempStr1 + '  *COG'
    else
      TempStr1 := TempStr1 + '   COG';
    if StrToIntDef(Piece(Piece(ScoreString, RECORD_END, 28), '^', 4), 0) < 90 then
      TempStr1 := TempStr1 + '  *SUI'
    else
      TempStr1 := TempStr1 + '   SUI';
    if StrToIntDef(Piece(Piece(ScoreString, RECORD_END, 29), '^', 4), 0) < 90 then
      TempStr1 := TempStr1 + '  *HLP'
    else
      TempStr1 := TempStr1 + '   HLP';
    if StrToIntDef(Piece(Piece(ScoreString, RECORD_END, 30), '^', 4), 0) < 90 then
      TempStr1 := TempStr1 + '  *SFD'
    else
      TempStr1 := TempStr1 + '   SFD';
    if StrToIntDef(Piece(Piece(ScoreString, RECORD_END, 31), '^', 4), 0) < 90 then
      TempStr1 := TempStr1 + '  *NFC'
    else
      TempStr1 := TempStr1 + '   NFC';
    if StrToIntDef(Piece(Piece(ScoreString, RECORD_END, 32), '^', 4), 0) < 90 then
      TempStr1 := TempStr1 + '  *STW'
    else
      TempStr1 := TempStr1 + '   STW';
    if StrToIntDef(Piece(Piece(ScoreString, RECORD_END, 33), '^', 4), 0) < 90 then
      TempStr1 := TempStr1 + '  *AXY'
    else
      TempStr1 := TempStr1 + '   AXY';
    if StrToIntDef(Piece(Piece(ScoreString, RECORD_END, 34), '^', 4), 0) < 90 then
      TempStr1 := TempStr1 + '  *ANP'
    else
      TempStr1 := TempStr1 + '   ANP';
    if StrToIntDef(Piece(Piece(ScoreString, RECORD_END, 35), '^', 4), 0) < 90 then
      TempStr1 := TempStr1 + '  *BRF'
    else
      TempStr1 := TempStr1 + '   BRF';
    if StrToIntDef(Piece(Piece(ScoreString, RECORD_END, 36), '^', 4), 0) < 90 then
      TempStr1 := TempStr1 + '  *MSF'
    else
      TempStr1 := TempStr1 + '   MSF';

    result := result + TempStr1 + #13 + #13;

    TempLine := '';
    TempLine := AddSpace('Raw', 8) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 23), '^', 2), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 24), '^', 2), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 25), '^', 2), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 26), '^', 2), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 27), '^', 2), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 28), '^', 2), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 29), '^', 2), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 30), '^', 2), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 31), '^', 2), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 32), '^', 2), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 33), '^', 2), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 34), '^', 2), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 35), '^', 2), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 36), '^', 2), 6);
    result := result + TempLine + #13;

    TempLine := '';
    TempLine := AddSpace('T score', 8) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 23), '^', 3), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 24), '^', 3), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 25), '^', 3), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 26), '^', 3), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 27), '^', 3), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 28), '^', 3), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 29), '^', 3), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 30), '^', 3), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 31), '^', 3), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 32), '^', 3), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 33), '^', 3), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 34), '^', 3), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 35), '^', 3), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 36), '^', 3), 6);
    result := result + TempLine + #13;

    TempLine := '';
    TempLine := AddSpace('Resp %', 8) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 23), '^', 4), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 24), '^', 4), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 25), '^', 4), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 26), '^', 4), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 27), '^', 4), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 28), '^', 4), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 29), '^', 4), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 30), '^', 4), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 31), '^', 4), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 32), '^', 4), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 33), '^', 4), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 34), '^', 4), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 35), '^', 4), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 36), '^', 4), 6);
    result := result + TempLine + #13 + #13;

    result := result +
      'The highest and lowest T scores possible on each scale are ' +
      'indicated by a "---"; T scores are non-gendered.' + #13;
    result := result + 'Scales marked with an asterisk have response percentages below 90%.' + #13 + #13;

    result := result +
      AddSpace('MLS', 4) + AddSpace('Malaise', 28) +
      AddSpace('SUI', 4) + AddSpace('Suicidal/Death Ideation', 26) + AddSpace('AXY', 4) + 'Anxiety' + #13 +
      AddSpace('GIC', 4) + AddSpace('Gastrointestinal Complaints', 28) +
      AddSpace('HLP', 4) + AddSpace('Helplessness/Hopelessness', 26) + AddSpace('ANP', 4) + 'Anger Proneness' + #13 +
      AddSpace('HPC', 4) + AddSpace('Head Pain Complaints', 28) +
      AddSpace('SFD', 4) + AddSpace('Self_Doubt', 26) + AddSpace('BRF', 4) + 'Behavior_Restricting Fears' + #13 +
      AddSpace('NUC', 4) + AddSpace('Neurological Complaints', 28) +
      AddSpace('NFC', 4) + AddSpace('Inefficacy', 26) + AddSpace('MSF', 4) + 'Multiple Specific Fears' + #13 +
      AddSpace('COG', 4) + AddSpace('Cognitive Complaints', 28) +
      AddSpace('STW', 4) + 'Stress/Worry' + #13 + #13;
  end;

  procedure DisplayEIIScales;
  var
    ii, jj, kk                : integer;
    JCP, SUB, AGG, ACT, FML, IPP, SAV, SHY, DSF, AES, MEC,
      Number,
      TempLine                : string;

    procedure DisplayMinMax;
    var
      kk                      : integer;
    begin
      for kk := 0 to 3 do
        begin
          if IntToStr(ii * 4 + 20 - kk) = JCP then
            begin
              TempLine[9] := '-';
              TempLine[10] := '-';
              TempLine[11] := '-';
            end;
          if IntToStr(ii * 4 + 20 - kk) = SUB then
            begin
              TempLine[15] := '-';
              TempLine[16] := '-';
              TempLine[17] := '-';
            end;
          if IntToStr(ii * 4 + 20 - kk) = AGG then
            begin
              TempLine[21] := '-';
              TempLine[22] := '-';
              TempLine[23] := '-';
            end;
          if IntToStr(ii * 4 + 20 - kk) = ACT then
            begin
              TempLine[27] := '-';
              TempLine[28] := '-';
              TempLine[29] := '-';
            end;
          if IntToStr(ii * 4 + 20 - kk) = FML then
            begin
              TempLine[33] := '-';
              TempLine[34] := '-';
              TempLine[35] := '-';
            end;
          if IntToStr(ii * 4 + 20 - kk) = IPP then
            begin
              TempLine[39] := '-';
              TempLine[40] := '-';
              TempLine[41] := '-';
            end;
          if IntToStr(ii * 4 + 20 - kk) = SAV then
            begin
              TempLine[45] := '-';
              TempLine[46] := '-';
              TempLine[47] := '-';
            end;
          if IntToStr(ii * 4 + 20 - kk) = SHY then
            begin
              TempLine[51] := '-';
              TempLine[52] := '-';
              TempLine[53] := '-';
            end;
          if IntToStr(ii * 4 + 20 - kk) = DSF then
            begin
              TempLine[57] := '-';
              TempLine[58] := '-';
              TempLine[59] := '-';
            end;
          if IntToStr(ii * 4 + 20 - kk) = AES then
            begin
              TempLine[63] := '-';
              TempLine[64] := '-';
              TempLine[65] := '-';
            end;
          if IntToStr(ii * 4 + 20 - kk) = MEC then
            begin
              TempLine[69] := '-';
              TempLine[70] := '-';
              TempLine[71] := '-';
            end;
        end; // kk
    end;

  begin

    result := result + EOP + #13 +
      '            MMPI-2-RF  EXTERNALIZING, INTERPERSONAL and INTEREST SCALES' + #13 + #13;
    result := result +
      '         Externalizing              Interpersonal              Interest' + #13;
    for ii := 25 downto 0 do
      begin
        TempLine := '';

        // Set t-score percentages, pips and line for y-axis.
        if (ii mod 5 = 0) then
          begin
            Number := IntToStr(ii * 4 + 20);
            if Length(Number) = 2 then
              Number := ' ' + Number + ' -|'
            else
              Number := Number + ' -|';
          end
        else
          Number := '     |';
        TempLine := Number;

        // patch 105 llh - 5/20/13 adding lines missed in original coding
        //if (ii = 8) or (ii = 0) then
        if (ii = 12) or (ii = 8) or (ii = 5) or (ii = 0) then
          for jj := 6 to 72 do
            TempLine := TempLine + '_'
        else
          for jj := 6 to 72 do
            TempLine := TempLine + ' ';

        TempLine[31] := '|';
        TempLine[61] := '|';

        // Max score
        JCP := '84';
        SUB := '93';
        AGG := '92';
        ACT := '83';
        FML := '90';
        IPP := '81';
        SAV := '80';
        SHY := '75';
        DSF := '100';
        AES := '73';
        MEC := '78';
        DisplayMinMax;

        // Min Score
        JCP := '40';
        SUB := '41';
        AGG := '37';
        ACT := '33';
        FML := '37';
        IPP := '34';
        SAV := '36';
        SHY := '37';
        DSF := '44';
        AES := '33';
        MEC := '38';
        DisplayMinMax;

        JCP := Piece(Piece(ScoreString, RECORD_END, 37), '^', 3);
        SUB := Piece(Piece(ScoreString, RECORD_END, 38), '^', 3);
        AGG := Piece(Piece(ScoreString, RECORD_END, 39), '^', 3);
        ACT := Piece(Piece(ScoreString, RECORD_END, 40), '^', 3);
        FML := Piece(Piece(ScoreString, RECORD_END, 41), '^', 3);
        IPP := Piece(Piece(ScoreString, RECORD_END, 42), '^', 3);
        SAV := Piece(Piece(ScoreString, RECORD_END, 43), '^', 3);
        SHY := Piece(Piece(ScoreString, RECORD_END, 44), '^', 3);
        DSF := Piece(Piece(ScoreString, RECORD_END, 45), '^', 3);
        AES := Piece(Piece(ScoreString, RECORD_END, 46), '^', 3);
        MEC := Piece(Piece(ScoreString, RECORD_END, 47), '^', 3);

        for kk := 0 to 3 do
          begin
            if IntToStr(ii * 4 + 20 - kk) = JCP then
              TempLine[10] := '*';
            if IntToStr(ii * 4 + 20 - kk) = SUB then
              TempLine[16] := '*';
            if IntToStr(ii * 4 + 20 - kk) = AGG then
              TempLine[22] := '*';
            if IntToStr(ii * 4 + 20 - kk) = ACT then
              TempLine[28] := '*';
            if IntToStr(ii * 4 + 20 - kk) = FML then
              TempLine[34] := '*';
            if IntToStr(ii * 4 + 20 - kk) = IPP then
              TempLine[40] := '*';
            if IntToStr(ii * 4 + 20 - kk) = SAV then
              TempLine[46] := '*';
            if IntToStr(ii * 4 + 20 - kk) = SHY then
              TempLine[52] := '*';
            if IntToStr(ii * 4 + 20 - kk) = DSF then
              TempLine[58] := '*';
            if IntToStr(ii * 4 + 20 - kk) = AES then
              TempLine[64] := '*';
            if IntToStr(ii * 4 + 20 - kk) = MEC then
              TempLine[70] := '*';
          end; // kk

        result := result + TempLine + #13;
      end;


    result := result +
      '         |' +
      '     |' +
      '     |' +
      '     |' +
      '     |' +
      '     |' +
      '     |' +
      '     |' +
      '     |' +
      '     |' +
      '     |' + #13;

    TempStr1 := '';
    if StrToIntDef(Piece(Piece(ScoreString, RECORD_END, 37), '^', 4), 0) < 90 then
      TempStr1 := '       *JCP'
    else
      TempStr1 := '        JCP';
    if StrToIntDef(Piece(Piece(ScoreString, RECORD_END, 38), '^', 4), 0) < 90 then
      TempStr1 := TempStr1 + '  *SUB'
    else
      TempStr1 := TempStr1 + '   SUB';
    if StrToIntDef(Piece(Piece(ScoreString, RECORD_END, 39), '^', 4), 0) < 90 then
      TempStr1 := TempStr1 + '  *AGG'
    else
      TempStr1 := TempStr1 + '   AGG';
    if StrToIntDef(Piece(Piece(ScoreString, RECORD_END, 40), '^', 4), 0) < 90 then
      TempStr1 := TempStr1 + '  *ACT'
    else
      TempStr1 := TempStr1 + '   ACT';
    if StrToIntDef(Piece(Piece(ScoreString, RECORD_END, 41), '^', 4), 0) < 90 then
      TempStr1 := TempStr1 + '   FML'
    else
      TempStr1 := TempStr1 + '   FML';
    if StrToIntDef(Piece(Piece(ScoreString, RECORD_END, 42), '^', 4), 0) < 90 then
      TempStr1 := TempStr1 + '  *IPP'
    else
      TempStr1 := TempStr1 + '   IPP';
    if StrToIntDef(Piece(Piece(ScoreString, RECORD_END, 43), '^', 4), 0) < 90 then
      TempStr1 := TempStr1 + '  *SAV'
    else
      TempStr1 := TempStr1 + '   SAV';
    if StrToIntDef(Piece(Piece(ScoreString, RECORD_END, 44), '^', 4), 0) < 90 then
      TempStr1 := TempStr1 + '  *SHY'
    else
      TempStr1 := TempStr1 + '   SHY';
    if StrToIntDef(Piece(Piece(ScoreString, RECORD_END, 45), '^', 4), 0) < 90 then
      TempStr1 := TempStr1 + '  *DFS'
    else
      TempStr1 := TempStr1 + '   DFS';
    if StrToIntDef(Piece(Piece(ScoreString, RECORD_END, 46), '^', 4), 0) < 90 then
      TempStr1 := TempStr1 + '  *AES'
    else
      TempStr1 := TempStr1 + '   AES';
    if StrToIntDef(Piece(Piece(ScoreString, RECORD_END, 47), '^', 4), 0) < 90 then
      TempStr1 := TempStr1 + '  *MEC'
    else
      TempStr1 := TempStr1 + '   MEC';

    result := result + TempStr1 + #13 + #13;

    TempLine := '';
    TempLine := AddSpace('Raw', 8) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 37), '^', 2), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 38), '^', 2), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 39), '^', 2), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 40), '^', 2), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 41), '^', 2), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 42), '^', 2), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 43), '^', 2), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 44), '^', 2), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 45), '^', 2), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 46), '^', 2), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 47), '^', 2), 6);
    result := result + TempLine + #13;

    TempLine := '';
    TempLine := AddSpace('T score', 8) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 37), '^', 3), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 38), '^', 3), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 39), '^', 3), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 40), '^', 3), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 41), '^', 3), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 42), '^', 3), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 43), '^', 3), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 44), '^', 3), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 45), '^', 3), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 46), '^', 3), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 47), '^', 3), 6);
    result := result + TempLine + #13;

    TempLine := '';
    TempLine := AddSpace('Resp %', 8) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 37), '^', 4), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 38), '^', 4), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 39), '^', 4), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 40), '^', 4), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 41), '^', 4), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 42), '^', 4), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 43), '^', 4), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 44), '^', 4), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 45), '^', 4), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 46), '^', 4), 6) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 47), '^', 4), 6);
    result := result + TempLine + #13 + #13;

    result := result +
      'The highest and lowest T scores possible on each scale are ' +
      'indicated by a "---"; T scores are non-gendered. ' + #13;
    result := result + 'Scales marked with an asterisk have response percentages below 90%.' + #13 + #13;

    result := result +

    AddSpace('JCP', 4) + AddSpace('Juvenile Conduct Problems', 26) +
      AddSpace('FML', 4) + AddSpace('Family Problems', 25) +
      AddSpace('AES', 4) + 'Aesthetic_Literary Interests' + #13 +
      AddSpace('SUB', 4) + AddSpace('Substance Abuse', 26) +
      AddSpace('IPP', 4) + AddSpace('Interpersonal Passivity', 25) +
      AddSpace('MEC', 4) + 'Mechanical_Physical Interests' + #13 +
      AddSpace('AGG', 4) + AddSpace('Aggression', 26) +
      AddSpace('SAV', 4) + AddSpace('Social Avoidance', 25) + #13 +
      AddSpace('ACT', 4) + AddSpace('Activation', 26) +
      AddSpace('SHY', 4) + 'Shyness' + #13 +
      AddSpace(' ', 30) + AddSpace('DSF', 4) + 'Disaffiliativeness' + #13;
  end;

  procedure DisplayPSYScales;
  var
    ii, jj, kk                : integer;
    AGGR, PSYC, DISC, NEGE, INTR,
      Number,
      TempLine                : string;

    procedure DisplayMinMax;
    var
      kk                      : integer;
    begin
      for kk := 0 to 3 do
        begin
          if IntToStr(ii * 4 + 20 - kk) = AGGR then
            begin
              TempLine[13] := '-';
              TempLine[14] := '-';
              TempLine[15] := '-';
            end;
          if IntToStr(ii * 4 + 20 - kk) = PSYC then
            begin
              TempLine[25] := '-';
              TempLine[26] := '-';
              TempLine[27] := '-';
            end;
          if IntToStr(ii * 4 + 20 - kk) = DISC then
            begin
              TempLine[37] := '-';
              TempLine[38] := '-';
              TempLine[39] := '-';
            end;
          if IntToStr(ii * 4 + 20 - kk) = NEGE then
            begin
              TempLine[49] := '-';
              TempLine[50] := '-';
              TempLine[51] := '-';
            end;
          if IntToStr(ii * 4 + 20 - kk) = INTR then
            begin
              TempLine[61] := '-';
              TempLine[62] := '-';
              TempLine[63] := '-';
            end;

        end; // kk
    end;

  begin
    result := result + EOP + #13 +
      '            MMPI-2-RF  PSY-5 SCALES' + #13 + #13;

    for ii := 25 downto 0 do
      begin
        TempLine := '';

       // Set t-score percentages, pips and line for y-axis.
        if (ii mod 5 = 0) then
          begin
            Number := IntToStr(ii * 4 + 20);
            if Length(Number) = 2 then
              Number := ' ' + Number + ' -|'
            else
              Number := Number + ' -|';
          end
        else
          Number := '     |';
        TempLine := Number;

        // patch 105 llh - 5/20/13 adding lines missed in original coding
        //if (ii = 8) or (ii = 0) then
        if (ii = 12) or (ii = 8) or (ii = 5) or (ii = 0) then

          for jj := 6 to 72 do
            TempLine := TempLine + '_'
        else
          for jj := 6 to 72 do
            TempLine := TempLine + ' ';

        // Max score
        AGGR := '88';
        PSYC := '100';
        DISC := '92';
        NEGE := '95';
        INTR := '93';
        DisplayMinMax;

        // Min Score
        AGGR := '28';
        PSYC := '38';
        DISC := '31';
        NEGE := '32';
        INTR := '32';
        DisplayMinMax;

        AGGR := Piece(Piece(ScoreString, RECORD_END, 48), '^', 3);
        PSYC := Piece(Piece(ScoreString, RECORD_END, 49), '^', 3);
        DISC := Piece(Piece(ScoreString, RECORD_END, 50), '^', 3);
        NEGE := Piece(Piece(ScoreString, RECORD_END, 51), '^', 3);
        INTR := Piece(Piece(ScoreString, RECORD_END, 52), '^', 3);

        for kk := 0 to 3 do
          begin
            if IntToStr(ii * 4 + 20 - kk) = AGGR then
              TempLine[14] := '*';
            if IntToStr(ii * 4 + 20 - kk) = PSYC then
              TempLine[26] := '*';
            if IntToStr(ii * 4 + 20 - kk) = DISC then
              TempLine[38] := '*';
            if IntToStr(ii * 4 + 20 - kk) = NEGE then
              TempLine[50] := '*';
            if IntToStr(ii * 4 + 20 - kk) = INTR then
              TempLine[62] := '*';
          end; // kk

        result := result + TempLine + #13;
      end;

    result := result +
      '             |' +
      '           |' +
      '           |' +
      '           |' +
      '           |' +
      #13;

    TempStr1 := '';
    if StrToIntDef(Piece(Piece(ScoreString, RECORD_END, 48), '^', 4), 0) < 90 then
      TempStr1 := '          *AGGR-r'
    else
      TempStr1 := '           AGGR-r';
    if StrToIntDef(Piece(Piece(ScoreString, RECORD_END, 49), '^', 4), 0) < 90 then
      TempStr1 := TempStr1 + '     *PSYC-r'
    else
      TempStr1 := TempStr1 + '      PSYC-r';
    if StrToIntDef(Piece(Piece(ScoreString, RECORD_END, 50), '^', 4), 0) < 90 then
      TempStr1 := TempStr1 + '     *DISC-r'
    else
      TempStr1 := TempStr1 + '      DISC-r';
    if StrToIntDef(Piece(Piece(ScoreString, RECORD_END, 51), '^', 4), 0) < 90 then
      TempStr1 := TempStr1 + '     *NEGE-r'
    else
      TempStr1 := TempStr1 + '      NEGE-r';
    if StrToIntDef(Piece(Piece(ScoreString, RECORD_END, 52), '^', 4), 0) < 90 then
      TempStr1 := TempStr1 + '     *INTR-r'
    else
      TempStr1 := TempStr1 + '      INTR-r';

    result := result + TempStr1 + #13 + #13;

    TempLine := '';
    TempLine := AddSpace('Raw Score:', 13) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 48), '^', 2), 12) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 49), '^', 2), 12) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 50), '^', 2), 12) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 51), '^', 2), 12) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 52), '^', 2), 12);

    result := result + TempLine + #13;

    TempLine := '';
    TempLine := AddSpace('T score:', 13) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 48), '^', 3), 12) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 49), '^', 3), 12) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 50), '^', 3), 12) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 51), '^', 3), 12) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 52), '^', 3), 12);

    result := result + TempLine + #13;

    TempLine := '';
    TempLine := AddSpace('Resp %', 13) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 48), '^', 4), 12) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 49), '^', 4), 12) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 50), '^', 4), 12) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 51), '^', 4), 12) +
      AddSpace(Piece(Piece(ScoreString, RECORD_END, 52), '^', 4), 12);

    result := result + TempLine + #13 + #13;

    result := result +
      'The highest and lowest T scores possible on each scale are ' +
      'indicated by a "---"; T scores are non-gendered.' + #13;
    result := result + 'Scales marked with an asterisk have response percentages below 90%.' + #13 + #13;

    result := result +
      AddSpace('AGGR-r', 8) + 'Aggressiveness-Revised' + #13 +
      AddSpace('PSYC-r', 8) + 'Psychoticism-Revised' + #13 +
      AddSpace('DISC-r', 8) + 'Disconstraint-Revised' + #13 +
      AddSpace('NEGE-r', 8) + 'Negative Emotionality/Neuroticism-Revised' + #13 +
      AddSpace('INTR-r', 8) + 'Introversion/Low Positive Emotionality-Revised' + #13;
  end;

begin
// The header, footer and questions and answers are handled else where; avoiding
// broker calls here. <.DLL_String.> provides location for this string in the
// report.
  Result := '';

  Invalid := false;
  ValidScale := false;

  if (ScoreString = '') then
    Exit;

// -------------- Validity Checks  ---------------------------------------------
  // Cannot Say
  TempStr := Piece(ScoreString, RECORD_END, 1);

  if StrToIntDef(Trim(Piece(TempStr, '^', 2)), 0) > 18 then
    begin
      result := result +
        'INVALID RESULTS ' + #13 + #13 +
        '   There are too many "Cannot Say" items for this administration ' +
        'to be considered valid. ' + Piece(TempStr, '^', 2) + ' items ' +
        'were skipped; 18 is the maximum allowed. ' + #13 + #13;
      Invalid := true;
    end;

   // VRIN-r T-score greater than 79
  TempStr := Piece(ScoreString, RECORD_END, 3);
  if StrToIntDef(Piece(TempStr, '^', 3), 0) > 79 then
    begin
      result := result +
        'INVALID RESULTS ' + #13 + #13 +
        '   The T score for the Variable Response Inconsistency Scale (VRIN-r) is equal to, or greater than, 80. ';
      result := result + #13 + #13;
      Invalid := true;
    end;

  // Is response percent below 70%
  if StrToIntDef(Piece(TempStr, '^', 4), 100) < 70 then
    ValidScale := true;

   // TRIN-r T-score greater than 79
  TempStr := Piece(ScoreString, RECORD_END, 4);
  if StrToIntDef(Piece(TempStr, '^', 3), 0) > 79 then
    begin
      result := result +
        'INVALID RESULTS ' + #13 + #13 +
        '   The T score for the True Response Inconsistency Scale (TRIN-r) is equal to, or greater than, 80. ';
      result := result + #13 + #13;
      Invalid := true;
    end;

      // Is response percent below 70%
  if StrToIntDef(Piece(TempStr, '^', 4), 100) < 70 then
    ValidScale := true;

   // F-r T-score equals 120
  TempStr := Piece(ScoreString, RECORD_END, 5);
  if StrToIntDef(Piece(TempStr, '^', 3), 0) = 120 then
    begin
      result := result +
        'INVALID RESULTS ' + #13 + #13 +
        '   The T score for the Infrequent Responses Scale (F-r) is equal to 120. ';
      result := result + #13 + #13;
      Invalid := true;
    end;

      // Is response percent below 70%
  if StrToIntDef(Piece(TempStr, '^', 4), 100) < 70 then
    ValidScale := true;

   // Fp-r T-score greater than 99
  TempStr := Piece(ScoreString, RECORD_END, 6);
  if StrToIntDef(Piece(TempStr, '^', 3), 0) > 99 then
    begin
      result := result +
        'INVALID RESULTS ' + #13 + #13 +
        '   The T score for the Infrequent Psychopathology Responses Scale (Fp-r) is equal to, or greater than, 100. ';
      result := result + #13 + #13;
      Invalid := true;
    end;

  // Is response percent below 70%
  if StrToIntDef(Piece(TempStr, '^', 4), 100) < 70 then
    ValidScale := true;

  if ValidScale then
    begin
      result := result +
        'INVALID RESULTS ' + #13 + #13 +
        '   One or more Validity Scale has a response percentage below 70%.';
      result := result + #13 + #13;
      Invalid := true;
    end;

  DisplayValidityScales;

//**
//  if not Invalid then
  begin
    DisplayHOandRCScales;
    DisplaySCIScales;
    DisplayEIIScales;
    DisplayPSYScales;

(*
    result := result + #13 + #13 + 'Number of "Cannot Say" responses: ' + Piece(Piece(ScoreString, RECORD_END, 1), '^', 2) + #13;
    result := result + 'Percent True responses of answered items: ' + Piece(Piece(ScoreString, RECORD_END, 2), '^', 2) + #13;
    result := result + 'T Scores are non-gendered.' + #13 + #13;

    for i := 3 to NumberofPieces(ScoreString, RECORD_END) - 3 do
      begin
        if i = 3 then
          begin
            result := result + #13 + #13 + 'Validity Scales' + #13;
            result := result + Header + #13;
          end;
        if i = 12 then
          begin
            result := result + #13 + #13 + 'Higher-Order (H-O) Scales' + #13;
            result := result + Header + #13;
          end;
        if i = 15 then
          begin
            result := result + #13 + #13 + 'Restructured Clinical (RC) Scales' + #13;
            result := result + Header + #13;
          end;
        if i = 24 then
          begin
            result := result + #13 + #13 + 'Somatic/Cognitive Scales' + #13;
            result := result + Header + #13;
          end;
        if i = 29 then
          begin
            result := result + #13 + #13 + 'Internalizing Scales' + #13;
            result := result + Header + #13;
          end;
        if i = 38 then
          begin
            result := result + #13 + #13 + 'Externalizing Scales' + #13;
            result := result + Header + #13;
          end;
        if i = 42 then
          begin
            result := result + #13 + #13 + 'Interpersonal Scales' + #13;
            result := result + Header + #13;
          end;
        if i = 47 then
          begin
            result := result + #13 + #13 + 'Interest Scales' + #13;
            result := result + Header + #13;
          end;
        if i = 49 then
          begin
            result := result + #13 + #13 + 'Personality Psychopathology Five (PSY-5) Scales' + #13;
            result := result + Header + #13;
          end;

        TempStr := Piece(ScoreString, RECORD_END, i);

        result := result + GetScores + #13;
      end;
  end;
*)
    result := result + #13 + #13 + EOP + #13 + #13;
  // SUI
    TempStr := Piece(ScoreString, RECORD_END, 28);
    if StrToIntDef(Piece(TempStr, '^', 3), 0) > 64 then
      result := result +
        'NOTE:  The T score for the Suicidal/Death Ideation Scale (SUI) is above 64.' + #13;

    result := result + #13 + #13 +
      'ITEM-LEVEL INFORMATION:' + #13 + #13;

   // Cannot Say
    TempStr := Piece(ScoreString, RECORD_END, 1);
    TempStr1 := Piece(TempStr, '^', 3);

    if TempStr1 = '' then
      result := result + #13 + #13 +
        'There were no unscorable responses to all the MMPI-2-RF items.' + #13 + #13
    else
      begin
        TempStr2 := Copy(TempStr1, 1, Length(TempStr1) - 2);
        result := result + #13 + #13 +
          'The "Cannot Say" questions are: ' + TempStr2 + '.' + #13 + #13;
      end;

  // Critical Items
    TempStr := Piece(ScoreString, RECORD_END, NumberofPieces(ScoreString, RECORD_END) - 2);
    if Length(TempStr) > 15 then
      begin
        // patch 105 llh - late in the game... change Items to Responses
        result := result + #13 + 'Critical Responses:' + #13;
        for i := 1 to NumberOfPieces(TempStr, '^') - 1 do
          result := result + '  ' + Piece(TempStr, '^', i) + #13;
      end;
  end;

  // Response summary
  result := result + #13 + #13 + #13;
  result := result + 'Responses: ' + #13;
//  result := result + Piece(ScoreString, RECORD_END, NumberofPieces(ScoreString, RECORD_END) - 1);
  i := 1;

  TempStr := Piece(ScoreString, RECORD_END, NumberofPieces(ScoreString, RECORD_END) - 1);
  TempStr2 := '  ' + Piece(TempStr, '.', 1);
  while i < 338 do
  begin
    result := result + #13 + TempStr2 +
      Piece(TempStr, '.', i + 1) +
      Piece(TempStr, '.', i + 2) + Piece(TempStr, '.', i + 3) +
      Piece(TempStr, '.', i + 4) + Piece(TempStr, '.', i + 5) +
      Piece(TempStr, '.', i + 6) + Piece(TempStr, '.', i + 7) +
      Piece(TempStr, '.', i + 8) + Piece(TempStr, '.', i + 9);
      TempStr1 := Piece(TempStr, '.', i + 10);
      TempStr2 := Copy(TempStr1, 3, Length(TempStr1));
      Result := result + Copy(TempStr1, 1, 2);
      i := i + 10;
  end;

(*
  result := result + #13 + #13 +
    'Scale abbreviation and spelled out name:' + #13 + #13 +
    Indent1 + 'Validity Scales' + #13 +
    Indent2 + 'VRIN-r Variable Response Inconsistency' + #13 +
    Indent2 + 'TRIN-r True Response Inconsistency' + #13 +
    Indent2 + 'F-r    Infrequent Responses' + #13 +
    Indent2 + 'Fp-r   Infrequent Psychopathology Responses' + #13 +
    Indent2 + 'Fs     Infrequent Somatic Responses' + #13 +
    Indent2 + 'FBS-r  Symptom Validity' + #13 +
    Indent2 + 'RBS    Response Bias Scale' + #13 +
    Indent2 + 'L-r    Uncommon Virtues' + #13 +
    Indent2 + 'K-r    Adjustment Validity' + #13 + #13 +
    Indent1 + 'Higher-Order (H-O) Scales' + #13 +
    Indent2 + 'EID  Emotional/Internalizing Dysfunction' + #13 +
    Indent2 + 'THD  Thought Dysfunction' + #13 +
    Indent2 + 'BXD  Behavioral/Externalizing Dysfunction' + #13 + #13 +
    Indent1 + 'Restructured Clinical (RC) Scales' + #13 +
    Indent2 + 'RCd  Demoralization (DEM)' + #13 +
    Indent2 + 'RC1  Somatic Complaints (SOM)' + #13 +
    Indent2 + 'RC2  Low Positive Emotions (LPE)' + #13 +
    Indent2 + 'RC3  Cynicism (CYN)' + #13 +
    Indent2 + 'RC4  Antisocial Behavior (ASB)' + #13 +
    Indent2 + 'RC6  Ideas of Persecution (PER)' + #13 +
    Indent2 + 'RC7  Dysfunctional Negative Emotions (DNE)' + #13 +
    Indent2 + 'RC8  Aberrant Experiences (ABX)' + #13 +
    Indent2 + 'RC9  Hypomanic Activation (HPM)' + #13 + #13 +
    Indent1 + 'Somatic/Cognitive Scales' + #13 +
    Indent2 + 'MLS  Malaise' + #13 +
    Indent2 + 'GIC  Gastrointestinal Complaints' + #13 +
    Indent2 + 'HPC  Head Pain Complaints' + #13 +
    Indent2 + 'NUC  Neurological Complaints' + #13 +
    Indent2 + 'COG  Cognitive Complaints' + #13 + #13 +
    Indent1 + 'Internalizing Scales' + #13 +
    Indent2 + 'SUI  Suicidal/Death Ideation' + #13 +
    Indent2 + 'HLP  Helplessness/Hopelessness' + #13 +
    Indent2 + 'SFD  Self_Doubt' + #13 +
    Indent2 + 'NFC  Inefficacy' + #13 +
    Indent2 + 'STW  Stress/Worry' + #13 +
    Indent2 + 'AXY  Anxiety' + #13 +
    Indent2 + 'ANP  Anger Proneness' + #13 +
    Indent2 + 'BRF  Behavior_Restricting Fears' + #13 +
    Indent2 + 'MSF  Multiple Specific Fears' + #13 + #13 +
    Indent1 + 'Externalizing Scales' + #13 +
    Indent2 + 'JCP  Juvenile Conduct Problems' + #13 +
    Indent2 + 'SUB  Substance Abuse' + #13 +
    Indent2 + 'AGG  Aggression' + #13 +
    Indent2 + 'ACT  Activation' + #13 + #13 +
    Indent1 + 'Interpersonal Scales' + #13 +
    Indent2 + 'FML  Family Problems' + #13 +
    Indent2 + 'IPP  Interpersonal Passivity' + #13 +
    Indent2 + 'SAV  Social Avoidance' + #13 +
    Indent2 + 'SHY  Shyness' + #13 +
    Indent2 + 'DSF  Disaffiliativeness' + #13 + #13 +
    Indent1 + 'Interest Scales' + #13 +
    Indent2 + 'AES  Aesthetic_Literary Interests' + #13 +
    Indent2 + 'MEC  Mechanical_Physical Interests' + #13 + #13 +
    Indent1 + 'Personality Psychopathology Five (PSY-5) Scales' + #13 +
    Indent2 + 'AGGR-r Aggressiveness-Revised' + #13 +
    Indent2 + 'PSYC-r Psychoticism-Revised' + #13 +
    Indent2 + 'DISC-r Disconstraint-Revised' + #13 +
    Indent2 + 'NEGE-r Negative Emotionality/Neuroticism-Revised' + #13 +
    Indent2 + 'INTR-r Introversion/Low Positive Emotionality-Revised' + #13;
    *)
end;




end.

